#!/usr/bin/env node
"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
const cli_options_1 = require("./cli-options");
const path = __importStar(require("path"));
const process_1 = require("process");
const save_declarations_1 = require("./lib/save-declarations");
const source_spec_1 = require("./lib/source-spec");
const logger_1 = require("./lib/logger");
if (cli_options_1.options.config) {
    const sourceSpecs = source_spec_1.convertConfigToSourceSpecs(cli_options_1.options.config);
    save_declarations_1.extractAndResolveDeclarations(sourceSpecs).then((declarations) => {
        if (cli_options_1.options.outputDir) {
            save_declarations_1.writeToFile(cli_options_1.options.outputDir, declarations, 'config-resolved', !cli_options_1.options.silent);
        }
        else {
            console.log(JSON.stringify(declarations));
        }
    });
}
else if (cli_options_1.options.help || !cli_options_1.options.sourceDir) {
    require('./cli-help');
}
else {
    const parserOptions = {
        eventPrefix: cli_options_1.options.eventPrefix,
        applyEndpoints: cli_options_1.options.applyEndpoints,
        patchDebugEvents: false,
        lowerCaseEvents: false,
        silenceOutput: cli_options_1.options.silenceOutput,
        verbose: cli_options_1.options.verbose
    };
    logger_1.logMessage('....running.', parserOptions.silenceOutput);
    const sourceSpec = {
        sourceDirs: resolveDirectories(cli_options_1.options.sourceDir),
        excludedDirs: cli_options_1.options.excludedDir ? resolveDirectories(cli_options_1.options.excludedDir) : [],
        parserOptions: parserOptions
    };
    save_declarations_1.extractAndResolveDeclarations([sourceSpec]).then((declarations) => {
        if (cli_options_1.options.outputDir) {
            save_declarations_1.writeToFile(cli_options_1.options.outputDir, declarations, 'declarations-resolved', !cli_options_1.options.silent);
        }
        else {
            console.log(JSON.stringify(declarations));
        }
    });
}
// Resolves an array of paths
function resolveDirectories(dirs, workingDir) {
    if (workingDir) {
        if (path.isAbsolute(workingDir)) {
            return dirs.map(s => path.resolve(workingDir, s));
        }
        else {
            return dirs.map(s => path.resolve(process_1.cwd(), workingDir, s));
        }
    }
    else {
        return dirs.map(s => path.resolve(process_1.cwd(), s));
    }
}
//# sourceMappingURL=extractor.js.map