"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
const fragments_1 = require("./fragments");
const events_1 = require("./events");
const common_properties_1 = require("./common-properties");
const keywords = __importStar(require("./keywords"));
function merge(target, source) {
    for (const item of source.dataPoints) {
        const found = target.dataPoints.find((f) => {
            return f.name == item.name;
        });
        // We combine their properties together if the event already exists
        if (found) {
            found.properties = found.properties.concat(item.properties);
        }
        else {
            target.dataPoints.push(item);
        }
    }
}
exports.merge = merge;
// Searches the object for an event or fragment of the specific name 
// If found returns, if not found creates it, places it in the array, and then returns
function findOrCreate(searchTarget, name) {
    let found = searchTarget.dataPoints.find((item) => {
        return item.name === name;
    });
    if (!found) {
        if (searchTarget instanceof events_1.Events) {
            found = new events_1.Event(name);
        }
        else {
            found = new fragments_1.Fragment(name);
        }
    }
    searchTarget.dataPoints.push(found);
    return found;
}
exports.findOrCreate = findOrCreate;
function mergeWildcards(wildcard, target, applyEndpoints) {
    let wildCard = target.properties.find((item) => {
        return item instanceof events_1.Wildcard;
    });
    // if we don't have a wildcard yet we make one
    if (!wildCard) {
        wildCard = new events_1.Wildcard();
        target.properties.push(wildCard);
    }
    wildcard.forEach(w => {
        if (applyEndpoints) {
            wildCard.entries.push(new events_1.WildcardEntry(w[keywords.prefix], w[keywords.classification], 'none'));
        }
        else {
            wildCard.entries.push(new events_1.WildcardEntry(w[keywords.prefix], w[keywords.classification]));
        }
    });
}
exports.mergeWildcards = mergeWildcards;
function populateProperties(properties, target, applyEndpoints = false) {
    for (const propertyName in properties) {
        const currentProperty = properties[propertyName];
        if (propertyName === keywords.include) {
            target.properties.push(new events_1.Include(currentProperty));
        }
        else if (currentProperty[keywords.inline]) {
            // We consider the property name the inline name so when we resolve we can do prop.Inline for the new properties
            target.properties.push(new events_1.Inline(propertyName, currentProperty[keywords.inline]));
        }
        else if (propertyName === keywords.wildcard) {
            mergeWildcards(currentProperty, target, applyEndpoints);
        }
        else {
            const prop = new common_properties_1.Property(propertyName, currentProperty.classification, currentProperty.purpose);
            if (applyEndpoints) {
                const endpoint = currentProperty.endpoint ? currentProperty.endpoint : 'none';
                prop.endPoint = endpoint;
            }
            if (currentProperty.isMeasurement) {
                prop.isMeasurement = currentProperty.isMeasurement;
            }
            target.properties.push(prop);
        }
    }
}
exports.populateProperties = populateProperties;
function makeExclusionsRelativeToSource(sourceDir, excludedDirs) {
    const relativeExclusions = [];
    for (const excluded of excludedDirs) {
        if (excluded.includes(sourceDir)) {
            relativeExclusions.push(excluded.replace(sourceDir, ''));
        }
    }
    return relativeExclusions;
}
exports.makeExclusionsRelativeToSource = makeExclusionsRelativeToSource;
//# sourceMappingURL=operations.js.map