"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
const fs_1 = require("fs");
const path = __importStar(require("path"));
const process_1 = require("process");
;
function convertConfigToSourceSpecs(file) {
    try {
        const config = JSON.parse(fs_1.readFileSync(file).toString());
        const sourceSpecs = [];
        for (const key in config) {
            const spec = config[key];
            // Some defaults
            spec.excludedDirs = spec.excludedDirs ? spec.excludedDirs : [];
            spec.workingDir = spec.workingDir ? spec.workingDir : process_1.cwd();
            spec.patchDebugEvents = spec.patchDebugEvents ? spec.patchDebugEvents : false;
            spec.lowerCaseEvents = spec.lowerCaseEvents ? spec.lowerCaseEvents : false;
            const parserOptions = {
                eventPrefix: spec.eventPrefix ? spec.eventPrefix : '',
                applyEndpoints: spec.applyEndpoints,
                patchDebugEvents: spec.patchDebugEvents,
                lowerCaseEvents: spec.lowerCaseEvents,
                silenceOutput: spec.silenceOuput,
                verbose: spec.verbose
            };
            const sourceSpec = {
                sourceDirs: resolveDirectories(spec.sourceDirs, spec.workingDir),
                excludedDirs: resolveDirectories(spec.excludedDirs, spec.workingDir),
                parserOptions: parserOptions
            };
            sourceSpecs.push(sourceSpec);
        }
        return sourceSpecs;
    }
    catch (err) {
        console.error(err);
        return [];
    }
}
exports.convertConfigToSourceSpecs = convertConfigToSourceSpecs;
// Resolves an array of paths
function resolveDirectories(dirs, workingDir) {
    if (workingDir) {
        if (path.isAbsolute(workingDir)) {
            return dirs.map(s => path.resolve(workingDir, s));
        }
        else {
            return dirs.map(s => path.resolve(process_1.cwd(), workingDir, s));
        }
    }
    else {
        return dirs.map(s => path.resolve(process_1.cwd(), s));
    }
}
//# sourceMappingURL=source-spec.js.map