"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("vs/base/common/path");
const uri_1 = require("vs/base/common/uri");
const errors_1 = require("vs/base/common/errors");
const platform_1 = require("vs/base/common/platform");
class DeferredPromise {
    constructor() {
        this.p = new Promise((c, e) => {
            this.completeCallback = c;
            this.errorCallback = e;
        });
    }
    complete(value) {
        return new Promise(resolve => {
            process.nextTick(() => {
                this.completeCallback(value);
                resolve();
            });
        });
    }
    error(err) {
        return new Promise(resolve => {
            process.nextTick(() => {
                this.errorCallback(err);
                resolve();
            });
        });
    }
    cancel() {
        process.nextTick(() => {
            this.errorCallback(errors_1.canceled());
        });
    }
}
exports.DeferredPromise = DeferredPromise;
function toResource(path) {
    if (platform_1.isWindows) {
        return uri_1.URI.file(path_1.join('C:\\', Buffer.from(this.test.fullTitle()).toString('base64'), path));
    }
    return uri_1.URI.file(path_1.join('/', Buffer.from(this.test.fullTitle()).toString('base64'), path));
}
exports.toResource = toResource;
function suiteRepeat(n, description, callback) {
    for (let i = 0; i < n; i++) {
        suite(`${description} (iteration ${i})`, callback);
    }
}
exports.suiteRepeat = suiteRepeat;
function testRepeat(n, description, callback) {
    for (let i = 0; i < n; i++) {
        test(`${description} (iteration ${i})`, callback);
    }
}
exports.testRepeat = testRepeat;
function testRepeatOnly(n, description, callback) {
    suite.only('repeat', () => testRepeat(n, description, callback));
}
exports.testRepeatOnly = testRepeatOnly;
//# sourceMappingURL=utils.js.map