"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
const platform = require("vs/base/common/platform");
suite('Keytar', () => {
    test('loads and is functional', function (done) {
        if (platform.isLinux) {
            // Skip test due to set up issue with Travis.
            this.skip();
            return;
        }
        (() => __awaiter(this, void 0, void 0, function* () {
            const keytar = yield Promise.resolve().then(() => require('keytar'));
            const name = `VSCode Test ${Math.floor(Math.random() * 1e9)}`;
            try {
                yield keytar.setPassword(name, 'foo', 'bar');
                assert.equal(yield keytar.findPassword(name), 'bar');
                assert.equal((yield keytar.findCredentials(name)).length, 1);
                assert.equal(yield keytar.getPassword(name, 'foo'), 'bar');
                yield keytar.deletePassword(name, 'foo');
                assert.equal(yield keytar.getPassword(name, 'foo'), undefined);
            }
            catch (err) {
                // try to clean up
                try {
                    yield keytar.deletePassword(name, 'foo');
                }
                finally {
                    // tslint:disable-next-line: no-unsafe-finally
                    throw err;
                }
            }
        }))().then(done, done);
    });
});
//# sourceMappingURL=keytar.test.js.map