"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const platform = require("vs/base/common/platform");
const product_1 = require("vs/platform/product/common/product");
const ipc_net_1 = require("vs/base/parts/ipc/node/ipc.net");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const descriptors_1 = require("vs/platform/instantiation/common/descriptors");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const environment_1 = require("vs/platform/environment/common/environment");
const environmentService_1 = require("vs/platform/environment/node/environmentService");
const extensionManagementIpc_1 = require("vs/platform/extensionManagement/common/extensionManagementIpc");
const extensionManagement_1 = require("vs/platform/extensionManagement/common/extensionManagement");
const extensionManagementService_1 = require("vs/platform/extensionManagement/node/extensionManagementService");
const extensionGalleryService_1 = require("vs/platform/extensionManagement/common/extensionGalleryService");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const configurationService_1 = require("vs/platform/configuration/node/configurationService");
const request_1 = require("vs/platform/request/common/request");
const requestService_1 = require("vs/platform/request/browser/requestService");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
const commonProperties_1 = require("vs/platform/telemetry/node/commonProperties");
const telemetryIpc_1 = require("vs/platform/telemetry/node/telemetryIpc");
const telemetryService_1 = require("vs/platform/telemetry/common/telemetryService");
const appInsightsAppender_1 = require("vs/platform/telemetry/node/appInsightsAppender");
const windows_1 = require("vs/platform/windows/common/windows");
const windowsService_1 = require("vs/platform/windows/electron-browser/windowsService");
const electron_1 = require("electron");
const log_1 = require("vs/platform/log/common/log");
const logIpc_1 = require("vs/platform/log/common/logIpc");
const localizations_1 = require("vs/platform/localizations/node/localizations");
const localizations_2 = require("vs/platform/localizations/common/localizations");
const localizationsIpc_1 = require("vs/platform/localizations/node/localizationsIpc");
const lifecycle_1 = require("vs/base/common/lifecycle");
const downloadService_1 = require("vs/platform/download/common/downloadService");
const download_1 = require("vs/platform/download/common/download");
const ipc_1 = require("vs/base/parts/ipc/common/ipc");
const nodeCachedDataCleaner_1 = require("vs/code/electron-browser/sharedProcess/contrib/nodeCachedDataCleaner");
const languagePackCachedDataCleaner_1 = require("vs/code/electron-browser/sharedProcess/contrib/languagePackCachedDataCleaner");
const storageDataCleaner_1 = require("vs/code/electron-browser/sharedProcess/contrib/storageDataCleaner");
const logsDataCleaner_1 = require("vs/code/electron-browser/sharedProcess/contrib/logsDataCleaner");
const mainProcessService_1 = require("vs/platform/ipc/electron-browser/mainProcessService");
const spdlogService_1 = require("vs/platform/log/node/spdlogService");
const diagnosticsService_1 = require("vs/platform/diagnostics/node/diagnosticsService");
const diagnosticsIpc_1 = require("vs/platform/diagnostics/node/diagnosticsIpc");
const fileService_1 = require("vs/platform/files/common/fileService");
const files_1 = require("vs/platform/files/common/files");
const diskFileSystemProvider_1 = require("vs/platform/files/electron-browser/diskFileSystemProvider");
const network_1 = require("vs/base/common/network");
const productService_1 = require("vs/platform/product/common/productService");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const userDataSyncService_1 = require("vs/platform/userDataSync/common/userDataSyncService");
const userDataSyncStoreService_1 = require("vs/platform/userDataSync/common/userDataSyncStoreService");
const userDataSyncIpc_1 = require("vs/platform/userDataSync/common/userDataSyncIpc");
const settingsSyncIpc_1 = require("vs/platform/userDataSync/common/settingsSyncIpc");
function startup(configuration) {
    handshake(configuration);
}
exports.startup = startup;
const eventPrefix = 'monacoworkbench';
class MainProcessService {
    constructor(server, mainRouter) {
        this.server = server;
        this.mainRouter = mainRouter;
    }
    getChannel(channelName) {
        return this.server.getChannel(channelName, this.mainRouter);
    }
    registerChannel(channelName, channel) {
        this.server.registerChannel(channelName, channel);
    }
}
function main(server, initData, configuration) {
    return __awaiter(this, void 0, void 0, function* () {
        const services = new serviceCollection_1.ServiceCollection();
        const disposables = new lifecycle_1.DisposableStore();
        const onExit = () => disposables.dispose();
        process.once('exit', onExit);
        electron_1.ipcRenderer.once('handshake:goodbye', onExit);
        disposables.add(server);
        const environmentService = new environmentService_1.EnvironmentService(initData.args, process.execPath);
        const mainRouter = new ipc_1.StaticRouter(ctx => ctx === 'main');
        const loggerClient = new logIpc_1.LoggerChannelClient(server.getChannel('logger', mainRouter));
        const logService = new logIpc_1.FollowerLogService(loggerClient, new spdlogService_1.SpdLogService('sharedprocess', environmentService.logsPath, initData.logLevel));
        disposables.add(logService);
        logService.info('main', JSON.stringify(configuration));
        const configurationService = new configurationService_1.ConfigurationService(environmentService.settingsResource);
        disposables.add(configurationService);
        yield configurationService.initialize();
        services.set(environment_1.IEnvironmentService, environmentService);
        services.set(productService_1.IProductService, Object.assign({ _serviceBrand: undefined }, product_1.default));
        services.set(log_1.ILogService, logService);
        services.set(configuration_1.IConfigurationService, configurationService);
        services.set(request_1.IRequestService, new descriptors_1.SyncDescriptor(requestService_1.RequestService));
        const mainProcessService = new MainProcessService(server, mainRouter);
        services.set(mainProcessService_1.IMainProcessService, mainProcessService);
        const windowsService = new windowsService_1.WindowsService(mainProcessService);
        services.set(windows_1.IWindowsService, windowsService);
        const activeWindowManager = new windows_1.ActiveWindowManager(windowsService);
        const activeWindowRouter = new ipc_1.StaticRouter(ctx => activeWindowManager.getActiveClientId().then(id => ctx === id));
        const settingsMergeChannel = server.getChannel('settingsMerge', activeWindowRouter);
        services.set(userDataSync_1.ISettingsMergeService, new settingsSyncIpc_1.SettingsMergeChannelClient(settingsMergeChannel));
        // Files
        const fileService = new fileService_1.FileService(logService);
        services.set(files_1.IFileService, fileService);
        disposables.add(fileService);
        const diskFileSystemProvider = new diskFileSystemProvider_1.DiskFileSystemProvider(logService);
        disposables.add(diskFileSystemProvider);
        fileService.registerProvider(network_1.Schemas.file, diskFileSystemProvider);
        services.set(download_1.IDownloadService, new descriptors_1.SyncDescriptor(downloadService_1.DownloadService));
        const instantiationService = new instantiationService_1.InstantiationService(services);
        let telemetryService;
        instantiationService.invokeFunction(accessor => {
            const services = new serviceCollection_1.ServiceCollection();
            const environmentService = accessor.get(environment_1.IEnvironmentService);
            const { appRoot, extensionsPath, extensionDevelopmentLocationURI: extensionDevelopmentLocationURI, isBuilt, installSourcePath } = environmentService;
            const telemetryLogService = new logIpc_1.FollowerLogService(loggerClient, new spdlogService_1.SpdLogService('telemetry', environmentService.logsPath, initData.logLevel));
            telemetryLogService.info('The below are logs for every telemetry event sent from VS Code once the log level is set to trace.');
            telemetryLogService.info('===========================================================');
            let appInsightsAppender = telemetryUtils_1.NullAppender;
            if (!extensionDevelopmentLocationURI && !environmentService.args['disable-telemetry'] && product_1.default.enableTelemetry) {
                if (product_1.default.aiConfig && product_1.default.aiConfig.asimovKey && isBuilt) {
                    appInsightsAppender = new appInsightsAppender_1.AppInsightsAppender(eventPrefix, null, product_1.default.aiConfig.asimovKey, telemetryLogService);
                    disposables.add(lifecycle_1.toDisposable(() => appInsightsAppender.flush())); // Ensure the AI appender is disposed so that it flushes remaining data
                }
                const config = {
                    appender: telemetryUtils_1.combinedAppender(appInsightsAppender, new telemetryUtils_1.LogAppender(logService)),
                    commonProperties: commonProperties_1.resolveCommonProperties(product_1.default.commit, product_1.default.version, configuration.machineId, product_1.default.msftInternalDomains, installSourcePath),
                    piiPaths: extensionsPath ? [appRoot, extensionsPath] : [appRoot]
                };
                telemetryService = new telemetryService_1.TelemetryService(config, configurationService);
                services.set(telemetry_1.ITelemetryService, telemetryService);
            }
            else {
                telemetryService = telemetryUtils_1.NullTelemetryService;
                services.set(telemetry_1.ITelemetryService, telemetryUtils_1.NullTelemetryService);
            }
            server.registerChannel('telemetryAppender', new telemetryIpc_1.TelemetryAppenderChannel(appInsightsAppender));
            services.set(extensionManagement_1.IExtensionManagementService, new descriptors_1.SyncDescriptor(extensionManagementService_1.ExtensionManagementService));
            services.set(extensionManagement_1.IExtensionGalleryService, new descriptors_1.SyncDescriptor(extensionGalleryService_1.ExtensionGalleryService));
            services.set(localizations_2.ILocalizationsService, new descriptors_1.SyncDescriptor(localizations_1.LocalizationsService));
            services.set(diagnosticsService_1.IDiagnosticsService, new descriptors_1.SyncDescriptor(diagnosticsService_1.DiagnosticsService));
            services.set(userDataSync_1.IUserDataSyncStoreService, new descriptors_1.SyncDescriptor(userDataSyncStoreService_1.UserDataSyncStoreService));
            services.set(userDataSync_1.IUserDataSyncService, new descriptors_1.SyncDescriptor(userDataSyncService_1.UserDataSyncService));
            const instantiationService2 = instantiationService.createChild(services);
            instantiationService2.invokeFunction(accessor => {
                const extensionManagementService = accessor.get(extensionManagement_1.IExtensionManagementService);
                const channel = new extensionManagementIpc_1.ExtensionManagementChannel(extensionManagementService, () => null);
                server.registerChannel('extensions', channel);
                const localizationsService = accessor.get(localizations_2.ILocalizationsService);
                const localizationsChannel = new localizationsIpc_1.LocalizationsChannel(localizationsService);
                server.registerChannel('localizations', localizationsChannel);
                const diagnosticsService = accessor.get(diagnosticsService_1.IDiagnosticsService);
                const diagnosticsChannel = new diagnosticsIpc_1.DiagnosticsChannel(diagnosticsService);
                server.registerChannel('diagnostics', diagnosticsChannel);
                const userDataSyncService = accessor.get(userDataSync_1.IUserDataSyncService);
                const userDataSyncChannel = new userDataSyncIpc_1.UserDataSyncChannel(userDataSyncService);
                server.registerChannel('userDataSync', userDataSyncChannel);
                // clean up deprecated extensions
                extensionManagementService.removeDeprecatedExtensions();
                // update localizations cache
                localizationsService.update();
                // cache clean ups
                disposables.add(lifecycle_1.combinedDisposable(instantiationService2.createInstance(nodeCachedDataCleaner_1.NodeCachedDataCleaner), instantiationService2.createInstance(languagePackCachedDataCleaner_1.LanguagePackCachedDataCleaner), instantiationService2.createInstance(storageDataCleaner_1.StorageDataCleaner), instantiationService2.createInstance(logsDataCleaner_1.LogsDataCleaner), instantiationService2.createInstance(userDataSyncService_1.UserDataAutoSync)));
                disposables.add(extensionManagementService);
            });
        });
    });
}
function setupIPC(hook) {
    function setup(retry) {
        return ipc_net_1.serve(hook).then(null, err => {
            if (!retry || platform.isWindows || err.code !== 'EADDRINUSE') {
                return Promise.reject(err);
            }
            // should retry, not windows and eaddrinuse
            return ipc_net_1.connect(hook, '').then(client => {
                // we could connect to a running instance. this is not good, abort
                client.dispose();
                return Promise.reject(new Error('There is an instance already running.'));
            }, err => {
                // it happens on Linux and OS X that the pipe is left behind
                // let's delete it, since we can't connect to it
                // and the retry the whole thing
                try {
                    fs.unlinkSync(hook);
                }
                catch (e) {
                    return Promise.reject(new Error('Error deleting the shared ipc hook.'));
                }
                return setup(false);
            });
        });
    }
    return setup(true);
}
function handshake(configuration) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = yield new Promise(c => {
            electron_1.ipcRenderer.once('handshake:hey there', (_, r) => c(r));
            electron_1.ipcRenderer.send('handshake:hello');
        });
        const server = yield setupIPC(data.sharedIPCHandle);
        yield main(server, data, configuration);
        electron_1.ipcRenderer.send('handshake:im ready');
    });
}
//# sourceMappingURL=sharedProcessMain.js.map