"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const platform_1 = require("vs/base/common/platform");
suite('Windows Native Helpers', () => {
    if (!platform_1.isWindows) {
        return;
    }
    test('windows-mutex', () => __awaiter(void 0, void 0, void 0, function* () {
        const mutex = yield Promise.resolve().then(() => require('windows-mutex'));
        assert.ok(mutex && typeof mutex.isActive === 'function', 'Unable to load windows-mutex dependency.');
        assert.ok(typeof mutex.isActive === 'function', 'Unable to load windows-mutex dependency.');
    }));
    test('windows-foreground-love', () => __awaiter(void 0, void 0, void 0, function* () {
        const foregroundLove = yield Promise.resolve().then(() => require('windows-foreground-love'));
        assert.ok(foregroundLove && typeof foregroundLove.allowSetForegroundWindow === 'function', 'Unable to load windows-foreground-love dependency.');
    }));
    test('windows-process-tree', () => __awaiter(void 0, void 0, void 0, function* () {
        const processTree = yield Promise.resolve().then(() => require('windows-process-tree'));
        assert.ok(processTree && typeof processTree.getProcessTree === 'function', 'Unable to load windows-process-tree dependency.');
    }));
    test('vscode-windows-ca-certs', () => __awaiter(void 0, void 0, void 0, function* () {
        const windowsCerts = yield Promise.resolve().then(() => require('vscode-windows-ca-certs'));
        assert.ok(windowsCerts, 'Unable to load vscode-windows-ca-certs dependency.');
    }));
    test('vscode-windows-registry', () => __awaiter(void 0, void 0, void 0, function* () {
        const windowsRegistry = yield Promise.resolve().then(() => require('vscode-windows-registry'));
        assert.ok(windowsRegistry && typeof windowsRegistry.GetStringRegKey === 'function', 'Unable to load vscode-windows-registry dependency.');
    }));
});
//# sourceMappingURL=nativeHelpers.test.js.map