"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const moveCaretCommand_1 = require("vs/editor/contrib/caretOperations/moveCaretCommand");
class MoveCaretAction extends editorExtensions_1.EditorAction {
    constructor(left, opts) {
        super(opts);
        this.left = left;
    }
    run(accessor, editor) {
        if (!editor.hasModel()) {
            return;
        }
        let commands = [];
        let selections = editor.getSelections();
        for (const selection of selections) {
            commands.push(new moveCaretCommand_1.MoveCaretCommand(selection, this.left));
        }
        editor.pushUndoStop();
        editor.executeCommands(this.id, commands);
        editor.pushUndoStop();
    }
}
class MoveCaretLeftAction extends MoveCaretAction {
    constructor() {
        super(true, {
            id: 'editor.action.moveCarretLeftAction',
            label: nls.localize('caret.moveLeft', "Move Caret Left"),
            alias: 'Move Caret Left',
            precondition: editorContextKeys_1.EditorContextKeys.writable
        });
    }
}
class MoveCaretRightAction extends MoveCaretAction {
    constructor() {
        super(false, {
            id: 'editor.action.moveCarretRightAction',
            label: nls.localize('caret.moveRight', "Move Caret Right"),
            alias: 'Move Caret Right',
            precondition: editorContextKeys_1.EditorContextKeys.writable
        });
    }
}
editorExtensions_1.registerEditorAction(MoveCaretLeftAction);
editorExtensions_1.registerEditorAction(MoveCaretRightAction);
//# sourceMappingURL=caretOperations.js.map