"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const selection_1 = require("vs/editor/common/core/selection");
const moveCaretCommand_1 = require("vs/editor/contrib/caretOperations/moveCaretCommand");
const testCommand_1 = require("vs/editor/test/browser/testCommand");
function testMoveCaretLeftCommand(lines, selection, expectedLines, expectedSelection) {
    testCommand_1.testCommand(lines, null, selection, (sel) => new moveCaretCommand_1.MoveCaretCommand(sel, true), expectedLines, expectedSelection);
}
function testMoveCaretRightCommand(lines, selection, expectedLines, expectedSelection) {
    testCommand_1.testCommand(lines, null, selection, (sel) => new moveCaretCommand_1.MoveCaretCommand(sel, false), expectedLines, expectedSelection);
}
suite('Editor Contrib - Move Caret Command', () => {
    test('move selection to left', function () {
        testMoveCaretLeftCommand([
            '012345'
        ], new selection_1.Selection(1, 3, 1, 5), [
            '023145'
        ], new selection_1.Selection(1, 2, 1, 4));
    });
    test('move selection to right', function () {
        testMoveCaretRightCommand([
            '012345'
        ], new selection_1.Selection(1, 3, 1, 5), [
            '014235'
        ], new selection_1.Selection(1, 4, 1, 6));
    });
    test('move selection to left - from first column - no change', function () {
        testMoveCaretLeftCommand([
            '012345'
        ], new selection_1.Selection(1, 1, 1, 1), [
            '012345'
        ], new selection_1.Selection(1, 1, 1, 1));
    });
    test('move selection to right - from last column - no change', function () {
        testMoveCaretRightCommand([
            '012345'
        ], new selection_1.Selection(1, 5, 1, 7), [
            '012345'
        ], new selection_1.Selection(1, 5, 1, 7));
    });
});
//# sourceMappingURL=moveCarretCommand.test.js.map