"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const arrays_1 = require("vs/base/common/arrays");
const cancellation_1 = require("vs/base/common/cancellation");
const errors_1 = require("vs/base/common/errors");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const modes_1 = require("vs/editor/common/modes");
function getDefinitions(model, position, registry, provide) {
    const provider = registry.ordered(model);
    // get results
    const promises = provider.map((provider) => {
        return Promise.resolve(provide(provider, model, position)).then(undefined, err => {
            errors_1.onUnexpectedExternalError(err);
            return undefined;
        });
    });
    return Promise.all(promises)
        .then(arrays_1.flatten)
        .then(arrays_1.coalesce);
}
function getDefinitionsAtPosition(model, position, token) {
    return getDefinitions(model, position, modes_1.DefinitionProviderRegistry, (provider, model, position) => {
        return provider.provideDefinition(model, position, token);
    });
}
exports.getDefinitionsAtPosition = getDefinitionsAtPosition;
function getDeclarationsAtPosition(model, position, token) {
    return getDefinitions(model, position, modes_1.DeclarationProviderRegistry, (provider, model, position) => {
        return provider.provideDeclaration(model, position, token);
    });
}
exports.getDeclarationsAtPosition = getDeclarationsAtPosition;
function getImplementationsAtPosition(model, position, token) {
    return getDefinitions(model, position, modes_1.ImplementationProviderRegistry, (provider, model, position) => {
        return provider.provideImplementation(model, position, token);
    });
}
exports.getImplementationsAtPosition = getImplementationsAtPosition;
function getTypeDefinitionsAtPosition(model, position, token) {
    return getDefinitions(model, position, modes_1.TypeDefinitionProviderRegistry, (provider, model, position) => {
        return provider.provideTypeDefinition(model, position, token);
    });
}
exports.getTypeDefinitionsAtPosition = getTypeDefinitionsAtPosition;
editorExtensions_1.registerDefaultLanguageCommand('_executeDefinitionProvider', (model, position) => getDefinitionsAtPosition(model, position, cancellation_1.CancellationToken.None));
editorExtensions_1.registerDefaultLanguageCommand('_executeDeclarationProvider', (model, position) => getDeclarationsAtPosition(model, position, cancellation_1.CancellationToken.None));
editorExtensions_1.registerDefaultLanguageCommand('_executeImplementationProvider', (model, position) => getImplementationsAtPosition(model, position, cancellation_1.CancellationToken.None));
editorExtensions_1.registerDefaultLanguageCommand('_executeTypeDefinitionProvider', (model, position) => getTypeDefinitionsAtPosition(model, position, cancellation_1.CancellationToken.None));
//# sourceMappingURL=goToDefinition.js.map