"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const arrays_1 = require("vs/base/common/arrays");
const cancellation_1 = require("vs/base/common/cancellation");
const errors_1 = require("vs/base/common/errors");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const modes_1 = require("vs/editor/common/modes");
function getHover(model, position, token) {
    const supports = modes_1.HoverProviderRegistry.ordered(model);
    const promises = supports.map(support => {
        return Promise.resolve(support.provideHover(model, position, token)).then(hover => {
            return hover && isValid(hover) ? hover : undefined;
        }, err => {
            errors_1.onUnexpectedExternalError(err);
            return undefined;
        });
    });
    return Promise.all(promises).then(arrays_1.coalesce);
}
exports.getHover = getHover;
editorExtensions_1.registerDefaultLanguageCommand('_executeHoverProvider', (model, position) => getHover(model, position, cancellation_1.CancellationToken.None));
function isValid(result) {
    const hasRange = (typeof result.range !== 'undefined');
    const hasHtmlContent = typeof result.contents !== 'undefined' && result.contents && result.contents.length > 0;
    return hasRange && hasHtmlContent;
}
//# sourceMappingURL=getHover.js.map