"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const markdownRenderer_1 = require("vs/base/browser/markdownRenderer");
const opener_1 = require("vs/platform/opener/common/opener");
const modeService_1 = require("vs/editor/common/services/modeService");
const uri_1 = require("vs/base/common/uri");
const errors_1 = require("vs/base/common/errors");
const textToHtmlTokenizer_1 = require("vs/editor/common/modes/textToHtmlTokenizer");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const event_1 = require("vs/base/common/event");
const lifecycle_1 = require("vs/base/common/lifecycle");
const modes_1 = require("vs/editor/common/modes");
let MarkdownRenderer = class MarkdownRenderer extends lifecycle_1.Disposable {
    constructor(_editor, _modeService, _openerService = opener_1.NullOpenerService) {
        super();
        this._editor = _editor;
        this._modeService = _modeService;
        this._openerService = _openerService;
        this._onDidRenderCodeBlock = this._register(new event_1.Emitter());
        this.onDidRenderCodeBlock = this._onDidRenderCodeBlock.event;
    }
    getOptions(disposeables) {
        return {
            codeBlockRenderer: (languageAlias, value) => {
                // In markdown,
                // it is possible that we stumble upon language aliases (e.g.js instead of javascript)
                // it is possible no alias is given in which case we fall back to the current editor lang
                let modeId = null;
                if (languageAlias) {
                    modeId = this._modeService.getModeIdForLanguageName(languageAlias);
                }
                else {
                    const model = this._editor.getModel();
                    if (model) {
                        modeId = model.getLanguageIdentifier().language;
                    }
                }
                this._modeService.triggerMode(modeId || '');
                return Promise.resolve(true).then(_ => {
                    const promise = modes_1.TokenizationRegistry.getPromise(modeId || '');
                    if (promise) {
                        return promise.then(support => textToHtmlTokenizer_1.tokenizeToString(value, support));
                    }
                    return textToHtmlTokenizer_1.tokenizeToString(value, undefined);
                }).then(code => {
                    return `<span style="font-family: ${this._editor.getOption(30 /* fontInfo */).fontFamily}">${code}</span>`;
                });
            },
            codeBlockRenderCallback: () => this._onDidRenderCodeBlock.fire(),
            actionHandler: {
                callback: (content) => {
                    let uri;
                    try {
                        uri = uri_1.URI.parse(content);
                    }
                    catch (_a) {
                        // ignore
                    }
                    if (uri && this._openerService) {
                        this._openerService.open(uri).catch(errors_1.onUnexpectedError);
                    }
                },
                disposeables
            }
        };
    }
    render(markdown) {
        const disposeables = new lifecycle_1.DisposableStore();
        let element;
        if (!markdown) {
            element = document.createElement('span');
        }
        else {
            element = markdownRenderer_1.renderMarkdown(markdown, this.getOptions(disposeables));
        }
        return {
            element,
            dispose: () => disposeables.dispose()
        };
    }
};
MarkdownRenderer = __decorate([
    __param(1, modeService_1.IModeService),
    __param(2, instantiation_1.optional(opener_1.IOpenerService))
], MarkdownRenderer);
exports.MarkdownRenderer = MarkdownRenderer;
//# sourceMappingURL=markdownRenderer.js.map