"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("vs/base/common/errors");
const uri_1 = require("vs/base/common/uri");
const range_1 = require("vs/editor/common/core/range");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const modelService_1 = require("vs/editor/common/services/modelService");
const cancellation_1 = require("vs/base/common/cancellation");
const resolverService_1 = require("vs/editor/common/services/resolverService");
const outlineModel_1 = require("vs/editor/contrib/documentSymbols/outlineModel");
const collections_1 = require("vs/base/common/collections");
function getDocumentSymbols(document, flat, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const model = yield outlineModel_1.OutlineModel.create(document, token);
        const roots = [];
        for (const child of collections_1.values(model.children)) {
            if (child instanceof outlineModel_1.OutlineElement) {
                roots.push(child.symbol);
            }
            else {
                roots.push(...collections_1.values(child.children).map(child => child.symbol));
            }
        }
        let flatEntries = [];
        if (token.isCancellationRequested) {
            return flatEntries;
        }
        if (flat) {
            flatten(flatEntries, roots, '');
        }
        else {
            flatEntries = roots;
        }
        return flatEntries.sort(compareEntriesUsingStart);
    });
}
exports.getDocumentSymbols = getDocumentSymbols;
function compareEntriesUsingStart(a, b) {
    return range_1.Range.compareRangesUsingStarts(a.range, b.range);
}
function flatten(bucket, entries, overrideContainerLabel) {
    for (let entry of entries) {
        bucket.push({
            kind: entry.kind,
            tags: entry.tags,
            name: entry.name,
            detail: entry.detail,
            containerName: entry.containerName || overrideContainerLabel,
            range: entry.range,
            selectionRange: entry.selectionRange,
            children: undefined,
        });
        if (entry.children) {
            flatten(bucket, entry.children, entry.name);
        }
    }
}
editorExtensions_1.registerLanguageCommand('_executeDocumentSymbolProvider', function (accessor, args) {
    const { resource } = args;
    if (!(resource instanceof uri_1.URI)) {
        throw errors_1.illegalArgument('resource');
    }
    const model = accessor.get(modelService_1.IModelService).getModel(resource);
    if (model) {
        return getDocumentSymbols(model, false, cancellation_1.CancellationToken.None);
    }
    return accessor.get(resolverService_1.ITextModelService).createModelReference(resource).then(reference => {
        return new Promise((resolve, reject) => {
            try {
                const result = getDocumentSymbols(reference.object.textEditorModel, false, cancellation_1.CancellationToken.None);
                resolve(result);
            }
            catch (err) {
                reject(err);
            }
        }).finally(() => {
            reference.dispose();
        });
    });
});
//# sourceMappingURL=quickOpen.js.map