"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
const uri_1 = require("vs/base/common/uri");
const modes_1 = require("vs/editor/common/modes");
const suggest_1 = require("vs/editor/contrib/suggest/suggest");
const position_1 = require("vs/editor/common/core/position");
const textModel_1 = require("vs/editor/common/model/textModel");
const range_1 = require("vs/editor/common/core/range");
suite('Suggest', function () {
    let model;
    let registration;
    setup(function () {
        model = textModel_1.TextModel.createFromString('FOO\nbar\BAR\nfoo', undefined, undefined, uri_1.URI.parse('foo:bar/path'));
        registration = modes_1.CompletionProviderRegistry.register({ pattern: 'bar/path', scheme: 'foo' }, {
            provideCompletionItems(_doc, pos) {
                return {
                    incomplete: false,
                    suggestions: [{
                            label: 'aaa',
                            kind: 25 /* Snippet */,
                            insertText: 'aaa',
                            range: range_1.Range.fromPositions(pos)
                        }, {
                            label: 'zzz',
                            kind: 25 /* Snippet */,
                            insertText: 'zzz',
                            range: range_1.Range.fromPositions(pos)
                        }, {
                            label: 'fff',
                            kind: 9 /* Property */,
                            insertText: 'fff',
                            range: range_1.Range.fromPositions(pos)
                        }]
                };
            }
        });
    });
    teardown(() => {
        registration.dispose();
        model.dispose();
    });
    test('sort - snippet inline', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const items = yield suggest_1.provideSuggestionItems(model, new position_1.Position(1, 1), new suggest_1.CompletionOptions(1 /* Inline */));
            assert.equal(items.length, 3);
            assert.equal(items[0].completion.label, 'aaa');
            assert.equal(items[1].completion.label, 'fff');
            assert.equal(items[2].completion.label, 'zzz');
        });
    });
    test('sort - snippet top', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const items = yield suggest_1.provideSuggestionItems(model, new position_1.Position(1, 1), new suggest_1.CompletionOptions(0 /* Top */));
            assert.equal(items.length, 3);
            assert.equal(items[0].completion.label, 'aaa');
            assert.equal(items[1].completion.label, 'zzz');
            assert.equal(items[2].completion.label, 'fff');
        });
    });
    test('sort - snippet bottom', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const items = yield suggest_1.provideSuggestionItems(model, new position_1.Position(1, 1), new suggest_1.CompletionOptions(2 /* Bottom */));
            assert.equal(items.length, 3);
            assert.equal(items[0].completion.label, 'fff');
            assert.equal(items[1].completion.label, 'aaa');
            assert.equal(items[2].completion.label, 'zzz');
        });
    });
    test('sort - snippet none', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const items = yield suggest_1.provideSuggestionItems(model, new position_1.Position(1, 1), new suggest_1.CompletionOptions(undefined, new Set().add(25 /* Snippet */)));
            assert.equal(items.length, 1);
            assert.equal(items[0].completion.label, 'fff');
        });
    });
    test('only from', function () {
        const foo = {
            triggerCharacters: [],
            provideCompletionItems() {
                return {
                    currentWord: '',
                    incomplete: false,
                    suggestions: [{
                            label: 'jjj',
                            type: 'property',
                            insertText: 'jjj'
                        }]
                };
            }
        };
        const registration = modes_1.CompletionProviderRegistry.register({ pattern: 'bar/path', scheme: 'foo' }, foo);
        suggest_1.provideSuggestionItems(model, new position_1.Position(1, 1), new suggest_1.CompletionOptions(undefined, undefined, new Set().add(foo))).then(items => {
            registration.dispose();
            assert.equal(items.length, 1);
            assert.ok(items[0].provider === foo);
        });
    });
});
//# sourceMappingURL=suggest.test.js.map