"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const dom_1 = require("vs/base/browser/dom");
const network_1 = require("vs/base/common/network");
const codeEditorServiceImpl_1 = require("vs/editor/browser/services/codeEditorServiceImpl");
class StandaloneCodeEditorServiceImpl extends codeEditorServiceImpl_1.CodeEditorServiceImpl {
    getActiveCodeEditor() {
        return undefined; // not supported in the standalone case
    }
    openCodeEditor(input, source, sideBySide) {
        if (!source) {
            return Promise.resolve(undefined);
        }
        return Promise.resolve(this.doOpenEditor(source, input));
    }
    doOpenEditor(editor, input) {
        const model = this.findModel(editor, input.resource);
        if (!model) {
            if (input.resource) {
                const schema = input.resource.scheme;
                if (schema === network_1.Schemas.http || schema === network_1.Schemas.https) {
                    // This is a fully qualified http or https URL
                    dom_1.windowOpenNoOpener(input.resource.toString());
                    return editor;
                }
            }
            return undefined;
        }
        const selection = (input.options ? input.options.selection : null);
        if (selection) {
            if (typeof selection.endLineNumber === 'number' && typeof selection.endColumn === 'number') {
                editor.setSelection(selection);
                editor.revealRangeInCenter(selection, 1 /* Immediate */);
            }
            else {
                const pos = {
                    lineNumber: selection.startLineNumber,
                    column: selection.startColumn
                };
                editor.setPosition(pos);
                editor.revealPositionInCenter(pos, 1 /* Immediate */);
            }
        }
        return editor;
    }
    findModel(editor, resource) {
        const model = editor.getModel();
        if (model && model.uri.toString() !== resource.toString()) {
            return null;
        }
        return model;
    }
}
exports.StandaloneCodeEditorServiceImpl = StandaloneCodeEditorServiceImpl;
//# sourceMappingURL=standaloneCodeServiceImpl.js.map