"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
const simpleServices_1 = require("vs/editor/standalone/browser/simpleServices");
const contextKeyService_1 = require("vs/platform/contextkey/browser/contextKeyService");
const instantiationService_1 = require("vs/platform/instantiation/common/instantiationService");
const serviceCollection_1 = require("vs/platform/instantiation/common/serviceCollection");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
suite('StandaloneKeybindingService', () => {
    class TestStandaloneKeybindingService extends simpleServices_1.StandaloneKeybindingService {
        testDispatch(e) {
            super._dispatch(e, null);
        }
    }
    test('issue Microsoft/monaco-editor#167', () => {
        let serviceCollection = new serviceCollection_1.ServiceCollection();
        const instantiationService = new instantiationService_1.InstantiationService(serviceCollection, true);
        let configurationService = new simpleServices_1.SimpleConfigurationService();
        let contextKeyService = new contextKeyService_1.ContextKeyService(configurationService);
        let commandService = new simpleServices_1.StandaloneCommandService(instantiationService);
        let notificationService = new simpleServices_1.SimpleNotificationService();
        let domElement = document.createElement('div');
        let keybindingService = new TestStandaloneKeybindingService(contextKeyService, commandService, telemetryUtils_1.NullTelemetryService, notificationService, domElement);
        let commandInvoked = false;
        keybindingService.addDynamicKeybinding('testCommand', 67 /* F9 */, () => {
            commandInvoked = true;
        }, undefined);
        keybindingService.testDispatch({
            _standardKeyboardEventBrand: true,
            ctrlKey: false,
            shiftKey: false,
            altKey: false,
            metaKey: false,
            keyCode: 67 /* F9 */,
            code: null
        });
        assert.ok(commandInvoked, 'command invoked');
    });
});
//# sourceMappingURL=simpleServices.test.js.map