"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const trimTrailingWhitespaceCommand_1 = require("vs/editor/common/commands/trimTrailingWhitespaceCommand");
const position_1 = require("vs/editor/common/core/position");
const range_1 = require("vs/editor/common/core/range");
const selection_1 = require("vs/editor/common/core/selection");
const testCommand_1 = require("vs/editor/test/browser/testCommand");
const editorTestUtils_1 = require("vs/editor/test/common/editorTestUtils");
/**
 * Create single edit operation
 */
function createInsertDeleteSingleEditOp(text, positionLineNumber, positionColumn, selectionLineNumber = positionLineNumber, selectionColumn = positionColumn) {
    return {
        range: new range_1.Range(selectionLineNumber, selectionColumn, positionLineNumber, positionColumn),
        text: text
    };
}
/**
 * Create single edit operation
 */
function createSingleEditOp(text, positionLineNumber, positionColumn, selectionLineNumber = positionLineNumber, selectionColumn = positionColumn) {
    return {
        range: new range_1.Range(selectionLineNumber, selectionColumn, positionLineNumber, positionColumn),
        text: text
    };
}
exports.createSingleEditOp = createSingleEditOp;
function assertTrimTrailingWhitespaceCommand(text, expected) {
    return editorTestUtils_1.withEditorModel(text, (model) => {
        let op = new trimTrailingWhitespaceCommand_1.TrimTrailingWhitespaceCommand(new selection_1.Selection(1, 1, 1, 1), []);
        let actual = testCommand_1.getEditOperation(model, op);
        assert.deepEqual(actual, expected);
    });
}
function assertTrimTrailingWhitespace(text, cursors, expected) {
    return editorTestUtils_1.withEditorModel(text, (model) => {
        let actual = trimTrailingWhitespaceCommand_1.trimTrailingWhitespace(model, cursors);
        assert.deepEqual(actual, expected);
    });
}
suite('Editor Commands - Trim Trailing Whitespace Command', () => {
    test('remove trailing whitespace', function () {
        assertTrimTrailingWhitespaceCommand([''], []);
        assertTrimTrailingWhitespaceCommand(['text'], []);
        assertTrimTrailingWhitespaceCommand(['text   '], [createSingleEditOp(null, 1, 5, 1, 8)]);
        assertTrimTrailingWhitespaceCommand(['text\t   '], [createSingleEditOp(null, 1, 5, 1, 9)]);
        assertTrimTrailingWhitespaceCommand(['\t   '], [createSingleEditOp(null, 1, 1, 1, 5)]);
        assertTrimTrailingWhitespaceCommand(['text\t'], [createSingleEditOp(null, 1, 5, 1, 6)]);
        assertTrimTrailingWhitespaceCommand([
            'some text\t',
            'some more text',
            '\t  ',
            'even more text  ',
            'and some mixed\t   \t'
        ], [
            createSingleEditOp(null, 1, 10, 1, 11),
            createSingleEditOp(null, 3, 1, 3, 4),
            createSingleEditOp(null, 4, 15, 4, 17),
            createSingleEditOp(null, 5, 15, 5, 20)
        ]);
        assertTrimTrailingWhitespace(['text   '], [new position_1.Position(1, 1), new position_1.Position(1, 2), new position_1.Position(1, 3)], [createInsertDeleteSingleEditOp(null, 1, 5, 1, 8)]);
        assertTrimTrailingWhitespace(['text   '], [new position_1.Position(1, 1), new position_1.Position(1, 5)], [createInsertDeleteSingleEditOp(null, 1, 5, 1, 8)]);
        assertTrimTrailingWhitespace(['text   '], [new position_1.Position(1, 1), new position_1.Position(1, 5), new position_1.Position(1, 6)], [createInsertDeleteSingleEditOp(null, 1, 6, 1, 8)]);
        assertTrimTrailingWhitespace([
            'some text\t',
            'some more text',
            '\t  ',
            'even more text  ',
            'and some mixed\t   \t'
        ], [], [
            createInsertDeleteSingleEditOp(null, 1, 10, 1, 11),
            createInsertDeleteSingleEditOp(null, 3, 1, 3, 4),
            createInsertDeleteSingleEditOp(null, 4, 15, 4, 17),
            createInsertDeleteSingleEditOp(null, 5, 15, 5, 20)
        ]);
        assertTrimTrailingWhitespace([
            'some text\t',
            'some more text',
            '\t  ',
            'even more text  ',
            'and some mixed\t   \t'
        ], [new position_1.Position(1, 11), new position_1.Position(3, 2), new position_1.Position(5, 1), new position_1.Position(4, 1), new position_1.Position(5, 10)], [
            createInsertDeleteSingleEditOp(null, 3, 2, 3, 4),
            createInsertDeleteSingleEditOp(null, 4, 15, 4, 17),
            createInsertDeleteSingleEditOp(null, 5, 15, 5, 20)
        ]);
    });
});
//# sourceMappingURL=trimTrailingWhitespaceCommand.test.js.map