"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const windows_1 = require("vs/platform/windows/electron-main/windows");
const electron_1 = require("electron");
const lifecycleMainService_1 = require("vs/platform/lifecycle/electron-main/lifecycleMainService");
const platform_1 = require("vs/base/common/platform");
let ElectronMainService = class ElectronMainService {
    constructor(windowsMainService, lifecycleMainService) {
        this.windowsMainService = windowsMainService;
        this.lifecycleMainService = lifecycleMainService;
    }
    //#region Window
    windowCount(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.windowsMainService.getWindowCount();
        });
    }
    openEmptyWindow(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.windowsMainService.openEmptyWindow(5 /* API */, options);
        });
    }
    toggleFullScreen(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                window.toggleFullScreen();
            }
        });
    }
    //#endregion
    //#region Dialog
    showMessageBox(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.windowsMainService.showMessageBox(options, this.windowsMainService.getWindowById(windowId));
        });
    }
    showSaveDialog(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.windowsMainService.showSaveDialog(options, this.windowsMainService.getWindowById(windowId));
        });
    }
    showOpenDialog(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.windowsMainService.showOpenDialog(options, this.windowsMainService.getWindowById(windowId));
        });
    }
    pickFileFolderAndOpen(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options.windowId = windowId;
            return this.windowsMainService.pickFileFolderAndOpen(options);
        });
    }
    pickFileAndOpen(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options.windowId = windowId;
            return this.windowsMainService.pickFileAndOpen(options);
        });
    }
    pickFolderAndOpen(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options.windowId = windowId;
            return this.windowsMainService.pickFolderAndOpen(options);
        });
    }
    pickWorkspaceAndOpen(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            options.windowId = windowId;
            return this.windowsMainService.pickWorkspaceAndOpen(options);
        });
    }
    //#endregion
    //#region OS
    showItemInFolder(windowId, path) {
        return __awaiter(this, void 0, void 0, function* () {
            electron_1.shell.showItemInFolder(path);
        });
    }
    setRepresentedFilename(windowId, path) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                window.setRepresentedFilename(path);
            }
        });
    }
    setDocumentEdited(windowId, edited) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                window.win.setDocumentEdited(edited);
            }
        });
    }
    //#endregion
    //#region Lifecycle
    relaunch(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.lifecycleMainService.relaunch(options);
        });
    }
    reload(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                return this.windowsMainService.reload(window);
            }
        });
    }
    //#endregion
    //#region Development
    openDevTools(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                window.win.webContents.openDevTools(options);
            }
        });
    }
    toggleDevTools(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            const window = this.windowsMainService.getWindowById(windowId);
            if (window) {
                const contents = window.win.webContents;
                if (platform_1.isMacintosh && window.hasHiddenTitleBarStyle() && !window.isFullScreen() && !contents.isDevToolsOpened()) {
                    contents.openDevTools({ mode: 'undocked' }); // due to https://github.com/electron/electron/issues/3647
                }
                else {
                    contents.toggleDevTools();
                }
            }
        });
    }
    //#endregion
    //#region Connectivity
    resolveProxy(windowId, url) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                const window = this.windowsMainService.getWindowById(windowId);
                if (window && window.win && window.win.webContents && window.win.webContents.session) {
                    window.win.webContents.session.resolveProxy(url, proxy => resolve(proxy));
                }
                else {
                    resolve();
                }
            });
        });
    }
};
ElectronMainService = __decorate([
    __param(0, windows_1.IWindowsMainService),
    __param(1, lifecycleMainService_1.ILifecycleMainService)
], ElectronMainService);
exports.ElectronMainService = ElectronMainService;
//# sourceMappingURL=electronMainService.js.map