"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const os = require("os");
const environmentService_1 = require("vs/platform/environment/node/environmentService");
const argv_1 = require("vs/platform/environment/node/argv");
const testUtils_1 = require("vs/base/test/node/testUtils");
const path_1 = require("vs/base/common/path");
const pfs_1 = require("vs/base/node/pfs");
const extensionGalleryService_1 = require("vs/platform/extensionManagement/common/extensionGalleryService");
const uuid_1 = require("vs/base/common/uuid");
const lifecycle_1 = require("vs/base/common/lifecycle");
const fileService_1 = require("vs/platform/files/common/fileService");
const log_1 = require("vs/platform/log/common/log");
const diskFileSystemProvider_1 = require("vs/platform/files/node/diskFileSystemProvider");
const network_1 = require("vs/base/common/network");
const product_1 = require("vs/platform/product/common/product");
suite('Extension Gallery Service', () => {
    const parentDir = testUtils_1.getRandomTestPath(os.tmpdir(), 'vsctests', 'extensiongalleryservice');
    const marketplaceHome = path_1.join(parentDir, 'Marketplace');
    let fileService;
    let disposables;
    setup(done => {
        disposables = new lifecycle_1.DisposableStore();
        fileService = new fileService_1.FileService(new log_1.NullLogService());
        disposables.add(fileService);
        const diskFileSystemProvider = new diskFileSystemProvider_1.DiskFileSystemProvider(new log_1.NullLogService());
        disposables.add(diskFileSystemProvider);
        fileService.registerProvider(network_1.Schemas.file, diskFileSystemProvider);
        // Delete any existing backups completely and then re-create it.
        pfs_1.rimraf(marketplaceHome, pfs_1.RimRafMode.MOVE).then(() => {
            pfs_1.mkdirp(marketplaceHome).then(() => {
                done();
            }, error => done(error));
        }, error => done(error));
    });
    teardown(done => {
        disposables.clear();
        pfs_1.rimraf(marketplaceHome, pfs_1.RimRafMode.MOVE).then(done, done);
    });
    test('marketplace machine id', () => {
        const args = ['--user-data-dir', marketplaceHome];
        const environmentService = new environmentService_1.EnvironmentService(argv_1.parseArgs(args, argv_1.OPTIONS), process.execPath);
        return extensionGalleryService_1.resolveMarketplaceHeaders(product_1.default.version, environmentService, fileService).then(headers => {
            assert.ok(uuid_1.isUUID(headers['X-Market-User-Id']));
            return extensionGalleryService_1.resolveMarketplaceHeaders(product_1.default.version, environmentService, fileService).then(headers2 => {
                assert.equal(headers['X-Market-User-Id'], headers2['X-Market-User-Id']);
            });
        });
    });
});
//# sourceMappingURL=extensionGalleryService.test.js.map