"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const diskFileSystemProvider_1 = require("vs/platform/files/node/diskFileSystemProvider");
const platform_1 = require("vs/base/common/platform");
const nls_1 = require("vs/nls");
const path_1 = require("vs/base/common/path");
class DiskFileSystemProvider extends diskFileSystemProvider_1.DiskFileSystemProvider {
    get capabilities() {
        if (!this._capabilities) {
            this._capabilities = super.capabilities | 4096 /* Trash */;
        }
        return this._capabilities;
    }
    doDelete(filePath, opts) {
        const _super = Object.create(null, {
            doDelete: { get: () => super.doDelete }
        });
        return __awaiter(this, void 0, void 0, function* () {
            if (!opts.useTrash) {
                return _super.doDelete.call(this, filePath, opts);
            }
            const result = electron_1.shell.moveItemToTrash(filePath);
            if (!result) {
                throw new Error(platform_1.isWindows ? nls_1.localize('binFailed', "Failed to move '{0}' to the recycle bin", path_1.basename(filePath)) : nls_1.localize('trashFailed', "Failed to move '{0}' to the trash", path_1.basename(filePath)));
            }
        });
    }
}
exports.DiskFileSystemProvider = DiskFileSystemProvider;
//# sourceMappingURL=diskFileSystemProvider.js.map