"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
function serializeContext(proxyContext) {
    if (proxyContext) {
        return { __$simpleIPCContextMarker: true, proxyContext };
    }
    return undefined;
}
function deserializeContext(candidate) {
    if (candidate && candidate.__$simpleIPCContextMarker === true) {
        return candidate.proxyContext;
    }
    return undefined;
}
class SimpleServiceProxyChannel {
    constructor(service) {
        this.service = service;
    }
    listen(_, event) {
        throw new Error(`Events are currently unsupported by SimpleServiceProxyChannel: ${event}`);
    }
    call(_, command, args) {
        const target = this.service[command];
        if (typeof target === 'function') {
            const context = deserializeContext(args[0]);
            if (context) {
                args[0] = context;
            }
            return target.apply(this.service, args);
        }
        throw new Error(`Method not found: ${command}`);
    }
}
exports.SimpleServiceProxyChannel = SimpleServiceProxyChannel;
function createSimpleChannelProxy(channel, context) {
    const serializedContext = serializeContext(context);
    return new Proxy({}, {
        get(_target, propKey, _receiver) {
            if (typeof propKey === 'string') {
                return function (...args) {
                    let methodArgs;
                    if (serializedContext) {
                        methodArgs = [context, ...args];
                    }
                    else {
                        methodArgs = args;
                    }
                    return channel.call(propKey, methodArgs);
                };
            }
            throw new Error(`Unable to provide main channel proxy implementation for: ${String(propKey)}`);
        }
    });
}
exports.createSimpleChannelProxy = createSimpleChannelProxy;
//# sourceMappingURL=simpleIpcProxy.js.map