"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const nls_1 = require("vs/nls");
const resources_1 = require("vs/base/common/resources");
const strings_1 = require("vs/base/common/strings");
const LABEL_SERVICE_ID = 'label';
function getSimpleWorkspaceLabel(workspace, workspaceHome) {
    if (workspaces_1.isSingleFolderWorkspaceIdentifier(workspace)) {
        return resources_1.basename(workspace);
    }
    // Workspace: Untitled
    if (resources_1.isEqualOrParent(workspace.configPath, workspaceHome)) {
        return nls_1.localize('untitledWorkspace', "Untitled (Workspace)");
    }
    let filename = resources_1.basename(workspace.configPath);
    if (strings_1.endsWith(filename, workspaces_1.WORKSPACE_EXTENSION)) {
        filename = filename.substr(0, filename.length - workspaces_1.WORKSPACE_EXTENSION.length - 1);
    }
    return nls_1.localize('workspaceName', "{0} (Workspace)", filename);
}
exports.getSimpleWorkspaceLabel = getSimpleWorkspaceLabel;
exports.ILabelService = instantiation_1.createDecorator(LABEL_SERVICE_ID);
//# sourceMappingURL=label.js.map