"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const event_1 = require("vs/base/common/event");
const async_1 = require("vs/base/common/async");
const lifecycle_1 = require("vs/base/common/lifecycle");
const lifecycle_2 = require("vs/platform/lifecycle/common/lifecycle");
const log_1 = require("vs/platform/log/common/log");
const performance_1 = require("vs/base/common/performance");
let AbstractLifecycleService = class AbstractLifecycleService extends lifecycle_1.Disposable {
    constructor(logService) {
        super();
        this.logService = logService;
        this._onBeforeShutdown = this._register(new event_1.Emitter());
        this.onBeforeShutdown = this._onBeforeShutdown.event;
        this._onWillShutdown = this._register(new event_1.Emitter());
        this.onWillShutdown = this._onWillShutdown.event;
        this._onShutdown = this._register(new event_1.Emitter());
        this.onShutdown = this._onShutdown.event;
        this._phase = 1 /* Starting */;
        this.phaseWhen = new Map();
    }
    get startupKind() { return this._startupKind; }
    get phase() { return this._phase; }
    set phase(value) {
        if (value < this.phase) {
            throw new Error('Lifecycle cannot go backwards');
        }
        if (this._phase === value) {
            return;
        }
        this.logService.trace(`lifecycle: phase changed (value: ${value})`);
        this._phase = value;
        performance_1.mark(`LifecyclePhase/${lifecycle_2.LifecyclePhaseToString(value)}`);
        const barrier = this.phaseWhen.get(this._phase);
        if (barrier) {
            barrier.open();
            this.phaseWhen.delete(this._phase);
        }
    }
    when(phase) {
        return __awaiter(this, void 0, void 0, function* () {
            if (phase <= this._phase) {
                return;
            }
            let barrier = this.phaseWhen.get(phase);
            if (!barrier) {
                barrier = new async_1.Barrier();
                this.phaseWhen.set(phase, barrier);
            }
            yield barrier.wait();
        });
    }
};
AbstractLifecycleService = __decorate([
    __param(0, log_1.ILogService)
], AbstractLifecycleService);
exports.AbstractLifecycleService = AbstractLifecycleService;
//# sourceMappingURL=lifecycleService.js.map