"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const platform_1 = require("vs/base/common/platform");
const environment_1 = require("vs/platform/environment/common/environment");
const electron_1 = require("electron");
const windows_1 = require("vs/platform/windows/common/windows");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const update_1 = require("vs/platform/update/common/update");
const product_1 = require("vs/platform/product/common/product");
const async_1 = require("vs/base/common/async");
const log_1 = require("vs/platform/log/common/log");
const labels_1 = require("vs/base/common/labels");
const windows_2 = require("vs/platform/windows/electron-main/windows");
const historyMainService_1 = require("vs/platform/history/electron-main/historyMainService");
const menubar_1 = require("vs/platform/menubar/node/menubar");
const uri_1 = require("vs/base/common/uri");
const state_1 = require("vs/platform/state/common/state");
const lifecycleMainService_1 = require("vs/platform/lifecycle/electron-main/lifecycleMainService");
const telemetryFrom = 'menu';
let Menubar = class Menubar {
    constructor(updateService, configurationService, windowsMainService, environmentService, telemetryService, historyMainService, stateService, lifecycleMainService, logService) {
        this.updateService = updateService;
        this.configurationService = configurationService;
        this.windowsMainService = windowsMainService;
        this.environmentService = environmentService;
        this.telemetryService = telemetryService;
        this.historyMainService = historyMainService;
        this.stateService = stateService;
        this.lifecycleMainService = lifecycleMainService;
        this.logService = logService;
        this.fallbackMenuHandlers = {};
        this.menuUpdater = new async_1.RunOnceScheduler(() => this.doUpdateMenu(), 0);
        this.menuGC = new async_1.RunOnceScheduler(() => { this.oldMenus = []; }, 10000);
        this.menubarMenus = Object.create(null);
        this.keybindings = Object.create(null);
        if (platform_1.isMacintosh || windows_1.getTitleBarStyle(this.configurationService, this.environmentService) === 'native') {
            this.restoreCachedMenubarData();
        }
        this.addFallbackHandlers();
        this.closedLastWindow = false;
        this.oldMenus = [];
        this.install();
        this.registerListeners();
    }
    restoreCachedMenubarData() {
        const menubarData = this.stateService.getItem(Menubar.lastKnownMenubarStorageKey);
        if (menubarData) {
            if (menubarData.menus) {
                this.menubarMenus = menubarData.menus;
            }
            if (menubarData.keybindings) {
                this.keybindings = menubarData.keybindings;
            }
        }
    }
    addFallbackHandlers() {
        // File Menu Items
        this.fallbackMenuHandlers['workbench.action.files.newUntitledFile'] = () => this.windowsMainService.openEmptyWindow(2 /* MENU */);
        this.fallbackMenuHandlers['workbench.action.newWindow'] = () => this.windowsMainService.openEmptyWindow(2 /* MENU */);
        this.fallbackMenuHandlers['workbench.action.files.openFileFolder'] = (menuItem, win, event) => this.windowsMainService.pickFileFolderAndOpen({ forceNewWindow: this.isOptionClick(event), telemetryExtraData: { from: telemetryFrom } });
        this.fallbackMenuHandlers['workbench.action.openWorkspace'] = (menuItem, win, event) => this.windowsMainService.pickWorkspaceAndOpen({ forceNewWindow: this.isOptionClick(event), telemetryExtraData: { from: telemetryFrom } });
        // Recent Menu Items
        this.fallbackMenuHandlers['workbench.action.clearRecentFiles'] = () => this.historyMainService.clearRecentlyOpened();
        // Help Menu Items
        const twitterUrl = product_1.default.twitterUrl;
        if (twitterUrl) {
            this.fallbackMenuHandlers['workbench.action.openTwitterUrl'] = () => this.openUrl(twitterUrl, 'openTwitterUrl');
        }
        const requestFeatureUrl = product_1.default.requestFeatureUrl;
        if (requestFeatureUrl) {
            this.fallbackMenuHandlers['workbench.action.openRequestFeatureUrl'] = () => this.openUrl(requestFeatureUrl, 'openUserVoiceUrl');
        }
        const reportIssueUrl = product_1.default.reportIssueUrl;
        if (reportIssueUrl) {
            this.fallbackMenuHandlers['workbench.action.openIssueReporter'] = () => this.openUrl(reportIssueUrl, 'openReportIssues');
        }
        const licenseUrl = product_1.default.licenseUrl;
        if (licenseUrl) {
            this.fallbackMenuHandlers['workbench.action.openLicenseUrl'] = () => {
                if (platform_1.language) {
                    const queryArgChar = licenseUrl.indexOf('?') > 0 ? '&' : '?';
                    this.openUrl(`${licenseUrl}${queryArgChar}lang=${platform_1.language}`, 'openLicenseUrl');
                }
                else {
                    this.openUrl(licenseUrl, 'openLicenseUrl');
                }
            };
        }
        const privacyStatementUrl = product_1.default.privacyStatementUrl;
        if (privacyStatementUrl && licenseUrl) {
            this.fallbackMenuHandlers['workbench.action.openPrivacyStatementUrl'] = () => {
                if (platform_1.language) {
                    const queryArgChar = licenseUrl.indexOf('?') > 0 ? '&' : '?';
                    this.openUrl(`${privacyStatementUrl}${queryArgChar}lang=${platform_1.language}`, 'openPrivacyStatement');
                }
                else {
                    this.openUrl(privacyStatementUrl, 'openPrivacyStatement');
                }
            };
        }
    }
    registerListeners() {
        // Keep flag when app quits
        this.lifecycleMainService.onWillShutdown(() => this.willShutdown = true);
        // // Listen to some events from window service to update menu
        this.windowsMainService.onWindowsCountChanged(e => this.onWindowsCountChanged(e));
    }
    get currentEnableMenuBarMnemonics() {
        let enableMenuBarMnemonics = this.configurationService.getValue('window.enableMenuBarMnemonics');
        if (typeof enableMenuBarMnemonics !== 'boolean') {
            enableMenuBarMnemonics = true;
        }
        return enableMenuBarMnemonics;
    }
    get currentEnableNativeTabs() {
        if (!platform_1.isMacintosh) {
            return false;
        }
        let enableNativeTabs = this.configurationService.getValue('window.nativeTabs');
        if (typeof enableNativeTabs !== 'boolean') {
            enableNativeTabs = false;
        }
        return enableNativeTabs;
    }
    updateMenu(menubarData, windowId) {
        this.menubarMenus = menubarData.menus;
        this.keybindings = menubarData.keybindings;
        // Save off new menu and keybindings
        this.stateService.setItem(Menubar.lastKnownMenubarStorageKey, menubarData);
        this.scheduleUpdateMenu();
    }
    scheduleUpdateMenu() {
        this.menuUpdater.schedule(); // buffer multiple attempts to update the menu
    }
    doUpdateMenu() {
        // Due to limitations in Electron, it is not possible to update menu items dynamically. The suggested
        // workaround from Electron is to set the application menu again.
        // See also https://github.com/electron/electron/issues/846
        //
        // Run delayed to prevent updating menu while it is open
        if (!this.willShutdown) {
            setTimeout(() => {
                if (!this.willShutdown) {
                    this.install();
                }
            }, 10 /* delay this because there is an issue with updating a menu when it is open */);
        }
    }
    onWindowsCountChanged(e) {
        if (!platform_1.isMacintosh) {
            return;
        }
        // Update menu if window count goes from N > 0 or 0 > N to update menu item enablement
        if ((e.oldCount === 0 && e.newCount > 0) || (e.oldCount > 0 && e.newCount === 0)) {
            this.closedLastWindow = e.newCount === 0;
            this.scheduleUpdateMenu();
        }
    }
    install() {
        // Store old menu in our array to avoid GC to collect the menu and crash. See #55347
        // TODO@sbatten Remove this when fixed upstream by Electron
        const oldMenu = electron_1.Menu.getApplicationMenu();
        if (oldMenu) {
            this.oldMenus.push(oldMenu);
        }
        // If we don't have a menu yet, set it to null to avoid the electron menu.
        // This should only happen on the first launch ever
        if (Object.keys(this.menubarMenus).length === 0) {
            electron_1.Menu.setApplicationMenu(platform_1.isMacintosh ? new electron_1.Menu() : null);
            return;
        }
        // Menus
        const menubar = new electron_1.Menu();
        // Mac: Application
        let macApplicationMenuItem;
        if (platform_1.isMacintosh) {
            const applicationMenu = new electron_1.Menu();
            macApplicationMenuItem = new electron_1.MenuItem({ label: product_1.default.nameShort, submenu: applicationMenu });
            this.setMacApplicationMenu(applicationMenu);
            menubar.append(macApplicationMenuItem);
        }
        // Mac: Dock
        if (platform_1.isMacintosh && !this.appMenuInstalled) {
            this.appMenuInstalled = true;
            const dockMenu = new electron_1.Menu();
            dockMenu.append(new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'miNewWindow', comment: ['&& denotes a mnemonic'] }, "New &&Window")), click: () => this.windowsMainService.openEmptyWindow(1 /* DOCK */) }));
            electron_1.app.dock.setMenu(dockMenu);
        }
        // File
        const fileMenu = new electron_1.Menu();
        const fileMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mFile', comment: ['&& denotes a mnemonic'] }, "&&File")), submenu: fileMenu });
        this.setMenuById(fileMenu, 'File');
        menubar.append(fileMenuItem);
        // Edit
        const editMenu = new electron_1.Menu();
        const editMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mEdit', comment: ['&& denotes a mnemonic'] }, "&&Edit")), submenu: editMenu });
        this.setMenuById(editMenu, 'Edit');
        menubar.append(editMenuItem);
        // Selection
        const selectionMenu = new electron_1.Menu();
        const selectionMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mSelection', comment: ['&& denotes a mnemonic'] }, "&&Selection")), submenu: selectionMenu });
        this.setMenuById(selectionMenu, 'Selection');
        menubar.append(selectionMenuItem);
        // View
        const viewMenu = new electron_1.Menu();
        const viewMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mView', comment: ['&& denotes a mnemonic'] }, "&&View")), submenu: viewMenu });
        this.setMenuById(viewMenu, 'View');
        menubar.append(viewMenuItem);
        // Go
        const gotoMenu = new electron_1.Menu();
        const gotoMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mGoto', comment: ['&& denotes a mnemonic'] }, "&&Go")), submenu: gotoMenu });
        this.setMenuById(gotoMenu, 'Go');
        menubar.append(gotoMenuItem);
        // Debug
        const debugMenu = new electron_1.Menu();
        const debugMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mDebug', comment: ['&& denotes a mnemonic'] }, "&&Debug")), submenu: debugMenu });
        this.setMenuById(debugMenu, 'Debug');
        menubar.append(debugMenuItem);
        // Terminal
        const terminalMenu = new electron_1.Menu();
        const terminalMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mTerminal', comment: ['&& denotes a mnemonic'] }, "&&Terminal")), submenu: terminalMenu });
        this.setMenuById(terminalMenu, 'Terminal');
        menubar.append(terminalMenuItem);
        // Mac: Window
        let macWindowMenuItem;
        if (this.shouldDrawMenu('Window')) {
            const windowMenu = new electron_1.Menu();
            macWindowMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize('mWindow', "Window")), submenu: windowMenu, role: 'window' });
            this.setMacWindowMenu(windowMenu);
        }
        if (macWindowMenuItem) {
            menubar.append(macWindowMenuItem);
        }
        // Help
        const helpMenu = new electron_1.Menu();
        const helpMenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'mHelp', comment: ['&& denotes a mnemonic'] }, "&&Help")), submenu: helpMenu, role: 'help' });
        this.setMenuById(helpMenu, 'Help');
        menubar.append(helpMenuItem);
        if (menubar.items && menubar.items.length > 0) {
            electron_1.Menu.setApplicationMenu(menubar);
        }
        else {
            electron_1.Menu.setApplicationMenu(null);
        }
        // Dispose of older menus after some time
        this.menuGC.schedule();
    }
    setMacApplicationMenu(macApplicationMenu) {
        const about = this.createMenuItem(nls.localize('mAbout', "About {0}", product_1.default.nameLong), 'workbench.action.showAboutDialog');
        const checkForUpdates = this.getUpdateMenuItems();
        let preferences;
        if (this.shouldDrawMenu('Preferences')) {
            const preferencesMenu = new electron_1.Menu();
            this.setMenuById(preferencesMenu, 'Preferences');
            preferences = new electron_1.MenuItem({ label: this.mnemonicLabel(nls.localize({ key: 'miPreferences', comment: ['&& denotes a mnemonic'] }, "&&Preferences")), submenu: preferencesMenu });
        }
        const servicesMenu = new electron_1.Menu();
        const services = new electron_1.MenuItem({ label: nls.localize('mServices', "Services"), role: 'services', submenu: servicesMenu });
        const hide = new electron_1.MenuItem({ label: nls.localize('mHide', "Hide {0}", product_1.default.nameLong), role: 'hide', accelerator: 'Command+H' });
        const hideOthers = new electron_1.MenuItem({ label: nls.localize('mHideOthers', "Hide Others"), role: 'hideOthers', accelerator: 'Command+Alt+H' });
        const showAll = new electron_1.MenuItem({ label: nls.localize('mShowAll', "Show All"), role: 'unhide' });
        const quit = new electron_1.MenuItem(this.likeAction('workbench.action.quit', {
            label: nls.localize('miQuit', "Quit {0}", product_1.default.nameLong), click: () => {
                if (this.windowsMainService.getWindowCount() === 0 || // allow to quit when no more windows are open
                    !!this.windowsMainService.getFocusedWindow() || // allow to quit when window has focus (fix for https://github.com/Microsoft/vscode/issues/39191)
                    this.windowsMainService.getLastActiveWindow().isMinimized() // allow to quit when window has no focus but is minimized (https://github.com/Microsoft/vscode/issues/63000)
                ) {
                    this.windowsMainService.quit();
                }
            }
        }));
        const actions = [about];
        actions.push(...checkForUpdates);
        if (preferences) {
            actions.push(...[
                __separator__(),
                preferences
            ]);
        }
        actions.push(...[
            __separator__(),
            services,
            __separator__(),
            hide,
            hideOthers,
            showAll,
            __separator__(),
            quit
        ]);
        actions.forEach(i => macApplicationMenu.append(i));
    }
    shouldDrawMenu(menuId) {
        // We need to draw an empty menu to override the electron default
        if (!platform_1.isMacintosh && windows_1.getTitleBarStyle(this.configurationService, this.environmentService) === 'custom') {
            return false;
        }
        switch (menuId) {
            case 'File':
            case 'Help':
                if (platform_1.isMacintosh) {
                    return (this.windowsMainService.getWindowCount() === 0 && this.closedLastWindow) || (!!this.menubarMenus && !!this.menubarMenus[menuId]);
                }
            case 'Window':
                if (platform_1.isMacintosh) {
                    return (this.windowsMainService.getWindowCount() === 0 && this.closedLastWindow) || !!this.menubarMenus;
                }
            default:
                return this.windowsMainService.getWindowCount() > 0 && (!!this.menubarMenus && !!this.menubarMenus[menuId]);
        }
    }
    setMenu(menu, items) {
        items.forEach((item) => {
            if (menubar_1.isMenubarMenuItemSeparator(item)) {
                menu.append(__separator__());
            }
            else if (menubar_1.isMenubarMenuItemSubmenu(item)) {
                const submenu = new electron_1.Menu();
                const submenuItem = new electron_1.MenuItem({ label: this.mnemonicLabel(item.label), submenu: submenu });
                this.setMenu(submenu, item.submenu.items);
                menu.append(submenuItem);
            }
            else if (menubar_1.isMenubarMenuItemUriAction(item)) {
                menu.append(this.createOpenRecentMenuItem(item.uri, item.label, item.id));
            }
            else if (menubar_1.isMenubarMenuItemAction(item)) {
                if (item.id === 'workbench.action.showAboutDialog') {
                    this.insertCheckForUpdatesItems(menu);
                }
                if (platform_1.isMacintosh) {
                    if (this.windowsMainService.getWindowCount() === 0 && this.closedLastWindow) {
                        // In the fallback scenario, we are either disabled or using a fallback handler
                        if (this.fallbackMenuHandlers[item.id]) {
                            menu.append(new electron_1.MenuItem(this.likeAction(item.id, { label: this.mnemonicLabel(item.label), click: this.fallbackMenuHandlers[item.id] })));
                        }
                        else {
                            menu.append(this.createMenuItem(item.label, item.id, false, item.checked));
                        }
                    }
                    else {
                        menu.append(this.createMenuItem(item.label, item.id, item.enabled === false ? false : true, !!item.checked));
                    }
                }
                else {
                    menu.append(this.createMenuItem(item.label, item.id, item.enabled === false ? false : true, !!item.checked));
                }
            }
        });
    }
    setMenuById(menu, menuId) {
        if (this.menubarMenus && this.menubarMenus[menuId]) {
            this.setMenu(menu, this.menubarMenus[menuId].items);
        }
    }
    insertCheckForUpdatesItems(menu) {
        const updateItems = this.getUpdateMenuItems();
        if (updateItems.length) {
            updateItems.forEach(i => menu.append(i));
            menu.append(__separator__());
        }
    }
    createOpenRecentMenuItem(uri, label, commandId) {
        const revivedUri = uri_1.URI.revive(uri);
        const uriToOpen = (commandId === 'openRecentFile') ? { fileUri: revivedUri } :
            (commandId === 'openRecentWorkspace') ? { workspaceUri: revivedUri } : { folderUri: revivedUri };
        return new electron_1.MenuItem(this.likeAction(commandId, {
            label,
            click: (menuItem, win, event) => {
                const openInNewWindow = this.isOptionClick(event);
                const success = this.windowsMainService.open({
                    context: 2 /* MENU */,
                    cli: this.environmentService.args,
                    urisToOpen: [uriToOpen],
                    forceNewWindow: openInNewWindow,
                    gotoLineMode: false
                }).length > 0;
                if (!success) {
                    this.historyMainService.removeFromRecentlyOpened([revivedUri]);
                }
            }
        }, false));
    }
    isOptionClick(event) {
        return !!(event && ((!platform_1.isMacintosh && (event.ctrlKey || event.shiftKey)) || (platform_1.isMacintosh && (event.metaKey || event.altKey))));
    }
    createRoleMenuItem(label, commandId, role) {
        const options = {
            label: this.mnemonicLabel(label),
            role,
            enabled: true
        };
        return new electron_1.MenuItem(this.withKeybinding(commandId, options));
    }
    setMacWindowMenu(macWindowMenu) {
        const minimize = new electron_1.MenuItem({ label: nls.localize('mMinimize', "Minimize"), role: 'minimize', accelerator: 'Command+M', enabled: this.windowsMainService.getWindowCount() > 0 });
        const zoom = new electron_1.MenuItem({ label: nls.localize('mZoom', "Zoom"), role: 'zoom', enabled: this.windowsMainService.getWindowCount() > 0 });
        const bringAllToFront = new electron_1.MenuItem({ label: nls.localize('mBringToFront', "Bring All to Front"), role: 'front', enabled: this.windowsMainService.getWindowCount() > 0 });
        const switchWindow = this.createMenuItem(nls.localize({ key: 'miSwitchWindow', comment: ['&& denotes a mnemonic'] }, "Switch &&Window..."), 'workbench.action.switchWindow');
        const nativeTabMenuItems = [];
        if (this.currentEnableNativeTabs) {
            nativeTabMenuItems.push(__separator__());
            nativeTabMenuItems.push(this.createMenuItem(nls.localize('mNewTab', "New Tab"), 'workbench.action.newWindowTab'));
            nativeTabMenuItems.push(this.createRoleMenuItem(nls.localize('mShowPreviousTab', "Show Previous Tab"), 'workbench.action.showPreviousWindowTab', 'selectPreviousTab'));
            nativeTabMenuItems.push(this.createRoleMenuItem(nls.localize('mShowNextTab', "Show Next Tab"), 'workbench.action.showNextWindowTab', 'selectNextTab'));
            nativeTabMenuItems.push(this.createRoleMenuItem(nls.localize('mMoveTabToNewWindow', "Move Tab to New Window"), 'workbench.action.moveWindowTabToNewWindow', 'moveTabToNewWindow'));
            nativeTabMenuItems.push(this.createRoleMenuItem(nls.localize('mMergeAllWindows', "Merge All Windows"), 'workbench.action.mergeAllWindowTabs', 'mergeAllWindows'));
        }
        [
            minimize,
            zoom,
            switchWindow,
            ...nativeTabMenuItems,
            __separator__(),
            bringAllToFront
        ].forEach(item => macWindowMenu.append(item));
    }
    getUpdateMenuItems() {
        const state = this.updateService.state;
        switch (state.type) {
            case "uninitialized" /* Uninitialized */:
                return [];
            case "idle" /* Idle */:
                return [new electron_1.MenuItem({
                        label: this.mnemonicLabel(nls.localize('miCheckForUpdates', "Check for &&Updates...")), click: () => setTimeout(() => {
                            this.reportMenuActionTelemetry('CheckForUpdate');
                            const focusedWindow = this.windowsMainService.getFocusedWindow();
                            const context = focusedWindow ? { windowId: focusedWindow.id } : null;
                            this.updateService.checkForUpdates(context);
                        }, 0)
                    })];
            case "checking for updates" /* CheckingForUpdates */:
                return [new electron_1.MenuItem({ label: nls.localize('miCheckingForUpdates', "Checking for Updates..."), enabled: false })];
            case "available for download" /* AvailableForDownload */:
                return [new electron_1.MenuItem({
                        label: this.mnemonicLabel(nls.localize('miDownloadUpdate', "D&&ownload Available Update")), click: () => {
                            this.updateService.downloadUpdate();
                        }
                    })];
            case "downloading" /* Downloading */:
                return [new electron_1.MenuItem({ label: nls.localize('miDownloadingUpdate', "Downloading Update..."), enabled: false })];
            case "downloaded" /* Downloaded */:
                return [new electron_1.MenuItem({
                        label: this.mnemonicLabel(nls.localize('miInstallUpdate', "Install &&Update...")), click: () => {
                            this.reportMenuActionTelemetry('InstallUpdate');
                            this.updateService.applyUpdate();
                        }
                    })];
            case "updating" /* Updating */:
                return [new electron_1.MenuItem({ label: nls.localize('miInstallingUpdate', "Installing Update..."), enabled: false })];
            case "ready" /* Ready */:
                return [new electron_1.MenuItem({
                        label: this.mnemonicLabel(nls.localize('miRestartToUpdate', "Restart to &&Update")), click: () => {
                            this.reportMenuActionTelemetry('RestartToUpdate');
                            this.updateService.quitAndInstall();
                        }
                    })];
        }
    }
    static _menuItemIsTriggeredViaKeybinding(event, userSettingsLabel) {
        // The event coming in from Electron will inform us only about the modifier keys pressed.
        // The strategy here is to check if the modifier keys match those of the keybinding,
        // since it is highly unlikely to use modifier keys when clicking with the mouse
        if (!userSettingsLabel) {
            // There is no keybinding
            return false;
        }
        let ctrlRequired = /ctrl/.test(userSettingsLabel);
        let shiftRequired = /shift/.test(userSettingsLabel);
        let altRequired = /alt/.test(userSettingsLabel);
        let metaRequired = /cmd/.test(userSettingsLabel) || /super/.test(userSettingsLabel);
        if (!ctrlRequired && !shiftRequired && !altRequired && !metaRequired) {
            // This keybinding does not use any modifier keys, so we cannot use this heuristic
            return false;
        }
        return (ctrlRequired === event.ctrlKey
            && shiftRequired === event.shiftKey
            && altRequired === event.altKey
            && metaRequired === event.metaKey);
    }
    createMenuItem(arg1, arg2, arg3, arg4) {
        const label = this.mnemonicLabel(arg1);
        const click = (typeof arg2 === 'function') ? arg2 : (menuItem, win, event) => {
            const userSettingsLabel = menuItem ? menuItem.userSettingsLabel : null;
            let commandId = arg2;
            if (Array.isArray(arg2)) {
                commandId = this.isOptionClick(event) ? arg2[1] : arg2[0]; // support alternative action if we got multiple action Ids and the option key was pressed while invoking
            }
            if (userSettingsLabel && Menubar._menuItemIsTriggeredViaKeybinding(event, userSettingsLabel)) {
                this.runActionInRenderer({ type: 'keybinding', userSettingsLabel });
            }
            else {
                this.runActionInRenderer({ type: 'commandId', commandId });
            }
        };
        const enabled = typeof arg3 === 'boolean' ? arg3 : this.windowsMainService.getWindowCount() > 0;
        const checked = typeof arg4 === 'boolean' ? arg4 : false;
        const options = {
            label,
            click,
            enabled
        };
        if (checked) {
            options.type = 'checkbox';
            options.checked = checked;
        }
        let commandId;
        if (typeof arg2 === 'string') {
            commandId = arg2;
        }
        else if (Array.isArray(arg2)) {
            commandId = arg2[0];
        }
        if (platform_1.isMacintosh) {
            // Add role for special case menu items
            if (commandId === 'editor.action.clipboardCutAction') {
                options.role = 'cut';
            }
            else if (commandId === 'editor.action.clipboardCopyAction') {
                options.role = 'copy';
            }
            else if (commandId === 'editor.action.clipboardPasteAction') {
                options.role = 'paste';
            }
            // Add context aware click handlers for special case menu items
            if (commandId === 'undo') {
                options.click = this.makeContextAwareClickHandler(click, {
                    inDevTools: devTools => devTools.undo(),
                    inNoWindow: () => electron_1.Menu.sendActionToFirstResponder('undo:')
                });
            }
            else if (commandId === 'redo') {
                options.click = this.makeContextAwareClickHandler(click, {
                    inDevTools: devTools => devTools.redo(),
                    inNoWindow: () => electron_1.Menu.sendActionToFirstResponder('redo:')
                });
            }
            else if (commandId === 'editor.action.selectAll') {
                options.click = this.makeContextAwareClickHandler(click, {
                    inDevTools: devTools => devTools.selectAll(),
                    inNoWindow: () => electron_1.Menu.sendActionToFirstResponder('selectAll:')
                });
            }
        }
        return new electron_1.MenuItem(this.withKeybinding(commandId, options));
    }
    makeContextAwareClickHandler(click, contextSpecificHandlers) {
        return () => {
            // No Active Window
            const activeWindow = this.windowsMainService.getFocusedWindow();
            if (!activeWindow) {
                return contextSpecificHandlers.inNoWindow();
            }
            // DevTools focused
            if (activeWindow.win.webContents.isDevToolsFocused()) {
                return contextSpecificHandlers.inDevTools(activeWindow.win.webContents.devToolsWebContents);
            }
            // Finally execute command in Window
            click();
        };
    }
    runActionInRenderer(invocation) {
        // We make sure to not run actions when the window has no focus, this helps
        // for https://github.com/Microsoft/vscode/issues/25907 and specifically for
        // https://github.com/Microsoft/vscode/issues/11928
        // Still allow to run when the last active window is minimized though for
        // https://github.com/Microsoft/vscode/issues/63000
        let activeWindow = this.windowsMainService.getFocusedWindow();
        if (!activeWindow) {
            const lastActiveWindow = this.windowsMainService.getLastActiveWindow();
            if (lastActiveWindow && lastActiveWindow.isMinimized()) {
                activeWindow = lastActiveWindow;
            }
        }
        if (activeWindow) {
            this.logService.trace('menubar#runActionInRenderer', invocation);
            if (platform_1.isMacintosh && !this.environmentService.isBuilt && !activeWindow.isReady) {
                if ((invocation.type === 'commandId' && invocation.commandId === 'workbench.action.toggleDevTools') || (invocation.type !== 'commandId' && invocation.userSettingsLabel === 'alt+cmd+i')) {
                    // prevent this action from running twice on macOS (https://github.com/Microsoft/vscode/issues/62719)
                    // we already register a keybinding in bootstrap-window.js for opening developer tools in case something
                    // goes wrong and that keybinding is only removed when the application has loaded (= window ready).
                    return;
                }
            }
            if (invocation.type === 'commandId') {
                activeWindow.sendWhenReady('vscode:runAction', { id: invocation.commandId, from: 'menu' });
            }
            else {
                activeWindow.sendWhenReady('vscode:runKeybinding', { userSettingsLabel: invocation.userSettingsLabel });
            }
        }
        else {
            this.logService.trace('menubar#runActionInRenderer: no active window found', invocation);
        }
    }
    withKeybinding(commandId, options) {
        const binding = typeof commandId === 'string' ? this.keybindings[commandId] : undefined;
        // Apply binding if there is one
        if (binding && binding.label) {
            // if the binding is native, we can just apply it
            if (binding.isNative !== false) {
                options.accelerator = binding.label;
                options.userSettingsLabel = binding.userSettingsLabel;
            }
            // the keybinding is not native so we cannot show it as part of the accelerator of
            // the menu item. we fallback to a different strategy so that we always display it
            else if (typeof options.label === 'string') {
                const bindingIndex = options.label.indexOf('[');
                if (bindingIndex >= 0) {
                    options.label = `${options.label.substr(0, bindingIndex)} [${binding.label}]`;
                }
                else {
                    options.label = `${options.label} [${binding.label}]`;
                }
            }
        }
        // Unset bindings if there is none
        else {
            options.accelerator = undefined;
        }
        return options;
    }
    likeAction(commandId, options, setAccelerator = !options.accelerator) {
        if (setAccelerator) {
            options = this.withKeybinding(commandId, options);
        }
        const originalClick = options.click;
        options.click = (item, window, event) => {
            this.reportMenuActionTelemetry(commandId);
            if (originalClick) {
                originalClick(item, window, event);
            }
        };
        return options;
    }
    openUrl(url, id) {
        electron_1.shell.openExternal(url);
        this.reportMenuActionTelemetry(id);
    }
    reportMenuActionTelemetry(id) {
        this.telemetryService.publicLog2('workbenchActionExecuted', { id, from: telemetryFrom });
    }
    mnemonicLabel(label) {
        return labels_1.mnemonicMenuLabel(label, !this.currentEnableMenuBarMnemonics);
    }
};
Menubar.lastKnownMenubarStorageKey = 'lastKnownMenubarData';
Menubar = __decorate([
    __param(0, update_1.IUpdateService),
    __param(1, configuration_1.IConfigurationService),
    __param(2, windows_2.IWindowsMainService),
    __param(3, environment_1.IEnvironmentService),
    __param(4, telemetry_1.ITelemetryService),
    __param(5, historyMainService_1.IHistoryMainService),
    __param(6, state_1.IStateService),
    __param(7, lifecycleMainService_1.ILifecycleMainService),
    __param(8, log_1.ILogService)
], Menubar);
exports.Menubar = Menubar;
function __separator__() {
    return new electron_1.MenuItem({ type: 'separator' });
}
//# sourceMappingURL=menubar.js.map