"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const zlib_1 = require("zlib");
const url_1 = require("url");
const lifecycle_1 = require("vs/base/common/lifecycle");
const objects_1 = require("vs/base/common/objects");
const types_1 = require("vs/base/common/types");
const errors_1 = require("vs/base/common/errors");
const proxy_1 = require("vs/platform/request/node/proxy");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const log_1 = require("vs/platform/log/common/log");
const buffer_1 = require("vs/base/common/buffer");
/**
 * This service exposes the `request` API, while using the global
 * or configured proxy settings.
 */
let RequestService = class RequestService extends lifecycle_1.Disposable {
    constructor(configurationService, logService) {
        super();
        this.logService = logService;
        this.configure(configurationService.getValue());
        this._register(configurationService.onDidChangeConfiguration(() => this.configure(configurationService.getValue()), this));
    }
    configure(config) {
        this.proxyUrl = config.http && config.http.proxy;
        this.strictSSL = !!(config.http && config.http.proxyStrictSSL);
        this.authorization = config.http && config.http.proxyAuthorization;
    }
    request(options, token) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('RequestService#request', options.url);
            const { proxyUrl, strictSSL } = this;
            const agent = options.agent ? options.agent : yield proxy_1.getProxyAgent(options.url || '', { proxyUrl, strictSSL });
            options.agent = agent;
            options.strictSSL = strictSSL;
            if (this.authorization) {
                options.headers = objects_1.assign(options.headers || {}, { 'Proxy-Authorization': this.authorization });
            }
            return this._request(options, token);
        });
    }
    getNodeRequest(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const endpoint = url_1.parse(options.url);
            const module = endpoint.protocol === 'https:' ? yield Promise.resolve().then(() => require('https')) : yield Promise.resolve().then(() => require('http'));
            return module.request;
        });
    }
    _request(options, token) {
        return new Promise((c, e) => __awaiter(this, void 0, void 0, function* () {
            let req;
            const endpoint = url_1.parse(options.url);
            const rawRequest = options.getRawRequest
                ? options.getRawRequest(options)
                : yield this.getNodeRequest(options);
            const opts = {
                hostname: endpoint.hostname,
                port: endpoint.port ? parseInt(endpoint.port) : (endpoint.protocol === 'https:' ? 443 : 80),
                protocol: endpoint.protocol,
                path: endpoint.path,
                method: options.type || 'GET',
                headers: options.headers,
                agent: options.agent,
                rejectUnauthorized: types_1.isBoolean(options.strictSSL) ? options.strictSSL : true
            };
            if (options.user && options.password) {
                opts.auth = options.user + ':' + options.password;
            }
            req = rawRequest(opts, (res) => {
                const followRedirects = types_1.isNumber(options.followRedirects) ? options.followRedirects : 3;
                if (res.statusCode && res.statusCode >= 300 && res.statusCode < 400 && followRedirects > 0 && res.headers['location']) {
                    this._request(objects_1.assign({}, options, {
                        url: res.headers['location'],
                        followRedirects: followRedirects - 1
                    }), token).then(c, e);
                }
                else {
                    let stream = res;
                    if (res.headers['content-encoding'] === 'gzip') {
                        stream = stream.pipe(zlib_1.createGunzip());
                    }
                    c({ res, stream: buffer_1.toVSBufferReadableStream(stream) });
                }
            });
            req.on('error', e);
            if (options.timeout) {
                req.setTimeout(options.timeout);
            }
            if (options.data) {
                if (typeof options.data === 'string') {
                    req.write(options.data);
                }
            }
            req.end();
            token.onCancellationRequested(() => {
                req.abort();
                e(errors_1.canceled());
            });
        }));
    }
    resolveProxy(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return undefined; // currently not implemented in node
        });
    }
};
RequestService = __decorate([
    __param(0, configuration_1.IConfigurationService),
    __param(1, log_1.ILogService)
], RequestService);
exports.RequestService = RequestService;
//# sourceMappingURL=requestService.js.map