"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = require("assert");
const storageService_1 = require("vs/platform/storage/browser/storageService");
const uuid_1 = require("vs/base/common/uuid");
const path_1 = require("vs/base/common/path");
const os_1 = require("os");
const pfs_1 = require("vs/base/node/pfs");
const log_1 = require("vs/platform/log/common/log");
const storage_1 = require("vs/base/parts/storage/common/storage");
const uri_1 = require("vs/base/common/uri");
const fileService_1 = require("vs/platform/files/common/fileService");
const testUtils_1 = require("vs/base/test/node/testUtils");
const diskFileSystemProvider_1 = require("vs/platform/files/node/diskFileSystemProvider");
const lifecycle_1 = require("vs/base/common/lifecycle");
const network_1 = require("vs/base/common/network");
suite('Storage', () => {
    const parentDir = testUtils_1.getRandomTestPath(os_1.tmpdir(), 'vsctests', 'storageservice');
    let fileService;
    let fileProvider;
    let testDir;
    const disposables = new lifecycle_1.DisposableStore();
    setup(() => __awaiter(void 0, void 0, void 0, function* () {
        const logService = new log_1.NullLogService();
        fileService = new fileService_1.FileService(logService);
        disposables.add(fileService);
        fileProvider = new diskFileSystemProvider_1.DiskFileSystemProvider(logService);
        disposables.add(fileService.registerProvider(network_1.Schemas.file, fileProvider));
        disposables.add(fileProvider);
        const id = uuid_1.generateUuid();
        testDir = path_1.join(parentDir, id);
    }));
    teardown(() => __awaiter(void 0, void 0, void 0, function* () {
        disposables.clear();
        yield pfs_1.rimraf(parentDir, pfs_1.RimRafMode.MOVE);
    }));
    test('File Based Storage', () => __awaiter(void 0, void 0, void 0, function* () {
        let storage = new storage_1.Storage(new storageService_1.FileStorageDatabase(uri_1.URI.file(path_1.join(testDir, 'storage.json')), false, fileService));
        yield storage.init();
        storage.set('bar', 'foo');
        storage.set('barNumber', 55);
        storage.set('barBoolean', true);
        assert_1.equal(storage.get('bar'), 'foo');
        assert_1.equal(storage.get('barNumber'), '55');
        assert_1.equal(storage.get('barBoolean'), 'true');
        yield storage.close();
        storage = new storage_1.Storage(new storageService_1.FileStorageDatabase(uri_1.URI.file(path_1.join(testDir, 'storage.json')), false, fileService));
        yield storage.init();
        assert_1.equal(storage.get('bar'), 'foo');
        assert_1.equal(storage.get('barNumber'), '55');
        assert_1.equal(storage.get('barBoolean'), 'true');
        storage.delete('bar');
        storage.delete('barNumber');
        storage.delete('barBoolean');
        assert_1.equal(storage.get('bar', 'undefined'), 'undefined');
        assert_1.equal(storage.get('barNumber', 'undefinedNumber'), 'undefinedNumber');
        assert_1.equal(storage.get('barBoolean', 'undefinedBoolean'), 'undefinedBoolean');
        yield storage.close();
        storage = new storage_1.Storage(new storageService_1.FileStorageDatabase(uri_1.URI.file(path_1.join(testDir, 'storage.json')), false, fileService));
        yield storage.init();
        assert_1.equal(storage.get('bar', 'undefined'), 'undefined');
        assert_1.equal(storage.get('barNumber', 'undefinedNumber'), 'undefinedNumber');
        assert_1.equal(storage.get('barBoolean', 'undefinedBoolean'), 'undefinedBoolean');
    }));
});
//# sourceMappingURL=storage.test.js.map