"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const lifecycle_1 = require("vs/base/common/lifecycle");
const platform = require("vs/platform/registry/common/platform");
const event_1 = require("vs/base/common/event");
exports.IThemeService = instantiation_1.createDecorator('themeService');
function themeColorFromId(id) {
    return { id };
}
exports.themeColorFromId = themeColorFromId;
exports.FileThemeIcon = { id: 'file' };
exports.FolderThemeIcon = { id: 'folder' };
// base themes
exports.DARK = 'dark';
exports.LIGHT = 'light';
exports.HIGH_CONTRAST = 'hc';
function getThemeTypeSelector(type) {
    switch (type) {
        case exports.DARK: return 'vs-dark';
        case exports.HIGH_CONTRAST: return 'hc-black';
        default: return 'vs';
    }
}
exports.getThemeTypeSelector = getThemeTypeSelector;
// static theming participant
exports.Extensions = {
    ThemingContribution: 'base.contributions.theming'
};
class ThemingRegistry {
    constructor() {
        this.themingParticipants = [];
        this.themingParticipants = [];
        this.onThemingParticipantAddedEmitter = new event_1.Emitter();
    }
    onThemeChange(participant) {
        this.themingParticipants.push(participant);
        this.onThemingParticipantAddedEmitter.fire(participant);
        return lifecycle_1.toDisposable(() => {
            const idx = this.themingParticipants.indexOf(participant);
            this.themingParticipants.splice(idx, 1);
        });
    }
    get onThemingParticipantAdded() {
        return this.onThemingParticipantAddedEmitter.event;
    }
    getThemingParticipants() {
        return this.themingParticipants;
    }
}
let themingRegistry = new ThemingRegistry();
platform.Registry.add(exports.Extensions.ThemingContribution, themingRegistry);
function registerThemingParticipant(participant) {
    return themingRegistry.onThemeChange(participant);
}
exports.registerThemingParticipant = registerThemingParticipant;
//# sourceMappingURL=themeService.js.map