"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uri_1 = require("vs/base/common/uri");
const arrays_1 = require("vs/base/common/arrays");
class URLServiceChannel {
    constructor(service) {
        this.service = service;
    }
    listen(_, event) {
        throw new Error(`Event not found: ${event}`);
    }
    call(_, command, arg) {
        switch (command) {
            case 'open': return this.service.open(uri_1.URI.revive(arg));
        }
        throw new Error(`Call not found: ${command}`);
    }
}
exports.URLServiceChannel = URLServiceChannel;
class URLServiceChannelClient {
    constructor(channel) {
        this.channel = channel;
    }
    open(url) {
        return this.channel.call('open', url.toJSON());
    }
    registerHandler(handler) {
        throw new Error('Not implemented.');
    }
    create(_options) {
        throw new Error('Method not implemented.');
    }
}
exports.URLServiceChannelClient = URLServiceChannelClient;
class URLHandlerChannel {
    constructor(handler) {
        this.handler = handler;
    }
    listen(_, event) {
        throw new Error(`Event not found: ${event}`);
    }
    call(_, command, arg) {
        switch (command) {
            case 'handleURL': return this.handler.handleURL(uri_1.URI.revive(arg));
        }
        throw new Error(`Call not found: ${command}`);
    }
}
exports.URLHandlerChannel = URLHandlerChannel;
class URLHandlerChannelClient {
    constructor(channel) {
        this.channel = channel;
    }
    handleURL(uri) {
        return this.channel.call('handleURL', uri.toJSON());
    }
}
exports.URLHandlerChannelClient = URLHandlerChannelClient;
class URLHandlerRouter {
    constructor(next) {
        this.next = next;
    }
    routeCall(hub, command, arg, cancellationToken) {
        return __awaiter(this, void 0, void 0, function* () {
            if (command !== 'handleURL') {
                throw new Error(`Call not found: ${command}`);
            }
            if (arg) {
                const uri = uri_1.URI.revive(arg);
                if (uri && uri.query) {
                    const match = /\bwindowId=([^&]+)/.exec(uri.query);
                    if (match) {
                        const windowId = match[1];
                        const connection = arrays_1.first(hub.connections, c => c.ctx === windowId);
                        if (connection) {
                            return connection;
                        }
                    }
                }
            }
            return this.next.routeCall(hub, command, arg, cancellationToken);
        });
    }
    routeEvent(_, event) {
        throw new Error(`Event not found: ${event}`);
    }
}
exports.URLHandlerRouter = URLHandlerRouter;
//# sourceMappingURL=urlIpc.js.map