"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const map_1 = require("vs/base/common/map");
const async_1 = require("vs/base/common/async");
const lifecycle_1 = require("vs/base/common/lifecycle");
class AbstractURLService extends lifecycle_1.Disposable {
    constructor() {
        super(...arguments);
        this.handlers = new Set();
    }
    open(uri) {
        const handlers = map_1.values(this.handlers);
        return async_1.first(handlers.map(h => () => h.handleURL(uri)), undefined, false).then(val => val || false);
    }
    registerHandler(handler) {
        this.handlers.add(handler);
        return lifecycle_1.toDisposable(() => this.handlers.delete(handler));
    }
}
exports.AbstractURLService = AbstractURLService;
//# sourceMappingURL=urlService.js.map