"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const lifecycle_1 = require("vs/base/common/lifecycle");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const settingsSync_1 = require("vs/platform/userDataSync/common/settingsSync");
const event_1 = require("vs/base/common/event");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const async_1 = require("vs/base/common/async");
let UserDataSyncService = class UserDataSyncService extends lifecycle_1.Disposable {
    constructor(userDataSyncStoreService, instantiationService) {
        super();
        this.userDataSyncStoreService = userDataSyncStoreService;
        this.instantiationService = instantiationService;
        this._status = userDataSync_1.SyncStatus.Uninitialized;
        this._onDidChangeStatus = this._register(new event_1.Emitter());
        this.onDidChangeStatus = this._onDidChangeStatus.event;
        this._conflictsSource = null;
        this.synchronisers = [
            this.instantiationService.createInstance(settingsSync_1.SettingsSynchroniser)
        ];
        this.updateStatus();
        this._register(event_1.Event.any(...this.synchronisers.map(s => event_1.Event.map(s.onDidChangeStatus, () => undefined)))(() => this.updateStatus()));
        this.onDidChangeLocal = event_1.Event.any(...this.synchronisers.map(s => s.onDidChangeLocal));
    }
    get status() { return this._status; }
    get conflictsSource() { return this._conflictsSource; }
    sync(_continue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.userDataSyncStoreService.enabled) {
                throw new Error('Not enabled');
            }
            for (const synchroniser of this.synchronisers) {
                if (!(yield synchroniser.sync(_continue))) {
                    return false;
                }
            }
            return true;
        });
    }
    updateStatus() {
        this._conflictsSource = this.computeConflictsSource();
        this.setStatus(this.computeStatus());
    }
    setStatus(status) {
        if (this._status !== status) {
            this._status = status;
            this._onDidChangeStatus.fire(status);
        }
    }
    computeStatus() {
        if (!this.userDataSyncStoreService.enabled) {
            return userDataSync_1.SyncStatus.Uninitialized;
        }
        if (this.synchronisers.some(s => s.status === userDataSync_1.SyncStatus.HasConflicts)) {
            return userDataSync_1.SyncStatus.HasConflicts;
        }
        if (this.synchronisers.some(s => s.status === userDataSync_1.SyncStatus.Syncing)) {
            return userDataSync_1.SyncStatus.Syncing;
        }
        return userDataSync_1.SyncStatus.Idle;
    }
    computeConflictsSource() {
        const source = this.synchronisers.filter(s => s.status === userDataSync_1.SyncStatus.HasConflicts)[0];
        if (source) {
            if (source instanceof settingsSync_1.SettingsSynchroniser) {
                return userDataSync_1.SyncSource.Settings;
            }
        }
        return null;
    }
};
UserDataSyncService = __decorate([
    __param(0, userDataSync_1.IUserDataSyncStoreService),
    __param(1, instantiation_1.IInstantiationService)
], UserDataSyncService);
exports.UserDataSyncService = UserDataSyncService;
let UserDataAutoSync = class UserDataAutoSync extends lifecycle_1.Disposable {
    constructor(configurationService, userDataSyncService, userDataSyncStoreService) {
        super();
        this.configurationService = configurationService;
        this.userDataSyncService = userDataSyncService;
        if (userDataSyncStoreService.enabled) {
            this.sync(true);
            this._register(event_1.Event.filter(this.configurationService.onDidChangeConfiguration, e => e.affectsConfiguration('userConfiguration.enableSync'))(() => this.sync(true)));
            // Sync immediately if there is a local change.
            this._register(event_1.Event.debounce(this.userDataSyncService.onDidChangeLocal, () => undefined, 500)(() => this.sync(false)));
        }
    }
    sync(loop) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isSyncEnabled()) {
                try {
                    yield this.userDataSyncService.sync();
                }
                catch (e) {
                    // Ignore errors
                }
                if (loop) {
                    yield async_1.timeout(1000 * 5); // Loop sync for every 5s.
                    this.sync(loop);
                }
            }
        });
    }
    isSyncEnabled() {
        const { user: userLocal } = this.configurationService.inspect('userConfiguration.enableSync');
        return userLocal === undefined || userLocal;
    }
};
UserDataAutoSync = __decorate([
    __param(0, configuration_1.IConfigurationService),
    __param(1, userDataSync_1.IUserDataSyncService),
    __param(2, userDataSync_1.IUserDataSyncStoreService)
], UserDataAutoSync);
exports.UserDataAutoSync = UserDataAutoSync;
//# sourceMappingURL=userDataSyncService.js.map