"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const productService_1 = require("vs/platform/product/common/productService");
const event_1 = require("vs/base/common/event");
const request_1 = require("vs/platform/request/common/request");
const uri_1 = require("vs/base/common/uri");
const resources_1 = require("vs/base/common/resources");
const cancellation_1 = require("vs/base/common/cancellation");
let UserDataSyncStoreService = class UserDataSyncStoreService extends lifecycle_1.Disposable {
    constructor(productService, requestService) {
        super();
        this.productService = productService;
        this.requestService = requestService;
        this._loggedIn = false;
        this._onDidChangeLoggedIn = this._register(new event_1.Emitter());
        this.onDidChangeLoggedIn = this._onDidChangeLoggedIn.event;
    }
    get enabled() { return !!this.productService.settingsSyncStoreUrl; }
    get loggedIn() { return this._loggedIn; }
    login() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
        });
    }
    read(key, oldValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.enabled) {
                return Promise.reject(new Error('No settings sync store url configured.'));
            }
            const url = resources_1.joinPath(uri_1.URI.parse(this.productService.settingsSyncStoreUrl), key).toString();
            const headers = {};
            if (oldValue) {
                headers['If-None-Match'] = oldValue.ref;
            }
            const context = yield this.requestService.request({ type: 'GET', url, headers }, cancellation_1.CancellationToken.None);
            if (context.res.statusCode === 304) {
                // There is no new value. Hence return the old value.
                return oldValue;
            }
            const ref = context.res.headers['etag'];
            if (!ref) {
                throw new Error('Server did not return the ref');
            }
            const content = yield request_1.asText(context);
            return { ref, content };
        });
    }
    write(key, data, ref) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.enabled) {
                return Promise.reject(new Error('No settings sync store url configured.'));
            }
            const url = resources_1.joinPath(uri_1.URI.parse(this.productService.settingsSyncStoreUrl), key).toString();
            const headers = { 'Content-Type': 'text/plain' };
            if (ref) {
                headers['If-Match'] = ref;
            }
            const context = yield this.requestService.request({ type: 'POST', url, data, headers }, cancellation_1.CancellationToken.None);
            if (context.res.statusCode === 412) {
                // There is a new value. Throw Rejected Error
                throw new userDataSync_1.UserDataSyncStoreError('New data exists', userDataSync_1.UserDataSyncStoreErrorCode.Rejected);
            }
            const newRef = context.res.headers['etag'];
            if (!newRef) {
                throw new Error('Server did not return the ref');
            }
            return newRef;
        });
    }
};
UserDataSyncStoreService = __decorate([
    __param(0, productService_1.IProductService),
    __param(1, request_1.IRequestService)
], UserDataSyncStoreService);
exports.UserDataSyncStoreService = UserDataSyncStoreService;
//# sourceMappingURL=userDataSyncStoreService.js.map