"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const history_1 = require("vs/platform/history/common/history");
const uri_1 = require("vs/base/common/uri");
const mainProcessService_1 = require("vs/platform/ipc/electron-browser/mainProcessService");
let WindowsService = class WindowsService {
    constructor(mainProcessService) {
        this.channel = mainProcessService.getChannel('windows');
    }
    get onWindowOpen() { return this.channel.listen('onWindowOpen'); }
    get onWindowFocus() { return this.channel.listen('onWindowFocus'); }
    get onWindowBlur() { return this.channel.listen('onWindowBlur'); }
    get onWindowMaximize() { return this.channel.listen('onWindowMaximize'); }
    get onWindowUnmaximize() { return this.channel.listen('onWindowUnmaximize'); }
    get onRecentlyOpenedChange() { return this.channel.listen('onRecentlyOpenedChange'); }
    closeWorkspace(windowId) {
        return this.channel.call('closeWorkspace', windowId);
    }
    enterWorkspace(windowId, path) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.channel.call('enterWorkspace', [windowId, path]);
            if (result) {
                result.workspace = workspaces_1.reviveWorkspaceIdentifier(result.workspace);
            }
            return result;
        });
    }
    addRecentlyOpened(recent) {
        return this.channel.call('addRecentlyOpened', recent);
    }
    removeFromRecentlyOpened(paths) {
        return this.channel.call('removeFromRecentlyOpened', paths);
    }
    clearRecentlyOpened() {
        return this.channel.call('clearRecentlyOpened');
    }
    getRecentlyOpened(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            const recentlyOpened = yield this.channel.call('getRecentlyOpened', windowId);
            recentlyOpened.workspaces.forEach(recent => history_1.isRecentWorkspace(recent) ? recent.workspace = workspaces_1.reviveWorkspaceIdentifier(recent.workspace) : recent.folderUri = uri_1.URI.revive(recent.folderUri));
            recentlyOpened.files.forEach(recent => recent.fileUri = uri_1.URI.revive(recent.fileUri));
            return recentlyOpened;
        });
    }
    newWindowTab() {
        return this.channel.call('newWindowTab');
    }
    showPreviousWindowTab() {
        return this.channel.call('showPreviousWindowTab');
    }
    showNextWindowTab() {
        return this.channel.call('showNextWindowTab');
    }
    moveWindowTabToNewWindow() {
        return this.channel.call('moveWindowTabToNewWindow');
    }
    mergeAllWindowTabs() {
        return this.channel.call('mergeAllWindowTabs');
    }
    toggleWindowTabsBar() {
        return this.channel.call('toggleWindowTabsBar');
    }
    focusWindow(windowId) {
        return this.channel.call('focusWindow', windowId);
    }
    closeWindow(windowId) {
        return this.channel.call('closeWindow', windowId);
    }
    isFocused(windowId) {
        return this.channel.call('isFocused', windowId);
    }
    isMaximized(windowId) {
        return this.channel.call('isMaximized', windowId);
    }
    maximizeWindow(windowId) {
        return this.channel.call('maximizeWindow', windowId);
    }
    unmaximizeWindow(windowId) {
        return this.channel.call('unmaximizeWindow', windowId);
    }
    minimizeWindow(windowId) {
        return this.channel.call('minimizeWindow', windowId);
    }
    onWindowTitleDoubleClick(windowId) {
        return this.channel.call('onWindowTitleDoubleClick', windowId);
    }
    quit() {
        return this.channel.call('quit');
    }
    whenSharedProcessReady() {
        return this.channel.call('whenSharedProcessReady');
    }
    toggleSharedProcess() {
        return this.channel.call('toggleSharedProcess');
    }
    openWindow(windowId, uris, options) {
        return this.channel.call('openWindow', [windowId, uris, options]);
    }
    openExtensionDevelopmentHostWindow(args, env) {
        return this.channel.call('openExtensionDevelopmentHostWindow', [args, env]);
    }
    getWindows() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.channel.call('getWindows');
            for (const win of result) {
                if (win.folderUri) {
                    win.folderUri = uri_1.URI.revive(win.folderUri);
                }
                if (win.workspace) {
                    win.workspace = workspaces_1.reviveWorkspaceIdentifier(win.workspace);
                }
            }
            return result;
        });
    }
    getActiveWindowId() {
        return this.channel.call('getActiveWindowId');
    }
    openExternal(url) {
        return this.channel.call('openExternal', url);
    }
    startCrashReporter(config) {
        return this.channel.call('startCrashReporter', config);
    }
    updateTouchBar(windowId, items) {
        return this.channel.call('updateTouchBar', [windowId, items]);
    }
};
WindowsService = __decorate([
    __param(0, mainProcessService_1.IMainProcessService)
], WindowsService);
exports.WindowsService = WindowsService;
//# sourceMappingURL=windowsService.js.map