"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const objects_1 = require("vs/base/common/objects");
const uri_1 = require("vs/base/common/uri");
const environment_1 = require("vs/platform/environment/common/environment");
const electron_1 = require("electron");
const event_1 = require("vs/base/common/event");
const url_1 = require("vs/platform/url/common/url");
const windows_1 = require("vs/platform/windows/electron-main/windows");
const historyMainService_1 = require("vs/platform/history/electron-main/historyMainService");
const network_1 = require("vs/base/common/network");
const platform_1 = require("vs/base/common/platform");
const log_1 = require("vs/platform/log/common/log");
// @deprecated this should eventually go away and be implemented by host & electron service
let LegacyWindowsMainService = class LegacyWindowsMainService extends lifecycle_1.Disposable {
    constructor(sharedProcess, windowsMainService, environmentService, urlService, historyMainService, logService) {
        super();
        this.sharedProcess = sharedProcess;
        this.windowsMainService = windowsMainService;
        this.environmentService = environmentService;
        this.historyMainService = historyMainService;
        this.logService = logService;
        this.disposables = this._register(new lifecycle_1.DisposableStore());
        this.onWindowOpen = event_1.Event.filter(event_1.Event.fromNodeEventEmitter(electron_1.app, 'browser-window-created', (_, w) => w.id), id => !!this.windowsMainService.getWindowById(id));
        this.onWindowBlur = event_1.Event.filter(event_1.Event.fromNodeEventEmitter(electron_1.app, 'browser-window-blur', (_, w) => w.id), id => !!this.windowsMainService.getWindowById(id));
        this.onWindowMaximize = event_1.Event.filter(event_1.Event.fromNodeEventEmitter(electron_1.app, 'browser-window-maximize', (_, w) => w.id), id => !!this.windowsMainService.getWindowById(id));
        this.onWindowUnmaximize = event_1.Event.filter(event_1.Event.fromNodeEventEmitter(electron_1.app, 'browser-window-unmaximize', (_, w) => w.id), id => !!this.windowsMainService.getWindowById(id));
        this.onWindowFocus = event_1.Event.any(event_1.Event.map(event_1.Event.filter(event_1.Event.map(this.windowsMainService.onWindowsCountChanged, () => this.windowsMainService.getLastActiveWindow()), w => !!w), w => w.id), event_1.Event.filter(event_1.Event.fromNodeEventEmitter(electron_1.app, 'browser-window-focus', (_, w) => w.id), id => !!this.windowsMainService.getWindowById(id)));
        this.onRecentlyOpenedChange = this.historyMainService.onRecentlyOpenedChange;
        urlService.registerHandler(this);
        // remember last active window id
        event_1.Event.latch(event_1.Event.any(this.onWindowOpen, this.onWindowFocus))(id => this._activeWindowId = id, null, this.disposables);
    }
    showMessageBox(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#showMessageBox', windowId);
            return this.withWindow(windowId, codeWindow => this.windowsMainService.showMessageBox(options, codeWindow), () => this.windowsMainService.showMessageBox(options));
        });
    }
    showSaveDialog(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#showSaveDialog', windowId);
            return this.withWindow(windowId, codeWindow => this.windowsMainService.showSaveDialog(options, codeWindow), () => this.windowsMainService.showSaveDialog(options));
        });
    }
    showOpenDialog(windowId, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#showOpenDialog', windowId);
            return this.withWindow(windowId, codeWindow => this.windowsMainService.showOpenDialog(options, codeWindow), () => this.windowsMainService.showOpenDialog(options));
        });
    }
    updateTouchBar(windowId, items) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#updateTouchBar', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.updateTouchBar(items));
        });
    }
    closeWorkspace(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#closeWorkspace', windowId);
            return this.withWindow(windowId, codeWindow => this.windowsMainService.closeWorkspace(codeWindow));
        });
    }
    enterWorkspace(windowId, path) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#enterWorkspace', windowId);
            return this.withWindow(windowId, codeWindow => this.windowsMainService.enterWorkspace(codeWindow, path));
        });
    }
    addRecentlyOpened(recents) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#addRecentlyOpened');
            this.historyMainService.addRecentlyOpened(recents);
        });
    }
    removeFromRecentlyOpened(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#removeFromRecentlyOpened');
            this.historyMainService.removeFromRecentlyOpened(paths);
        });
    }
    clearRecentlyOpened() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#clearRecentlyOpened');
            this.historyMainService.clearRecentlyOpened();
        });
    }
    getRecentlyOpened(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#getRecentlyOpened', windowId);
            return this.withWindow(windowId, codeWindow => this.historyMainService.getRecentlyOpened(codeWindow.config.workspace, codeWindow.config.folderUri, codeWindow.config.filesToOpenOrCreate), () => this.historyMainService.getRecentlyOpened());
        });
    }
    newWindowTab() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#newWindowTab');
            this.windowsMainService.openNewTabbedWindow(5 /* API */);
        });
    }
    showPreviousWindowTab() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#showPreviousWindowTab');
            electron_1.Menu.sendActionToFirstResponder('selectPreviousTab:');
        });
    }
    showNextWindowTab() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#showNextWindowTab');
            electron_1.Menu.sendActionToFirstResponder('selectNextTab:');
        });
    }
    moveWindowTabToNewWindow() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#moveWindowTabToNewWindow');
            electron_1.Menu.sendActionToFirstResponder('moveTabToNewWindow:');
        });
    }
    mergeAllWindowTabs() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#mergeAllWindowTabs');
            electron_1.Menu.sendActionToFirstResponder('mergeAllWindows:');
        });
    }
    toggleWindowTabsBar() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#toggleWindowTabsBar');
            electron_1.Menu.sendActionToFirstResponder('toggleTabBar:');
        });
    }
    focusWindow(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#focusWindow', windowId);
            if (platform_1.isMacintosh) {
                return this.withWindow(windowId, codeWindow => codeWindow.win.show());
            }
            else {
                return this.withWindow(windowId, codeWindow => codeWindow.win.focus());
            }
        });
    }
    closeWindow(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#closeWindow', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.close());
        });
    }
    isFocused(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#isFocused', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.isFocused(), () => false);
        });
    }
    isMaximized(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#isMaximized', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.isMaximized(), () => false);
        });
    }
    maximizeWindow(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#maximizeWindow', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.maximize());
        });
    }
    unmaximizeWindow(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#unmaximizeWindow', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.unmaximize());
        });
    }
    minimizeWindow(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#minimizeWindow', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.win.minimize());
        });
    }
    onWindowTitleDoubleClick(windowId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#onWindowTitleDoubleClick', windowId);
            return this.withWindow(windowId, codeWindow => codeWindow.onWindowTitleDoubleClick());
        });
    }
    openWindow(windowId, urisToOpen, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#openWindow');
            if (!urisToOpen || !urisToOpen.length) {
                return undefined;
            }
            this.windowsMainService.open({
                context: 5 /* API */,
                contextWindowId: windowId,
                urisToOpen: urisToOpen,
                cli: options.args ? Object.assign(Object.assign({}, this.environmentService.args), options.args) : this.environmentService.args,
                forceNewWindow: options.forceNewWindow,
                forceReuseWindow: options.forceReuseWindow,
                diffMode: options.diffMode,
                addMode: options.addMode,
                gotoLineMode: options.gotoLineMode,
                noRecentEntry: options.noRecentEntry,
                waitMarkerFileURI: options.waitMarkerFileURI
            });
        });
    }
    openExtensionDevelopmentHostWindow(args, env) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#openExtensionDevelopmentHostWindow ' + JSON.stringify(args));
            const extDevPaths = args.extensionDevelopmentPath;
            if (extDevPaths) {
                this.windowsMainService.openExtensionDevelopmentHostWindow(extDevPaths, {
                    context: 5 /* API */,
                    cli: args,
                    userEnv: Object.keys(env).length > 0 ? env : undefined
                });
            }
        });
    }
    getWindows() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#getWindows');
            const windows = this.windowsMainService.getWindows();
            return windows.map(window => ({
                id: window.id,
                workspace: window.openedWorkspace,
                folderUri: window.openedFolderUri,
                title: window.win.getTitle(),
                filename: window.getRepresentedFilename()
            }));
        });
    }
    getWindowCount() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#getWindowCount');
            return this.windowsMainService.getWindows().length;
        });
    }
    getActiveWindowId() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._activeWindowId;
        });
    }
    openExternal(url) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.windowsMainService.openExternal(url);
        });
    }
    startCrashReporter(config) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#startCrashReporter');
            electron_1.crashReporter.start(config);
        });
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#quit');
            this.windowsMainService.quit();
        });
    }
    whenSharedProcessReady() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#whenSharedProcessReady');
            return this.sharedProcess.whenReady();
        });
    }
    toggleSharedProcess() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logService.trace('windowsService#toggleSharedProcess');
            this.sharedProcess.toggle();
        });
    }
    handleURL(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            // Catch file URLs
            if (uri.authority === network_1.Schemas.file && !!uri.path) {
                this.openFileForURI({ fileUri: uri_1.URI.file(uri.fsPath) }); // using fsPath on a non-file URI...
                return true;
            }
            return false;
        });
    }
    openFileForURI(uri) {
        const cli = objects_1.assign(Object.create(null), this.environmentService.args);
        const urisToOpen = [uri];
        this.windowsMainService.open({ context: 5 /* API */, cli, urisToOpen, gotoLineMode: true });
    }
    withWindow(windowId, fn, fallback) {
        const codeWindow = this.windowsMainService.getWindowById(windowId);
        if (codeWindow) {
            return fn(codeWindow);
        }
        if (fallback) {
            return fallback();
        }
        return undefined;
    }
};
LegacyWindowsMainService = __decorate([
    __param(1, windows_1.IWindowsMainService),
    __param(2, environment_1.IEnvironmentService),
    __param(3, url_1.IURLService),
    __param(4, historyMainService_1.IHistoryMainService),
    __param(5, log_1.ILogService)
], LegacyWindowsMainService);
exports.LegacyWindowsMainService = LegacyWindowsMainService;
//# sourceMappingURL=legacyWindowsMainService.js.map