"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const event_1 = require("vs/base/common/event");
const lifecycle_1 = require("vs/base/common/lifecycle");
const uri_1 = require("vs/base/common/uri");
const opener_1 = require("vs/platform/opener/common/opener");
const windows_1 = require("vs/platform/windows/common/windows");
const extHostCustomers_1 = require("vs/workbench/api/common/extHostCustomers");
const extHost_protocol_1 = require("../common/extHost.protocol");
let MainThreadWindow = class MainThreadWindow {
    constructor(extHostContext, windowService, openerService) {
        this.windowService = windowService;
        this.openerService = openerService;
        this.handlePool = 1;
        this.disposables = new lifecycle_1.DisposableStore();
        this.resolved = new Map();
        this.proxy = extHostContext.getProxy(extHost_protocol_1.ExtHostContext.ExtHostWindow);
        event_1.Event.latch(windowService.onDidChangeFocus)(this.proxy.$onDidChangeWindowFocus, this.proxy, this.disposables);
    }
    dispose() {
        this.disposables.dispose();
        for (const value of this.resolved.values()) {
            value.dispose();
        }
        this.resolved.clear();
    }
    $getWindowVisibility() {
        return this.windowService.isFocused();
    }
    $openUri(uriComponents, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = uri_1.URI.from(uriComponents);
            return this.openerService.open(uri, { openExternal: true, allowTunneling: options.allowTunneling });
        });
    }
    $resolveExternalUri(uriComponents, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = uri_1.URI.revive(uriComponents);
            const handle = ++this.handlePool;
            const result = yield this.openerService.resolveExternalUri(uri, options);
            this.resolved.set(handle, result);
            return { handle, result: result.resolved };
        });
    }
    $releaseResolvedExternalUri(handle) {
        return __awaiter(this, void 0, void 0, function* () {
            const entry = this.resolved.get(handle);
            if (entry) {
                entry.dispose();
                this.resolved.delete(handle);
                return true;
            }
            return false;
        });
    }
};
MainThreadWindow = __decorate([
    extHostCustomers_1.extHostNamedCustomer(extHost_protocol_1.MainContext.MainThreadWindow),
    __param(1, windows_1.IWindowService),
    __param(2, opener_1.IOpenerService)
], MainThreadWindow);
exports.MainThreadWindow = MainThreadWindow;
//# sourceMappingURL=mainThreadWindow.js.map