"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const extHost_protocol_1 = require("./extHost.protocol");
const event_1 = require("vs/base/common/event");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
class ExtHostStorage {
    constructor(mainContext) {
        this._onDidChangeStorage = new event_1.Emitter();
        this.onDidChangeStorage = this._onDidChangeStorage.event;
        this._proxy = mainContext.getProxy(extHost_protocol_1.MainContext.MainThreadStorage);
    }
    getValue(shared, key, defaultValue) {
        return this._proxy.$getValue(shared, key).then(value => value || defaultValue);
    }
    setValue(shared, key, value) {
        return this._proxy.$setValue(shared, key, value);
    }
    $acceptValue(shared, key, value) {
        this._onDidChangeStorage.fire({ shared, key, value });
    }
}
exports.ExtHostStorage = ExtHostStorage;
exports.IExtHostStorage = instantiation_1.createDecorator('IExtHostStorage');
//# sourceMappingURL=extHostStorage.js.map