"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/css!./media/actions");
const nls = require("vs/nls");
const actions_1 = require("vs/base/common/actions");
const windows_1 = require("vs/platform/windows/common/windows");
const dialogs_1 = require("vs/platform/dialogs/common/dialogs");
const actions_2 = require("vs/platform/actions/common/actions");
const platform_1 = require("vs/platform/registry/common/platform");
const contextkeys_1 = require("vs/workbench/browser/contextkeys");
const actions_3 = require("vs/workbench/common/actions");
const keybindingsRegistry_1 = require("vs/platform/keybinding/common/keybindingsRegistry");
const quickInput_1 = require("vs/platform/quickinput/common/quickInput");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const label_1 = require("vs/platform/label/common/label");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const modelService_1 = require("vs/editor/common/services/modelService");
const modeService_1 = require("vs/editor/common/services/modeService");
const history_1 = require("vs/platform/history/common/history");
const getIconClasses_1 = require("vs/editor/common/services/getIconClasses");
const files_1 = require("vs/platform/files/common/files");
const labels_1 = require("vs/base/common/labels");
const platform_2 = require("vs/base/common/platform");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const quickopen_1 = require("vs/workbench/browser/parts/quickopen/quickopen");
const host_1 = require("vs/workbench/services/host/browser/host");
exports.inRecentFilesPickerContextKey = 'inRecentFilesPicker';
class BaseOpenRecentAction extends actions_1.Action {
    constructor(id, label, windowService, quickInputService, contextService, labelService, keybindingService, modelService, modeService) {
        super(id, label);
        this.windowService = windowService;
        this.quickInputService = quickInputService;
        this.contextService = contextService;
        this.labelService = labelService;
        this.keybindingService = keybindingService;
        this.modelService = modelService;
        this.modeService = modeService;
        this.removeFromRecentlyOpened = {
            iconClass: 'action-remove-from-recently-opened',
            tooltip: nls.localize('remove', "Remove from Recently Opened")
        };
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { workspaces, files } = yield this.windowService.getRecentlyOpened();
            this.openRecent(workspaces, files);
        });
    }
    openRecent(recentWorkspaces, recentFiles) {
        return __awaiter(this, void 0, void 0, function* () {
            const toPick = (recent, labelService, buttons) => {
                let uriToOpen;
                let iconClasses;
                let fullLabel;
                let resource;
                // Folder
                if (history_1.isRecentFolder(recent)) {
                    resource = recent.folderUri;
                    iconClasses = getIconClasses_1.getIconClasses(this.modelService, this.modeService, resource, files_1.FileKind.FOLDER);
                    uriToOpen = { folderUri: resource };
                    fullLabel = recent.label || labelService.getWorkspaceLabel(resource, { verbose: true });
                }
                // Workspace
                else if (history_1.isRecentWorkspace(recent)) {
                    resource = recent.workspace.configPath;
                    iconClasses = getIconClasses_1.getIconClasses(this.modelService, this.modeService, resource, files_1.FileKind.ROOT_FOLDER);
                    uriToOpen = { workspaceUri: resource };
                    fullLabel = recent.label || labelService.getWorkspaceLabel(recent.workspace, { verbose: true });
                }
                // File
                else {
                    resource = recent.fileUri;
                    iconClasses = getIconClasses_1.getIconClasses(this.modelService, this.modeService, resource, files_1.FileKind.FILE);
                    uriToOpen = { fileUri: resource };
                    fullLabel = recent.label || labelService.getUriLabel(resource);
                }
                const { name, parentPath } = labels_1.splitName(fullLabel);
                return {
                    iconClasses,
                    label: name,
                    description: parentPath,
                    buttons,
                    uriToOpen,
                    resource
                };
            };
            const workspacePicks = recentWorkspaces.map(workspace => toPick(workspace, this.labelService, !this.isQuickNavigate() ? [this.removeFromRecentlyOpened] : undefined));
            const filePicks = recentFiles.map(p => toPick(p, this.labelService, !this.isQuickNavigate() ? [this.removeFromRecentlyOpened] : undefined));
            // focus second entry if the first recent workspace is the current workspace
            const firstEntry = recentWorkspaces[0];
            let autoFocusSecondEntry = firstEntry && this.contextService.isCurrentWorkspace(history_1.isRecentWorkspace(firstEntry) ? firstEntry.workspace : firstEntry.folderUri);
            let keyMods;
            const workspaceSeparator = { type: 'separator', label: nls.localize('workspaces', "workspaces") };
            const fileSeparator = { type: 'separator', label: nls.localize('files', "files") };
            const picks = [workspaceSeparator, ...workspacePicks, fileSeparator, ...filePicks];
            const pick = yield this.quickInputService.pick(picks, {
                contextKey: exports.inRecentFilesPickerContextKey,
                activeItem: [...workspacePicks, ...filePicks][autoFocusSecondEntry ? 1 : 0],
                placeHolder: platform_2.isMacintosh ? nls.localize('openRecentPlaceHolderMac', "Select to open (hold Cmd-key to open in new window)") : nls.localize('openRecentPlaceHolder', "Select to open (hold Ctrl-key to open in new window)"),
                matchOnDescription: true,
                onKeyMods: mods => keyMods = mods,
                quickNavigate: this.isQuickNavigate() ? { keybindings: this.keybindingService.lookupKeybindings(this.id) } : undefined,
                onDidTriggerItemButton: (context) => __awaiter(this, void 0, void 0, function* () {
                    yield this.windowService.removeFromRecentlyOpened([context.item.resource]);
                    context.removeItem();
                })
            });
            if (pick) {
                return this.windowService.openWindow([pick.uriToOpen], { forceNewWindow: keyMods && keyMods.ctrlCmd });
            }
        });
    }
}
let OpenRecentAction = class OpenRecentAction extends BaseOpenRecentAction {
    constructor(id, label, windowService, quickInputService, contextService, keybindingService, modelService, modeService, labelService) {
        super(id, label, windowService, quickInputService, contextService, labelService, keybindingService, modelService, modeService);
    }
    isQuickNavigate() {
        return false;
    }
};
OpenRecentAction.ID = 'workbench.action.openRecent';
OpenRecentAction.LABEL = nls.localize('openRecent', "Open Recent...");
OpenRecentAction = __decorate([
    __param(2, windows_1.IWindowService),
    __param(3, quickInput_1.IQuickInputService),
    __param(4, workspace_1.IWorkspaceContextService),
    __param(5, keybinding_1.IKeybindingService),
    __param(6, modelService_1.IModelService),
    __param(7, modeService_1.IModeService),
    __param(8, label_1.ILabelService)
], OpenRecentAction);
exports.OpenRecentAction = OpenRecentAction;
let QuickOpenRecentAction = class QuickOpenRecentAction extends BaseOpenRecentAction {
    constructor(id, label, windowService, quickInputService, contextService, keybindingService, modelService, modeService, labelService) {
        super(id, label, windowService, quickInputService, contextService, labelService, keybindingService, modelService, modeService);
    }
    isQuickNavigate() {
        return true;
    }
};
QuickOpenRecentAction.ID = 'workbench.action.quickOpenRecent';
QuickOpenRecentAction.LABEL = nls.localize('quickOpenRecent', "Quick Open Recent...");
QuickOpenRecentAction = __decorate([
    __param(2, windows_1.IWindowService),
    __param(3, quickInput_1.IQuickInputService),
    __param(4, workspace_1.IWorkspaceContextService),
    __param(5, keybinding_1.IKeybindingService),
    __param(6, modelService_1.IModelService),
    __param(7, modeService_1.IModeService),
    __param(8, label_1.ILabelService)
], QuickOpenRecentAction);
let ToggleFullScreenAction = class ToggleFullScreenAction extends actions_1.Action {
    constructor(id, label, hostService) {
        super(id, label);
        this.hostService = hostService;
    }
    run() {
        return this.hostService.toggleFullScreen();
    }
};
ToggleFullScreenAction.ID = 'workbench.action.toggleFullScreen';
ToggleFullScreenAction.LABEL = nls.localize('toggleFullScreen', "Toggle Full Screen");
ToggleFullScreenAction = __decorate([
    __param(2, host_1.IHostService)
], ToggleFullScreenAction);
let ReloadWindowAction = class ReloadWindowAction extends actions_1.Action {
    constructor(id, label, hostService) {
        super(id, label);
        this.hostService = hostService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.hostService.reload();
            return true;
        });
    }
};
ReloadWindowAction.ID = 'workbench.action.reloadWindow';
ReloadWindowAction.LABEL = nls.localize('reloadWindow', "Reload Window");
ReloadWindowAction = __decorate([
    __param(2, host_1.IHostService)
], ReloadWindowAction);
exports.ReloadWindowAction = ReloadWindowAction;
let ShowAboutDialogAction = class ShowAboutDialogAction extends actions_1.Action {
    constructor(id, label, dialogService) {
        super(id, label);
        this.dialogService = dialogService;
    }
    run() {
        return this.dialogService.about();
    }
};
ShowAboutDialogAction.ID = 'workbench.action.showAboutDialog';
ShowAboutDialogAction.LABEL = nls.localize('about', "About");
ShowAboutDialogAction = __decorate([
    __param(2, dialogs_1.IDialogService)
], ShowAboutDialogAction);
let NewWindowAction = class NewWindowAction extends actions_1.Action {
    constructor(id, label, hostService) {
        super(id, label);
        this.hostService = hostService;
    }
    run() {
        return this.hostService.openEmptyWindow();
    }
};
NewWindowAction.ID = 'workbench.action.newWindow';
NewWindowAction.LABEL = nls.localize('newWindow', "New Window");
NewWindowAction = __decorate([
    __param(2, host_1.IHostService)
], NewWindowAction);
exports.NewWindowAction = NewWindowAction;
const registry = platform_1.Registry.as(actions_3.Extensions.WorkbenchActions);
// --- Actions Registration
const fileCategory = nls.localize('file', "File");
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(NewWindowAction, NewWindowAction.ID, NewWindowAction.LABEL, { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 44 /* KEY_N */ }), 'New Window');
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(QuickOpenRecentAction, QuickOpenRecentAction.ID, QuickOpenRecentAction.LABEL), 'File: Quick Open Recent...', fileCategory);
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(OpenRecentAction, OpenRecentAction.ID, OpenRecentAction.LABEL, { primary: 2048 /* CtrlCmd */ | 48 /* KEY_R */, mac: { primary: 256 /* WinCtrl */ | 48 /* KEY_R */ } }), 'File: Open Recent...', fileCategory);
const viewCategory = nls.localize('view', "View");
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ToggleFullScreenAction, ToggleFullScreenAction.ID, ToggleFullScreenAction.LABEL, { primary: 69 /* F11 */, mac: { primary: 2048 /* CtrlCmd */ | 256 /* WinCtrl */ | 36 /* KEY_F */ } }), 'View: Toggle Full Screen', viewCategory);
const developerCategory = nls.localize('developer', "Developer");
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ReloadWindowAction, ReloadWindowAction.ID, ReloadWindowAction.LABEL), 'Developer: Reload Window', developerCategory);
const helpCategory = nls.localize('help', "Help");
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(ShowAboutDialogAction, ShowAboutDialogAction.ID, ShowAboutDialogAction.LABEL), `Help: About`, helpCategory);
// --- Commands/Keybindings Registration
const recentFilesPickerContext = contextkey_1.ContextKeyExpr.and(quickopen_1.inQuickOpenContext, contextkey_1.ContextKeyExpr.has(exports.inRecentFilesPickerContextKey));
const quickOpenNavigateNextInRecentFilesPickerId = 'workbench.action.quickOpenNavigateNextInRecentFilesPicker';
keybindingsRegistry_1.KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickOpenNavigateNextInRecentFilesPickerId,
    weight: 200 /* WorkbenchContrib */ + 50,
    handler: quickopen_1.getQuickNavigateHandler(quickOpenNavigateNextInRecentFilesPickerId, true),
    when: recentFilesPickerContext,
    primary: 2048 /* CtrlCmd */ | 48 /* KEY_R */,
    mac: { primary: 256 /* WinCtrl */ | 48 /* KEY_R */ }
});
const quickOpenNavigatePreviousInRecentFilesPicker = 'workbench.action.quickOpenNavigatePreviousInRecentFilesPicker';
keybindingsRegistry_1.KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: quickOpenNavigatePreviousInRecentFilesPicker,
    weight: 200 /* WorkbenchContrib */ + 50,
    handler: quickopen_1.getQuickNavigateHandler(quickOpenNavigatePreviousInRecentFilesPicker, false),
    when: recentFilesPickerContext,
    primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 48 /* KEY_R */,
    mac: { primary: 256 /* WinCtrl */ | 1024 /* Shift */ | 48 /* KEY_R */ }
});
keybindingsRegistry_1.KeybindingsRegistry.registerKeybindingRule({
    id: ReloadWindowAction.ID,
    weight: 200 /* WorkbenchContrib */ + 50,
    when: contextkeys_1.IsDevelopmentContext,
    primary: 2048 /* CtrlCmd */ | 48 /* KEY_R */
});
// --- Menu Registration
actions_2.MenuRegistry.appendMenuItem(15 /* MenubarFileMenu */, {
    group: '1_new',
    command: {
        id: NewWindowAction.ID,
        title: nls.localize({ key: 'miNewWindow', comment: ['&& denotes a mnemonic'] }, "New &&Window")
    },
    order: 2
});
actions_2.MenuRegistry.appendMenuItem(15 /* MenubarFileMenu */, {
    title: nls.localize({ key: 'miOpenRecent', comment: ['&& denotes a mnemonic'] }, "Open &&Recent"),
    submenu: 21 /* MenubarRecentMenu */,
    group: '2_open',
    order: 4
});
actions_2.MenuRegistry.appendMenuItem(21 /* MenubarRecentMenu */, {
    group: 'y_more',
    command: {
        id: OpenRecentAction.ID,
        title: nls.localize({ key: 'miMore', comment: ['&& denotes a mnemonic'] }, "&&More...")
    },
    order: 1
});
actions_2.MenuRegistry.appendMenuItem(12 /* MenubarAppearanceMenu */, {
    group: '1_toggle_view',
    command: {
        id: ToggleFullScreenAction.ID,
        title: nls.localize({ key: 'miToggleFullScreen', comment: ['&& denotes a mnemonic'] }, "&&Full Screen"),
        toggled: contextkeys_1.IsFullscreenContext
    },
    order: 1
});
actions_2.MenuRegistry.appendMenuItem(17 /* MenubarHelpMenu */, {
    group: 'z_about',
    command: {
        id: ShowAboutDialogAction.ID,
        title: nls.localize({ key: 'miAbout', comment: ['&& denotes a mnemonic'] }, "&&About")
    },
    order: 1,
    when: contextkeys_1.IsMacNativeContext.toNegated()
});
//# sourceMappingURL=windowActions.js.map