"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const actions_1 = require("vs/base/common/actions");
const nls = require("vs/nls");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const workspaceEditing_1 = require("vs/workbench/services/workspace/common/workspaceEditing");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const commands_1 = require("vs/platform/commands/common/commands");
const workspaceCommands_1 = require("vs/workbench/browser/actions/workspaceCommands");
const dialogs_1 = require("vs/platform/dialogs/common/dialogs");
const actions_2 = require("vs/platform/actions/common/actions");
const contextkeys_1 = require("vs/workbench/browser/contextkeys");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const platform_1 = require("vs/platform/registry/common/platform");
const actions_3 = require("vs/workbench/common/actions");
let OpenFileAction = class OpenFileAction extends actions_1.Action {
    constructor(id, label, dialogService) {
        super(id, label);
        this.dialogService = dialogService;
    }
    run(event, data) {
        return this.dialogService.pickFileAndOpen({ forceNewWindow: false, telemetryExtraData: data });
    }
};
OpenFileAction.ID = 'workbench.action.files.openFile';
OpenFileAction.LABEL = nls.localize('openFile', "Open File...");
OpenFileAction = __decorate([
    __param(2, dialogs_1.IFileDialogService)
], OpenFileAction);
exports.OpenFileAction = OpenFileAction;
let OpenFolderAction = class OpenFolderAction extends actions_1.Action {
    constructor(id, label, dialogService) {
        super(id, label);
        this.dialogService = dialogService;
    }
    run(event, data) {
        return this.dialogService.pickFolderAndOpen({ forceNewWindow: false, telemetryExtraData: data });
    }
};
OpenFolderAction.ID = 'workbench.action.files.openFolder';
OpenFolderAction.LABEL = nls.localize('openFolder', "Open Folder...");
OpenFolderAction = __decorate([
    __param(2, dialogs_1.IFileDialogService)
], OpenFolderAction);
exports.OpenFolderAction = OpenFolderAction;
let OpenFileFolderAction = class OpenFileFolderAction extends actions_1.Action {
    constructor(id, label, dialogService) {
        super(id, label);
        this.dialogService = dialogService;
    }
    run(event, data) {
        return this.dialogService.pickFileFolderAndOpen({ forceNewWindow: false, telemetryExtraData: data });
    }
};
OpenFileFolderAction.ID = 'workbench.action.files.openFileFolder';
OpenFileFolderAction.LABEL = nls.localize('openFileFolder', "Open...");
OpenFileFolderAction = __decorate([
    __param(2, dialogs_1.IFileDialogService)
], OpenFileFolderAction);
exports.OpenFileFolderAction = OpenFileFolderAction;
let OpenWorkspaceAction = class OpenWorkspaceAction extends actions_1.Action {
    constructor(id, label, dialogService) {
        super(id, label);
        this.dialogService = dialogService;
    }
    run(event, data) {
        return this.dialogService.pickWorkspaceAndOpen({ telemetryExtraData: data });
    }
};
OpenWorkspaceAction.ID = 'workbench.action.openWorkspace';
OpenWorkspaceAction.LABEL = nls.localize('openWorkspaceAction', "Open Workspace...");
OpenWorkspaceAction = __decorate([
    __param(2, dialogs_1.IFileDialogService)
], OpenWorkspaceAction);
exports.OpenWorkspaceAction = OpenWorkspaceAction;
let OpenWorkspaceConfigFileAction = class OpenWorkspaceConfigFileAction extends actions_1.Action {
    constructor(id, label, workspaceContextService, editorService) {
        super(id, label);
        this.workspaceContextService = workspaceContextService;
        this.editorService = editorService;
        this.enabled = !!this.workspaceContextService.getWorkspace().configuration;
    }
    run() {
        const configuration = this.workspaceContextService.getWorkspace().configuration;
        if (configuration) {
            return this.editorService.openEditor({ resource: configuration });
        }
        return Promise.resolve();
    }
};
OpenWorkspaceConfigFileAction.ID = 'workbench.action.openWorkspaceConfigFile';
OpenWorkspaceConfigFileAction.LABEL = nls.localize('openWorkspaceConfigFile', "Open Workspace Configuration File");
OpenWorkspaceConfigFileAction = __decorate([
    __param(2, workspace_1.IWorkspaceContextService),
    __param(3, editorService_1.IEditorService)
], OpenWorkspaceConfigFileAction);
exports.OpenWorkspaceConfigFileAction = OpenWorkspaceConfigFileAction;
let AddRootFolderAction = class AddRootFolderAction extends actions_1.Action {
    constructor(id, label, commandService) {
        super(id, label);
        this.commandService = commandService;
    }
    run() {
        return this.commandService.executeCommand(workspaceCommands_1.ADD_ROOT_FOLDER_COMMAND_ID);
    }
};
AddRootFolderAction.ID = 'workbench.action.addRootFolder';
AddRootFolderAction.LABEL = workspaceCommands_1.ADD_ROOT_FOLDER_LABEL;
AddRootFolderAction = __decorate([
    __param(2, commands_1.ICommandService)
], AddRootFolderAction);
exports.AddRootFolderAction = AddRootFolderAction;
let GlobalRemoveRootFolderAction = class GlobalRemoveRootFolderAction extends actions_1.Action {
    constructor(id, label, workspaceEditingService, contextService, commandService) {
        super(id, label);
        this.workspaceEditingService = workspaceEditingService;
        this.contextService = contextService;
        this.commandService = commandService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const state = this.contextService.getWorkbenchState();
            // Workspace / Folder
            if (state === 3 /* WORKSPACE */ || state === 2 /* FOLDER */) {
                const folder = yield this.commandService.executeCommand(workspaceCommands_1.PICK_WORKSPACE_FOLDER_COMMAND_ID);
                if (folder) {
                    yield this.workspaceEditingService.removeFolders([folder.uri]);
                }
            }
            return true;
        });
    }
};
GlobalRemoveRootFolderAction.ID = 'workbench.action.removeRootFolder';
GlobalRemoveRootFolderAction.LABEL = nls.localize('globalRemoveFolderFromWorkspace', "Remove Folder from Workspace...");
GlobalRemoveRootFolderAction = __decorate([
    __param(2, workspaceEditing_1.IWorkspaceEditingService),
    __param(3, workspace_1.IWorkspaceContextService),
    __param(4, commands_1.ICommandService)
], GlobalRemoveRootFolderAction);
exports.GlobalRemoveRootFolderAction = GlobalRemoveRootFolderAction;
// --- Actions Registration
const registry = platform_1.Registry.as(actions_3.Extensions.WorkbenchActions);
const workspacesCategory = nls.localize('workspaces', "Workspaces");
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(AddRootFolderAction, AddRootFolderAction.ID, AddRootFolderAction.LABEL), 'Workspaces: Add Folder to Workspace...', workspacesCategory, contextkeys_1.SupportsWorkspacesContext);
registry.registerWorkbenchAction(new actions_2.SyncActionDescriptor(GlobalRemoveRootFolderAction, GlobalRemoveRootFolderAction.ID, GlobalRemoveRootFolderAction.LABEL), 'Workspaces: Remove Folder from Workspace...', workspacesCategory, contextkeys_1.SupportsWorkspacesContext);
// --- Menu Registration
commands_1.CommandsRegistry.registerCommand(OpenWorkspaceConfigFileAction.ID, serviceAccessor => {
    serviceAccessor.get(instantiation_1.IInstantiationService).createInstance(OpenWorkspaceConfigFileAction, OpenWorkspaceConfigFileAction.ID, OpenWorkspaceConfigFileAction.LABEL).run();
});
actions_2.MenuRegistry.appendMenuItem(15 /* MenubarFileMenu */, {
    group: '3_workspace',
    command: {
        id: workspaceCommands_1.ADD_ROOT_FOLDER_COMMAND_ID,
        title: nls.localize({ key: 'miAddFolderToWorkspace', comment: ['&& denotes a mnemonic'] }, "A&&dd Folder to Workspace...")
    },
    order: 1,
    when: contextkeys_1.SupportsWorkspacesContext
});
actions_2.MenuRegistry.appendMenuItem(0 /* CommandPalette */, {
    command: {
        id: OpenWorkspaceConfigFileAction.ID,
        title: { value: `${workspacesCategory}: ${OpenWorkspaceConfigFileAction.LABEL}`, original: 'Workspaces: Open Workspace Configuration File' },
    },
    when: contextkeys_1.WorkbenchStateContext.isEqualTo('workspace')
});
//# sourceMappingURL=workspaceActions.js.map