"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const aria_1 = require("vs/base/browser/ui/aria/aria");
const nls_1 = require("vs/nls");
const lifecycle_1 = require("vs/base/common/lifecycle");
const errorMessage_1 = require("vs/base/common/errorMessage");
const notification_1 = require("vs/platform/notification/common/notification");
const event_1 = require("vs/base/common/event");
class NotificationsAlerts extends lifecycle_1.Disposable {
    constructor(model) {
        super();
        this.model = model;
        // Alert initial notifications if any
        model.notifications.forEach(n => this.triggerAriaAlert(n));
        this.registerListeners();
    }
    registerListeners() {
        this._register(this.model.onDidNotificationChange(e => this.onDidNotificationChange(e)));
    }
    onDidNotificationChange(e) {
        if (e.kind === 0 /* ADD */) {
            // ARIA alert for screen readers
            this.triggerAriaAlert(e.item);
            // Always log errors to console with full details
            if (e.item.severity === notification_1.Severity.Error) {
                if (e.item.message.original instanceof Error) {
                    console.error(e.item.message.original);
                }
                else {
                    console.error(errorMessage_1.toErrorMessage(e.item.message.value, true));
                }
            }
        }
    }
    triggerAriaAlert(notifiation) {
        // Trigger the alert again whenever the label changes
        const listener = notifiation.onDidLabelChange(e => {
            if (e.kind === 1 /* MESSAGE */) {
                this.doTriggerAriaAlert(notifiation);
            }
        });
        event_1.Event.once(notifiation.onDidClose)(() => listener.dispose());
        this.doTriggerAriaAlert(notifiation);
    }
    doTriggerAriaAlert(notifiation) {
        let alertText;
        if (notifiation.severity === notification_1.Severity.Error) {
            alertText = nls_1.localize('alertErrorMessage', "Error: {0}", notifiation.message.value);
        }
        else if (notifiation.severity === notification_1.Severity.Warning) {
            alertText = nls_1.localize('alertWarningMessage', "Warning: {0}", notifiation.message.value);
        }
        else {
            alertText = nls_1.localize('alertInfoMessage', "Info: {0}", notifiation.message.value);
        }
        aria_1.alert(alertText);
    }
}
exports.NotificationsAlerts = NotificationsAlerts;
//# sourceMappingURL=notificationsAlerts.js.map