"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/css!./media/panelviewlet");
const nls = require("vs/nls");
const event_1 = require("vs/base/common/event");
const styler_1 = require("vs/platform/theme/common/styler");
const theme_1 = require("vs/workbench/common/theme");
const dom_1 = require("vs/base/browser/dom");
const lifecycle_1 = require("vs/base/common/lifecycle");
const arrays_1 = require("vs/base/common/arrays");
const platform_1 = require("vs/platform/registry/common/platform");
const actions_1 = require("vs/workbench/browser/actions");
const viewlet_1 = require("vs/workbench/browser/viewlet");
const toolbar_1 = require("vs/base/browser/ui/toolbar/toolbar");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const themeService_1 = require("vs/platform/theme/common/themeService");
const panelview_1 = require("vs/base/browser/ui/splitview/panelview");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const layoutService_1 = require("vs/workbench/services/layout/browser/layoutService");
const mouseEvent_1 = require("vs/base/browser/mouseEvent");
const views_1 = require("vs/workbench/common/views");
const storage_1 = require("vs/platform/storage/common/storage");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
let ViewletPanel = class ViewletPanel extends panelview_1.Panel {
    constructor(options, keybindingService, contextMenuService, configurationService, contextKeyService) {
        super(options);
        this.keybindingService = keybindingService;
        this.contextMenuService = contextMenuService;
        this.configurationService = configurationService;
        this._onDidFocus = this._register(new event_1.Emitter());
        this.onDidFocus = this._onDidFocus.event;
        this._onDidBlur = this._register(new event_1.Emitter());
        this.onDidBlur = this._onDidBlur.event;
        this._onDidChangeBodyVisibility = this._register(new event_1.Emitter());
        this.onDidChangeBodyVisibility = this._onDidChangeBodyVisibility.event;
        this._onDidChangeTitleArea = this._register(new event_1.Emitter());
        this.onDidChangeTitleArea = this._onDidChangeTitleArea.event;
        this._isVisible = false;
        this.showActionsAlways = false;
        this.id = options.id;
        this.title = options.title;
        this.actionRunner = options.actionRunner;
        this.showActionsAlways = !!options.showActionsAlways;
        this.focusedViewContextKey = views_1.FocusedViewContext.bindTo(contextKeyService);
    }
    setVisible(visible) {
        if (this._isVisible !== visible) {
            this._isVisible = visible;
            if (this.isExpanded()) {
                this._onDidChangeBodyVisibility.fire(visible);
            }
        }
    }
    isVisible() {
        return this._isVisible;
    }
    isBodyVisible() {
        return this._isVisible && this.isExpanded();
    }
    setExpanded(expanded) {
        const changed = super.setExpanded(expanded);
        if (changed) {
            this._onDidChangeBodyVisibility.fire(expanded);
        }
        return changed;
    }
    render() {
        super.render();
        const focusTracker = dom_1.trackFocus(this.element);
        this._register(focusTracker);
        this._register(focusTracker.onDidFocus(() => {
            this.focusedViewContextKey.set(this.id);
            this._onDidFocus.fire();
        }));
        this._register(focusTracker.onDidBlur(() => {
            this.focusedViewContextKey.reset();
            this._onDidBlur.fire();
        }));
    }
    renderHeader(container) {
        this.headerContainer = container;
        this.renderHeaderTitle(container, this.title);
        const actions = dom_1.append(container, dom_1.$('.actions'));
        dom_1.toggleClass(actions, 'show', this.showActionsAlways);
        this.toolbar = new toolbar_1.ToolBar(actions, this.contextMenuService, {
            orientation: 0 /* HORIZONTAL */,
            actionViewItemProvider: action => this.getActionViewItem(action),
            ariaLabel: nls.localize('viewToolbarAriaLabel', "{0} actions", this.title),
            getKeyBinding: action => this.keybindingService.lookupKeybinding(action.id),
            actionRunner: this.actionRunner
        });
        this._register(this.toolbar);
        this.setActions();
        const onDidRelevantConfigurationChange = event_1.Event.filter(this.configurationService.onDidChangeConfiguration, e => e.affectsConfiguration(ViewletPanel.AlwaysShowActionsConfig));
        this._register(onDidRelevantConfigurationChange(this.updateActionsVisibility, this));
        this.updateActionsVisibility();
    }
    renderHeaderTitle(container, title) {
        this.titleContainer = dom_1.append(container, dom_1.$('h3.title', undefined, title));
    }
    updateTitle(title) {
        this.titleContainer.textContent = title;
        this._onDidChangeTitleArea.fire();
    }
    focus() {
        if (this.element) {
            this.element.focus();
            this._onDidFocus.fire();
        }
    }
    setActions() {
        this.toolbar.setActions(actions_1.prepareActions(this.getActions()), actions_1.prepareActions(this.getSecondaryActions()))();
        this.toolbar.context = this.getActionsContext();
    }
    updateActionsVisibility() {
        const shouldAlwaysShowActions = this.configurationService.getValue('workbench.view.alwaysShowHeaderActions');
        dom_1.toggleClass(this.headerContainer, 'actions-always-visible', shouldAlwaysShowActions);
    }
    updateActions() {
        this.setActions();
        this._onDidChangeTitleArea.fire();
    }
    getActions() {
        return [];
    }
    getSecondaryActions() {
        return [];
    }
    getActionViewItem(action) {
        return undefined;
    }
    getActionsContext() {
        return undefined;
    }
    getOptimalWidth() {
        return 0;
    }
    saveState() {
        // Subclasses to implement for saving state
    }
};
ViewletPanel.AlwaysShowActionsConfig = 'workbench.view.alwaysShowHeaderActions';
ViewletPanel = __decorate([
    __param(1, keybinding_1.IKeybindingService),
    __param(2, contextView_1.IContextMenuService),
    __param(3, configuration_1.IConfigurationService),
    __param(4, contextkey_1.IContextKeyService)
], ViewletPanel);
exports.ViewletPanel = ViewletPanel;
let PanelViewlet = class PanelViewlet extends viewlet_1.Viewlet {
    constructor(id, options, configurationService, layoutService, contextMenuService, telemetryService, themeService, storageService) {
        super(id, configurationService, layoutService, telemetryService, themeService, storageService);
        this.options = options;
        this.contextMenuService = contextMenuService;
        this.panelItems = [];
    }
    get onDidSashChange() {
        return this.panelview.onDidSashChange;
    }
    get panels() {
        return this.panelItems.map(i => i.panel);
    }
    get length() {
        return this.panelItems.length;
    }
    create(parent) {
        super.create(parent);
        this.panelview = this._register(new panelview_1.PanelView(parent, this.options));
        this._register(this.panelview.onDidDrop(({ from, to }) => this.movePanel(from, to)));
        this._register(dom_1.addDisposableListener(parent, dom_1.EventType.CONTEXT_MENU, (e) => this.showContextMenu(new mouseEvent_1.StandardMouseEvent(e))));
    }
    showContextMenu(event) {
        for (const panelItem of this.panelItems) {
            // Do not show context menu if target is coming from inside panel views
            if (dom_1.isAncestor(event.target, panelItem.panel.element)) {
                return;
            }
        }
        event.stopPropagation();
        event.preventDefault();
        let anchor = { x: event.posx, y: event.posy };
        this.contextMenuService.showContextMenu({
            getAnchor: () => anchor,
            getActions: () => this.getContextMenuActions()
        });
    }
    getTitle() {
        let title = platform_1.Registry.as(viewlet_1.Extensions.Viewlets).getViewlet(this.getId()).name;
        if (this.isSingleView()) {
            const panelItemTitle = this.panelItems[0].panel.title;
            title = panelItemTitle ? `${title}: ${panelItemTitle}` : title;
        }
        return title;
    }
    getActions() {
        if (this.isSingleView()) {
            return this.panelItems[0].panel.getActions();
        }
        return [];
    }
    getSecondaryActions() {
        if (this.isSingleView()) {
            return this.panelItems[0].panel.getSecondaryActions();
        }
        return [];
    }
    getActionViewItem(action) {
        if (this.isSingleView()) {
            return this.panelItems[0].panel.getActionViewItem(action);
        }
        return super.getActionViewItem(action);
    }
    focus() {
        super.focus();
        if (this.lastFocusedPanel) {
            this.lastFocusedPanel.focus();
        }
        else if (this.panelItems.length > 0) {
            for (const { panel } of this.panelItems) {
                if (panel.isExpanded()) {
                    panel.focus();
                    return;
                }
            }
        }
    }
    layout(dimension) {
        this.panelview.layout(dimension.height, dimension.width);
    }
    getOptimalWidth() {
        const sizes = this.panelItems
            .map(panelItem => panelItem.panel.getOptimalWidth() || 0);
        return Math.max(...sizes);
    }
    addPanels(panels) {
        const wasSingleView = this.isSingleView();
        for (const { panel, size, index } of panels) {
            this.addPanel(panel, size, index);
        }
        this.updateViewHeaders();
        if (this.isSingleView() !== wasSingleView) {
            this.updateTitleArea();
        }
    }
    addPanel(panel, size, index = this.panelItems.length - 1) {
        const onDidFocus = panel.onDidFocus(() => this.lastFocusedPanel = panel);
        const onDidChangeTitleArea = panel.onDidChangeTitleArea(() => {
            if (this.isSingleView()) {
                this.updateTitleArea();
            }
        });
        const onDidChange = panel.onDidChange(() => {
            if (panel === this.lastFocusedPanel && !panel.isExpanded()) {
                this.lastFocusedPanel = undefined;
            }
        });
        const panelStyler = styler_1.attachStyler(this.themeService, {
            headerForeground: theme_1.SIDE_BAR_SECTION_HEADER_FOREGROUND,
            headerBackground: theme_1.SIDE_BAR_SECTION_HEADER_BACKGROUND,
            headerBorder: theme_1.SIDE_BAR_SECTION_HEADER_BORDER,
            dropBackground: theme_1.SIDE_BAR_DRAG_AND_DROP_BACKGROUND
        }, panel);
        const disposable = lifecycle_1.combinedDisposable(onDidFocus, onDidChangeTitleArea, panelStyler, onDidChange);
        const panelItem = { panel, disposable };
        this.panelItems.splice(index, 0, panelItem);
        this.panelview.addPanel(panel, size, index);
    }
    removePanels(panels) {
        const wasSingleView = this.isSingleView();
        panels.forEach(panel => this.removePanel(panel));
        this.updateViewHeaders();
        if (wasSingleView !== this.isSingleView()) {
            this.updateTitleArea();
        }
    }
    removePanel(panel) {
        const index = arrays_1.firstIndex(this.panelItems, i => i.panel === panel);
        if (index === -1) {
            return;
        }
        if (this.lastFocusedPanel === panel) {
            this.lastFocusedPanel = undefined;
        }
        this.panelview.removePanel(panel);
        const [panelItem] = this.panelItems.splice(index, 1);
        panelItem.disposable.dispose();
    }
    movePanel(from, to) {
        const fromIndex = arrays_1.firstIndex(this.panelItems, item => item.panel === from);
        const toIndex = arrays_1.firstIndex(this.panelItems, item => item.panel === to);
        if (fromIndex < 0 || fromIndex >= this.panelItems.length) {
            return;
        }
        if (toIndex < 0 || toIndex >= this.panelItems.length) {
            return;
        }
        const [panelItem] = this.panelItems.splice(fromIndex, 1);
        this.panelItems.splice(toIndex, 0, panelItem);
        this.panelview.movePanel(from, to);
    }
    resizePanel(panel, size) {
        this.panelview.resizePanel(panel, size);
    }
    getPanelSize(panel) {
        return this.panelview.getPanelSize(panel);
    }
    updateViewHeaders() {
        if (this.isSingleView()) {
            this.panelItems[0].panel.setExpanded(true);
            this.panelItems[0].panel.headerVisible = false;
        }
        else {
            this.panelItems.forEach(i => i.panel.headerVisible = true);
        }
    }
    isSingleView() {
        return this.options.showHeaderInTitleWhenSingleView && this.panelItems.length === 1;
    }
    dispose() {
        super.dispose();
        this.panelItems.forEach(i => i.disposable.dispose());
        this.panelview.dispose();
    }
};
PanelViewlet = __decorate([
    __param(2, configuration_1.IConfigurationService),
    __param(3, layoutService_1.IWorkbenchLayoutService),
    __param(4, contextView_1.IContextMenuService),
    __param(5, telemetry_1.ITelemetryService),
    __param(6, themeService_1.IThemeService),
    __param(7, storage_1.IStorageService)
], PanelViewlet);
exports.PanelViewlet = PanelViewlet;
//# sourceMappingURL=panelViewlet.js.map