"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const browser = require("vs/base/browser/browser");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const event_1 = require("vs/base/common/event");
const log_1 = require("vs/platform/log/common/log");
const lifecycle_1 = require("vs/base/common/lifecycle");
const storage_1 = require("vs/platform/storage/common/storage");
const windows_1 = require("vs/platform/windows/common/windows");
const history_1 = require("vs/platform/history/common/history");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const dom_1 = require("vs/base/browser/dom");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const editor_1 = require("vs/workbench/common/editor");
const files_1 = require("vs/platform/files/common/files");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const historyStorage_1 = require("vs/platform/history/common/historyStorage");
//#region Window
let SimpleWindowService = class SimpleWindowService extends lifecycle_1.Disposable {
    constructor(editorService, fileService, configurationService, storageService, workspaceService, logService) {
        super();
        this.editorService = editorService;
        this.fileService = fileService;
        this.configurationService = configurationService;
        this.storageService = storageService;
        this.workspaceService = workspaceService;
        this.logService = logService;
        this.onDidChangeFocus = event_1.Event.None;
        this.onDidChangeMaximize = event_1.Event.None;
        this.hasFocus = true;
        this.windowId = 0;
        this.addWorkspaceToRecentlyOpened();
        this.registerListeners();
    }
    addWorkspaceToRecentlyOpened() {
        const workspace = this.workspaceService.getWorkspace();
        switch (this.workspaceService.getWorkbenchState()) {
            case 2 /* FOLDER */:
                this.addRecentlyOpened([{ folderUri: workspace.folders[0].uri }]);
                break;
            case 3 /* WORKSPACE */:
                this.addRecentlyOpened([{ workspace: { id: workspace.id, configPath: workspace.configuration } }]);
                break;
        }
    }
    registerListeners() {
        this._register(dom_1.addDisposableListener(document, dom_1.EventType.FULLSCREEN_CHANGE, () => {
            if (document.fullscreenElement || document.webkitFullscreenElement) {
                browser.setFullscreen(true);
            }
            else {
                browser.setFullscreen(false);
            }
        }));
        this._register(dom_1.addDisposableListener(document, dom_1.EventType.WK_FULLSCREEN_CHANGE, () => {
            if (document.fullscreenElement || document.webkitFullscreenElement || document.webkitIsFullScreen) {
                browser.setFullscreen(true);
            }
            else {
                browser.setFullscreen(false);
            }
        }));
    }
    isFocused() {
        return Promise.resolve(this.hasFocus);
    }
    isMaximized() {
        return Promise.resolve(false);
    }
    closeWorkspace() {
        return Promise.resolve();
    }
    enterWorkspace(_path) {
        return Promise.resolve(undefined);
    }
    getRecentlyOpened() {
        return __awaiter(this, void 0, void 0, function* () {
            const recentlyOpenedRaw = this.storageService.get(SimpleWindowService.RECENTLY_OPENED_KEY, 0 /* GLOBAL */);
            if (recentlyOpenedRaw) {
                return historyStorage_1.restoreRecentlyOpened(JSON.parse(recentlyOpenedRaw), this.logService);
            }
            return { workspaces: [], files: [] };
        });
    }
    addRecentlyOpened(recents) {
        return __awaiter(this, void 0, void 0, function* () {
            const recentlyOpened = yield this.getRecentlyOpened();
            recents.forEach(recent => {
                if (history_1.isRecentFile(recent)) {
                    this.doRemoveFromRecentlyOpened(recentlyOpened, [recent.fileUri]);
                    recentlyOpened.files.unshift(recent);
                }
                else if (history_1.isRecentFolder(recent)) {
                    this.doRemoveFromRecentlyOpened(recentlyOpened, [recent.folderUri]);
                    recentlyOpened.workspaces.unshift(recent);
                }
                else {
                    this.doRemoveFromRecentlyOpened(recentlyOpened, [recent.workspace.configPath]);
                    recentlyOpened.workspaces.unshift(recent);
                }
            });
            return this.saveRecentlyOpened(recentlyOpened);
        });
    }
    removeFromRecentlyOpened(paths) {
        return __awaiter(this, void 0, void 0, function* () {
            const recentlyOpened = yield this.getRecentlyOpened();
            this.doRemoveFromRecentlyOpened(recentlyOpened, paths);
            return this.saveRecentlyOpened(recentlyOpened);
        });
    }
    doRemoveFromRecentlyOpened(recentlyOpened, paths) {
        recentlyOpened.files = recentlyOpened.files.filter(file => {
            return !paths.some(path => path.toString() === file.fileUri.toString());
        });
        recentlyOpened.workspaces = recentlyOpened.workspaces.filter(workspace => {
            return !paths.some(path => path.toString() === (history_1.isRecentFolder(workspace) ? workspace.folderUri.toString() : workspace.workspace.configPath.toString()));
        });
    }
    saveRecentlyOpened(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.storageService.store(SimpleWindowService.RECENTLY_OPENED_KEY, JSON.stringify(historyStorage_1.toStoreData(data)), 0 /* GLOBAL */);
        });
    }
    focusWindow() {
        return Promise.resolve();
    }
    maximizeWindow() {
        return Promise.resolve();
    }
    unmaximizeWindow() {
        return Promise.resolve();
    }
    minimizeWindow() {
        return Promise.resolve();
    }
    openWindow(_uris, _options) {
        return __awaiter(this, void 0, void 0, function* () {
            const { openFolderInNewWindow } = this.shouldOpenNewWindow(_options);
            for (let i = 0; i < _uris.length; i++) {
                const uri = _uris[i];
                if ('folderUri' in uri) {
                    const newAddress = `${document.location.origin}${document.location.pathname}?folder=${uri.folderUri.path}`;
                    if (openFolderInNewWindow) {
                        window.open(newAddress);
                    }
                    else {
                        window.location.href = newAddress;
                    }
                }
                if ('workspaceUri' in uri) {
                    const newAddress = `${document.location.origin}${document.location.pathname}?workspace=${uri.workspaceUri.path}`;
                    if (openFolderInNewWindow) {
                        window.open(newAddress);
                    }
                    else {
                        window.location.href = newAddress;
                    }
                }
                if ('fileUri' in uri) {
                    const inputs = yield editor_1.pathsToEditors([uri], this.fileService);
                    this.editorService.openEditors(inputs);
                }
            }
            return Promise.resolve();
        });
    }
    shouldOpenNewWindow(_options = {}) {
        const windowConfig = this.configurationService.getValue('window');
        const openFolderInNewWindowConfig = (windowConfig && windowConfig.openFoldersInNewWindow) || 'default' /* default */;
        let openFolderInNewWindow = !!_options.forceNewWindow && !_options.forceReuseWindow;
        if (!_options.forceNewWindow && !_options.forceReuseWindow && (openFolderInNewWindowConfig === 'on' || openFolderInNewWindowConfig === 'off')) {
            openFolderInNewWindow = (openFolderInNewWindowConfig === 'on');
        }
        return { openFolderInNewWindow };
    }
    closeWindow() {
        window.close();
        return Promise.resolve();
    }
    onWindowTitleDoubleClick() {
        return Promise.resolve();
    }
    updateTouchBar(_items) {
        return Promise.resolve();
    }
};
SimpleWindowService.RECENTLY_OPENED_KEY = 'recently.opened';
SimpleWindowService = __decorate([
    __param(0, editorService_1.IEditorService),
    __param(1, files_1.IFileService),
    __param(2, configuration_1.IConfigurationService),
    __param(3, storage_1.IStorageService),
    __param(4, workspace_1.IWorkspaceContextService),
    __param(5, log_1.ILogService)
], SimpleWindowService);
exports.SimpleWindowService = SimpleWindowService;
extensions_1.registerSingleton(windows_1.IWindowService, SimpleWindowService);
//#endregion
//#region Window
class SimpleWindowsService {
    constructor() {
        this.onWindowOpen = event_1.Event.None;
        this.onWindowFocus = event_1.Event.None;
        this.onWindowBlur = event_1.Event.None;
        this.onWindowMaximize = event_1.Event.None;
        this.onWindowUnmaximize = event_1.Event.None;
        this.onRecentlyOpenedChange = event_1.Event.None;
    }
    isFocused(_windowId) {
        return Promise.resolve(true);
    }
    closeWorkspace(_windowId) {
        return Promise.resolve();
    }
    enterWorkspace(_windowId, _path) {
        return Promise.resolve(undefined);
    }
    addRecentlyOpened(recents) {
        return Promise.resolve();
    }
    removeFromRecentlyOpened(_paths) {
        return Promise.resolve();
    }
    clearRecentlyOpened() {
        return Promise.resolve();
    }
    getRecentlyOpened(_windowId) {
        return Promise.resolve({
            workspaces: [],
            files: []
        });
    }
    focusWindow(_windowId) {
        return Promise.resolve();
    }
    closeWindow(_windowId) {
        return Promise.resolve();
    }
    isMaximized(_windowId) {
        return Promise.resolve(false);
    }
    maximizeWindow(_windowId) {
        return Promise.resolve();
    }
    minimizeWindow(_windowId) {
        return Promise.resolve();
    }
    unmaximizeWindow(_windowId) {
        return Promise.resolve();
    }
    onWindowTitleDoubleClick(_windowId) {
        return Promise.resolve();
    }
    quit() {
        return Promise.resolve();
    }
    whenSharedProcessReady() {
        return Promise.resolve();
    }
    toggleSharedProcess() {
        return Promise.resolve();
    }
    // Global methods
    openWindow(_windowId, _uris, _options) {
        return Promise.resolve();
    }
    openExtensionDevelopmentHostWindow(args, env) {
        return Promise.resolve();
    }
    getWindows() {
        return Promise.resolve([]);
    }
    newWindowTab() {
        return Promise.resolve();
    }
    showPreviousWindowTab() {
        return Promise.resolve();
    }
    showNextWindowTab() {
        return Promise.resolve();
    }
    moveWindowTabToNewWindow() {
        return Promise.resolve();
    }
    mergeAllWindowTabs() {
        return Promise.resolve();
    }
    toggleWindowTabsBar() {
        return Promise.resolve();
    }
    updateTouchBar(_windowId, _items) {
        return Promise.resolve();
    }
    getActiveWindowId() {
        return Promise.resolve(0);
    }
    // This needs to be handled from browser process to prevent
    // foreground ordering issues on Windows
    openExternal(_url) {
        dom_1.windowOpenNoOpener(_url);
        return Promise.resolve(true);
    }
    // TODO: this is a bit backwards
    startCrashReporter(_config) {
        return Promise.resolve();
    }
}
exports.SimpleWindowsService = SimpleWindowsService;
extensions_1.registerSingleton(windows_1.IWindowsService, SimpleWindowsService);
//#endregion
//# sourceMappingURL=web.simpleservices.js.map