"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cancellation_1 = require("vs/base/common/cancellation");
const languageFeatureRegistry_1 = require("vs/editor/common/modes/languageFeatureRegistry");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const arrays_1 = require("vs/base/common/arrays");
const errors_1 = require("vs/base/common/errors");
var CallHierarchyDirection;
(function (CallHierarchyDirection) {
    CallHierarchyDirection[CallHierarchyDirection["CallsTo"] = 1] = "CallsTo";
    CallHierarchyDirection[CallHierarchyDirection["CallsFrom"] = 2] = "CallsFrom";
})(CallHierarchyDirection = exports.CallHierarchyDirection || (exports.CallHierarchyDirection = {}));
exports.CallHierarchyProviderRegistry = new languageFeatureRegistry_1.LanguageFeatureRegistry();
function provideIncomingCalls(model, position, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const [provider] = exports.CallHierarchyProviderRegistry.ordered(model);
        if (!provider) {
            return [];
        }
        try {
            const result = yield provider.provideIncomingCalls(model, position, token);
            if (arrays_1.isNonEmptyArray(result)) {
                return result;
            }
        }
        catch (e) {
            errors_1.onUnexpectedExternalError(e);
        }
        return [];
    });
}
exports.provideIncomingCalls = provideIncomingCalls;
function provideOutgoingCalls(model, position, token) {
    return __awaiter(this, void 0, void 0, function* () {
        const [provider] = exports.CallHierarchyProviderRegistry.ordered(model);
        if (!provider) {
            return [];
        }
        try {
            const result = yield provider.provideOutgoingCalls(model, position, token);
            if (arrays_1.isNonEmptyArray(result)) {
                return result;
            }
        }
        catch (e) {
            errors_1.onUnexpectedExternalError(e);
        }
        return [];
    });
}
exports.provideOutgoingCalls = provideOutgoingCalls;
editorExtensions_1.registerDefaultLanguageCommand('_executeCallHierarchyIncomingCalls', (model, position) => __awaiter(void 0, void 0, void 0, function* () { return provideIncomingCalls(model, position, cancellation_1.CancellationToken.None); }));
editorExtensions_1.registerDefaultLanguageCommand('_executeCallHierarchyOutgoingCalls', (model, position) => __awaiter(void 0, void 0, void 0, function* () { return provideOutgoingCalls(model, position, cancellation_1.CancellationToken.None); }));
//# sourceMappingURL=callHierarchy.js.map