"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/css!./simpleFindWidget");
const nls = require("vs/nls");
const dom = require("vs/base/browser/dom");
const widget_1 = require("vs/base/browser/ui/widget");
const async_1 = require("vs/base/common/async");
const findState_1 = require("vs/editor/contrib/find/findState");
const findWidget_1 = require("vs/editor/contrib/find/findWidget");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const themeService_1 = require("vs/platform/theme/common/themeService");
const contextScopedHistoryWidget_1 = require("vs/platform/browser/contextScopedHistoryWidget");
const NLS_FIND_INPUT_LABEL = nls.localize('label.find', "Find");
const NLS_FIND_INPUT_PLACEHOLDER = nls.localize('placeholder.find', "Find");
const NLS_PREVIOUS_MATCH_BTN_LABEL = nls.localize('label.previousMatchButton', "Previous match");
const NLS_NEXT_MATCH_BTN_LABEL = nls.localize('label.nextMatchButton', "Next match");
const NLS_CLOSE_BTN_LABEL = nls.localize('label.closeButton', "Close");
let SimpleFindWidget = class SimpleFindWidget extends widget_1.Widget {
    constructor(_contextViewService, contextKeyService, _state = new findState_1.FindReplaceState(), showOptionButtons) {
        super();
        this._contextViewService = _contextViewService;
        this._state = _state;
        this._isVisible = false;
        this.foundMatch = false;
        this._findInput = this._register(new contextScopedHistoryWidget_1.ContextScopedFindInput(null, this._contextViewService, {
            label: NLS_FIND_INPUT_LABEL,
            placeholder: NLS_FIND_INPUT_PLACEHOLDER,
            validation: (value) => {
                if (value.length === 0 || !this._findInput.getRegex()) {
                    return null;
                }
                try {
                    /* tslint:disable-next-line:no-unused-expression */
                    new RegExp(value);
                    return null;
                }
                catch (e) {
                    this.foundMatch = false;
                    this.updateButtons(this.foundMatch);
                    return { content: e.message };
                }
            }
        }, contextKeyService, showOptionButtons));
        // Find History with update delayer
        this._updateHistoryDelayer = new async_1.Delayer(500);
        this.oninput(this._findInput.domNode, (e) => {
            this.foundMatch = this.onInputChanged();
            this.updateButtons(this.foundMatch);
            this._delayedUpdateHistory();
        });
        this._findInput.setRegex(!!this._state.isRegex);
        this._findInput.setCaseSensitive(!!this._state.matchCase);
        this._findInput.setWholeWords(!!this._state.wholeWord);
        this._register(this._findInput.onDidOptionChange(() => {
            this._state.change({
                isRegex: this._findInput.getRegex(),
                wholeWord: this._findInput.getWholeWords(),
                matchCase: this._findInput.getCaseSensitive()
            }, true);
        }));
        this._register(this._state.onFindReplaceStateChange(() => {
            this._findInput.setRegex(this._state.isRegex);
            this._findInput.setWholeWords(this._state.wholeWord);
            this._findInput.setCaseSensitive(this._state.matchCase);
            this.findFirst();
        }));
        this.prevBtn = this._register(new findWidget_1.SimpleButton({
            label: NLS_PREVIOUS_MATCH_BTN_LABEL,
            className: 'previous',
            onTrigger: () => {
                this.find(true);
            }
        }));
        this.nextBtn = this._register(new findWidget_1.SimpleButton({
            label: NLS_NEXT_MATCH_BTN_LABEL,
            className: 'next',
            onTrigger: () => {
                this.find(false);
            }
        }));
        const closeBtn = this._register(new findWidget_1.SimpleButton({
            label: NLS_CLOSE_BTN_LABEL,
            className: 'close-fw',
            onTrigger: () => {
                this.hide();
            }
        }));
        this._innerDomNode = document.createElement('div');
        this._innerDomNode.classList.add('simple-find-part');
        this._innerDomNode.appendChild(this._findInput.domNode);
        this._innerDomNode.appendChild(this.prevBtn.domNode);
        this._innerDomNode.appendChild(this.nextBtn.domNode);
        this._innerDomNode.appendChild(closeBtn.domNode);
        // _domNode wraps _innerDomNode, ensuring that
        this._domNode = document.createElement('div');
        this._domNode.classList.add('simple-find-part-wrapper');
        this._domNode.appendChild(this._innerDomNode);
        this.onkeyup(this._innerDomNode, e => {
            if (e.equals(9 /* Escape */)) {
                this.hide();
                e.preventDefault();
                return;
            }
        });
        this._focusTracker = this._register(dom.trackFocus(this._innerDomNode));
        this._register(this._focusTracker.onDidFocus(this.onFocusTrackerFocus.bind(this)));
        this._register(this._focusTracker.onDidBlur(this.onFocusTrackerBlur.bind(this)));
        this._findInputFocusTracker = this._register(dom.trackFocus(this._findInput.domNode));
        this._register(this._findInputFocusTracker.onDidFocus(this.onFindInputFocusTrackerFocus.bind(this)));
        this._register(this._findInputFocusTracker.onDidBlur(this.onFindInputFocusTrackerBlur.bind(this)));
        this._register(dom.addDisposableListener(this._innerDomNode, 'click', (event) => {
            event.stopPropagation();
        }));
    }
    get inputValue() {
        return this._findInput.getValue();
    }
    get focusTracker() {
        return this._focusTracker;
    }
    updateTheme(theme) {
        const inputStyles = {
            inputActiveOptionBorder: theme.getColor(colorRegistry_1.inputActiveOptionBorder),
            inputActiveOptionBackground: theme.getColor(colorRegistry_1.inputActiveOptionBackground),
            inputBackground: theme.getColor(colorRegistry_1.inputBackground),
            inputForeground: theme.getColor(colorRegistry_1.inputForeground),
            inputBorder: theme.getColor(colorRegistry_1.inputBorder),
            inputValidationInfoBackground: theme.getColor(colorRegistry_1.inputValidationInfoBackground),
            inputValidationInfoForeground: theme.getColor(colorRegistry_1.inputValidationInfoForeground),
            inputValidationInfoBorder: theme.getColor(colorRegistry_1.inputValidationInfoBorder),
            inputValidationWarningBackground: theme.getColor(colorRegistry_1.inputValidationWarningBackground),
            inputValidationWarningForeground: theme.getColor(colorRegistry_1.inputValidationWarningForeground),
            inputValidationWarningBorder: theme.getColor(colorRegistry_1.inputValidationWarningBorder),
            inputValidationErrorBackground: theme.getColor(colorRegistry_1.inputValidationErrorBackground),
            inputValidationErrorForeground: theme.getColor(colorRegistry_1.inputValidationErrorForeground),
            inputValidationErrorBorder: theme.getColor(colorRegistry_1.inputValidationErrorBorder)
        };
        this._findInput.style(inputStyles);
    }
    dispose() {
        super.dispose();
        if (this._domNode && this._domNode.parentElement) {
            this._domNode.parentElement.removeChild(this._domNode);
        }
    }
    getDomNode() {
        return this._domNode;
    }
    reveal(initialInput) {
        if (initialInput) {
            this._findInput.setValue(initialInput);
        }
        if (this._isVisible) {
            this._findInput.select();
            return;
        }
        this._isVisible = true;
        this.updateButtons(this.foundMatch);
        setTimeout(() => {
            dom.addClass(this._innerDomNode, 'visible');
            dom.addClass(this._innerDomNode, 'visible-transition');
            this._innerDomNode.setAttribute('aria-hidden', 'false');
            this._findInput.select();
        }, 0);
    }
    show(initialInput) {
        if (initialInput && !this._isVisible) {
            this._findInput.setValue(initialInput);
        }
        this._isVisible = true;
        setTimeout(() => {
            dom.addClass(this._innerDomNode, 'visible');
            dom.addClass(this._innerDomNode, 'visible-transition');
            this._innerDomNode.setAttribute('aria-hidden', 'false');
        }, 0);
    }
    hide() {
        if (this._isVisible) {
            dom.removeClass(this._innerDomNode, 'visible-transition');
            this._innerDomNode.setAttribute('aria-hidden', 'true');
            // Need to delay toggling visibility until after Transition, then visibility hidden - removes from tabIndex list
            setTimeout(() => {
                this._isVisible = false;
                this.updateButtons(this.foundMatch);
                dom.removeClass(this._innerDomNode, 'visible');
            }, 200);
        }
    }
    _delayedUpdateHistory() {
        this._updateHistoryDelayer.trigger(this._updateHistory.bind(this));
    }
    _updateHistory() {
        this._findInput.inputBox.addToHistory();
    }
    _getRegexValue() {
        return this._findInput.getRegex();
    }
    _getWholeWordValue() {
        return this._findInput.getWholeWords();
    }
    _getCaseSensitiveValue() {
        return this._findInput.getCaseSensitive();
    }
    updateButtons(foundMatch) {
        const hasInput = this.inputValue.length > 0;
        this.prevBtn.setEnabled(this._isVisible && hasInput && foundMatch);
        this.nextBtn.setEnabled(this._isVisible && hasInput && foundMatch);
    }
};
SimpleFindWidget = __decorate([
    __param(0, contextView_1.IContextViewService),
    __param(1, contextkey_1.IContextKeyService)
], SimpleFindWidget);
exports.SimpleFindWidget = SimpleFindWidget;
// theming
themeService_1.registerThemingParticipant((theme, collector) => {
    const findWidgetBGColor = theme.getColor(colorRegistry_1.editorWidgetBackground);
    if (findWidgetBGColor) {
        collector.addRule(`.monaco-workbench .simple-find-part { background-color: ${findWidgetBGColor} !important; }`);
    }
    const widgetShadowColor = theme.getColor(colorRegistry_1.widgetShadow);
    if (widgetShadowColor) {
        collector.addRule(`.monaco-workbench .simple-find-part { box-shadow: 0 2px 8px ${widgetShadowColor}; }`);
    }
});
//# sourceMappingURL=simpleFindWidget.js.map