"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const dom = require("vs/base/browser/dom");
const modes = require("vs/editor/common/modes");
const actionbar_1 = require("vs/base/browser/ui/actionbar/actionbar");
const actions_1 = require("vs/base/common/actions");
const lifecycle_1 = require("vs/base/common/lifecycle");
const uri_1 = require("vs/base/common/uri");
const modelService_1 = require("vs/editor/common/services/modelService");
const modeService_1 = require("vs/editor/common/services/modeService");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const themeService_1 = require("vs/platform/theme/common/themeService");
const commentService_1 = require("vs/workbench/contrib/comments/browser/commentService");
const simpleCommentEditor_1 = require("vs/workbench/contrib/comments/browser/simpleCommentEditor");
const selection_1 = require("vs/editor/common/core/selection");
const event_1 = require("vs/base/common/event");
const notification_1 = require("vs/platform/notification/common/notification");
const toolbar_1 = require("vs/base/browser/ui/toolbar/toolbar");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const dropdown_1 = require("vs/base/browser/ui/dropdown/dropdown");
const reactionsAction_1 = require("./reactionsAction");
const actions_2 = require("vs/platform/actions/common/actions");
const menuEntryActionViewItem_1 = require("vs/platform/actions/browser/menuEntryActionViewItem");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const commentFormActions_1 = require("vs/workbench/contrib/comments/browser/commentFormActions");
let CommentNode = class CommentNode extends lifecycle_1.Disposable {
    constructor(commentThread, comment, owner, resource, parentEditor, parentThread, markdownRenderer, themeService, instantiationService, commentService, modelService, modeService, keybindingService, notificationService, contextMenuService, contextKeyService) {
        super();
        this.commentThread = commentThread;
        this.comment = comment;
        this.owner = owner;
        this.resource = resource;
        this.parentEditor = parentEditor;
        this.parentThread = parentThread;
        this.markdownRenderer = markdownRenderer;
        this.themeService = themeService;
        this.instantiationService = instantiationService;
        this.commentService = commentService;
        this.modelService = modelService;
        this.modeService = modeService;
        this.keybindingService = keybindingService;
        this.notificationService = notificationService;
        this.contextMenuService = contextMenuService;
        this._editAction = null;
        this._commentEditContainer = null;
        this._commentEditor = null;
        this._commentEditorDisposables = [];
        this._commentEditorModel = null;
        this._commentFormActions = null;
        this._onDidDelete = new event_1.Emitter();
        this.isEditing = false;
        this._domNode = dom.$('div.review-comment');
        this._contextKeyService = contextKeyService.createScoped(this._domNode);
        this._commentContextValue = this._contextKeyService.createKey('comment', comment.contextValue);
        this._domNode.tabIndex = 0;
        const avatar = dom.append(this._domNode, dom.$('div.avatar-container'));
        if (comment.userIconPath) {
            const img = dom.append(avatar, dom.$('img.avatar'));
            img.src = comment.userIconPath.toString();
            img.onerror = _ => img.remove();
        }
        this._commentDetailsContainer = dom.append(this._domNode, dom.$('.review-comment-contents'));
        this.createHeader(this._commentDetailsContainer);
        this._body = dom.append(this._commentDetailsContainer, dom.$('div.comment-body'));
        this._md = this.markdownRenderer.render(comment.body).element;
        this._body.appendChild(this._md);
        if (this.comment.commentReactions && this.comment.commentReactions.length && this.comment.commentReactions.filter(reaction => !!reaction.count).length) {
            this.createReactionsContainer(this._commentDetailsContainer);
        }
        this._domNode.setAttribute('aria-label', `${comment.userName}, ${comment.body.value}`);
        this._domNode.setAttribute('role', 'treeitem');
        this._clearTimeout = null;
    }
    get domNode() {
        return this._domNode;
    }
    get onDidDelete() {
        return this._onDidDelete.event;
    }
    createHeader(commentDetailsContainer) {
        const header = dom.append(commentDetailsContainer, dom.$('div.comment-title'));
        const author = dom.append(header, dom.$('strong.author'));
        author.innerText = this.comment.userName;
        this._isPendingLabel = dom.append(header, dom.$('span.isPending'));
        if (this.comment.label) {
            this._isPendingLabel.innerText = this.comment.label;
        }
        else {
            this._isPendingLabel.innerText = '';
        }
        this._actionsToolbarContainer = dom.append(header, dom.$('.comment-actions.hidden'));
        this.createActionsToolbar();
    }
    getToolbarActions(menu) {
        const contributedActions = menu.getActions({ shouldForwardArgs: true });
        const primary = [];
        const secondary = [];
        const result = { primary, secondary };
        fillInActions(contributedActions, result, false, g => /^inline/.test(g));
        return result;
    }
    createToolbar() {
        this.toolbar = new toolbar_1.ToolBar(this._actionsToolbarContainer, this.contextMenuService, {
            actionViewItemProvider: action => {
                if (action.id === reactionsAction_1.ToggleReactionsAction.ID) {
                    return new dropdown_1.DropdownMenuActionViewItem(action, action.menuActions, this.contextMenuService, action => {
                        return this.actionViewItemProvider(action);
                    }, this.actionRunner, undefined, 'toolbar-toggle-pickReactions', () => { return 1 /* RIGHT */; });
                }
                return this.actionViewItemProvider(action);
            },
            orientation: 0 /* HORIZONTAL */
        });
        this.toolbar.context = {
            thread: this.commentThread,
            commentUniqueId: this.comment.uniqueIdInThread,
            $mid: 9
        };
        this.registerActionBarListeners(this._actionsToolbarContainer);
        this._register(this.toolbar);
    }
    createActionsToolbar() {
        const actions = [];
        let hasReactionHandler = this.commentService.hasReactionHandler(this.owner);
        if (hasReactionHandler) {
            let toggleReactionAction = this.createReactionPicker(this.comment.commentReactions || []);
            actions.push(toggleReactionAction);
        }
        let commentMenus = this.commentService.getCommentMenus(this.owner);
        const menu = commentMenus.getCommentTitleActions(this.comment, this._contextKeyService);
        this._register(menu);
        this._register(menu.onDidChange(e => {
            const { primary, secondary } = this.getToolbarActions(menu);
            if (!this.toolbar && (primary.length || secondary.length)) {
                this.createToolbar();
            }
            this.toolbar.setActions(primary, secondary)();
        }));
        const { primary, secondary } = this.getToolbarActions(menu);
        actions.push(...primary);
        if (actions.length || secondary.length) {
            this.createToolbar();
            this.toolbar.setActions(actions, secondary)();
        }
    }
    actionViewItemProvider(action) {
        let options = {};
        if (action.id === reactionsAction_1.ToggleReactionsAction.ID) {
            options = { label: false, icon: true };
        }
        else {
            options = { label: false, icon: true };
        }
        if (action.id === reactionsAction_1.ReactionAction.ID) {
            let item = new reactionsAction_1.ReactionActionViewItem(action);
            return item;
        }
        else if (action instanceof actions_2.MenuItemAction) {
            let item = new menuEntryActionViewItem_1.ContextAwareMenuEntryActionViewItem(action, this.keybindingService, this.notificationService, this.contextMenuService);
            return item;
        }
        else {
            let item = new actionbar_1.ActionViewItem({}, action, options);
            return item;
        }
    }
    createReactionPicker(reactionGroup) {
        let toggleReactionActionViewItem;
        let toggleReactionAction = this._register(new reactionsAction_1.ToggleReactionsAction(() => {
            if (toggleReactionActionViewItem) {
                toggleReactionActionViewItem.show();
            }
        }, nls.localize('commentToggleReaction', "Toggle Reaction")));
        let reactionMenuActions = [];
        if (reactionGroup && reactionGroup.length) {
            reactionMenuActions = reactionGroup.map((reaction) => {
                return new actions_1.Action(`reaction.command.${reaction.label}`, `${reaction.label}`, '', true, () => __awaiter(this, void 0, void 0, function* () {
                    try {
                        yield this.commentService.toggleReaction(this.owner, this.resource, this.commentThread, this.comment, reaction);
                    }
                    catch (e) {
                        const error = e.message
                            ? nls.localize('commentToggleReactionError', "Toggling the comment reaction failed: {0}.", e.message)
                            : nls.localize('commentToggleReactionDefaultError', "Toggling the comment reaction failed");
                        this.notificationService.error(error);
                    }
                }));
            });
        }
        toggleReactionAction.menuActions = reactionMenuActions;
        toggleReactionActionViewItem = new dropdown_1.DropdownMenuActionViewItem(toggleReactionAction, toggleReactionAction.menuActions, this.contextMenuService, action => {
            if (action.id === reactionsAction_1.ToggleReactionsAction.ID) {
                return toggleReactionActionViewItem;
            }
            return this.actionViewItemProvider(action);
        }, this.actionRunner, undefined, 'toolbar-toggle-pickReactions', () => { return 1 /* RIGHT */; });
        return toggleReactionAction;
    }
    createReactionsContainer(commentDetailsContainer) {
        this._reactionActionsContainer = dom.append(commentDetailsContainer, dom.$('div.comment-reactions'));
        this._reactionsActionBar = new actionbar_1.ActionBar(this._reactionActionsContainer, {
            actionViewItemProvider: action => {
                if (action.id === reactionsAction_1.ToggleReactionsAction.ID) {
                    return new dropdown_1.DropdownMenuActionViewItem(action, action.menuActions, this.contextMenuService, action => {
                        return this.actionViewItemProvider(action);
                    }, this.actionRunner, undefined, 'toolbar-toggle-pickReactions', () => { return 1 /* RIGHT */; });
                }
                return this.actionViewItemProvider(action);
            }
        });
        this._register(this._reactionsActionBar);
        let hasReactionHandler = this.commentService.hasReactionHandler(this.owner);
        this.comment.commentReactions.filter(reaction => !!reaction.count).map(reaction => {
            let action = new reactionsAction_1.ReactionAction(`reaction.${reaction.label}`, `${reaction.label}`, reaction.hasReacted && (reaction.canEdit || hasReactionHandler) ? 'active' : '', (reaction.canEdit || hasReactionHandler), () => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.commentService.toggleReaction(this.owner, this.resource, this.commentThread, this.comment, reaction);
                }
                catch (e) {
                    let error;
                    if (reaction.hasReacted) {
                        error = e.message
                            ? nls.localize('commentDeleteReactionError', "Deleting the comment reaction failed: {0}.", e.message)
                            : nls.localize('commentDeleteReactionDefaultError', "Deleting the comment reaction failed");
                    }
                    else {
                        error = e.message
                            ? nls.localize('commentAddReactionError', "Deleting the comment reaction failed: {0}.", e.message)
                            : nls.localize('commentAddReactionDefaultError', "Deleting the comment reaction failed");
                    }
                    this.notificationService.error(error);
                }
            }), reaction.iconPath, reaction.count);
            if (this._reactionsActionBar) {
                this._reactionsActionBar.push(action, { label: true, icon: true });
            }
        });
        if (hasReactionHandler) {
            let toggleReactionAction = this.createReactionPicker(this.comment.commentReactions || []);
            this._reactionsActionBar.push(toggleReactionAction, { label: false, icon: true });
        }
    }
    createCommentEditor(editContainer) {
        const container = dom.append(editContainer, dom.$('.edit-textarea'));
        this._commentEditor = this.instantiationService.createInstance(simpleCommentEditor_1.SimpleCommentEditor, container, simpleCommentEditor_1.SimpleCommentEditor.getEditorOptions(), this.parentEditor, this.parentThread);
        const resource = uri_1.URI.parse(`comment:commentinput-${this.comment.uniqueIdInThread}-${Date.now()}.md`);
        this._commentEditorModel = this.modelService.createModel('', this.modeService.createByFilepathOrFirstLine(resource), resource, false);
        this._commentEditor.setModel(this._commentEditorModel);
        this._commentEditor.setValue(this.comment.body.value);
        this._commentEditor.layout({ width: container.clientWidth - 14, height: 90 });
        this._commentEditor.focus();
        const lastLine = this._commentEditorModel.getLineCount();
        const lastColumn = this._commentEditorModel.getLineContent(lastLine).length + 1;
        this._commentEditor.setSelection(new selection_1.Selection(lastLine, lastColumn, lastLine, lastColumn));
        let commentThread = this.commentThread;
        commentThread.input = {
            uri: this._commentEditor.getModel().uri,
            value: this.comment.body.value
        };
        this.commentService.setActiveCommentThread(commentThread);
        this._commentEditorDisposables.push(this._commentEditor.onDidFocusEditorWidget(() => {
            commentThread.input = {
                uri: this._commentEditor.getModel().uri,
                value: this.comment.body.value
            };
            this.commentService.setActiveCommentThread(commentThread);
        }));
        this._commentEditorDisposables.push(this._commentEditor.onDidChangeModelContent(e => {
            if (commentThread.input && this._commentEditor && this._commentEditor.getModel().uri === commentThread.input.uri) {
                let newVal = this._commentEditor.getValue();
                if (newVal !== commentThread.input.value) {
                    let input = commentThread.input;
                    input.value = newVal;
                    commentThread.input = input;
                    this.commentService.setActiveCommentThread(commentThread);
                }
            }
        }));
        this._register(this._commentEditor);
        this._register(this._commentEditorModel);
    }
    removeCommentEditor() {
        this.isEditing = false;
        if (this._editAction) {
            this._editAction.enabled = true;
        }
        this._body.classList.remove('hidden');
        if (this._commentEditorModel) {
            this._commentEditorModel.dispose();
        }
        this._commentEditorDisposables.forEach(dispose => dispose.dispose());
        this._commentEditorDisposables = [];
        if (this._commentEditor) {
            this._commentEditor.dispose();
            this._commentEditor = null;
        }
        this._commentEditContainer.remove();
    }
    switchToEditMode() {
        if (this.isEditing) {
            return;
        }
        this.isEditing = true;
        this._body.classList.add('hidden');
        this._commentEditContainer = dom.append(this._commentDetailsContainer, dom.$('.edit-container'));
        this.createCommentEditor(this._commentEditContainer);
        const formActions = dom.append(this._commentEditContainer, dom.$('.form-actions'));
        const menus = this.commentService.getCommentMenus(this.owner);
        const menu = menus.getCommentActions(this.comment, this._contextKeyService);
        this._register(menu);
        this._register(menu.onDidChange(() => {
            if (this._commentFormActions) {
                this._commentFormActions.setActions(menu);
            }
        }));
        this._commentFormActions = new commentFormActions_1.CommentFormActions(formActions, (action) => {
            let text = this._commentEditor.getValue();
            action.run({
                thread: this.commentThread,
                commentUniqueId: this.comment.uniqueIdInThread,
                text: text,
                $mid: 10
            });
            this.removeCommentEditor();
        }, this.themeService);
        this._commentFormActions.setActions(menu);
    }
    registerActionBarListeners(actionsContainer) {
        this._register(dom.addDisposableListener(this._domNode, 'mouseenter', () => {
            actionsContainer.classList.remove('hidden');
        }));
        this._register(dom.addDisposableListener(this._domNode, 'focus', () => {
            actionsContainer.classList.remove('hidden');
        }));
        this._register(dom.addDisposableListener(this._domNode, 'mouseleave', () => {
            if (!this._domNode.contains(document.activeElement)) {
                actionsContainer.classList.add('hidden');
            }
        }));
    }
    update(newComment) {
        if (newComment.body !== this.comment.body) {
            this._body.removeChild(this._md);
            this._md = this.markdownRenderer.render(newComment.body).element;
            this._body.appendChild(this._md);
        }
        if (newComment.mode !== undefined && newComment.mode !== this.comment.mode) {
            if (newComment.mode === modes.CommentMode.Editing) {
                this.switchToEditMode();
            }
            else {
                this.removeCommentEditor();
            }
        }
        this.comment = newComment;
        if (newComment.label) {
            this._isPendingLabel.innerText = newComment.label;
        }
        else {
            this._isPendingLabel.innerText = '';
        }
        // update comment reactions
        if (this._reactionActionsContainer) {
            this._reactionActionsContainer.remove();
        }
        if (this._reactionsActionBar) {
            this._reactionsActionBar.clear();
        }
        if (this.comment.commentReactions && this.comment.commentReactions.some(reaction => !!reaction.count)) {
            this.createReactionsContainer(this._commentDetailsContainer);
        }
        if (this.comment.contextValue) {
            this._commentContextValue.set(this.comment.contextValue);
        }
        else {
            this._commentContextValue.reset();
        }
    }
    focus() {
        this.domNode.focus();
        if (!this._clearTimeout) {
            dom.addClass(this.domNode, 'focus');
            this._clearTimeout = setTimeout(() => {
                dom.removeClass(this.domNode, 'focus');
            }, 3000);
        }
    }
};
CommentNode = __decorate([
    __param(7, themeService_1.IThemeService),
    __param(8, instantiation_1.IInstantiationService),
    __param(9, commentService_1.ICommentService),
    __param(10, modelService_1.IModelService),
    __param(11, modeService_1.IModeService),
    __param(12, keybinding_1.IKeybindingService),
    __param(13, notification_1.INotificationService),
    __param(14, contextView_1.IContextMenuService),
    __param(15, contextkey_1.IContextKeyService)
], CommentNode);
exports.CommentNode = CommentNode;
function fillInActions(groups, target, useAlternativeActions, isPrimaryGroup = group => group === 'navigation') {
    for (let tuple of groups) {
        let [group, actions] = tuple;
        if (useAlternativeActions) {
            actions = actions.map(a => (a instanceof actions_2.MenuItemAction) && !!a.alt ? a.alt : a);
        }
        if (isPrimaryGroup(group)) {
            const to = Array.isArray(target) ? target : target.primary;
            to.unshift(...actions);
        }
        else {
            const to = Array.isArray(target) ? target : target.secondary;
            if (to.length > 0) {
                to.push(new actionbar_1.Separator());
            }
            to.push(...actions);
        }
    }
}
//# sourceMappingURL=commentNode.js.map