"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const network_1 = require("vs/base/common/network");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const nls = require("vs/nls");
const actions_1 = require("vs/platform/actions/common/actions");
const keybindingsRegistry_1 = require("vs/platform/keybinding/common/keybindingsRegistry");
const listService_1 = require("vs/platform/list/browser/listService");
const resources_1 = require("vs/workbench/common/resources");
const customEditor_1 = require("vs/workbench/contrib/customEditor/common/customEditor");
const files_1 = require("vs/workbench/contrib/files/browser/files");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const arrays_1 = require("vs/base/common/arrays");
const viewCategory = nls.localize('viewCategory', "View");
// #region Open With
const OPEN_WITH_COMMAND_ID = 'openWith';
const OPEN_WITH_TITLE = { value: nls.localize('openWith.title', 'Open With'), original: 'Open With' };
keybindingsRegistry_1.KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: OPEN_WITH_COMMAND_ID,
    weight: 200 /* WorkbenchContrib */,
    when: editorContextKeys_1.EditorContextKeys.focus.toNegated(),
    handler: (accessor, resource) => __awaiter(void 0, void 0, void 0, function* () {
        const editorService = accessor.get(editorService_1.IEditorService);
        const resources = files_1.getMultiSelectedResources(resource, accessor.get(listService_1.IListService), editorService);
        const targetResource = arrays_1.firstOrDefault(resources);
        if (!targetResource) {
            return;
        }
        return accessor.get(customEditor_1.ICustomEditorService).promptOpenWith(targetResource, undefined, undefined);
    })
});
actions_1.MenuRegistry.appendMenuItem(11 /* ExplorerContext */, {
    group: 'navigation',
    order: 20,
    command: {
        id: OPEN_WITH_COMMAND_ID,
        title: OPEN_WITH_TITLE,
    },
    when: resources_1.ResourceContextKey.Scheme.isEqualTo(network_1.Schemas.file)
});
// #endregion
// #region Reopen With
const REOPEN_WITH_COMMAND_ID = 'reOpenWith';
const REOPEN_WITH_TITLE = { value: nls.localize('reopenWith.title', 'Reopen With'), original: 'Reopen With' };
keybindingsRegistry_1.KeybindingsRegistry.registerCommandAndKeybindingRule({
    id: REOPEN_WITH_COMMAND_ID,
    weight: 200 /* WorkbenchContrib */,
    when: undefined,
    handler: (accessor, resource, editorContext) => __awaiter(void 0, void 0, void 0, function* () {
        const customEditorService = accessor.get(customEditor_1.ICustomEditorService);
        const editorService = accessor.get(editorService_1.IEditorService);
        const editorGroupService = accessor.get(editorGroupsService_1.IEditorGroupsService);
        let group;
        if (editorContext) {
            group = editorGroupService.getGroup(editorContext.groupId);
        }
        else if (!resource) {
            if (editorService.activeEditor) {
                resource = editorService.activeEditor.getResource();
                group = editorGroupService.activeGroup;
            }
        }
        if (!resource) {
            return;
        }
        // Make sure the context menu has been dismissed before we prompt.
        // Otherwise with webviews, we will sometimes close the prompt instantly when the webview is
        // refocused by the workbench
        setTimeout(() => {
            customEditorService.promptOpenWith(resource, undefined, group);
        }, 10);
    })
});
actions_1.MenuRegistry.appendMenuItem(9 /* EditorTitleContext */, {
    order: 40,
    command: {
        id: REOPEN_WITH_COMMAND_ID,
        title: REOPEN_WITH_TITLE,
    }
});
actions_1.MenuRegistry.appendMenuItem(0 /* CommandPalette */, {
    command: {
        id: REOPEN_WITH_COMMAND_ID,
        title: REOPEN_WITH_TITLE,
        category: viewCategory,
    }
});
// #endregion
//# sourceMappingURL=commands.js.map