"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("vs/base/common/decorators");
const event_1 = require("vs/base/common/event");
const path_1 = require("vs/base/common/path");
const dialogs_1 = require("vs/platform/dialogs/common/dialogs");
const label_1 = require("vs/platform/label/common/label");
const webviewEditorInput_1 = require("vs/workbench/contrib/webview/browser/webviewEditorInput");
const webviewEditorService_1 = require("vs/workbench/contrib/webview/browser/webviewEditorService");
const extensions_1 = require("vs/workbench/services/extensions/common/extensions");
const textFileService_1 = require("vs/workbench/services/textfile/browser/textFileService");
let CustomFileEditorInput = class CustomFileEditorInput extends webviewEditorInput_1.WebviewInput {
    constructor(resource, viewType, id, webview, labelService, _webviewEditorService, _extensionService, dialogService) {
        super(id, viewType, '', undefined, webview);
        this.labelService = labelService;
        this._webviewEditorService = _webviewEditorService;
        this._extensionService = _extensionService;
        this.dialogService = dialogService;
        this._hasResolved = false;
        this._state = 1 /* Readonly */;
        this._onWillSave = this._register(new event_1.Emitter());
        this.onWillSave = this._onWillSave.event;
        this._editorResource = resource;
    }
    getTypeId() {
        return CustomFileEditorInput.typeId;
    }
    getResource() {
        return this._editorResource;
    }
    getName() {
        if (!this.name) {
            this.name = path_1.basename(this.labelService.getUriLabel(this.getResource()));
        }
        return this.name;
    }
    matches(other) {
        return this === other || (other instanceof CustomFileEditorInput
            && this.viewType === other.viewType
            && this.getResource().toString() === other.getResource().toString());
    }
    get shortTitle() {
        return this.getName();
    }
    get mediumTitle() {
        return this.labelService.getUriLabel(this.getResource(), { relative: true });
    }
    get longTitle() {
        return this.labelService.getUriLabel(this.getResource());
    }
    getTitle(verbosity) {
        switch (verbosity) {
            case 0 /* SHORT */:
                return this.shortTitle;
            default:
            case 1 /* MEDIUM */:
                return this.mediumTitle;
            case 2 /* LONG */:
                return this.longTitle;
        }
    }
    resolve() {
        const _super = Object.create(null, {
            resolve: { get: () => super.resolve }
        });
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._hasResolved) {
                this._hasResolved = true;
                this._extensionService.activateByEvent(`onWebviewEditor:${this.viewType}`);
                yield this._webviewEditorService.resolveWebview(this);
            }
            return _super.resolve.call(this);
        });
    }
    setState(newState) {
        this._state = newState;
        this._onDidChangeDirty.fire();
    }
    isDirty() {
        return this._state === 3 /* Dirty */;
    }
    confirmSave() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isDirty()) {
                return 1 /* DONT_SAVE */;
            }
            return textFileService_1.promptSave(this.dialogService, [this.getResource()]);
        });
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isDirty) {
                return true;
            }
            const waitingOn = [];
            this._onWillSave.fire({
                waitUntil: (thenable) => { waitingOn.push(thenable); },
            });
            const result = yield Promise.all(waitingOn);
            return result.every(x => x);
        });
    }
};
CustomFileEditorInput.typeId = 'workbench.editors.webviewEditor';
__decorate([
    decorators_1.memoize
], CustomFileEditorInput.prototype, "shortTitle", null);
__decorate([
    decorators_1.memoize
], CustomFileEditorInput.prototype, "mediumTitle", null);
__decorate([
    decorators_1.memoize
], CustomFileEditorInput.prototype, "longTitle", null);
CustomFileEditorInput = __decorate([
    __param(4, label_1.ILabelService),
    __param(5, webviewEditorService_1.IWebviewEditorService),
    __param(6, extensions_1.IExtensionService),
    __param(7, dialogs_1.IDialogService)
], CustomFileEditorInput);
exports.CustomFileEditorInput = CustomFileEditorInput;
//# sourceMappingURL=customEditorInput.js.map