"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const extensionsRegistry_1 = require("vs/workbench/services/extensions/common/extensionsRegistry");
const workbenchModeService_1 = require("vs/workbench/services/mode/common/workbenchModeService");
var WebviewEditorContribution;
(function (WebviewEditorContribution) {
    WebviewEditorContribution.viewType = 'viewType';
    WebviewEditorContribution.displayName = 'displayName';
    WebviewEditorContribution.selector = 'selector';
    WebviewEditorContribution.discretion = 'discretion';
})(WebviewEditorContribution || (WebviewEditorContribution = {}));
const webviewEditorsContribution = {
    description: nls.localize('contributes.webviewEditors', 'Contributes webview editors.'),
    type: 'array',
    defaultSnippets: [{ body: [{ viewType: '', displayName: '' }] }],
    items: {
        type: 'object',
        required: [
            WebviewEditorContribution.viewType,
            WebviewEditorContribution.displayName,
            WebviewEditorContribution.selector,
        ],
        properties: {
            [WebviewEditorContribution.viewType]: {
                type: 'string',
                description: nls.localize('contributes.viewType', 'Unique identifier of the custom editor.'),
            },
            [WebviewEditorContribution.displayName]: {
                type: 'string',
                description: nls.localize('contributes.displayName', 'Name of the custom editor displayed to users.'),
            },
            [WebviewEditorContribution.selector]: {
                type: 'array',
                description: nls.localize('contributes.selector', 'Set of globs that the custom editor is enabled for.'),
                items: {
                    type: 'object',
                    properties: {
                        filenamePattern: {
                            type: 'string',
                            description: nls.localize('contributes.selector.filenamePattern', 'Glob that the custom editor is enabled for.'),
                        },
                        scheme: {
                            type: 'string',
                            description: nls.localize('contributes.selector.scheme', 'File scheme that the custom editor is enabled for.'),
                        }
                    }
                }
            },
            [WebviewEditorContribution.discretion]: {
                type: 'string',
                description: nls.localize('contributes.discretion', 'Controls when the custom editor is used. May be overridden by users.'),
                enum: [
                    "default" /* default */,
                    "option" /* option */
                ],
                enumDescriptions: [
                    nls.localize('contributes.discretion.default', 'Editor is automatically used for a resource if no other default custom editors are registered for it.'),
                    nls.localize('contributes.discretion.option', 'Editor is not automatically used but can be selected by a user.'),
                ],
                default: 'default'
            }
        }
    }
};
exports.webviewEditorsExtensionPoint = extensionsRegistry_1.ExtensionsRegistry.registerExtensionPoint({
    extensionPoint: 'webviewEditors',
    deps: [workbenchModeService_1.languagesExtPoint],
    jsonSchema: webviewEditorsContribution
});
//# sourceMappingURL=extensionPoint.js.map