"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const configurationRegistry_1 = require("vs/platform/configuration/common/configurationRegistry");
const descriptors_1 = require("vs/platform/instantiation/common/descriptors");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const platform_1 = require("vs/platform/registry/common/platform");
const editor_1 = require("vs/workbench/browser/editor");
const contributions_1 = require("vs/workbench/common/contributions");
const editor_2 = require("vs/workbench/common/editor");
const customEditorInputFactory_1 = require("vs/workbench/contrib/customEditor/browser/customEditorInputFactory");
const customEditor_1 = require("vs/workbench/contrib/customEditor/common/customEditor");
const webviewEditor_1 = require("vs/workbench/contrib/webview/browser/webviewEditor");
require("./commands");
const customEditorInput_1 = require("./customEditorInput");
const customEditors_1 = require("./customEditors");
extensions_1.registerSingleton(customEditor_1.ICustomEditorService, customEditors_1.CustomEditorService);
platform_1.Registry.as(contributions_1.Extensions.Workbench)
    .registerWorkbenchContribution(customEditors_1.CustomEditorContribution, 1 /* Starting */);
platform_1.Registry.as(editor_1.Extensions.Editors).registerEditor(new editor_1.EditorDescriptor(webviewEditor_1.WebviewEditor, webviewEditor_1.WebviewEditor.ID, 'Webview Editor'), [
    new descriptors_1.SyncDescriptor(customEditorInput_1.CustomFileEditorInput)
]);
platform_1.Registry.as(editor_2.Extensions.EditorInputFactories).registerEditorInputFactory(customEditorInputFactory_1.CustomEditoInputFactory.ID, customEditorInputFactory_1.CustomEditoInputFactory);
platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration)
    .registerConfiguration({
    'id': 'workbench',
    'order': 7,
    'title': nls.localize('workbenchConfigurationTitle', "Workbench"),
    'type': 'object',
    'properties': {
        [customEditors_1.customEditorsAssociationsKey]: {
            type: 'array',
            markdownDescription: nls.localize('editor.editorAssociations', "Configure which editor to use for a resource."),
            items: {
                type: 'object',
                properties: {
                    'viewType': {
                        type: 'string',
                        description: nls.localize('editor.editorAssociations.viewType', "Editor view type."),
                    },
                    'scheme': {
                        type: 'string',
                        description: nls.localize('editor.editorAssociations.scheme', "Uri scheme the editor should be used for."),
                    },
                    'filenamePattern': {
                        type: 'string',
                        description: nls.localize('editor.editorAssociations.filenamePattern', "Glob pattern the the editor should be used for."),
                    }
                }
            }
        }
    }
});
//# sourceMappingURL=webviewEditor.contribution.js.map