"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/css!./media/exceptionWidget");
const nls = require("vs/nls");
const dom = require("vs/base/browser/dom");
const zoneWidget_1 = require("vs/editor/contrib/zoneWidget/zoneWidget");
const async_1 = require("vs/base/common/async");
const themeService_1 = require("vs/platform/theme/common/themeService");
const color_1 = require("vs/base/common/color");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const linkDetector_1 = require("vs/workbench/contrib/debug/browser/linkDetector");
const $ = dom.$;
// theming
exports.debugExceptionWidgetBorder = colorRegistry_1.registerColor('debugExceptionWidget.border', { dark: '#a31515', light: '#a31515', hc: '#a31515' }, nls.localize('debugExceptionWidgetBorder', 'Exception widget border color.'));
exports.debugExceptionWidgetBackground = colorRegistry_1.registerColor('debugExceptionWidget.background', { dark: '#420b0d', light: '#f1dfde', hc: '#420b0d' }, nls.localize('debugExceptionWidgetBackground', 'Exception widget background color.'));
let ExceptionWidget = class ExceptionWidget extends zoneWidget_1.ZoneWidget {
    constructor(editor, exceptionInfo, themeService, instantiationService) {
        super(editor, { showFrame: true, showArrow: true, frameWidth: 1, className: 'exception-widget-container' });
        this.exceptionInfo = exceptionInfo;
        this.instantiationService = instantiationService;
        this._backgroundColor = color_1.Color.white;
        this._applyTheme(themeService.getTheme());
        this._disposables.add(themeService.onThemeChange(this._applyTheme.bind(this)));
        this.create();
        const onDidLayoutChangeScheduler = new async_1.RunOnceScheduler(() => this._doLayout(undefined, undefined), 50);
        this._disposables.add(this.editor.onDidLayoutChange(() => onDidLayoutChangeScheduler.schedule()));
        this._disposables.add(onDidLayoutChangeScheduler);
    }
    _applyTheme(theme) {
        this._backgroundColor = theme.getColor(exports.debugExceptionWidgetBackground);
        const frameColor = theme.getColor(exports.debugExceptionWidgetBorder);
        this.style({
            arrowColor: frameColor,
            frameColor: frameColor
        }); // style() will trigger _applyStyles
    }
    _applyStyles() {
        if (this.container) {
            this.container.style.backgroundColor = this._backgroundColor ? this._backgroundColor.toString() : '';
        }
        super._applyStyles();
    }
    _fillContainer(container) {
        this.setCssClass('exception-widget');
        // Set the font size and line height to the one from the editor configuration.
        const fontInfo = this.editor.getOption(30 /* fontInfo */);
        container.style.fontSize = `${fontInfo.fontSize}px`;
        container.style.lineHeight = `${fontInfo.lineHeight}px`;
        let title = $('.title');
        title.textContent = this.exceptionInfo.id ? nls.localize('exceptionThrownWithId', 'Exception has occurred: {0}', this.exceptionInfo.id) : nls.localize('exceptionThrown', 'Exception has occurred.');
        dom.append(container, title);
        if (this.exceptionInfo.description) {
            let description = $('.description');
            description.textContent = this.exceptionInfo.description;
            dom.append(container, description);
        }
        if (this.exceptionInfo.details && this.exceptionInfo.details.stackTrace) {
            let stackTrace = $('.stack-trace');
            const linkDetector = this.instantiationService.createInstance(linkDetector_1.LinkDetector);
            const linkedStackTrace = linkDetector.handleLinks(this.exceptionInfo.details.stackTrace);
            stackTrace.appendChild(linkedStackTrace);
            dom.append(container, stackTrace);
        }
    }
    _doLayout(_heightInPixel, _widthInPixel) {
        // Reload the height with respect to the exception text content and relayout it to match the line count.
        this.container.style.height = 'initial';
        const lineHeight = this.editor.getOption(45 /* lineHeight */);
        const arrowHeight = Math.round(lineHeight / 3);
        const computedLinesNumber = Math.ceil((this.container.offsetHeight + arrowHeight) / lineHeight);
        this._relayout(computedLinesNumber);
    }
};
ExceptionWidget = __decorate([
    __param(2, themeService_1.IThemeService),
    __param(3, instantiation_1.IInstantiationService)
], ExceptionWidget);
exports.ExceptionWidget = ExceptionWidget;
//# sourceMappingURL=exceptionWidget.js.map