"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const themeService_1 = require("vs/platform/theme/common/themeService");
const nls_1 = require("vs/nls");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const layoutService_1 = require("vs/workbench/services/layout/browser/layoutService");
const debug_1 = require("vs/workbench/contrib/debug/common/debug");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const theme_1 = require("vs/workbench/common/theme");
const dom_1 = require("vs/base/browser/dom");
// colors for theming
exports.STATUS_BAR_DEBUGGING_BACKGROUND = colorRegistry_1.registerColor('statusBar.debuggingBackground', {
    dark: '#CC6633',
    light: '#CC6633',
    hc: '#CC6633'
}, nls_1.localize('statusBarDebuggingBackground', "Status bar background color when a program is being debugged. The status bar is shown in the bottom of the window"));
exports.STATUS_BAR_DEBUGGING_FOREGROUND = colorRegistry_1.registerColor('statusBar.debuggingForeground', {
    dark: theme_1.STATUS_BAR_FOREGROUND,
    light: theme_1.STATUS_BAR_FOREGROUND,
    hc: theme_1.STATUS_BAR_FOREGROUND
}, nls_1.localize('statusBarDebuggingForeground', "Status bar foreground color when a program is being debugged. The status bar is shown in the bottom of the window"));
exports.STATUS_BAR_DEBUGGING_BORDER = colorRegistry_1.registerColor('statusBar.debuggingBorder', {
    dark: theme_1.STATUS_BAR_BORDER,
    light: theme_1.STATUS_BAR_BORDER,
    hc: theme_1.STATUS_BAR_BORDER
}, nls_1.localize('statusBarDebuggingBorder', "Status bar border color separating to the sidebar and editor when a program is being debugged. The status bar is shown in the bottom of the window"));
let StatusBarColorProvider = class StatusBarColorProvider extends theme_1.Themable {
    constructor(themeService, debugService, contextService, layoutService) {
        super(themeService);
        this.debugService = debugService;
        this.contextService = contextService;
        this.layoutService = layoutService;
        this.registerListeners();
        this.updateStyles();
    }
    registerListeners() {
        this._register(this.debugService.onDidChangeState(state => this.updateStyles()));
        this._register(this.contextService.onDidChangeWorkbenchState(state => this.updateStyles()));
    }
    updateStyles() {
        super.updateStyles();
        const container = this.layoutService.getContainer("workbench.parts.statusbar" /* STATUSBAR_PART */);
        if (isStatusbarInDebugMode(this.debugService)) {
            dom_1.addClass(container, 'debugging');
        }
        else {
            dom_1.removeClass(container, 'debugging');
        }
        // Container Colors
        const backgroundColor = this.getColor(this.getColorKey(theme_1.STATUS_BAR_NO_FOLDER_BACKGROUND, exports.STATUS_BAR_DEBUGGING_BACKGROUND, theme_1.STATUS_BAR_BACKGROUND));
        container.style.backgroundColor = backgroundColor;
        container.style.color = this.getColor(this.getColorKey(theme_1.STATUS_BAR_NO_FOLDER_FOREGROUND, exports.STATUS_BAR_DEBUGGING_FOREGROUND, theme_1.STATUS_BAR_FOREGROUND));
        // Border Color
        const borderColor = this.getColor(this.getColorKey(theme_1.STATUS_BAR_NO_FOLDER_BORDER, exports.STATUS_BAR_DEBUGGING_BORDER, theme_1.STATUS_BAR_BORDER)) || this.getColor(colorRegistry_1.contrastBorder);
        if (borderColor) {
            dom_1.addClass(container, 'status-border-top');
            container.style.setProperty('--status-border-top-color', borderColor.toString());
        }
        else {
            dom_1.removeClass(container, 'status-border-top');
            container.style.removeProperty('--status-border-top-color');
        }
        // Notification Beak
        if (!this.styleElement) {
            this.styleElement = dom_1.createStyleSheet(container);
        }
        this.styleElement.innerHTML = `.monaco-workbench .part.statusbar > .items-container > .statusbar-item.has-beak:before { border-bottom-color: ${backgroundColor} !important; }`;
    }
    getColorKey(noFolderColor, debuggingColor, normalColor) {
        // Not debugging
        if (!isStatusbarInDebugMode(this.debugService)) {
            if (this.contextService.getWorkbenchState() !== 1 /* EMPTY */) {
                return normalColor;
            }
            return noFolderColor;
        }
        // Debugging
        return debuggingColor;
    }
};
StatusBarColorProvider = __decorate([
    __param(0, themeService_1.IThemeService),
    __param(1, debug_1.IDebugService),
    __param(2, workspace_1.IWorkspaceContextService),
    __param(3, layoutService_1.IWorkbenchLayoutService)
], StatusBarColorProvider);
exports.StatusBarColorProvider = StatusBarColorProvider;
function isStatusbarInDebugMode(debugService) {
    if (debugService.state === 0 /* Inactive */ || debugService.state === 1 /* Initializing */) {
        return false;
    }
    const session = debugService.getViewModel().focusedSession;
    const isRunningWithoutDebug = session && session.configuration && session.configuration.noDebug;
    if (isRunningWithoutDebug) {
        return false;
    }
    return true;
}
exports.isStatusbarInDebugMode = isStatusbarInDebugMode;
//# sourceMappingURL=statusbarColorProvider.js.map