"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const async_1 = require("vs/base/common/async");
const dom = require("vs/base/browser/dom");
const viewlet_1 = require("vs/workbench/browser/viewlet");
const debug_1 = require("vs/workbench/contrib/debug/common/debug");
const debugModel_1 = require("vs/workbench/contrib/debug/common/debugModel");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const baseDebugView_1 = require("vs/workbench/contrib/debug/browser/baseDebugView");
const actions_1 = require("vs/base/common/actions");
const debugActions_1 = require("vs/workbench/contrib/debug/browser/debugActions");
const actionbar_1 = require("vs/base/browser/ui/actionbar/actionbar");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const panelViewlet_1 = require("vs/workbench/browser/parts/views/panelViewlet");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const event_1 = require("vs/base/common/event");
const listService_1 = require("vs/platform/list/browser/listService");
const errors_1 = require("vs/base/common/errors");
const filters_1 = require("vs/base/common/filters");
const highlightedLabel_1 = require("vs/base/browser/ui/highlightedlabel/highlightedLabel");
const clipboardService_1 = require("vs/platform/clipboard/common/clipboardService");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const lifecycle_1 = require("vs/base/common/lifecycle");
const $ = dom.$;
let forgetScopes = true;
exports.variableSetEmitter = new event_1.Emitter();
let VariablesView = class VariablesView extends panelViewlet_1.ViewletPanel {
    constructor(options, contextMenuService, debugService, keybindingService, configurationService, instantiationService, clipboardService, contextKeyService) {
        super(Object.assign(Object.assign({}, options), { ariaHeaderLabel: nls.localize('variablesSection', "Variables Section") }), keybindingService, contextMenuService, configurationService, contextKeyService);
        this.debugService = debugService;
        this.instantiationService = instantiationService;
        this.clipboardService = clipboardService;
        this.needsRefresh = false;
        // Use scheduler to prevent unnecessary flashing
        this.onFocusStackFrameScheduler = new async_1.RunOnceScheduler(() => {
            const stackFrame = this.debugService.getViewModel().focusedStackFrame;
            this.needsRefresh = false;
            if (stackFrame && this.savedViewState) {
                this.tree.setInput(this.debugService.getViewModel(), this.savedViewState).then(null, errors_1.onUnexpectedError);
                this.savedViewState = undefined;
            }
            else {
                if (!stackFrame) {
                    // We have no stackFrame, save tree state before it is cleared
                    this.savedViewState = this.tree.getViewState();
                }
                this.tree.updateChildren().then(() => {
                    if (stackFrame) {
                        stackFrame.getScopes().then(scopes => {
                            // Expand the first scope if it is not expensive and if there is no expansion state (all are collapsed)
                            if (scopes.every(s => this.tree.getNode(s).collapsed) && scopes.length > 0 && !scopes[0].expensive) {
                                this.tree.expand(scopes[0]).then(undefined, errors_1.onUnexpectedError);
                            }
                        });
                    }
                }, errors_1.onUnexpectedError);
            }
        }, 400);
    }
    renderBody(container) {
        dom.addClass(container, 'debug-variables');
        const treeContainer = baseDebugView_1.renderViewTree(container);
        this.tree = this.instantiationService.createInstance(listService_1.WorkbenchAsyncDataTree, 'VariablesView', treeContainer, new VariablesDelegate(), [this.instantiationService.createInstance(VariablesRenderer), new ScopesRenderer()], new VariablesDataSource(), {
            ariaLabel: nls.localize('variablesAriaTreeLabel', "Debug Variables"),
            accessibilityProvider: new VariablesAccessibilityProvider(),
            identityProvider: { getId: (element) => element.getId() },
            keyboardNavigationLabelProvider: { getKeyboardNavigationLabel: (e) => e }
        });
        this.tree.setInput(this.debugService.getViewModel()).then(null, errors_1.onUnexpectedError);
        debug_1.CONTEXT_VARIABLES_FOCUSED.bindTo(this.tree.contextKeyService);
        const collapseAction = new viewlet_1.CollapseAction(this.tree, true, 'explorer-action collapse-explorer');
        this.toolbar.setActions([collapseAction])();
        this.tree.updateChildren();
        this._register(this.debugService.getViewModel().onDidFocusStackFrame(sf => {
            if (!this.isBodyVisible()) {
                this.needsRefresh = true;
                return;
            }
            // Refresh the tree immediately if the user explictly changed stack frames.
            // Otherwise postpone the refresh until user stops stepping.
            const timeout = sf.explicit ? 0 : undefined;
            this.onFocusStackFrameScheduler.schedule(timeout);
        }));
        this._register(exports.variableSetEmitter.event(() => {
            const stackFrame = this.debugService.getViewModel().focusedStackFrame;
            if (stackFrame && forgetScopes) {
                stackFrame.forgetScopes();
            }
            forgetScopes = true;
            this.tree.updateChildren();
        }));
        this._register(this.tree.onMouseDblClick(e => this.onMouseDblClick(e)));
        this._register(this.tree.onContextMenu((e) => __awaiter(this, void 0, void 0, function* () { return yield this.onContextMenu(e); })));
        this._register(this.onDidChangeBodyVisibility(visible => {
            if (visible && this.needsRefresh) {
                this.onFocusStackFrameScheduler.schedule();
            }
        }));
        this._register(this.debugService.getViewModel().onDidSelectExpression(e => {
            if (e instanceof debugModel_1.Variable) {
                this.tree.rerender(e);
            }
        }));
    }
    layoutBody(width, height) {
        this.tree.layout(width, height);
    }
    focus() {
        this.tree.domFocus();
    }
    onMouseDblClick(e) {
        const session = this.debugService.getViewModel().focusedSession;
        if (session && e.element instanceof debugModel_1.Variable && session.capabilities.supportsSetVariable) {
            this.debugService.getViewModel().setSelectedExpression(e.element);
        }
    }
    onContextMenu(e) {
        return __awaiter(this, void 0, void 0, function* () {
            const variable = e.element;
            if (variable instanceof debugModel_1.Variable && !!variable.value) {
                const actions = [];
                const session = this.debugService.getViewModel().focusedSession;
                if (session && session.capabilities.supportsSetVariable) {
                    actions.push(new actions_1.Action('workbench.setValue', nls.localize('setValue', "Set Value"), undefined, true, () => {
                        this.debugService.getViewModel().setSelectedExpression(variable);
                        return Promise.resolve();
                    }));
                }
                actions.push(this.instantiationService.createInstance(debugActions_1.CopyValueAction, debugActions_1.CopyValueAction.ID, debugActions_1.CopyValueAction.LABEL, variable, 'variables'));
                if (variable.evaluateName) {
                    actions.push(new actions_1.Action('debug.copyEvaluatePath', nls.localize('copyAsExpression', "Copy as Expression"), undefined, true, () => {
                        return this.clipboardService.writeText(variable.evaluateName);
                    }));
                    actions.push(new actionbar_1.Separator());
                    actions.push(new actions_1.Action('debug.addToWatchExpressions', nls.localize('addToWatchExpressions', "Add to Watch"), undefined, true, () => {
                        this.debugService.addWatchExpression(variable.evaluateName);
                        return Promise.resolve(undefined);
                    }));
                }
                if (session && session.capabilities.supportsDataBreakpoints) {
                    const response = yield session.dataBreakpointInfo(variable.name, variable.parent.reference);
                    const dataid = response.dataId;
                    if (dataid) {
                        actions.push(new actionbar_1.Separator());
                        actions.push(new actions_1.Action('debug.breakWhenValueChanges', nls.localize('breakWhenValueChanges', "Break When Value Changes"), undefined, true, () => {
                            return this.debugService.addDataBreakpoint(response.description, dataid, !!response.canPersist);
                        }));
                    }
                }
                this.contextMenuService.showContextMenu({
                    getAnchor: () => e.anchor,
                    getActions: () => actions,
                    getActionsContext: () => variable,
                    onHide: () => lifecycle_1.dispose(actions)
                });
            }
        });
    }
};
VariablesView = __decorate([
    __param(1, contextView_1.IContextMenuService),
    __param(2, debug_1.IDebugService),
    __param(3, keybinding_1.IKeybindingService),
    __param(4, configuration_1.IConfigurationService),
    __param(5, instantiation_1.IInstantiationService),
    __param(6, clipboardService_1.IClipboardService),
    __param(7, contextkey_1.IContextKeyService)
], VariablesView);
exports.VariablesView = VariablesView;
function isViewModel(obj) {
    return typeof obj.getSelectedExpression === 'function';
}
class VariablesDataSource {
    hasChildren(element) {
        if (isViewModel(element) || element instanceof debugModel_1.Scope) {
            return true;
        }
        return element.hasChildren;
    }
    getChildren(element) {
        if (isViewModel(element)) {
            const stackFrame = element.focusedStackFrame;
            return stackFrame ? stackFrame.getScopes() : Promise.resolve([]);
        }
        return element.getChildren();
    }
}
exports.VariablesDataSource = VariablesDataSource;
class VariablesDelegate {
    getHeight(element) {
        return 22;
    }
    getTemplateId(element) {
        if (element instanceof debugModel_1.Scope) {
            return ScopesRenderer.ID;
        }
        return VariablesRenderer.ID;
    }
}
class ScopesRenderer {
    get templateId() {
        return ScopesRenderer.ID;
    }
    renderTemplate(container) {
        const name = dom.append(container, $('.scope'));
        const label = new highlightedLabel_1.HighlightedLabel(name, false);
        return { name, label };
    }
    renderElement(element, index, templateData) {
        templateData.label.set(element.element.name, filters_1.createMatches(element.filterData));
    }
    disposeTemplate(templateData) {
        // noop
    }
}
ScopesRenderer.ID = 'scope';
class VariablesRenderer extends baseDebugView_1.AbstractExpressionsRenderer {
    get templateId() {
        return VariablesRenderer.ID;
    }
    renderExpression(expression, data, highlights) {
        baseDebugView_1.renderVariable(expression, data, true, highlights);
    }
    getInputBoxOptions(expression) {
        const variable = expression;
        return {
            initialValue: expression.value,
            ariaLabel: nls.localize('variableValueAriaLabel', "Type new variable value"),
            validationOptions: {
                validation: () => variable.errorMessage ? ({ content: variable.errorMessage }) : null
            },
            onFinish: (value, success) => {
                variable.errorMessage = undefined;
                if (success && variable.value !== value) {
                    variable.setVariable(value)
                        // Need to force watch expressions and variables to update since a variable change can have an effect on both
                        .then(() => {
                        // Do not refresh scopes due to a node limitation #15520
                        forgetScopes = false;
                        exports.variableSetEmitter.fire();
                    });
                }
            }
        };
    }
}
exports.VariablesRenderer = VariablesRenderer;
VariablesRenderer.ID = 'variable';
class VariablesAccessibilityProvider {
    getAriaLabel(element) {
        if (element instanceof debugModel_1.Scope) {
            return nls.localize('variableScopeAriaLabel', "Scope {0}, variables, debug", element.name);
        }
        if (element instanceof debugModel_1.Variable) {
            return nls.localize('variableAriaLabel', "{0} value {1}, variables, debug", element.name, element.value);
        }
        return null;
    }
}
//# sourceMappingURL=variablesView.js.map