"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const debugViewModel_1 = require("vs/workbench/contrib/debug/common/debugViewModel");
const debugModel_1 = require("vs/workbench/contrib/debug/common/debugModel");
const mockDebug_1 = require("vs/workbench/contrib/debug/test/common/mockDebug");
const mockKeybindingService_1 = require("vs/platform/keybinding/test/common/mockKeybindingService");
suite('Debug - View Model', () => {
    let model;
    setup(() => {
        model = new debugViewModel_1.ViewModel(new mockKeybindingService_1.MockContextKeyService());
    });
    test('focused stack frame', () => {
        assert.equal(model.focusedStackFrame, null);
        assert.equal(model.focusedThread, null);
        const session = new mockDebug_1.MockSession();
        const thread = new debugModel_1.Thread(session, 'myThread', 1);
        const frame = new debugModel_1.StackFrame(thread, 1, undefined, 'app.js', 'normal', { startColumn: 1, startLineNumber: 1, endColumn: 1, endLineNumber: 1 }, 0);
        model.setFocus(frame, thread, session, false);
        assert.equal(model.focusedStackFrame.getId(), frame.getId());
        assert.equal(model.focusedThread.threadId, 1);
        assert.equal(model.focusedSession.getId(), session.getId());
    });
    test('selected expression', () => {
        assert.equal(model.getSelectedExpression(), null);
        const expression = new debugModel_1.Expression('my expression');
        model.setSelectedExpression(expression);
        assert.equal(model.getSelectedExpression(), expression);
    });
    test('multi session view and changed workbench state', () => {
        assert.equal(model.isMultiSessionView(), false);
        model.setMultiSessionView(true);
        assert.equal(model.isMultiSessionView(), true);
    });
});
//# sourceMappingURL=debugViewModel.test.js.map