"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls_1 = require("vs/nls");
const actions_1 = require("vs/base/common/actions");
const files_1 = require("vs/platform/files/common/files");
const uri_1 = require("vs/base/common/uri");
const environment_1 = require("vs/platform/environment/common/environment");
const electron_1 = require("vs/platform/electron/node/electron");
const network_1 = require("vs/base/common/network");
let OpenExtensionsFolderAction = class OpenExtensionsFolderAction extends actions_1.Action {
    constructor(id, label, electronService, fileService, environmentService) {
        super(id, label, undefined, true);
        this.electronService = electronService;
        this.fileService = fileService;
        this.environmentService = environmentService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.environmentService.extensionsPath) {
                const extensionsHome = uri_1.URI.file(this.environmentService.extensionsPath);
                const file = yield this.fileService.resolve(extensionsHome);
                let itemToShow;
                if (file.children && file.children.length > 0) {
                    itemToShow = file.children[0].resource;
                }
                else {
                    itemToShow = extensionsHome;
                }
                if (itemToShow.scheme === network_1.Schemas.file) {
                    return this.electronService.showItemInFolder(itemToShow.fsPath);
                }
            }
        });
    }
};
OpenExtensionsFolderAction.ID = 'workbench.extensions.action.openExtensionsFolder';
OpenExtensionsFolderAction.LABEL = nls_1.localize('openExtensionsFolder', "Open Extensions Folder");
OpenExtensionsFolderAction = __decorate([
    __param(2, electron_1.IElectronService),
    __param(3, files_1.IFileService),
    __param(4, environment_1.IEnvironmentService)
], OpenExtensionsFolderAction);
exports.OpenExtensionsFolderAction = OpenExtensionsFolderAction;
//# sourceMappingURL=extensionsActions.js.map