"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const objects_1 = require("vs/base/common/objects");
const uuid_1 = require("vs/base/common/uuid");
const extensionsViews_1 = require("vs/workbench/contrib/extensions/browser/extensionsViews");
const instantiationServiceMock_1 = require("vs/platform/instantiation/test/common/instantiationServiceMock");
const extensions_1 = require("vs/workbench/contrib/extensions/common/extensions");
const extensionsWorkbenchService_1 = require("vs/workbench/contrib/extensions/browser/extensionsWorkbenchService");
const extensionManagement_1 = require("vs/platform/extensionManagement/common/extensionManagement");
const extensionManagement_2 = require("vs/workbench/services/extensionManagement/common/extensionManagement");
const extensionManagementUtil_1 = require("vs/platform/extensionManagement/common/extensionManagementUtil");
const extensionManagementService_1 = require("vs/platform/extensionManagement/node/extensionManagementService");
const extensionTipsService_1 = require("vs/workbench/contrib/extensions/browser/extensionTipsService");
const extensionEnablementService_test_1 = require("vs/workbench/services/extensionManagement/test/electron-browser/extensionEnablementService.test");
const extensionGalleryService_1 = require("vs/platform/extensionManagement/common/extensionGalleryService");
const url_1 = require("vs/platform/url/common/url");
const event_1 = require("vs/base/common/event");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
const extensions_2 = require("vs/workbench/services/extensions/common/extensions");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const log_1 = require("vs/platform/log/common/log");
const windows_1 = require("vs/platform/windows/common/windows");
const urlService_1 = require("vs/platform/url/node/urlService");
const uri_1 = require("vs/base/common/uri");
const testConfigurationService_1 = require("vs/platform/configuration/test/common/testConfigurationService");
const experimentService_1 = require("vs/workbench/contrib/experiments/common/experimentService");
const remoteAgentService_1 = require("vs/workbench/services/remote/common/remoteAgentService");
const remoteAgentServiceImpl_1 = require("vs/workbench/services/remote/electron-browser/remoteAgentServiceImpl");
const extensions_3 = require("vs/platform/extensions/common/extensions");
const sharedProcessService_1 = require("vs/platform/ipc/electron-browser/sharedProcessService");
const extensionManagementServerService_1 = require("vs/workbench/services/extensionManagement/electron-browser/extensionManagementServerService");
const productService_1 = require("vs/platform/product/common/productService");
const label_1 = require("vs/platform/label/common/label");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const mockKeybindingService_1 = require("vs/platform/keybinding/test/common/mockKeybindingService");
suite('ExtensionsListView Tests', () => {
    let instantiationService;
    let testableView;
    let installEvent, didInstallEvent, uninstallEvent, didUninstallEvent;
    const localEnabledTheme = aLocalExtension('first-enabled-extension', { categories: ['Themes', 'random'] });
    const localEnabledLanguage = aLocalExtension('second-enabled-extension', { categories: ['Programming languages'] });
    const localDisabledTheme = aLocalExtension('first-disabled-extension', { categories: ['themes'] });
    const localDisabledLanguage = aLocalExtension('second-disabled-extension', { categories: ['programming languages'] });
    const localRandom = aLocalExtension('random-enabled-extension', { categories: ['random'] });
    const builtInTheme = aLocalExtension('my-theme', { contributes: { themes: ['my-theme'] } }, { type: 0 /* System */ });
    const builtInBasic = aLocalExtension('my-lang', { contributes: { grammars: [{ language: 'my-language' }] } }, { type: 0 /* System */ });
    const workspaceRecommendationA = aGalleryExtension('workspace-recommendation-A');
    const workspaceRecommendationB = aGalleryExtension('workspace-recommendation-B');
    const fileBasedRecommendationA = aGalleryExtension('filebased-recommendation-A');
    const fileBasedRecommendationB = aGalleryExtension('filebased-recommendation-B');
    const otherRecommendationA = aGalleryExtension('other-recommendation-A');
    suiteSetup(() => {
        installEvent = new event_1.Emitter();
        didInstallEvent = new event_1.Emitter();
        uninstallEvent = new event_1.Emitter();
        didUninstallEvent = new event_1.Emitter();
        instantiationService = new instantiationServiceMock_1.TestInstantiationService();
        instantiationService.stub(telemetry_1.ITelemetryService, telemetryUtils_1.NullTelemetryService);
        instantiationService.stub(log_1.ILogService, log_1.NullLogService);
        instantiationService.stub(windows_1.IWindowService, workbenchTestServices_1.TestWindowService);
        instantiationService.stub(workspace_1.IWorkspaceContextService, new workbenchTestServices_1.TestContextService());
        instantiationService.stub(configuration_1.IConfigurationService, new testConfigurationService_1.TestConfigurationService());
        instantiationService.stub(extensionManagement_1.IExtensionGalleryService, extensionGalleryService_1.ExtensionGalleryService);
        instantiationService.stub(sharedProcessService_1.ISharedProcessService, workbenchTestServices_1.TestSharedProcessService);
        instantiationService.stub(experimentService_1.IExperimentService, experimentService_1.ExperimentService);
        instantiationService.stub(extensionManagement_1.IExtensionManagementService, extensionManagementService_1.ExtensionManagementService);
        instantiationService.stub(extensionManagement_1.IExtensionManagementService, 'onInstallExtension', installEvent.event);
        instantiationService.stub(extensionManagement_1.IExtensionManagementService, 'onDidInstallExtension', didInstallEvent.event);
        instantiationService.stub(extensionManagement_1.IExtensionManagementService, 'onUninstallExtension', uninstallEvent.event);
        instantiationService.stub(extensionManagement_1.IExtensionManagementService, 'onDidUninstallExtension', didUninstallEvent.event);
        instantiationService.stub(remoteAgentService_1.IRemoteAgentService, remoteAgentServiceImpl_1.RemoteAgentService);
        instantiationService.stub(contextkey_1.IContextKeyService, mockKeybindingService_1.MockContextKeyService);
        instantiationService.stub(extensionManagement_2.IExtensionManagementServerService, new class extends extensionManagementServerService_1.ExtensionManagementServerService {
            constructor() {
                super(instantiationService.get(sharedProcessService_1.ISharedProcessService), instantiationService.get(remoteAgentService_1.IRemoteAgentService), instantiationService.get(extensionManagement_1.IExtensionGalleryService), instantiationService.get(configuration_1.IConfigurationService), instantiationService.get(productService_1.IProductService), instantiationService.get(log_1.ILogService), instantiationService.get(label_1.ILabelService));
                this._localExtensionManagementServer = { extensionManagementService: instantiationService.get(extensionManagement_1.IExtensionManagementService), label: 'local', authority: 'vscode-local' };
            }
            get localExtensionManagementServer() { return this._localExtensionManagementServer; }
            set localExtensionManagementServer(server) { }
        }());
        instantiationService.stub(extensionManagement_2.IExtensionEnablementService, new extensionEnablementService_test_1.TestExtensionEnablementService(instantiationService));
        instantiationService.stub(extensionManagement_2.IExtensionTipsService, extensionTipsService_1.ExtensionTipsService);
        instantiationService.stub(url_1.IURLService, urlService_1.URLService);
        instantiationService.stubPromise(extensionManagement_2.IExtensionTipsService, 'getWorkspaceRecommendations', [
            { extensionId: workspaceRecommendationA.identifier.id },
            { extensionId: workspaceRecommendationB.identifier.id }
        ]);
        instantiationService.stub(extensionManagement_2.IExtensionTipsService, 'getFileBasedRecommendations', [
            { extensionId: fileBasedRecommendationA.identifier.id },
            { extensionId: fileBasedRecommendationB.identifier.id }
        ]);
        instantiationService.stubPromise(extensionManagement_2.IExtensionTipsService, 'getOtherRecommendations', [
            { extensionId: otherRecommendationA.identifier.id }
        ]);
        const reasons = {};
        reasons[workspaceRecommendationA.identifier.id] = { reasonId: 0 /* Workspace */ };
        reasons[workspaceRecommendationB.identifier.id] = { reasonId: 0 /* Workspace */ };
        reasons[fileBasedRecommendationA.identifier.id] = { reasonId: 1 /* File */ };
        reasons[fileBasedRecommendationB.identifier.id] = { reasonId: 1 /* File */ };
        reasons[otherRecommendationA.identifier.id] = { reasonId: 2 /* Executable */ };
        instantiationService.stub(extensionManagement_2.IExtensionTipsService, 'getAllRecommendationsWithReason', reasons);
    });
    setup(() => __awaiter(void 0, void 0, void 0, function* () {
        instantiationService.stubPromise(extensionManagement_1.IExtensionManagementService, 'getInstalled', [localEnabledTheme, localEnabledLanguage, localRandom, localDisabledTheme, localDisabledLanguage, builtInTheme, builtInBasic]);
        instantiationService.stubPromise(extensionManagement_1.IExtensionManagementService, 'getExtensionsReport', []);
        instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage());
        instantiationService.stubPromise(experimentService_1.IExperimentService, 'getExperimentsByType', []);
        instantiationService.stub(extensions_2.IExtensionService, {
            getExtensions: () => {
                return Promise.resolve([
                    toExtensionDescription(localEnabledTheme),
                    toExtensionDescription(localEnabledLanguage),
                    toExtensionDescription(localRandom),
                    toExtensionDescription(builtInTheme),
                    toExtensionDescription(builtInBasic)
                ]);
            }
        });
        yield instantiationService.get(extensionManagement_2.IExtensionEnablementService).setEnablement([localDisabledTheme], 2 /* DisabledGlobally */);
        yield instantiationService.get(extensionManagement_2.IExtensionEnablementService).setEnablement([localDisabledLanguage], 2 /* DisabledGlobally */);
        instantiationService.set(extensions_1.IExtensionsWorkbenchService, instantiationService.createInstance(extensionsWorkbenchService_1.ExtensionsWorkbenchService));
        testableView = instantiationService.createInstance(extensionsViews_1.ExtensionsListView, {});
    }));
    teardown(() => {
        instantiationService.get(extensions_1.IExtensionsWorkbenchService).dispose();
        testableView.dispose();
    });
    test('Test query types', () => {
        assert.equal(extensionsViews_1.ExtensionsListView.isBuiltInExtensionsQuery('@builtin'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@installed'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@enabled'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@disabled'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@outdated'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@installed searchText'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@enabled searchText'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@disabled searchText'), true);
        assert.equal(extensionsViews_1.ExtensionsListView.isLocalExtensionsQuery('@outdated searchText'), true);
    });
    test('Test empty query equates to sort by install count', () => {
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage());
        return testableView.show('').then(() => {
            assert.ok(target.calledOnce);
            const options = target.args[0][0];
            assert.equal(options.sortBy, 4 /* InstallCount */);
        });
    });
    test('Test non empty query without sort doesnt use sortBy', () => {
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage());
        return testableView.show('some extension').then(() => {
            assert.ok(target.calledOnce);
            const options = target.args[0][0];
            assert.equal(options.sortBy, undefined);
        });
    });
    test('Test query with sort uses sortBy', () => {
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage());
        return testableView.show('some extension @sort:rating').then(() => {
            assert.ok(target.calledOnce);
            const options = target.args[0][0];
            assert.equal(options.sortBy, 12 /* WeightedRating */);
        });
    });
    test('Test installed query results', () => __awaiter(void 0, void 0, void 0, function* () {
        yield testableView.show('@installed').then(result => {
            assert.equal(result.length, 5, 'Unexpected number of results for @installed query');
            const actual = [result.get(0).name, result.get(1).name, result.get(2).name, result.get(3).name, result.get(4).name].sort();
            const expected = [localDisabledTheme.manifest.name, localEnabledTheme.manifest.name, localRandom.manifest.name, localDisabledLanguage.manifest.name, localEnabledLanguage.manifest.name];
            for (let i = 0; i < result.length; i++) {
                assert.equal(actual[i], expected[i], 'Unexpected extension for @installed query.');
            }
        });
        yield testableView.show('@installed first').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @installed query');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @installed query with search text.');
            assert.equal(result.get(1).name, localDisabledTheme.manifest.name, 'Unexpected extension for @installed query with search text.');
        });
        yield testableView.show('@disabled').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @disabled query');
            assert.equal(result.get(0).name, localDisabledTheme.manifest.name, 'Unexpected extension for @disabled query.');
            assert.equal(result.get(1).name, localDisabledLanguage.manifest.name, 'Unexpected extension for @disabled query.');
        });
        yield testableView.show('@enabled').then(result => {
            assert.equal(result.length, 3, 'Unexpected number of results for @enabled query');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @enabled query.');
            assert.equal(result.get(1).name, localRandom.manifest.name, 'Unexpected extension for @enabled query.');
            assert.equal(result.get(2).name, localEnabledLanguage.manifest.name, 'Unexpected extension for @enabled query.');
        });
        yield testableView.show('@builtin:themes').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @builtin:themes query');
            assert.equal(result.get(0).name, builtInTheme.manifest.name, 'Unexpected extension for @builtin:themes query.');
        });
        yield testableView.show('@builtin:basics').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @builtin:basics query');
            assert.equal(result.get(0).name, builtInBasic.manifest.name, 'Unexpected extension for @builtin:basics query.');
        });
        yield testableView.show('@builtin').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @builtin query');
            assert.equal(result.get(0).name, builtInBasic.manifest.name, 'Unexpected extension for @builtin query.');
            assert.equal(result.get(1).name, builtInTheme.manifest.name, 'Unexpected extension for @builtin query.');
        });
        yield testableView.show('@builtin my-theme').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @builtin query');
            assert.equal(result.get(0).name, builtInTheme.manifest.name, 'Unexpected extension for @builtin query.');
        });
    }));
    test('Test installed query with category', () => __awaiter(void 0, void 0, void 0, function* () {
        yield testableView.show('@installed category:themes').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @installed query with category');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @installed query with category.');
            assert.equal(result.get(1).name, localDisabledTheme.manifest.name, 'Unexpected extension for @installed query with category.');
        });
        yield testableView.show('@installed category:"themes"').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @installed query with quoted category');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @installed query with quoted category.');
            assert.equal(result.get(1).name, localDisabledTheme.manifest.name, 'Unexpected extension for @installed query with quoted category.');
        });
        yield testableView.show('@installed category:"programming languages"').then(result => {
            assert.equal(result.length, 2, 'Unexpected number of results for @installed query with quoted category including space');
            assert.equal(result.get(0).name, localEnabledLanguage.manifest.name, 'Unexpected extension for @installed query with quoted category including space.');
            assert.equal(result.get(1).name, localDisabledLanguage.manifest.name, 'Unexpected extension for @installed query with quoted category inlcuding space.');
        });
        yield testableView.show('@installed category:themes category:random').then(result => {
            assert.equal(result.length, 3, 'Unexpected number of results for @installed query with multiple category');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @installed query with multiple category.');
            assert.equal(result.get(1).name, localRandom.manifest.name, 'Unexpected extension for @installed query with multiple category.');
            assert.equal(result.get(2).name, localDisabledTheme.manifest.name, 'Unexpected extension for @installed query with multiple category.');
        });
        yield testableView.show('@enabled category:themes').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @enabled query with category');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @enabled query with category.');
        });
        yield testableView.show('@enabled category:"themes"').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @enabled query with quoted category');
            assert.equal(result.get(0).name, localEnabledTheme.manifest.name, 'Unexpected extension for @enabled query with quoted category.');
        });
        yield testableView.show('@enabled category:"programming languages"').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @enabled query with quoted category inlcuding space');
            assert.equal(result.get(0).name, localEnabledLanguage.manifest.name, 'Unexpected extension for @enabled query with quoted category including space.');
        });
        yield testableView.show('@disabled category:themes').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @disabled query with category');
            assert.equal(result.get(0).name, localDisabledTheme.manifest.name, 'Unexpected extension for @disabled query with category.');
        });
        yield testableView.show('@disabled category:"themes"').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @disabled query with quoted category');
            assert.equal(result.get(0).name, localDisabledTheme.manifest.name, 'Unexpected extension for @disabled query with quoted category.');
        });
        yield testableView.show('@disabled category:"programming languages"').then(result => {
            assert.equal(result.length, 1, 'Unexpected number of results for @disabled query with quoted category inlcuding space');
            assert.equal(result.get(0).name, localDisabledLanguage.manifest.name, 'Unexpected extension for @disabled query with quoted category including space.');
        });
    }));
    test('Test @recommended:workspace query', () => {
        const workspaceRecommendedExtensions = [
            workspaceRecommendationA,
            workspaceRecommendationB
        ];
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...workspaceRecommendedExtensions));
        return testableView.show('@recommended:workspace').then(result => {
            assert.ok(target.calledOnce);
            const options = target.args[0][0];
            assert.equal(options.names.length, workspaceRecommendedExtensions.length);
            assert.equal(result.length, workspaceRecommendedExtensions.length);
            for (let i = 0; i < workspaceRecommendedExtensions.length; i++) {
                assert.equal(options.names[i], workspaceRecommendedExtensions[i].identifier.id);
                assert.equal(result.get(i).identifier.id, workspaceRecommendedExtensions[i].identifier.id);
            }
        });
    });
    test('Test @recommended query', () => {
        const allRecommendedExtensions = [
            fileBasedRecommendationA,
            fileBasedRecommendationB,
            otherRecommendationA
        ];
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...allRecommendedExtensions));
        return testableView.show('@recommended').then(result => {
            const options = target.args[0][0];
            assert.ok(target.calledOnce);
            assert.equal(options.names.length, allRecommendedExtensions.length);
            assert.equal(result.length, allRecommendedExtensions.length);
            for (let i = 0; i < allRecommendedExtensions.length; i++) {
                assert.equal(options.names[i], allRecommendedExtensions[i].identifier.id);
                assert.equal(result.get(i).identifier.id, allRecommendedExtensions[i].identifier.id);
            }
        });
    });
    test('Test @recommended:all query', () => {
        const allRecommendedExtensions = [
            workspaceRecommendationA,
            workspaceRecommendationB,
            fileBasedRecommendationA,
            fileBasedRecommendationB,
            otherRecommendationA
        ];
        const target = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...allRecommendedExtensions));
        return testableView.show('@recommended:all').then(result => {
            const options = target.args[0][0];
            assert.ok(target.calledOnce);
            assert.equal(options.names.length, allRecommendedExtensions.length);
            assert.equal(result.length, allRecommendedExtensions.length);
            for (let i = 0; i < allRecommendedExtensions.length; i++) {
                assert.equal(options.names[i], allRecommendedExtensions[i].identifier.id);
                assert.equal(result.get(i).identifier.id, allRecommendedExtensions[i].identifier.id);
            }
        });
    });
    test('Test curated list experiment', () => {
        const curatedList = [
            workspaceRecommendationA,
            fileBasedRecommendationA
        ];
        const experimentTarget = instantiationService.stubPromise(experimentService_1.IExperimentService, 'getCuratedExtensionsList', curatedList.map(e => e.identifier.id));
        const queryTarget = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...curatedList));
        return testableView.show('curated:mykey').then(result => {
            const curatedKey = experimentTarget.args[0][0];
            const options = queryTarget.args[0][0];
            assert.ok(experimentTarget.calledOnce);
            assert.ok(queryTarget.calledOnce);
            assert.equal(options.names.length, curatedList.length);
            assert.equal(result.length, curatedList.length);
            for (let i = 0; i < curatedList.length; i++) {
                assert.equal(options.names[i], curatedList[i].identifier.id);
                assert.equal(result.get(i).identifier.id, curatedList[i].identifier.id);
            }
            assert.equal(curatedKey, 'mykey');
        });
    });
    test('Test search', () => {
        const searchText = 'search-me';
        const results = [
            fileBasedRecommendationA,
            workspaceRecommendationA,
            otherRecommendationA,
            workspaceRecommendationB
        ];
        const queryTarget = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...results));
        return testableView.show('search-me').then(result => {
            const options = queryTarget.args[0][0];
            assert.ok(queryTarget.calledOnce);
            assert.equal(options.text, searchText);
            assert.equal(result.length, results.length);
            for (let i = 0; i < results.length; i++) {
                assert.equal(result.get(i).identifier.id, results[i].identifier.id);
            }
        });
    });
    test('Test preferred search experiment', () => {
        const searchText = 'search-me';
        const actual = [
            fileBasedRecommendationA,
            workspaceRecommendationA,
            otherRecommendationA,
            workspaceRecommendationB
        ];
        const expected = [
            workspaceRecommendationA,
            workspaceRecommendationB,
            fileBasedRecommendationA,
            otherRecommendationA
        ];
        const queryTarget = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...actual));
        const experimentTarget = instantiationService.stubPromise(experimentService_1.IExperimentService, 'getExperimentsByType', [{
                id: 'someId',
                enabled: true,
                state: 2 /* Run */,
                action: {
                    type: experimentService_1.ExperimentActionType.ExtensionSearchResults,
                    properties: {
                        searchText: 'search-me',
                        preferredResults: [
                            workspaceRecommendationA.identifier.id,
                            'something-that-wasnt-in-first-page',
                            workspaceRecommendationB.identifier.id
                        ]
                    }
                }
            }]);
        testableView.dispose();
        testableView = instantiationService.createInstance(extensionsViews_1.ExtensionsListView, {});
        return testableView.show('search-me').then(result => {
            const options = queryTarget.args[0][0];
            assert.ok(experimentTarget.calledOnce);
            assert.ok(queryTarget.calledOnce);
            assert.equal(options.text, searchText);
            assert.equal(result.length, expected.length);
            for (let i = 0; i < expected.length; i++) {
                assert.equal(result.get(i).identifier.id, expected[i].identifier.id);
            }
        });
    });
    test('Skip preferred search experiment when user defines sort order', () => {
        const searchText = 'search-me';
        const realResults = [
            fileBasedRecommendationA,
            workspaceRecommendationA,
            otherRecommendationA,
            workspaceRecommendationB
        ];
        const queryTarget = instantiationService.stubPromise(extensionManagement_1.IExtensionGalleryService, 'query', aPage(...realResults));
        testableView.dispose();
        testableView = instantiationService.createInstance(extensionsViews_1.ExtensionsListView, {});
        return testableView.show('search-me @sort:installs').then(result => {
            const options = queryTarget.args[0][0];
            assert.ok(queryTarget.calledOnce);
            assert.equal(options.text, searchText);
            assert.equal(result.length, realResults.length);
            for (let i = 0; i < realResults.length; i++) {
                assert.equal(result.get(i).identifier.id, realResults[i].identifier.id);
            }
        });
    });
    function aLocalExtension(name = 'someext', manifest = {}, properties = {}) {
        manifest = objects_1.assign({ name, publisher: 'pub', version: '1.0.0' }, manifest);
        properties = objects_1.assign({
            type: 1 /* User */,
            location: uri_1.URI.file(`pub.${name}`),
            identifier: { id: extensionManagementUtil_1.getGalleryExtensionId(manifest.publisher, manifest.name), uuid: undefined },
            metadata: { id: extensionManagementUtil_1.getGalleryExtensionId(manifest.publisher, manifest.name), publisherId: manifest.publisher, publisherDisplayName: 'somename' }
        }, properties);
        return Object.create(Object.assign({ manifest }, properties));
    }
    function aGalleryExtension(name, properties = {}, galleryExtensionProperties = {}, assets = {}) {
        const galleryExtension = Object.create({});
        objects_1.assign(galleryExtension, { name, publisher: 'pub', version: '1.0.0', properties: {}, assets: {} }, properties);
        objects_1.assign(galleryExtension.properties, { dependencies: [] }, galleryExtensionProperties);
        objects_1.assign(galleryExtension.assets, assets);
        galleryExtension.identifier = { id: extensionManagementUtil_1.getGalleryExtensionId(galleryExtension.publisher, galleryExtension.name), uuid: uuid_1.generateUuid() };
        return galleryExtension;
    }
    function aPage(...objects) {
        return { firstPage: objects, total: objects.length, pageSize: objects.length, getPage: () => null };
    }
    function toExtensionDescription(local) {
        return Object.assign({ identifier: new extensions_3.ExtensionIdentifier(local.identifier.id), isBuiltin: local.type === 0 /* System */, isUnderDevelopment: false, extensionLocation: local.location }, local.manifest);
    }
});
//# sourceMappingURL=extensionsViews.test.js.map