"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const errorMessage_1 = require("vs/base/common/errorMessage");
const resources_1 = require("vs/base/common/resources");
const actions_1 = require("vs/base/common/actions");
const uri_1 = require("vs/base/common/uri");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const lifecycle_1 = require("vs/base/common/lifecycle");
const textFileEditorModel_1 = require("vs/workbench/services/textfile/common/textFileEditorModel");
const resolverService_1 = require("vs/editor/common/services/resolverService");
const map_1 = require("vs/base/common/map");
const diffEditorInput_1 = require("vs/workbench/common/editor/diffEditorInput");
const resourceEditorInput_1 = require("vs/workbench/common/editor/resourceEditorInput");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const files_1 = require("vs/workbench/contrib/files/common/files");
const fileEditorInput_1 = require("vs/workbench/contrib/files/common/editors/fileEditorInput");
const modelService_1 = require("vs/editor/common/services/modelService");
const fileCommands_1 = require("vs/workbench/contrib/files/browser/fileCommands");
const textModel_1 = require("vs/editor/common/model/textModel");
const notification_1 = require("vs/platform/notification/common/notification");
const opener_1 = require("vs/platform/opener/common/opener");
const storage_1 = require("vs/platform/storage/common/storage");
const actions_2 = require("vs/platform/actions/common/actions");
const environment_1 = require("vs/platform/environment/common/environment");
const event_1 = require("vs/base/common/event");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const platform_1 = require("vs/base/common/platform");
const network_1 = require("vs/base/common/network");
exports.CONFLICT_RESOLUTION_CONTEXT = 'saveConflictResolutionContext';
exports.CONFLICT_RESOLUTION_SCHEME = 'conflictResolution';
const LEARN_MORE_DIRTY_WRITE_IGNORE_KEY = 'learnMoreDirtyWriteError';
const conflictEditorHelp = nls.localize('userGuide', "Use the actions in the editor tool bar to either undo your changes or overwrite the content of the file with your changes.");
// A handler for save error happening with conflict resolution actions
let SaveErrorHandler = class SaveErrorHandler extends lifecycle_1.Disposable {
    constructor(notificationService, textFileService, contextKeyService, editorService, textModelService, instantiationService, storageService) {
        super();
        this.notificationService = notificationService;
        this.textFileService = textFileService;
        this.editorService = editorService;
        this.instantiationService = instantiationService;
        this.storageService = storageService;
        this.messages = new map_1.ResourceMap();
        this.conflictResolutionContext = new contextkey_1.RawContextKey(exports.CONFLICT_RESOLUTION_CONTEXT, false).bindTo(contextKeyService);
        const provider = this._register(instantiationService.createInstance(files_1.TextFileContentProvider));
        this._register(textModelService.registerTextModelContentProvider(exports.CONFLICT_RESOLUTION_SCHEME, provider));
        // Hook into model
        textFileEditorModel_1.TextFileEditorModel.setSaveErrorHandler(this);
        this.registerListeners();
    }
    registerListeners() {
        this._register(this.textFileService.models.onModelSaved(e => this.onFileSavedOrReverted(e.resource)));
        this._register(this.textFileService.models.onModelReverted(e => this.onFileSavedOrReverted(e.resource)));
        this._register(this.editorService.onDidActiveEditorChange(() => this.onActiveEditorChanged()));
    }
    onActiveEditorChanged() {
        let isActiveEditorSaveConflictResolution = false;
        let activeConflictResolutionResource;
        const activeInput = this.editorService.activeEditor;
        if (activeInput instanceof diffEditorInput_1.DiffEditorInput && activeInput.originalInput instanceof resourceEditorInput_1.ResourceEditorInput && activeInput.modifiedInput instanceof fileEditorInput_1.FileEditorInput) {
            const resource = activeInput.originalInput.getResource();
            if (resource && resource.scheme === exports.CONFLICT_RESOLUTION_SCHEME) {
                isActiveEditorSaveConflictResolution = true;
                activeConflictResolutionResource = activeInput.modifiedInput.getResource();
            }
        }
        this.conflictResolutionContext.set(isActiveEditorSaveConflictResolution);
        this.activeConflictResolutionResource = activeConflictResolutionResource;
    }
    onFileSavedOrReverted(resource) {
        const messageHandle = this.messages.get(resource);
        if (messageHandle) {
            messageHandle.close();
            this.messages.delete(resource);
        }
    }
    onSaveError(error, model) {
        const fileOperationError = error;
        const resource = model.getResource();
        let message;
        const primaryActions = [];
        const secondaryActions = [];
        // Dirty write prevention
        if (fileOperationError.fileOperationResult === 3 /* FILE_MODIFIED_SINCE */) {
            // If the user tried to save from the opened conflict editor, show its message again
            if (this.activeConflictResolutionResource && this.activeConflictResolutionResource.toString() === model.getResource().toString()) {
                if (this.storageService.getBoolean(LEARN_MORE_DIRTY_WRITE_IGNORE_KEY, 0 /* GLOBAL */)) {
                    return; // return if this message is ignored
                }
                message = conflictEditorHelp;
                primaryActions.push(this.instantiationService.createInstance(ResolveConflictLearnMoreAction));
                secondaryActions.push(this.instantiationService.createInstance(DoNotShowResolveConflictLearnMoreAction));
            }
            // Otherwise show the message that will lead the user into the save conflict editor.
            else {
                message = nls.localize('staleSaveError', "Failed to save '{0}': The content of the file is newer. Please compare your version with the file contents.", resources_1.basename(resource));
                primaryActions.push(this.instantiationService.createInstance(ResolveSaveConflictAction, model));
            }
        }
        // Any other save error
        else {
            const isReadonly = fileOperationError.fileOperationResult === 5 /* FILE_READ_ONLY */;
            const triedToMakeWriteable = isReadonly && fileOperationError.options && fileOperationError.options.overwriteReadonly;
            const isPermissionDenied = fileOperationError.fileOperationResult === 6 /* FILE_PERMISSION_DENIED */;
            const canHandlePermissionOrReadonlyErrors = resource.scheme === network_1.Schemas.file; // https://github.com/Microsoft/vscode/issues/48659
            // Save Elevated
            if (canHandlePermissionOrReadonlyErrors && (isPermissionDenied || triedToMakeWriteable)) {
                primaryActions.push(this.instantiationService.createInstance(SaveElevatedAction, model, triedToMakeWriteable));
            }
            // Overwrite
            else if (canHandlePermissionOrReadonlyErrors && isReadonly) {
                primaryActions.push(this.instantiationService.createInstance(OverwriteReadonlyAction, model));
            }
            // Retry
            else {
                primaryActions.push(this.instantiationService.createInstance(actions_2.ExecuteCommandAction, fileCommands_1.SAVE_FILE_COMMAND_ID, nls.localize('retry', "Retry")));
            }
            // Save As
            primaryActions.push(this.instantiationService.createInstance(actions_2.ExecuteCommandAction, fileCommands_1.SAVE_FILE_AS_COMMAND_ID, fileCommands_1.SAVE_FILE_AS_LABEL));
            // Discard
            primaryActions.push(this.instantiationService.createInstance(actions_2.ExecuteCommandAction, fileCommands_1.REVERT_FILE_COMMAND_ID, nls.localize('discard', "Discard")));
            // Message
            if (canHandlePermissionOrReadonlyErrors && isReadonly) {
                if (triedToMakeWriteable) {
                    message = platform_1.isWindows ? nls.localize('readonlySaveErrorAdmin', "Failed to save '{0}': File is read-only. Select 'Overwrite as Admin' to retry as administrator.", resources_1.basename(resource)) : nls.localize('readonlySaveErrorSudo', "Failed to save '{0}': File is read-only. Select 'Overwrite as Sudo' to retry as superuser.", resources_1.basename(resource));
                }
                else {
                    message = nls.localize('readonlySaveError', "Failed to save '{0}': File is read-only. Select 'Overwrite' to attempt to make it writeable.", resources_1.basename(resource));
                }
            }
            else if (canHandlePermissionOrReadonlyErrors && isPermissionDenied) {
                message = platform_1.isWindows ? nls.localize('permissionDeniedSaveError', "Failed to save '{0}': Insufficient permissions. Select 'Retry as Admin' to retry as administrator.", resources_1.basename(resource)) : nls.localize('permissionDeniedSaveErrorSudo', "Failed to save '{0}': Insufficient permissions. Select 'Retry as Sudo' to retry as superuser.", resources_1.basename(resource));
            }
            else {
                message = nls.localize('genericSaveError', "Failed to save '{0}': {1}", resources_1.basename(resource), errorMessage_1.toErrorMessage(error, false));
            }
        }
        // Show message and keep function to hide in case the file gets saved/reverted
        const actions = { primary: primaryActions, secondary: secondaryActions };
        const handle = this.notificationService.notify({ severity: notification_1.Severity.Error, message, actions });
        event_1.Event.once(handle.onDidClose)(() => { lifecycle_1.dispose(primaryActions), lifecycle_1.dispose(secondaryActions); });
        this.messages.set(model.getResource(), handle);
    }
    dispose() {
        super.dispose();
        this.messages.clear();
    }
};
SaveErrorHandler = __decorate([
    __param(0, notification_1.INotificationService),
    __param(1, textfiles_1.ITextFileService),
    __param(2, contextkey_1.IContextKeyService),
    __param(3, editorService_1.IEditorService),
    __param(4, resolverService_1.ITextModelService),
    __param(5, instantiation_1.IInstantiationService),
    __param(6, storage_1.IStorageService)
], SaveErrorHandler);
exports.SaveErrorHandler = SaveErrorHandler;
const pendingResolveSaveConflictMessages = [];
function clearPendingResolveSaveConflictMessages() {
    while (pendingResolveSaveConflictMessages.length > 0) {
        const item = pendingResolveSaveConflictMessages.pop();
        if (item) {
            item.close();
        }
    }
}
let ResolveConflictLearnMoreAction = class ResolveConflictLearnMoreAction extends actions_1.Action {
    constructor(openerService) {
        super('workbench.files.action.resolveConflictLearnMore', nls.localize('learnMore', "Learn More"));
        this.openerService = openerService;
    }
    run() {
        return this.openerService.open(uri_1.URI.parse('https://go.microsoft.com/fwlink/?linkid=868264'));
    }
};
ResolveConflictLearnMoreAction = __decorate([
    __param(0, opener_1.IOpenerService)
], ResolveConflictLearnMoreAction);
let DoNotShowResolveConflictLearnMoreAction = class DoNotShowResolveConflictLearnMoreAction extends actions_1.Action {
    constructor(storageService) {
        super('workbench.files.action.resolveConflictLearnMoreDoNotShowAgain', nls.localize('dontShowAgain', "Don't Show Again"));
        this.storageService = storageService;
    }
    run(notification) {
        this.storageService.store(LEARN_MORE_DIRTY_WRITE_IGNORE_KEY, true, 0 /* GLOBAL */);
        // Hide notification
        notification.dispose();
        return Promise.resolve();
    }
};
DoNotShowResolveConflictLearnMoreAction = __decorate([
    __param(0, storage_1.IStorageService)
], DoNotShowResolveConflictLearnMoreAction);
let ResolveSaveConflictAction = class ResolveSaveConflictAction extends actions_1.Action {
    constructor(model, editorService, notificationService, instantiationService, environmentService) {
        super('workbench.files.action.resolveConflict', nls.localize('compareChanges', "Compare"));
        this.model = model;
        this.editorService = editorService;
        this.notificationService = notificationService;
        this.instantiationService = instantiationService;
        this.environmentService = environmentService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.model.isDisposed()) {
                const resource = this.model.getResource();
                const name = resources_1.basename(resource);
                const editorLabel = nls.localize('saveConflictDiffLabel', "{0} (in file) ↔ {1} (in {2}) - Resolve save conflict", name, name, this.environmentService.appNameLong);
                yield files_1.TextFileContentProvider.open(resource, exports.CONFLICT_RESOLUTION_SCHEME, editorLabel, this.editorService, { pinned: true });
                // Show additional help how to resolve the save conflict
                const actions = { primary: [this.instantiationService.createInstance(ResolveConflictLearnMoreAction)] };
                const handle = this.notificationService.notify({
                    severity: notification_1.Severity.Info,
                    message: conflictEditorHelp,
                    actions,
                    neverShowAgain: { id: LEARN_MORE_DIRTY_WRITE_IGNORE_KEY, isSecondary: true }
                });
                event_1.Event.once(handle.onDidClose)(() => lifecycle_1.dispose(actions.primary));
                pendingResolveSaveConflictMessages.push(handle);
            }
            return Promise.resolve(true);
        });
    }
};
ResolveSaveConflictAction = __decorate([
    __param(1, editorService_1.IEditorService),
    __param(2, notification_1.INotificationService),
    __param(3, instantiation_1.IInstantiationService),
    __param(4, environment_1.IEnvironmentService)
], ResolveSaveConflictAction);
class SaveElevatedAction extends actions_1.Action {
    constructor(model, triedToMakeWriteable) {
        super('workbench.files.action.saveElevated', triedToMakeWriteable ? platform_1.isWindows ? nls.localize('overwriteElevated', "Overwrite as Admin...") : nls.localize('overwriteElevatedSudo', "Overwrite as Sudo...") : platform_1.isWindows ? nls.localize('saveElevated', "Retry as Admin...") : nls.localize('saveElevatedSudo', "Retry as Sudo..."));
        this.model = model;
        this.triedToMakeWriteable = triedToMakeWriteable;
    }
    run() {
        if (!this.model.isDisposed()) {
            this.model.save({
                writeElevated: true,
                overwriteReadonly: this.triedToMakeWriteable
            });
        }
        return Promise.resolve(true);
    }
}
class OverwriteReadonlyAction extends actions_1.Action {
    constructor(model) {
        super('workbench.files.action.overwrite', nls.localize('overwrite', "Overwrite"));
        this.model = model;
    }
    run() {
        if (!this.model.isDisposed()) {
            this.model.save({ overwriteReadonly: true });
        }
        return Promise.resolve(true);
    }
}
exports.acceptLocalChangesCommand = (accessor, resource) => {
    const editorService = accessor.get(editorService_1.IEditorService);
    const resolverService = accessor.get(resolverService_1.ITextModelService);
    const modelService = accessor.get(modelService_1.IModelService);
    const control = editorService.activeControl;
    if (!control) {
        return;
    }
    const editor = control.input;
    const group = control.group;
    resolverService.createModelReference(resource).then((reference) => __awaiter(void 0, void 0, void 0, function* () {
        const model = reference.object;
        const localModelSnapshot = model.createSnapshot();
        clearPendingResolveSaveConflictMessages(); // hide any previously shown message about how to use these actions
        // Revert to be able to save
        yield model.revert();
        // Restore user value (without loosing undo stack)
        modelService.updateModel(model.textEditorModel, textModel_1.createTextBufferFactoryFromSnapshot(localModelSnapshot));
        // Trigger save
        yield model.save();
        // Reopen file input
        yield editorService.openEditor({ resource: model.getResource() }, group);
        // Clean up
        group.closeEditor(editor);
        editor.dispose();
        reference.dispose();
    }));
};
exports.revertLocalChangesCommand = (accessor, resource) => {
    const editorService = accessor.get(editorService_1.IEditorService);
    const resolverService = accessor.get(resolverService_1.ITextModelService);
    const control = editorService.activeControl;
    if (!control) {
        return;
    }
    const editor = control.input;
    const group = control.group;
    resolverService.createModelReference(resource).then((reference) => __awaiter(void 0, void 0, void 0, function* () {
        const model = reference.object;
        clearPendingResolveSaveConflictMessages(); // hide any previously shown message about how to use these actions
        // Revert on model
        yield model.revert();
        // Reopen file input
        yield editorService.openEditor({ resource: model.getResource() }, group);
        // Clean up
        group.closeEditor(editor);
        editor.dispose();
        reference.dispose();
    }));
};
//# sourceMappingURL=saveErrorHandler.js.map