"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const modes_1 = require("vs/editor/common/modes");
const nls = require("vs/nls");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const quickInput_1 = require("vs/platform/quickinput/common/quickInput");
const cancellation_1 = require("vs/base/common/cancellation");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const format_1 = require("vs/editor/contrib/format/format");
const range_1 = require("vs/editor/common/core/range");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const extensions_1 = require("vs/platform/extensions/common/extensions");
const platform_1 = require("vs/platform/registry/common/platform");
const configurationRegistry_1 = require("vs/platform/configuration/common/configurationRegistry");
const contributions_1 = require("vs/workbench/common/contributions");
const extensions_2 = require("vs/workbench/services/extensions/common/extensions");
const lifecycle_1 = require("vs/base/common/lifecycle");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const notification_1 = require("vs/platform/notification/common/notification");
const modeService_1 = require("vs/editor/common/services/modeService");
const label_1 = require("vs/platform/label/common/label");
const extensionManagement_1 = require("vs/workbench/services/extensionManagement/common/extensionManagement");
let DefaultFormatter = class DefaultFormatter extends lifecycle_1.Disposable {
    constructor(_extensionService, _extensionEnablementService, _configService, _notificationService, _quickInputService, _modeService, _labelService) {
        super();
        this._extensionService = _extensionService;
        this._extensionEnablementService = _extensionEnablementService;
        this._configService = _configService;
        this._notificationService = _notificationService;
        this._quickInputService = _quickInputService;
        this._modeService = _modeService;
        this._labelService = _labelService;
        this._register(this._extensionService.onDidChangeExtensions(this._updateConfigValues, this));
        this._register(format_1.FormattingConflicts.setFormatterSelector((formatter, document, mode) => this._selectFormatter(formatter, document, mode)));
        this._updateConfigValues();
    }
    _updateConfigValues() {
        return __awaiter(this, void 0, void 0, function* () {
            const extensions = yield this._extensionService.getExtensions();
            DefaultFormatter.extensionIds.length = 0;
            DefaultFormatter.extensionDescriptions.length = 0;
            DefaultFormatter.extensionIds.push(null);
            DefaultFormatter.extensionDescriptions.push(nls.localize('nullFormatterDescription', "None"));
            for (const extension of extensions) {
                if (extension.main) {
                    DefaultFormatter.extensionIds.push(extension.identifier.value);
                    DefaultFormatter.extensionDescriptions.push(extension.description || '');
                }
            }
        });
    }
    static _maybeQuotes(s) {
        return s.match(/\s/) ? `'${s}'` : s;
    }
    _selectFormatter(formatter, document, mode) {
        return __awaiter(this, void 0, void 0, function* () {
            const defaultFormatterId = this._configService.getValue(DefaultFormatter.configName, {
                resource: document.uri,
                overrideIdentifier: document.getModeId()
            });
            if (defaultFormatterId) {
                // good -> formatter configured
                const [defaultFormatter] = formatter.filter(formatter => extensions_1.ExtensionIdentifier.equals(formatter.extensionId, defaultFormatterId));
                if (defaultFormatter) {
                    // formatter available
                    return defaultFormatter;
                }
                // bad -> formatter gone
                const extension = yield this._extensionService.getExtension(defaultFormatterId);
                if (extension && this._extensionEnablementService.isEnabled(extensions_2.toExtension(extension))) {
                    // formatter does not target this file
                    const label = this._labelService.getUriLabel(document.uri, { relative: true });
                    const message = nls.localize('miss', "Extension '{0}' cannot format '{1}'", extension.displayName || extension.name, label);
                    this._notificationService.status(message, { hideAfter: 4000 });
                    return undefined;
                }
            }
            else if (formatter.length === 1) {
                // ok -> nothing configured but only one formatter available
                return formatter[0];
            }
            const langName = this._modeService.getLanguageName(document.getModeId()) || document.getModeId();
            const silent = mode === 2 /* Silent */;
            const message = !defaultFormatterId
                ? nls.localize('config.needed', "There are multiple formatters for {0}-files. Select a default formatter to continue.", DefaultFormatter._maybeQuotes(langName))
                : nls.localize('config.bad', "Extension '{0}' is configured as formatter but not available. Select a different default formatter to continue.", defaultFormatterId);
            return new Promise((resolve, reject) => {
                this._notificationService.prompt(notification_1.Severity.Info, message, [{ label: nls.localize('do.config', "Configure..."), run: () => this._pickAndPersistDefaultFormatter(formatter, document).then(resolve, reject) }], { silent, onCancel: resolve });
                if (silent) {
                    // don't wait when formatting happens without interaction
                    // but pick some formatter...
                    resolve(formatter[0]);
                }
            });
        });
    }
    _pickAndPersistDefaultFormatter(formatter, document) {
        return __awaiter(this, void 0, void 0, function* () {
            const picks = formatter.map((formatter, index) => {
                return {
                    index,
                    label: formatter.displayName || formatter.extensionId || '?',
                    description: formatter.extensionId && formatter.extensionId.value
                };
            });
            const langName = this._modeService.getLanguageName(document.getModeId()) || document.getModeId();
            const pick = yield this._quickInputService.pick(picks, { placeHolder: nls.localize('select', "Select a default formatter for {0}-files", DefaultFormatter._maybeQuotes(langName)) });
            if (!pick || !formatter[pick.index].extensionId) {
                return undefined;
            }
            this._configService.updateValue(DefaultFormatter.configName, formatter[pick.index].extensionId.value, {
                resource: document.uri,
                overrideIdentifier: document.getModeId()
            });
            return formatter[pick.index];
        });
    }
};
DefaultFormatter.configName = 'editor.defaultFormatter';
DefaultFormatter.extensionIds = [];
DefaultFormatter.extensionDescriptions = [];
DefaultFormatter = __decorate([
    __param(0, extensions_2.IExtensionService),
    __param(1, extensionManagement_1.IExtensionEnablementService),
    __param(2, configuration_1.IConfigurationService),
    __param(3, notification_1.INotificationService),
    __param(4, quickInput_1.IQuickInputService),
    __param(5, modeService_1.IModeService),
    __param(6, label_1.ILabelService)
], DefaultFormatter);
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(DefaultFormatter, 3 /* Restored */);
platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration).registerConfiguration({
    id: 'editor',
    order: 5,
    type: 'object',
    overridable: true,
    properties: {
        [DefaultFormatter.configName]: {
            description: nls.localize('formatter.default', "Defines a default formatter which takes precedence over all other formatter settings. Must be the identifier of an extension contributing a formatter."),
            type: ['string', 'null'],
            default: null,
            enum: DefaultFormatter.extensionIds,
            markdownEnumDescriptions: DefaultFormatter.extensionDescriptions
        }
    }
});
function logFormatterTelemetry(telemetryService, mode, options, pick) {
    function extKey(obj) {
        return obj.extensionId ? extensions_1.ExtensionIdentifier.toKey(obj.extensionId) : 'unknown';
    }
    /*
     * __GDPR__
        "formatterpick" : {
            "mode" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
            "extensions" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
            "pick" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" }
        }
     */
    telemetryService.publicLog('formatterpick', {
        mode,
        extensions: options.map(extKey),
        pick: pick ? extKey(pick) : 'none'
    });
}
function showFormatterPick(accessor, model, formatters) {
    return __awaiter(this, void 0, void 0, function* () {
        const quickPickService = accessor.get(quickInput_1.IQuickInputService);
        const configService = accessor.get(configuration_1.IConfigurationService);
        const modeService = accessor.get(modeService_1.IModeService);
        const overrides = { resource: model.uri, overrideIdentifier: model.getModeId() };
        const defaultFormatter = configService.getValue(DefaultFormatter.configName, overrides);
        let defaultFormatterPick;
        const picks = formatters.map((provider, index) => {
            const isDefault = extensions_1.ExtensionIdentifier.equals(provider.extensionId, defaultFormatter);
            const pick = {
                index,
                label: provider.displayName || '',
                description: isDefault ? nls.localize('def', "(default)") : undefined,
            };
            if (isDefault) {
                // autofocus default pick
                defaultFormatterPick = pick;
            }
            return pick;
        });
        const configurePick = {
            label: nls.localize('config', "Configure Default Formatter...")
        };
        const pick = yield quickPickService.pick([...picks, { type: 'separator' }, configurePick], {
            placeHolder: nls.localize('format.placeHolder', "Select a formatter"),
            activeItem: defaultFormatterPick
        });
        if (!pick) {
            // dismissed
            return undefined;
        }
        else if (pick === configurePick) {
            // config default
            const langName = modeService.getLanguageName(model.getModeId()) || model.getModeId();
            const pick = yield quickPickService.pick(picks, { placeHolder: nls.localize('select', "Select a default formatter for {0}-files", DefaultFormatter._maybeQuotes(langName)) });
            if (pick && formatters[pick.index].extensionId) {
                configService.updateValue(DefaultFormatter.configName, formatters[pick.index].extensionId.value, overrides);
            }
            return undefined;
        }
        else {
            // picked one
            return pick.index;
        }
    });
}
editorExtensions_1.registerEditorAction(class FormatDocumentMultipleAction extends editorExtensions_1.EditorAction {
    constructor() {
        super({
            id: 'editor.action.formatDocument.multiple',
            label: nls.localize('formatDocument.label.multiple', "Format Document With..."),
            alias: 'Format Document...',
            precondition: contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.writable, editorContextKeys_1.EditorContextKeys.hasMultipleDocumentFormattingProvider),
            menuOpts: {
                group: '1_modification',
                order: 1.3
            }
        });
    }
    run(accessor, editor, args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!editor.hasModel()) {
                return;
            }
            const instaService = accessor.get(instantiation_1.IInstantiationService);
            const telemetryService = accessor.get(telemetry_1.ITelemetryService);
            const model = editor.getModel();
            const provider = format_1.getRealAndSyntheticDocumentFormattersOrdered(model);
            const pick = yield instaService.invokeFunction(showFormatterPick, model, provider);
            if (typeof pick === 'number') {
                yield instaService.invokeFunction(format_1.formatDocumentWithProvider, provider[pick], editor, 1 /* Explicit */, cancellation_1.CancellationToken.None);
            }
            logFormatterTelemetry(telemetryService, 'document', provider, typeof pick === 'number' && provider[pick] || undefined);
        });
    }
});
editorExtensions_1.registerEditorAction(class FormatSelectionMultipleAction extends editorExtensions_1.EditorAction {
    constructor() {
        super({
            id: 'editor.action.formatSelection.multiple',
            label: nls.localize('formatSelection.label.multiple', "Format Selection With..."),
            alias: 'Format Code...',
            precondition: contextkey_1.ContextKeyExpr.and(contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.writable), editorContextKeys_1.EditorContextKeys.hasMultipleDocumentSelectionFormattingProvider),
            menuOpts: {
                when: contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.hasNonEmptySelection),
                group: '1_modification',
                order: 1.31
            }
        });
    }
    run(accessor, editor) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!editor.hasModel()) {
                return;
            }
            const instaService = accessor.get(instantiation_1.IInstantiationService);
            const telemetryService = accessor.get(telemetry_1.ITelemetryService);
            const model = editor.getModel();
            let range = editor.getSelection();
            if (range.isEmpty()) {
                range = new range_1.Range(range.startLineNumber, 1, range.startLineNumber, model.getLineMaxColumn(range.startLineNumber));
            }
            const provider = modes_1.DocumentRangeFormattingEditProviderRegistry.ordered(model);
            const pick = yield instaService.invokeFunction(showFormatterPick, model, provider);
            if (typeof pick === 'number') {
                yield instaService.invokeFunction(format_1.formatDocumentRangeWithProvider, provider[pick], editor, range, cancellation_1.CancellationToken.None);
            }
            logFormatterTelemetry(telemetryService, 'range', provider, typeof pick === 'number' && provider[pick] || undefined);
        });
    }
});
//# sourceMappingURL=formatActionsMultiple.js.map