"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const modes_1 = require("vs/editor/common/modes");
const nls = require("vs/nls");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const commands_1 = require("vs/platform/commands/common/commands");
const viewlet_1 = require("vs/workbench/services/viewlet/browser/viewlet");
const notification_1 = require("vs/platform/notification/common/notification");
const showExtensionQuery_1 = require("vs/workbench/contrib/format/browser/showExtensionQuery");
editorExtensions_1.registerEditorAction(class FormatDocumentMultipleAction extends editorExtensions_1.EditorAction {
    constructor() {
        super({
            id: 'editor.action.formatDocument.none',
            label: nls.localize('formatDocument.label.multiple', "Format Document"),
            alias: 'Format Document',
            precondition: contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.writable, editorContextKeys_1.EditorContextKeys.hasDocumentFormattingProvider.toNegated()),
            kbOpts: {
                kbExpr: contextkey_1.ContextKeyExpr.and(editorContextKeys_1.EditorContextKeys.editorTextFocus, editorContextKeys_1.EditorContextKeys.hasDocumentFormattingProvider.toNegated()),
                primary: 1024 /* Shift */ | 512 /* Alt */ | 36 /* KEY_F */,
                linux: { primary: 2048 /* CtrlCmd */ | 1024 /* Shift */ | 39 /* KEY_I */ },
                weight: 100 /* EditorContrib */,
            }
        });
    }
    run(accessor, editor, args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!editor.hasModel()) {
                return;
            }
            const commandService = accessor.get(commands_1.ICommandService);
            const viewletService = accessor.get(viewlet_1.IViewletService);
            const notificationService = accessor.get(notification_1.INotificationService);
            const model = editor.getModel();
            const formatterCount = modes_1.DocumentFormattingEditProviderRegistry.all(model).length;
            if (formatterCount > 1) {
                return commandService.executeCommand('editor.action.formatDocument.multiple');
            }
            else if (formatterCount === 1) {
                return commandService.executeCommand('editor.action.formatDocument');
            }
            else {
                const langName = model.getLanguageIdentifier().language;
                const message = nls.localize('no.provider', "There is no formatter for '{0}'-files installed.", langName);
                const choice = {
                    label: nls.localize('install.formatter', "Install Formatter..."),
                    run: () => showExtensionQuery_1.showExtensionQuery(viewletService, `category:formatters ${langName}`)
                };
                notificationService.prompt(notification_1.Severity.Info, message, [choice]);
            }
        });
    }
});
//# sourceMappingURL=formatActionsNone.js.map