"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const files_1 = require("vs/platform/files/common/files");
const resources_1 = require("vs/base/common/resources");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const uri_1 = require("vs/base/common/uri");
const lifecycle_2 = require("vs/platform/lifecycle/common/lifecycle");
let LogsDataCleaner = class LogsDataCleaner extends lifecycle_1.Disposable {
    constructor(environmentService, fileService, lifecycleService) {
        super();
        this.environmentService = environmentService;
        this.fileService = fileService;
        this.lifecycleService = lifecycleService;
        this.cleanUpOldLogsSoon();
    }
    cleanUpOldLogsSoon() {
        let handle = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
            handle = undefined;
            const logsPath = uri_1.URI.file(this.environmentService.logsPath).with({ scheme: this.environmentService.logFile.scheme });
            const stat = yield this.fileService.resolve(resources_1.dirname(logsPath));
            if (stat.children) {
                const currentLog = resources_1.basename(logsPath);
                const allSessions = stat.children.filter(stat => stat.isDirectory && /^\d{8}T\d{6}$/.test(stat.name));
                const oldSessions = allSessions.sort().filter((d, i) => d.name !== currentLog);
                const toDelete = oldSessions.slice(0, Math.max(0, oldSessions.length - 49));
                Promise.all(toDelete.map(stat => this.fileService.del(stat.resource, { recursive: true })));
            }
        }), 10 * 1000);
        this.lifecycleService.onWillShutdown(() => {
            if (handle) {
                clearTimeout(handle);
                handle = undefined;
            }
        });
    }
};
LogsDataCleaner = __decorate([
    __param(0, environmentService_1.IWorkbenchEnvironmentService),
    __param(1, files_1.IFileService),
    __param(2, lifecycle_2.ILifecycleService)
], LogsDataCleaner);
exports.LogsDataCleaner = LogsDataCleaner;
//# sourceMappingURL=logsDataCleaner.js.map