"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const markersModel_1 = require("./markersModel");
const lifecycle_1 = require("vs/base/common/lifecycle");
const markers_1 = require("vs/platform/markers/common/markers");
const activity_1 = require("vs/workbench/services/activity/common/activity");
const nls_1 = require("vs/nls");
const constants_1 = require("./constants");
const arrays_1 = require("vs/base/common/arrays");
const event_1 = require("vs/base/common/event");
const map_1 = require("vs/base/common/map");
exports.IMarkersWorkbenchService = instantiation_1.createDecorator('markersWorkbenchService');
let MarkersWorkbenchService = class MarkersWorkbenchService extends lifecycle_1.Disposable {
    constructor(markerService, instantiationService) {
        super();
        this.markerService = markerService;
        this.markersModel = this._register(instantiationService.createInstance(markersModel_1.MarkersModel, this.readMarkers()));
        this.markersModel.setResourceMarkers(arrays_1.groupBy(this.readMarkers(), markersModel_1.compareMarkersByUri).map(group => [group[0].resource, group]));
        this._register(event_1.Event.debounce(markerService.onMarkerChanged, (resourcesMap, resources) => {
            resourcesMap = resourcesMap ? resourcesMap : new map_1.ResourceMap();
            resources.forEach(resource => resourcesMap.set(resource, resource));
            return resourcesMap;
        }, 0)(resourcesMap => this.onMarkerChanged(resourcesMap.values())));
    }
    onMarkerChanged(resources) {
        this.markersModel.setResourceMarkers(resources.map(resource => [resource, this.readMarkers(resource)]));
    }
    readMarkers(resource) {
        return this.markerService.read({ resource, severities: markers_1.MarkerSeverity.Error | markers_1.MarkerSeverity.Warning | markers_1.MarkerSeverity.Info });
    }
};
MarkersWorkbenchService = __decorate([
    __param(0, markers_1.IMarkerService),
    __param(1, instantiation_1.IInstantiationService)
], MarkersWorkbenchService);
exports.MarkersWorkbenchService = MarkersWorkbenchService;
let ActivityUpdater = class ActivityUpdater extends lifecycle_1.Disposable {
    constructor(activityService, markerService) {
        super();
        this.activityService = activityService;
        this.markerService = markerService;
        this.activity = this._register(new lifecycle_1.MutableDisposable());
        this._register(this.markerService.onMarkerChanged(() => this.updateBadge()));
        this.updateBadge();
    }
    updateBadge() {
        const { errors, warnings, infos } = this.markerService.getStatistics();
        const total = errors + warnings + infos;
        const message = nls_1.localize('totalProblems', 'Total {0} Problems', total);
        this.activity.value = this.activityService.showActivity(constants_1.default.MARKERS_PANEL_ID, new activity_1.NumberBadge(total, () => message));
    }
};
ActivityUpdater = __decorate([
    __param(0, activity_1.IActivityService),
    __param(1, markers_1.IMarkerService)
], ActivityUpdater);
exports.ActivityUpdater = ActivityUpdater;
//# sourceMappingURL=markers.js.map