"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls_1 = require("vs/nls");
const actions_1 = require("vs/platform/actions/common/actions");
const commands_1 = require("vs/platform/commands/common/commands");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const platform_1 = require("vs/platform/registry/common/platform");
const contributions_1 = require("vs/workbench/common/contributions");
const editor_1 = require("vs/workbench/common/editor");
const perfviewEditor_1 = require("vs/workbench/contrib/performance/electron-browser/perfviewEditor");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const startupProfiler_1 = require("./startupProfiler");
const startupTimings_1 = require("./startupTimings");
// -- startup performance view
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(perfviewEditor_1.PerfviewContrib, 2 /* Ready */);
platform_1.Registry.as(editor_1.Extensions.EditorInputFactories).registerEditorInputFactory(perfviewEditor_1.PerfviewInput.Id, class {
    serialize() {
        return '';
    }
    deserialize(instantiationService) {
        return instantiationService.createInstance(perfviewEditor_1.PerfviewInput);
    }
});
commands_1.CommandsRegistry.registerCommand('perfview.show', accessor => {
    const editorService = accessor.get(editorService_1.IEditorService);
    const instaService = accessor.get(instantiation_1.IInstantiationService);
    return editorService.openEditor(instaService.createInstance(perfviewEditor_1.PerfviewInput));
});
actions_1.MenuRegistry.addCommand({
    id: 'perfview.show',
    category: nls_1.localize('show.cat', "Developer"),
    title: nls_1.localize('show.label', "Startup Performance")
});
// -- startup profiler
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(startupProfiler_1.StartupProfiler, 3 /* Restored */);
// -- startup timings
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(startupTimings_1.StartupTimings, 4 /* Eventually */);
//# sourceMappingURL=performance.contribution.js.map