"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("vs/base/common/path");
const pfs_1 = require("vs/base/node/pfs");
const resolverService_1 = require("vs/editor/common/services/resolverService");
const nls_1 = require("vs/nls");
const dialogs_1 = require("vs/platform/dialogs/common/dialogs");
const environment_1 = require("vs/platform/environment/common/environment");
const lifecycle_1 = require("vs/platform/lifecycle/common/lifecycle");
const product_1 = require("vs/platform/product/common/product");
const perfviewEditor_1 = require("vs/workbench/contrib/performance/electron-browser/perfviewEditor");
const extensions_1 = require("vs/workbench/services/extensions/common/extensions");
const clipboardService_1 = require("vs/platform/clipboard/common/clipboardService");
const uri_1 = require("vs/base/common/uri");
const opener_1 = require("vs/platform/opener/common/opener");
const electron_1 = require("vs/platform/electron/node/electron");
let StartupProfiler = class StartupProfiler {
    constructor(_dialogService, _environmentService, _textModelResolverService, _clipboardService, lifecycleService, extensionService, _openerService, _electronService) {
        this._dialogService = _dialogService;
        this._environmentService = _environmentService;
        this._textModelResolverService = _textModelResolverService;
        this._clipboardService = _clipboardService;
        this._openerService = _openerService;
        this._electronService = _electronService;
        // wait for everything to be ready
        Promise.all([
            lifecycleService.when(4 /* Eventually */),
            extensionService.whenInstalledExtensionsRegistered()
        ]).then(() => {
            this._stopProfiling();
        });
    }
    _stopProfiling() {
        const profileFilenamePrefix = this._environmentService.args['prof-startup-prefix'];
        if (!profileFilenamePrefix) {
            return;
        }
        const dir = path_1.dirname(profileFilenamePrefix);
        const prefix = path_1.basename(profileFilenamePrefix);
        const removeArgs = ['--prof-startup'];
        const markerFile = pfs_1.readFile(profileFilenamePrefix).then(value => removeArgs.push(...value.toString().split('|')))
            .then(() => pfs_1.rimraf(profileFilenamePrefix)) // (1) delete the file to tell the main process to stop profiling
            .then(() => new Promise(resolve => {
            const check = () => {
                pfs_1.exists(profileFilenamePrefix).then(exists => {
                    if (exists) {
                        resolve();
                    }
                    else {
                        setTimeout(check, 500);
                    }
                });
            };
            check();
        }))
            .then(() => pfs_1.rimraf(profileFilenamePrefix)); // (3) finally delete the file again
        markerFile.then(() => {
            return pfs_1.readdir(dir).then(files => files.filter(value => value.indexOf(prefix) === 0));
        }).then(files => {
            const profileFiles = files.reduce((prev, cur) => `${prev}${path_1.join(dir, cur)}\n`, '\n');
            return this._dialogService.confirm({
                type: 'info',
                message: nls_1.localize('prof.message', "Successfully created profiles."),
                detail: nls_1.localize('prof.detail', "Please create an issue and manually attach the following files:\n{0}", profileFiles),
                primaryButton: nls_1.localize('prof.restartAndFileIssue', "Create Issue and Restart"),
                secondaryButton: nls_1.localize('prof.restart', "Restart")
            }).then(res => {
                if (res.confirmed) {
                    Promise.all([
                        this._electronService.showItemInFolder(uri_1.URI.file(path_1.join(dir, files[0])).fsPath),
                        this._createPerfIssue(files)
                    ]).then(() => {
                        // keep window stable until restart is selected
                        return this._dialogService.confirm({
                            type: 'info',
                            message: nls_1.localize('prof.thanks', "Thanks for helping us."),
                            detail: nls_1.localize('prof.detail.restart', "A final restart is required to continue to use '{0}'. Again, thank you for your contribution.", this._environmentService.appNameLong),
                            primaryButton: nls_1.localize('prof.restart', "Restart"),
                            secondaryButton: undefined
                        }).then(() => {
                            // now we are ready to restart
                            this._electronService.relaunch({ removeArgs });
                        });
                    });
                }
                else {
                    // simply restart
                    this._electronService.relaunch({ removeArgs });
                }
            });
        });
    }
    _createPerfIssue(files) {
        return __awaiter(this, void 0, void 0, function* () {
            const reportIssueUrl = product_1.default.reportIssueUrl;
            if (!reportIssueUrl) {
                return;
            }
            const ref = yield this._textModelResolverService.createModelReference(perfviewEditor_1.PerfviewInput.Uri);
            yield this._clipboardService.writeText(ref.object.textEditorModel.getValue());
            ref.dispose();
            const body = `
1. :warning: We have copied additional data to your clipboard. Make sure to **paste** here. :warning:
1. :warning: Make sure to **attach** these files from your *home*-directory: :warning:\n${files.map(file => `-\`${file}\``).join('\n')}
`;
            const baseUrl = reportIssueUrl;
            const queryStringPrefix = baseUrl.indexOf('?') === -1 ? '?' : '&';
            this._openerService.open(uri_1.URI.parse(`${baseUrl}${queryStringPrefix}body=${encodeURIComponent(body)}`));
        });
    }
};
StartupProfiler = __decorate([
    __param(0, dialogs_1.IDialogService),
    __param(1, environment_1.IEnvironmentService),
    __param(2, resolverService_1.ITextModelService),
    __param(3, clipboardService_1.IClipboardService),
    __param(4, lifecycle_1.ILifecycleService),
    __param(5, extensions_1.IExtensionService),
    __param(6, opener_1.IOpenerService),
    __param(7, electron_1.IElectronService)
], StartupProfiler);
exports.StartupProfiler = StartupProfiler;
//# sourceMappingURL=startupProfiler.js.map