"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const async_1 = require("vs/base/common/async");
const util_1 = require("util");
const errors_1 = require("vs/base/common/errors");
const editorBrowser_1 = require("vs/editor/browser/editorBrowser");
const environment_1 = require("vs/platform/environment/common/environment");
const lifecycle_1 = require("vs/platform/lifecycle/common/lifecycle");
const product_1 = require("vs/platform/product/common/product");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const update_1 = require("vs/platform/update/common/update");
const windows_1 = require("vs/platform/windows/common/windows");
const files = require("vs/workbench/contrib/files/common/files");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const panelService_1 = require("vs/workbench/services/panel/common/panelService");
const timerService_1 = require("vs/workbench/services/timer/electron-browser/timerService");
const viewlet_1 = require("vs/workbench/services/viewlet/browser/viewlet");
const performance_1 = require("vs/base/common/performance");
const host_1 = require("vs/workbench/services/host/browser/host");
let StartupTimings = class StartupTimings {
    constructor(_timerService, _windowsService, _editorService, _viewletService, _panelService, _telemetryService, _lifecycleService, _updateService, _envService, _hostService) {
        this._timerService = _timerService;
        this._windowsService = _windowsService;
        this._editorService = _editorService;
        this._viewletService = _viewletService;
        this._panelService = _panelService;
        this._telemetryService = _telemetryService;
        this._lifecycleService = _lifecycleService;
        this._updateService = _updateService;
        this._envService = _envService;
        this._hostService = _hostService;
        //
        this._report().catch(errors_1.onUnexpectedError);
    }
    _report() {
        return __awaiter(this, void 0, void 0, function* () {
            const isStandardStartup = yield this._isStandardStartup();
            this._reportStartupTimes().catch(errors_1.onUnexpectedError);
            this._appendStartupTimes(isStandardStartup).catch(errors_1.onUnexpectedError);
            this._reportPerfTicks();
        });
    }
    _reportStartupTimes() {
        return __awaiter(this, void 0, void 0, function* () {
            const metrics = yield this._timerService.startupMetrics;
            /* __GDPR__
                "startupTimeVaried" : {
                    "${include}": [
                        "${IStartupMetrics}"
                    ]
                }
            */
            this._telemetryService.publicLog('startupTimeVaried', metrics);
        });
    }
    _appendStartupTimes(isStandardStartup) {
        return __awaiter(this, void 0, void 0, function* () {
            const appendTo = this._envService.args['prof-append-timers'];
            if (!appendTo) {
                // nothing to do
                return;
            }
            const { sessionId } = yield this._telemetryService.getTelemetryInfo();
            Promise.all([
                this._timerService.startupMetrics,
                async_1.timeout(15000),
            ]).then(([startupMetrics]) => {
                return util_1.promisify(fs_1.appendFile)(appendTo, `${startupMetrics.ellapsed}\t${product_1.default.nameShort}\t${(product_1.default.commit || '').slice(0, 10) || '0000000000'}\t${sessionId}\t${isStandardStartup ? 'standard_start' : 'NO_standard_start'}\n`);
            }).then(() => {
                this._windowsService.quit();
            }).catch(err => {
                console.error(err);
                this._windowsService.quit();
            });
        });
    }
    _isStandardStartup() {
        return __awaiter(this, void 0, void 0, function* () {
            // check for standard startup:
            // * new window (no reload)
            // * just one window
            // * explorer viewlet visible
            // * one text editor (not multiple, not webview, welcome etc...)
            // * cached data present (not rejected, not created)
            if (this._lifecycleService.startupKind !== 1 /* NewWindow */) {
                return false;
            }
            if ((yield this._hostService.windowCount) !== 1) {
                return false;
            }
            const activeViewlet = this._viewletService.getActiveViewlet();
            if (!activeViewlet || activeViewlet.getId() !== files.VIEWLET_ID) {
                return false;
            }
            const visibleControls = this._editorService.visibleControls;
            if (visibleControls.length !== 1 || !editorBrowser_1.isCodeEditor(visibleControls[0].getControl())) {
                return false;
            }
            if (this._panelService.getActivePanel()) {
                return false;
            }
            if (!timerService_1.didUseCachedData()) {
                return false;
            }
            if (!(yield this._updateService.isLatestVersion())) {
                return false;
            }
            return true;
        });
    }
    _reportPerfTicks() {
        const entries = Object.create(null);
        for (const entry of performance_1.getEntries()) {
            entries[entry.name] = entry.timestamp;
        }
        /* __GDPR__
            "startupRawTimers" : {
                "entries": { "classification": "SystemMetaData", "purpose": "PerformanceAndHealth" }
            }
        */
        this._telemetryService.publicLog('startupRawTimers', { entries });
    }
};
StartupTimings = __decorate([
    __param(0, timerService_1.ITimerService),
    __param(1, windows_1.IWindowsService),
    __param(2, editorService_1.IEditorService),
    __param(3, viewlet_1.IViewletService),
    __param(4, panelService_1.IPanelService),
    __param(5, telemetry_1.ITelemetryService),
    __param(6, lifecycle_1.ILifecycleService),
    __param(7, update_1.IUpdateService),
    __param(8, environment_1.IEnvironmentService),
    __param(9, host_1.IHostService)
], StartupTimings);
exports.StartupTimings = StartupTimings;
//# sourceMappingURL=startupTimings.js.map