"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const contributions_1 = require("vs/workbench/common/contributions");
const platform_1 = require("vs/platform/registry/common/platform");
const label_1 = require("vs/platform/label/common/label");
const platform_2 = require("vs/base/common/platform");
const network_1 = require("vs/base/common/network");
const remoteAgentService_1 = require("vs/workbench/services/remote/common/remoteAgentService");
const log_1 = require("vs/platform/log/common/log");
const logIpc_1 = require("vs/platform/log/common/logIpc");
const output_1 = require("vs/workbench/contrib/output/common/output");
const nls_1 = require("vs/nls");
const resources_1 = require("vs/base/common/resources");
const lifecycle_1 = require("vs/base/common/lifecycle");
const views_1 = require("vs/workbench/common/views");
exports.VIEWLET_ID = 'workbench.view.remote';
exports.VIEW_CONTAINER = platform_1.Registry.as(views_1.Extensions.ViewContainersRegistry).registerViewContainer(exports.VIEWLET_ID, true, undefined, {
    getOrder: (group) => {
        if (!group) {
            return;
        }
        let matches = /^targets@(\d+)$/.exec(group);
        if (matches) {
            return -1000;
        }
        matches = /^details@(\d+)$/.exec(group);
        if (matches) {
            return -500;
        }
        return;
    }
});
let LabelContribution = class LabelContribution {
    constructor(labelService, remoteAgentService) {
        this.labelService = labelService;
        this.remoteAgentService = remoteAgentService;
        this.registerFormatters();
    }
    registerFormatters() {
        this.remoteAgentService.getEnvironment().then(remoteEnvironment => {
            if (remoteEnvironment) {
                this.labelService.registerFormatter({
                    scheme: network_1.Schemas.vscodeRemote,
                    formatting: {
                        label: '${path}',
                        separator: remoteEnvironment.os === 1 /* Windows */ ? '\\' : '/',
                        tildify: remoteEnvironment.os !== 1 /* Windows */,
                        normalizeDriveLetter: remoteEnvironment.os === 1 /* Windows */,
                        workspaceSuffix: platform_2.isWeb ? undefined : network_1.Schemas.vscodeRemote
                    }
                });
            }
        });
    }
};
LabelContribution = __decorate([
    __param(0, label_1.ILabelService),
    __param(1, remoteAgentService_1.IRemoteAgentService)
], LabelContribution);
exports.LabelContribution = LabelContribution;
let RemoteChannelsContribution = class RemoteChannelsContribution extends lifecycle_1.Disposable {
    constructor(logService, remoteAgentService) {
        super();
        const connection = remoteAgentService.getConnection();
        if (connection) {
            const loggerClient = new logIpc_1.LoggerChannelClient(connection.getChannel('logger'));
            loggerClient.setLevel(logService.getLevel());
            this._register(logService.onDidChangeLogLevel(level => loggerClient.setLevel(level)));
        }
    }
};
RemoteChannelsContribution = __decorate([
    __param(0, log_1.ILogService),
    __param(1, remoteAgentService_1.IRemoteAgentService)
], RemoteChannelsContribution);
let RemoteLogOutputChannels = class RemoteLogOutputChannels {
    constructor(remoteAgentService) {
        remoteAgentService.getEnvironment().then(remoteEnv => {
            if (remoteEnv) {
                const outputChannelRegistry = platform_1.Registry.as(output_1.Extensions.OutputChannels);
                outputChannelRegistry.registerChannel({ id: 'remoteExtensionLog', label: nls_1.localize('remoteExtensionLog', "Remote Server"), file: resources_1.joinPath(remoteEnv.logsPath, `${remoteAgentService_1.RemoteExtensionLogFileName}.log`), log: true });
            }
        });
    }
};
RemoteLogOutputChannels = __decorate([
    __param(0, remoteAgentService_1.IRemoteAgentService)
], RemoteLogOutputChannels);
const workbenchContributionsRegistry = platform_1.Registry.as(contributions_1.Extensions.Workbench);
workbenchContributionsRegistry.registerWorkbenchContribution(LabelContribution, 1 /* Starting */);
workbenchContributionsRegistry.registerWorkbenchContribution(RemoteChannelsContribution, 1 /* Starting */);
workbenchContributionsRegistry.registerWorkbenchContribution(RemoteLogOutputChannels, 3 /* Restored */);
//# sourceMappingURL=remote.contribution.js.map