"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
require("vs/css!./media/scmViewlet");
const event_1 = require("vs/base/common/event");
const event_2 = require("vs/base/browser/event");
const resources_1 = require("vs/base/common/resources");
const lifecycle_1 = require("vs/base/common/lifecycle");
const panelViewlet_1 = require("vs/workbench/browser/parts/views/panelViewlet");
const dom_1 = require("vs/base/browser/dom");
const labels_1 = require("vs/workbench/browser/labels");
const countBadge_1 = require("vs/base/browser/ui/countBadge/countBadge");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const commands_1 = require("vs/platform/commands/common/commands");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const actions_1 = require("vs/platform/actions/common/actions");
const actions_2 = require("vs/base/common/actions");
const menuEntryActionViewItem_1 = require("vs/platform/actions/browser/menuEntryActionViewItem");
const menus_1 = require("./menus");
const actionbar_1 = require("vs/base/browser/ui/actionbar/actionbar");
const themeService_1 = require("vs/platform/theme/common/themeService");
const util_1 = require("./util");
const styler_1 = require("vs/platform/theme/common/styler");
const inputBox_1 = require("vs/base/browser/ui/inputbox/inputBox");
const strings_1 = require("vs/base/common/strings");
const listService_1 = require("vs/platform/list/browser/listService");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const async_1 = require("vs/base/common/async");
const notification_1 = require("vs/platform/notification/common/notification");
const platform = require("vs/base/common/platform");
const resourceTree_1 = require("vs/base/common/resourceTree");
const iterator_1 = require("vs/base/common/iterator");
const uri_1 = require("vs/base/common/uri");
const files_1 = require("vs/platform/files/common/files");
const comparers_1 = require("vs/base/common/comparers");
const filters_1 = require("vs/base/common/filters");
const nls_1 = require("vs/nls");
const arrays_1 = require("vs/base/common/arrays");
const decorators_1 = require("vs/base/common/decorators");
const workbenchThemeService_1 = require("vs/workbench/services/themes/common/workbenchThemeService");
class ResourceGroupRenderer {
    constructor(actionViewItemProvider, themeService, menus) {
        this.actionViewItemProvider = actionViewItemProvider;
        this.themeService = themeService;
        this.menus = menus;
    }
    get templateId() { return ResourceGroupRenderer.TEMPLATE_ID; }
    renderTemplate(container) {
        // hack
        dom_1.addClass(container.parentElement.parentElement.querySelector('.monaco-tl-twistie'), 'force-twistie');
        const element = dom_1.append(container, dom_1.$('.resource-group'));
        const name = dom_1.append(element, dom_1.$('.name'));
        const actionsContainer = dom_1.append(element, dom_1.$('.actions'));
        const actionBar = new actionbar_1.ActionBar(actionsContainer, { actionViewItemProvider: this.actionViewItemProvider });
        const countContainer = dom_1.append(element, dom_1.$('.count'));
        const count = new countBadge_1.CountBadge(countContainer);
        const styler = styler_1.attachBadgeStyler(count, this.themeService);
        const elementDisposables = lifecycle_1.Disposable.None;
        const disposables = lifecycle_1.combinedDisposable(actionBar, styler);
        return { name, count, actionBar, elementDisposables, disposables };
    }
    renderElement(node, index, template) {
        template.elementDisposables.dispose();
        const group = node.element;
        template.name.textContent = group.label;
        template.actionBar.clear();
        template.actionBar.context = group;
        template.count.setCount(group.elements.length);
        const disposables = new lifecycle_1.DisposableStore();
        disposables.add(util_1.connectPrimaryMenuToInlineActionBar(this.menus.getResourceGroupMenu(group), template.actionBar));
        template.elementDisposables = disposables;
    }
    renderCompressedElements(node, index, templateData, height) {
        throw new Error('Should never happen since node is incompressible');
    }
    disposeElement(group, index, template) {
        template.elementDisposables.dispose();
    }
    disposeTemplate(template) {
        template.elementDisposables.dispose();
        template.disposables.dispose();
    }
}
ResourceGroupRenderer.TEMPLATE_ID = 'resource group';
class MultipleSelectionActionRunner extends actions_2.ActionRunner {
    constructor(getSelectedResources) {
        super();
        this.getSelectedResources = getSelectedResources;
    }
    runAction(action, context) {
        if (!(action instanceof actions_1.MenuItemAction)) {
            return super.runAction(action, context);
        }
        const selection = this.getSelectedResources();
        const contextIsSelected = selection.some(s => s === context);
        const actualContext = contextIsSelected ? selection : [context];
        const args = arrays_1.flatten(actualContext.map(e => resourceTree_1.ResourceTree.isBranchNode(e) ? resourceTree_1.ResourceTree.collect(e) : [e]));
        return action.run(...args);
    }
}
class ResourceRenderer {
    constructor(viewModelProvider, labels, actionViewItemProvider, getSelectedResources, themeService, menus) {
        this.viewModelProvider = viewModelProvider;
        this.labels = labels;
        this.actionViewItemProvider = actionViewItemProvider;
        this.getSelectedResources = getSelectedResources;
        this.themeService = themeService;
        this.menus = menus;
    }
    get templateId() { return ResourceRenderer.TEMPLATE_ID; }
    renderTemplate(container) {
        const element = dom_1.append(container, dom_1.$('.resource'));
        const name = dom_1.append(element, dom_1.$('.name'));
        const fileLabel = this.labels.create(name, { supportHighlights: true });
        const actionsContainer = dom_1.append(fileLabel.element, dom_1.$('.actions'));
        const actionBar = new actionbar_1.ActionBar(actionsContainer, {
            actionViewItemProvider: this.actionViewItemProvider,
            actionRunner: new MultipleSelectionActionRunner(this.getSelectedResources)
        });
        const decorationIcon = dom_1.append(element, dom_1.$('.decoration-icon'));
        const disposables = lifecycle_1.combinedDisposable(actionBar, fileLabel);
        return { element, name, fileLabel, decorationIcon, actionBar, elementDisposables: lifecycle_1.Disposable.None, disposables };
    }
    renderElement(node, index, template) {
        template.elementDisposables.dispose();
        const elementDisposables = new lifecycle_1.DisposableStore();
        const resourceOrFolder = node.element;
        const theme = this.themeService.getTheme();
        const icon = !resourceTree_1.ResourceTree.isBranchNode(resourceOrFolder) && (theme.type === themeService_1.LIGHT ? resourceOrFolder.decorations.icon : resourceOrFolder.decorations.iconDark);
        const uri = resourceTree_1.ResourceTree.isBranchNode(resourceOrFolder) ? resourceOrFolder.uri : resourceOrFolder.sourceUri;
        const fileKind = resourceTree_1.ResourceTree.isBranchNode(resourceOrFolder) ? files_1.FileKind.FOLDER : files_1.FileKind.FILE;
        const viewModel = this.viewModelProvider();
        template.fileLabel.setFile(uri, {
            fileDecorations: { colors: false, badges: !icon },
            hidePath: viewModel.mode === "tree" /* Tree */,
            fileKind,
            matches: filters_1.createMatches(node.filterData)
        });
        template.actionBar.clear();
        template.actionBar.context = resourceOrFolder;
        if (resourceTree_1.ResourceTree.isBranchNode(resourceOrFolder)) {
            elementDisposables.add(util_1.connectPrimaryMenuToInlineActionBar(this.menus.getResourceFolderMenu(resourceOrFolder.context), template.actionBar));
            dom_1.removeClass(template.name, 'strike-through');
            dom_1.removeClass(template.element, 'faded');
        }
        else {
            elementDisposables.add(util_1.connectPrimaryMenuToInlineActionBar(this.menus.getResourceMenu(resourceOrFolder.resourceGroup), template.actionBar));
            dom_1.toggleClass(template.name, 'strike-through', resourceOrFolder.decorations.strikeThrough);
            dom_1.toggleClass(template.element, 'faded', resourceOrFolder.decorations.faded);
        }
        const tooltip = !resourceTree_1.ResourceTree.isBranchNode(resourceOrFolder) && resourceOrFolder.decorations.tooltip || '';
        if (icon) {
            template.decorationIcon.style.display = '';
            template.decorationIcon.style.backgroundImage = `url('${icon}')`;
            template.decorationIcon.title = tooltip;
        }
        else {
            template.decorationIcon.style.display = 'none';
            template.decorationIcon.style.backgroundImage = '';
            template.decorationIcon.title = '';
        }
        template.element.setAttribute('data-tooltip', tooltip);
        template.elementDisposables = elementDisposables;
    }
    disposeElement(resource, index, template) {
        template.elementDisposables.dispose();
    }
    renderCompressedElements(node, index, template, height) {
        template.elementDisposables.dispose();
        const elementDisposables = new lifecycle_1.DisposableStore();
        const compressed = node.element;
        const folder = compressed.elements[compressed.elements.length - 1];
        const label = compressed.elements.map(e => e.name).join('/');
        const fileKind = files_1.FileKind.FOLDER;
        template.fileLabel.setResource({ resource: folder.uri, name: label }, {
            fileDecorations: { colors: false, badges: true },
            fileKind,
            matches: filters_1.createMatches(node.filterData)
        });
        template.actionBar.clear();
        template.actionBar.context = folder;
        elementDisposables.add(util_1.connectPrimaryMenuToInlineActionBar(this.menus.getResourceFolderMenu(folder.context), template.actionBar));
        dom_1.removeClass(template.name, 'strike-through');
        dom_1.removeClass(template.element, 'faded');
        template.decorationIcon.style.display = 'none';
        template.decorationIcon.style.backgroundImage = '';
        template.element.setAttribute('data-tooltip', '');
        template.elementDisposables = elementDisposables;
    }
    disposeCompressedElements(node, index, template, height) {
        template.elementDisposables.dispose();
    }
    disposeTemplate(template) {
        template.elementDisposables.dispose();
        template.disposables.dispose();
    }
}
ResourceRenderer.TEMPLATE_ID = 'resource';
class ProviderListDelegate {
    getHeight() { return 22; }
    getTemplateId(element) {
        if (resourceTree_1.ResourceTree.isBranchNode(element) || util_1.isSCMResource(element)) {
            return ResourceRenderer.TEMPLATE_ID;
        }
        else {
            return ResourceGroupRenderer.TEMPLATE_ID;
        }
    }
}
class SCMTreeFilter {
    filter(element) {
        if (resourceTree_1.ResourceTree.isBranchNode(element)) {
            return true;
        }
        else if (util_1.isSCMResourceGroup(element)) {
            return element.elements.length > 0 || !element.hideWhenEmpty;
        }
        else {
            return true;
        }
    }
}
class SCMTreeSorter {
    constructor(viewModelProvider) {
        this.viewModelProvider = viewModelProvider;
    }
    get viewModel() { return this.viewModelProvider(); }
    compare(one, other) {
        if (this.viewModel.mode === "list" /* List */) {
            return 0;
        }
        if (util_1.isSCMResourceGroup(one) && util_1.isSCMResourceGroup(other)) {
            return 0;
        }
        const oneIsDirectory = resourceTree_1.ResourceTree.isBranchNode(one);
        const otherIsDirectory = resourceTree_1.ResourceTree.isBranchNode(other);
        if (oneIsDirectory !== otherIsDirectory) {
            return oneIsDirectory ? -1 : 1;
        }
        const oneName = resourceTree_1.ResourceTree.isBranchNode(one) ? one.name : resources_1.basename(one.sourceUri);
        const otherName = resourceTree_1.ResourceTree.isBranchNode(other) ? other.name : resources_1.basename(other.sourceUri);
        return comparers_1.compareFileNames(oneName, otherName);
    }
}
__decorate([
    decorators_1.memoize
], SCMTreeSorter.prototype, "viewModel", null);
exports.SCMTreeSorter = SCMTreeSorter;
class SCMTreeKeyboardNavigationLabelProvider {
    getKeyboardNavigationLabel(element) {
        if (resourceTree_1.ResourceTree.isBranchNode(element)) {
            return element.name;
        }
        else if (util_1.isSCMResourceGroup(element)) {
            return element.label;
        }
        else {
            return resources_1.basename(element.sourceUri);
        }
    }
    getCompressedNodeKeyboardNavigationLabel(elements) {
        const folders = elements;
        return folders.map(e => e.name).join('/');
    }
}
exports.SCMTreeKeyboardNavigationLabelProvider = SCMTreeKeyboardNavigationLabelProvider;
class SCMResourceIdentityProvider {
    getId(element) {
        if (resourceTree_1.ResourceTree.isBranchNode(element)) {
            const group = element.context;
            return `${group.provider.contextValue}/${group.id}/$FOLDER/${element.uri.toString()}`;
        }
        else if (util_1.isSCMResource(element)) {
            const group = element.resourceGroup;
            const provider = group.provider;
            return `${provider.contextValue}/${group.id}/${element.sourceUri.toString()}`;
        }
        else {
            const provider = element.provider;
            return `${provider.contextValue}/${element.id}`;
        }
    }
}
function groupItemAsTreeElement(item, mode) {
    const children = mode === "list" /* List */
        ? iterator_1.Iterator.map(iterator_1.Iterator.fromArray(item.resources), element => ({ element, incompressible: true }))
        : iterator_1.Iterator.map(item.tree.root.children, node => asTreeElement(node, true));
    return { element: item.group, children, incompressible: true };
}
function asTreeElement(node, incompressible) {
    if (resourceTree_1.ResourceTree.isBranchNode(node)) {
        return {
            element: node,
            children: iterator_1.Iterator.map(node.children, node => asTreeElement(node, false)),
            incompressible,
            collapsed: false
        };
    }
    return { element: node.element, incompressible: true };
}
var ViewModelMode;
(function (ViewModelMode) {
    ViewModelMode["List"] = "list";
    ViewModelMode["Tree"] = "tree";
})(ViewModelMode || (ViewModelMode = {}));
class ViewModel {
    constructor(groups, tree) {
        this.groups = groups;
        this.tree = tree;
        this._mode = "tree" /* Tree */;
        this._onDidChangeMode = new event_1.Emitter();
        this.onDidChangeMode = this._onDidChangeMode.event;
        this.items = [];
        this.visibilityDisposables = new lifecycle_1.DisposableStore();
        this.disposables = new lifecycle_1.DisposableStore();
    }
    get mode() { return this._mode; }
    set mode(mode) {
        this._mode = mode;
        this.refresh();
        this._onDidChangeMode.fire(mode);
    }
    onDidSpliceGroups({ start, deleteCount, toInsert }) {
        const itemsToInsert = [];
        for (const group of toInsert) {
            const tree = new resourceTree_1.ResourceTree(group, group.provider.rootUri || uri_1.URI.file('/'));
            const resources = [...group.elements];
            const disposable = lifecycle_1.combinedDisposable(group.onDidChange(() => this.tree.refilter()), group.onDidSplice(splice => this.onDidSpliceGroup(item, splice)));
            const item = { group, resources, tree, disposable };
            for (const resource of resources) {
                item.tree.add(resource.sourceUri, resource);
            }
            itemsToInsert.push(item);
        }
        const itemsToDispose = this.items.splice(start, deleteCount, ...itemsToInsert);
        for (const item of itemsToDispose) {
            item.disposable.dispose();
        }
        this.refresh();
    }
    onDidSpliceGroup(item, { start, deleteCount, toInsert }) {
        for (const resource of toInsert) {
            item.tree.add(resource.sourceUri, resource);
        }
        const deleted = item.resources.splice(start, deleteCount, ...toInsert);
        for (const resource of deleted) {
            item.tree.delete(resource.sourceUri);
        }
        this.refresh(item);
    }
    setVisible(visible) {
        if (visible) {
            this.visibilityDisposables = new lifecycle_1.DisposableStore();
            this.groups.onDidSplice(this.onDidSpliceGroups, this, this.visibilityDisposables);
            this.onDidSpliceGroups({ start: 0, deleteCount: this.items.length, toInsert: this.groups.elements });
            if (typeof this.scrollTop === 'number') {
                this.tree.scrollTop = this.scrollTop;
                this.scrollTop = undefined;
            }
        }
        else {
            this.visibilityDisposables.dispose();
            this.onDidSpliceGroups({ start: 0, deleteCount: this.items.length, toInsert: [] });
            this.scrollTop = this.tree.scrollTop;
        }
    }
    refresh(item) {
        if (item) {
            this.tree.setChildren(item.group, groupItemAsTreeElement(item, this.mode).children);
        }
        else {
            this.tree.setChildren(null, this.items.map(item => groupItemAsTreeElement(item, this.mode)));
        }
    }
    dispose() {
        this.visibilityDisposables.dispose();
        this.disposables.dispose();
    }
}
class ToggleViewModeAction extends actions_2.Action {
    constructor(viewModel) {
        super(ToggleViewModeAction.ID, ToggleViewModeAction.LABEL);
        this.viewModel = viewModel;
        this._register(this.viewModel.onDidChangeMode(this.onDidChangeMode, this));
        this.onDidChangeMode(this.viewModel.mode);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this.viewModel.mode = this.viewModel.mode === "list" /* List */ ? "tree" /* Tree */ : "list" /* List */;
        });
    }
    onDidChangeMode(mode) {
        this.class = `scm-action toggle-view-mode ${mode}`;
    }
}
exports.ToggleViewModeAction = ToggleViewModeAction;
ToggleViewModeAction.ID = 'workbench.scm.action.toggleViewMode';
ToggleViewModeAction.LABEL = nls_1.localize('toggleViewMode', "ToggleViewMode");
function convertValidationType(type) {
    switch (type) {
        case 2 /* Information */: return 1 /* INFO */;
        case 1 /* Warning */: return 2 /* WARNING */;
        case 0 /* Error */: return 3 /* ERROR */;
    }
}
let RepositoryPanel = class RepositoryPanel extends panelViewlet_1.ViewletPanel {
    constructor(repository, options, keybindingService, themeService, contextMenuService, contextViewService, commandService, notificationService, editorService, instantiationService, configurationService, contextKeyService, menuService) {
        super(options, keybindingService, contextMenuService, configurationService, contextKeyService);
        this.repository = repository;
        this.keybindingService = keybindingService;
        this.themeService = themeService;
        this.contextMenuService = contextMenuService;
        this.contextViewService = contextViewService;
        this.commandService = commandService;
        this.notificationService = notificationService;
        this.editorService = editorService;
        this.instantiationService = instantiationService;
        this.configurationService = configurationService;
        this.menuService = menuService;
        this.cachedHeight = undefined;
        this.cachedWidth = undefined;
        this.menus = instantiationService.createInstance(menus_1.SCMMenus, this.repository.provider);
        this._register(this.menus);
        this._register(this.menus.onDidChangeTitle(this._onDidChangeTitleArea.fire, this._onDidChangeTitleArea));
        this.contextKeyService = contextKeyService.createScoped(this.element);
        this.contextKeyService.createKey('scmRepository', this.repository);
    }
    render() {
        super.render();
        this._register(this.menus.onDidChangeTitle(this.updateActions, this));
    }
    renderHeaderTitle(container) {
        let title;
        let type;
        if (this.repository.provider.rootUri) {
            title = resources_1.basename(this.repository.provider.rootUri);
            type = this.repository.provider.label;
        }
        else {
            title = this.repository.provider.label;
            type = '';
        }
        super.renderHeaderTitle(container, title);
        dom_1.addClass(container, 'scm-provider');
        dom_1.append(container, dom_1.$('span.type', undefined, type));
    }
    renderBody(container) {
        const focusTracker = dom_1.trackFocus(container);
        this._register(focusTracker.onDidFocus(() => this.repository.focus()));
        this._register(focusTracker);
        // Input
        this.inputBoxContainer = dom_1.append(container, dom_1.$('.scm-editor'));
        const updatePlaceholder = () => {
            const binding = this.keybindingService.lookupKeybinding('scm.acceptInput');
            const label = binding ? binding.getLabel() : (platform.isMacintosh ? 'Cmd+Enter' : 'Ctrl+Enter');
            const placeholder = strings_1.format(this.repository.input.placeholder, label);
            this.inputBox.setPlaceHolder(placeholder);
        };
        const validationDelayer = new async_1.ThrottledDelayer(200);
        const validate = () => {
            return this.repository.input.validateInput(this.inputBox.value, this.inputBox.inputElement.selectionStart || 0).then(result => {
                if (!result) {
                    this.inputBox.inputElement.removeAttribute('aria-invalid');
                    this.inputBox.hideMessage();
                }
                else {
                    this.inputBox.inputElement.setAttribute('aria-invalid', 'true');
                    this.inputBox.showMessage({ content: result.message, type: convertValidationType(result.type) });
                }
            });
        };
        const triggerValidation = () => validationDelayer.trigger(validate);
        this.inputBox = new inputBox_1.InputBox(this.inputBoxContainer, this.contextViewService, { flexibleHeight: true, flexibleMaxHeight: 134 });
        this.inputBox.setEnabled(this.isBodyVisible());
        this._register(styler_1.attachInputBoxStyler(this.inputBox, this.themeService));
        this._register(this.inputBox);
        this._register(this.inputBox.onDidChange(triggerValidation, null));
        const onKeyUp = event_2.domEvent(this.inputBox.inputElement, 'keyup');
        const onMouseUp = event_2.domEvent(this.inputBox.inputElement, 'mouseup');
        this._register(event_1.Event.any(onKeyUp, onMouseUp)(triggerValidation, null));
        this.inputBox.value = this.repository.input.value;
        this._register(this.inputBox.onDidChange(value => this.repository.input.value = value, null));
        this._register(this.repository.input.onDidChange(value => this.inputBox.value = value, null));
        updatePlaceholder();
        this._register(this.repository.input.onDidChangePlaceholder(updatePlaceholder, null));
        this._register(this.keybindingService.onDidUpdateKeybindings(updatePlaceholder, null));
        this._register(this.inputBox.onDidHeightChange(() => this.layoutBody()));
        if (this.repository.provider.onDidChangeCommitTemplate) {
            this._register(this.repository.provider.onDidChangeCommitTemplate(this.updateInputBox, this));
        }
        this.updateInputBox();
        // Input box visibility
        this._register(this.repository.input.onDidChangeVisibility(this.updateInputBoxVisibility, this));
        this.updateInputBoxVisibility();
        // List
        this.listContainer = dom_1.append(container, dom_1.$('.scm-status.show-file-icons'));
        const updateActionsVisibility = () => dom_1.toggleClass(this.listContainer, 'show-actions', this.configurationService.getValue('scm.alwaysShowActions'));
        event_1.Event.filter(this.configurationService.onDidChangeConfiguration, e => e.affectsConfiguration('scm.alwaysShowActions'))(updateActionsVisibility);
        updateActionsVisibility();
        const delegate = new ProviderListDelegate();
        const actionViewItemProvider = (action) => this.getActionViewItem(action);
        this.listLabels = this.instantiationService.createInstance(labels_1.ResourceLabels, { onDidChangeVisibility: this.onDidChangeBodyVisibility });
        this._register(this.listLabels);
        const renderers = [
            new ResourceGroupRenderer(actionViewItemProvider, this.themeService, this.menus),
            new ResourceRenderer(() => this.viewModel, this.listLabels, actionViewItemProvider, () => this.getSelectedResources(), this.themeService, this.menus)
        ];
        const filter = new SCMTreeFilter();
        const sorter = new SCMTreeSorter(() => this.viewModel);
        const keyboardNavigationLabelProvider = new SCMTreeKeyboardNavigationLabelProvider();
        const identityProvider = new SCMResourceIdentityProvider();
        this.tree = this.instantiationService.createInstance(listService_1.WorkbenchCompressibleObjectTree, 'SCM Tree Repo', this.listContainer, delegate, renderers, {
            identityProvider,
            horizontalScrolling: false,
            filter,
            sorter,
            keyboardNavigationLabelProvider
        });
        this._register(event_1.Event.chain(this.tree.onDidOpen)
            .map(e => e.elements[0])
            .filter(e => !!e && !resourceTree_1.ResourceTree.isBranchNode(e) && util_1.isSCMResource(e))
            .on(this.open, this));
        this._register(event_1.Event.chain(this.tree.onDidPin)
            .map(e => e.elements[0])
            .filter(e => !!e && !resourceTree_1.ResourceTree.isBranchNode(e) && util_1.isSCMResource(e))
            .on(this.pin, this));
        this._register(this.tree.onContextMenu(this.onListContextMenu, this));
        this._register(this.tree);
        this.viewModel = new ViewModel(this.repository.provider.groups, this.tree);
        this._register(this.viewModel);
        dom_1.addClass(this.listContainer, 'file-icon-themable-tree');
        dom_1.addClass(this.listContainer, 'show-file-icons');
        const updateIndentStyles = (theme) => {
            dom_1.toggleClass(this.listContainer, 'list-view-mode', this.viewModel.mode === "list" /* List */);
            dom_1.toggleClass(this.listContainer, 'align-icons-and-twisties', this.viewModel.mode === "tree" /* Tree */ && theme.hasFileIcons && !theme.hasFolderIcons);
            dom_1.toggleClass(this.listContainer, 'hide-arrows', this.viewModel.mode === "tree" /* Tree */ && theme.hidesExplorerArrows === true);
        };
        updateIndentStyles(this.themeService.getFileIconTheme());
        this._register(this.themeService.onDidFileIconThemeChange(updateIndentStyles));
        this._register(this.viewModel.onDidChangeMode(() => updateIndentStyles(this.themeService.getFileIconTheme())));
        this.toggleViewModelModeAction = new ToggleViewModeAction(this.viewModel);
        this._register(this.toggleViewModelModeAction);
        this._register(this.onDidChangeBodyVisibility(this._onDidChangeVisibility, this));
        this.updateActions();
    }
    layoutBody(height = this.cachedHeight, width = this.cachedWidth) {
        if (height === undefined) {
            return;
        }
        this.cachedHeight = height;
        if (this.repository.input.visible) {
            dom_1.removeClass(this.inputBoxContainer, 'hidden');
            this.inputBox.layout();
            const editorHeight = this.inputBox.height;
            const listHeight = height - (editorHeight + 12 /* margin */);
            this.listContainer.style.height = `${listHeight}px`;
            this.tree.layout(listHeight, width);
        }
        else {
            dom_1.addClass(this.inputBoxContainer, 'hidden');
            this.listContainer.style.height = `${height}px`;
            this.tree.layout(height, width);
        }
    }
    focus() {
        super.focus();
        if (this.isExpanded()) {
            if (this.repository.input.visible) {
                this.inputBox.focus();
            }
            else {
                this.tree.domFocus();
            }
            this.repository.focus();
        }
    }
    _onDidChangeVisibility(visible) {
        this.inputBox.setEnabled(visible);
        this.viewModel.setVisible(visible);
    }
    getActions() {
        if (this.toggleViewModelModeAction) {
            return [
                this.toggleViewModelModeAction,
                ...this.menus.getTitleActions()
            ];
        }
        else {
            return this.menus.getTitleActions();
        }
    }
    getSecondaryActions() {
        return this.menus.getTitleSecondaryActions();
    }
    getActionViewItem(action) {
        if (!(action instanceof actions_1.MenuItemAction)) {
            return undefined;
        }
        return new menuEntryActionViewItem_1.ContextAwareMenuEntryActionViewItem(action, this.keybindingService, this.notificationService, this.contextMenuService);
    }
    getActionsContext() {
        return this.repository.provider;
    }
    open(e) {
        e.open();
    }
    pin() {
        const activeControl = this.editorService.activeControl;
        if (activeControl) {
            activeControl.group.pinEditor(activeControl.input);
        }
    }
    onListContextMenu(e) {
        if (!e.element) {
            return;
        }
        const element = e.element;
        let actions = [];
        if (resourceTree_1.ResourceTree.isBranchNode(element)) {
            actions = this.menus.getResourceFolderContextActions(element.context);
        }
        else if (util_1.isSCMResource(element)) {
            actions = this.menus.getResourceContextActions(element);
        }
        else {
            actions = this.menus.getResourceGroupContextActions(element);
        }
        this.contextMenuService.showContextMenu({
            getAnchor: () => e.anchor,
            getActions: () => actions,
            getActionsContext: () => element,
            actionRunner: new MultipleSelectionActionRunner(() => this.getSelectedResources())
        });
    }
    getSelectedResources() {
        return this.tree.getSelection()
            .filter(r => !!r && !util_1.isSCMResourceGroup(r));
    }
    updateInputBox() {
        if (typeof this.repository.provider.commitTemplate === 'undefined' || !this.repository.input.visible || this.inputBox.value) {
            return;
        }
        this.inputBox.value = this.repository.provider.commitTemplate;
    }
    updateInputBoxVisibility() {
        if (this.cachedHeight) {
            this.layoutBody(this.cachedHeight);
        }
    }
};
RepositoryPanel = __decorate([
    __param(2, keybinding_1.IKeybindingService),
    __param(3, workbenchThemeService_1.IWorkbenchThemeService),
    __param(4, contextView_1.IContextMenuService),
    __param(5, contextView_1.IContextViewService),
    __param(6, commands_1.ICommandService),
    __param(7, notification_1.INotificationService),
    __param(8, editorService_1.IEditorService),
    __param(9, instantiation_1.IInstantiationService),
    __param(10, configuration_1.IConfigurationService),
    __param(11, contextkey_1.IContextKeyService),
    __param(12, actions_1.IMenuService)
], RepositoryPanel);
exports.RepositoryPanel = RepositoryPanel;
class RepositoryViewDescriptor {
    constructor(repository, hideByDefault) {
        this.repository = repository;
        this.hideByDefault = hideByDefault;
        this.canToggleVisibility = true;
        this.order = -500;
        this.workspace = true;
        const repoId = repository.provider.rootUri ? repository.provider.rootUri.toString() : `#${RepositoryViewDescriptor.counter++}`;
        this.id = `scm:repository:${repository.provider.label}:${repoId}`;
        this.name = repository.provider.rootUri ? resources_1.basename(repository.provider.rootUri) : repository.provider.label;
        this.ctorDescriptor = { ctor: RepositoryPanel, arguments: [repository] };
    }
}
exports.RepositoryViewDescriptor = RepositoryViewDescriptor;
RepositoryViewDescriptor.counter = 0;
//# sourceMappingURL=repositoryPanel.js.map