"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const lifecycle_1 = require("vs/base/common/lifecycle");
const menuEntryActionViewItem_1 = require("vs/platform/actions/browser/menuEntryActionViewItem");
const arrays_1 = require("vs/base/common/arrays");
function isSCMRepository(element) {
    return !!element.provider && typeof element.setSelected === 'function';
}
exports.isSCMRepository = isSCMRepository;
function isSCMResourceGroup(element) {
    return !!element.provider && !!element.elements;
}
exports.isSCMResourceGroup = isSCMResourceGroup;
function isSCMResource(element) {
    return !!element.sourceUri && isSCMResourceGroup(element.resourceGroup);
}
exports.isSCMResource = isSCMResource;
function connectPrimaryMenuToInlineActionBar(menu, actionBar) {
    let cachedDisposable = lifecycle_1.Disposable.None;
    let cachedPrimary = [];
    const updateActions = () => {
        const primary = [];
        const secondary = [];
        const disposable = menuEntryActionViewItem_1.createAndFillInActionBarActions(menu, { shouldForwardArgs: true }, { primary, secondary }, g => /^inline/.test(g));
        if (arrays_1.equals(cachedPrimary, primary, (a, b) => a.id === b.id)) {
            disposable.dispose();
            return;
        }
        cachedDisposable = disposable;
        cachedPrimary = primary;
        actionBar.clear();
        actionBar.push(primary, { icon: true, label: false });
    };
    updateActions();
    return lifecycle_1.combinedDisposable(menu.onDidChange(updateActions), lifecycle_1.toDisposable(() => {
        cachedDisposable.dispose();
    }));
}
exports.connectPrimaryMenuToInlineActionBar = connectPrimaryMenuToInlineActionBar;
//# sourceMappingURL=util.js.map