"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
const uri_1 = require("vs/base/common/uri");
const modelService_1 = require("vs/editor/common/services/modelService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const testConfigurationService_1 = require("vs/platform/configuration/test/common/testConfigurationService");
const instantiationServiceMock_1 = require("vs/platform/instantiation/test/common/instantiationServiceMock");
const search_1 = require("vs/workbench/services/search/common/search");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const testWorkspace_1 = require("vs/platform/workspace/test/common/testWorkspace");
const searchModel_1 = require("vs/workbench/contrib/search/common/searchModel");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
suite('Search - Viewlet', () => {
    let instantiation;
    setup(() => {
        instantiation = new instantiationServiceMock_1.TestInstantiationService();
        instantiation.stub(modelService_1.IModelService, stubModelService(instantiation));
        instantiation.set(workspace_1.IWorkspaceContextService, new workbenchTestServices_1.TestContextService(testWorkspace_1.TestWorkspace));
    });
    test('Data Source', function () {
        let result = instantiation.createInstance(searchModel_1.SearchResult, null);
        result.query = {
            type: 2 /* Text */,
            contentPattern: { pattern: 'foo' },
            folderQueries: [{
                    folder: uri_1.URI.parse('file://c:/')
                }]
        };
        result.add([{
                resource: uri_1.URI.parse('file:///c:/foo'),
                results: [{
                        preview: {
                            text: 'bar',
                            matches: {
                                startLineNumber: 0,
                                startColumn: 0,
                                endLineNumber: 0,
                                endColumn: 1
                            }
                        },
                        ranges: {
                            startLineNumber: 1,
                            startColumn: 0,
                            endLineNumber: 1,
                            endColumn: 1
                        }
                    }]
            }]);
        let fileMatch = result.matches()[0];
        let lineMatch = fileMatch.matches()[0];
        assert.equal(fileMatch.id(), 'file:///c%3A/foo');
        assert.equal(lineMatch.id(), 'file:///c%3A/foo>[2,1 -> 2,2]b');
    });
    test('Comparer', () => {
        let fileMatch1 = aFileMatch('C:\\foo');
        let fileMatch2 = aFileMatch('C:\\with\\path');
        let fileMatch3 = aFileMatch('C:\\with\\path\\foo');
        let lineMatch1 = new searchModel_1.Match(fileMatch1, ['bar'], new search_1.OneLineRange(0, 1, 1), new search_1.OneLineRange(0, 1, 1));
        let lineMatch2 = new searchModel_1.Match(fileMatch1, ['bar'], new search_1.OneLineRange(0, 1, 1), new search_1.OneLineRange(2, 1, 1));
        let lineMatch3 = new searchModel_1.Match(fileMatch1, ['bar'], new search_1.OneLineRange(0, 1, 1), new search_1.OneLineRange(2, 1, 1));
        assert(searchModel_1.searchMatchComparer(fileMatch1, fileMatch2) < 0);
        assert(searchModel_1.searchMatchComparer(fileMatch2, fileMatch1) > 0);
        assert(searchModel_1.searchMatchComparer(fileMatch1, fileMatch1) === 0);
        assert(searchModel_1.searchMatchComparer(fileMatch2, fileMatch3) < 0);
        assert(searchModel_1.searchMatchComparer(lineMatch1, lineMatch2) < 0);
        assert(searchModel_1.searchMatchComparer(lineMatch2, lineMatch1) > 0);
        assert(searchModel_1.searchMatchComparer(lineMatch2, lineMatch3) === 0);
    });
    function aFileMatch(path, searchResult, ...lineMatches) {
        let rawMatch = {
            resource: uri_1.URI.file('C:\\' + path),
            results: lineMatches
        };
        return instantiation.createInstance(searchModel_1.FileMatch, null, null, null, searchResult, rawMatch);
    }
    function stubModelService(instantiationService) {
        instantiationService.stub(configuration_1.IConfigurationService, new testConfigurationService_1.TestConfigurationService());
        return instantiationService.createInstance(modelServiceImpl_1.ModelServiceImpl);
    }
});
//# sourceMappingURL=searchViewlet.test.js.map