"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const electron_1 = require("electron");
const path_1 = require("vs/base/common/path");
const browser_1 = require("vs/base/browser/browser");
const dom_1 = require("vs/base/browser/dom");
const color_1 = require("vs/base/common/color");
const event_1 = require("vs/base/common/event");
const lifecycle_1 = require("vs/base/common/lifecycle");
const lifecycle_2 = require("vs/platform/lifecycle/common/lifecycle");
const platform_1 = require("vs/platform/registry/common/platform");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const themeService_1 = require("vs/platform/theme/common/themeService");
const editor_1 = require("vs/workbench/browser/parts/editor/editor");
const contributions_1 = require("vs/workbench/common/contributions");
const themes = require("vs/workbench/common/theme");
const layoutService_1 = require("vs/workbench/services/layout/browser/layoutService");
const environment_1 = require("vs/platform/environment/common/environment");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const uri_1 = require("vs/base/common/uri");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const windows_1 = require("vs/platform/windows/common/windows");
const perf = require("vs/base/common/performance");
let PartsSplash = class PartsSplash {
    constructor(_themeService, _layoutService, _textFileService, _envService, windowService, lifecycleService, editorGroupsService, configService) {
        this._themeService = _themeService;
        this._layoutService = _layoutService;
        this._textFileService = _textFileService;
        this._envService = _envService;
        this.windowService = windowService;
        this._disposables = new lifecycle_1.DisposableStore();
        lifecycleService.when(3 /* Restored */).then(_ => {
            this._removePartsSplash();
            perf.mark('didRemovePartsSplash');
        });
        event_1.Event.debounce(event_1.Event.any(browser_1.onDidChangeFullscreen, editorGroupsService.onDidLayout), () => { }, 800)(this._savePartsSplash, this, this._disposables);
        configService.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration('window.titleBarStyle')) {
                this._didChangeTitleBarStyle = true;
                this._savePartsSplash();
            }
        }, this, this._disposables);
    }
    dispose() {
        this._disposables.dispose();
    }
    _savePartsSplash() {
        const baseTheme = themeService_1.getThemeTypeSelector(this._themeService.getTheme().type);
        const colorInfo = {
            foreground: this._getThemeColor(colorRegistry_1.foreground),
            editorBackground: this._getThemeColor(colorRegistry_1.editorBackground),
            titleBarBackground: this._getThemeColor(themes.TITLE_BAR_ACTIVE_BACKGROUND),
            activityBarBackground: this._getThemeColor(themes.ACTIVITY_BAR_BACKGROUND),
            sideBarBackground: this._getThemeColor(themes.SIDE_BAR_BACKGROUND),
            statusBarBackground: this._getThemeColor(themes.STATUS_BAR_BACKGROUND),
            statusBarNoFolderBackground: this._getThemeColor(themes.STATUS_BAR_NO_FOLDER_BACKGROUND),
        };
        const layoutInfo = !this._shouldSaveLayoutInfo() ? undefined : {
            sideBarSide: this._layoutService.getSideBarPosition() === 1 /* RIGHT */ ? 'right' : 'left',
            editorPartMinWidth: editor_1.DEFAULT_EDITOR_MIN_DIMENSIONS.width,
            titleBarHeight: this._layoutService.isVisible("workbench.parts.titlebar" /* TITLEBAR_PART */) ? dom_1.getTotalHeight(this._layoutService.getContainer("workbench.parts.titlebar" /* TITLEBAR_PART */)) : 0,
            activityBarWidth: this._layoutService.isVisible("workbench.parts.activitybar" /* ACTIVITYBAR_PART */) ? dom_1.getTotalWidth(this._layoutService.getContainer("workbench.parts.activitybar" /* ACTIVITYBAR_PART */)) : 0,
            sideBarWidth: this._layoutService.isVisible("workbench.parts.sidebar" /* SIDEBAR_PART */) ? dom_1.getTotalWidth(this._layoutService.getContainer("workbench.parts.sidebar" /* SIDEBAR_PART */)) : 0,
            statusBarHeight: this._layoutService.isVisible("workbench.parts.statusbar" /* STATUSBAR_PART */) ? dom_1.getTotalHeight(this._layoutService.getContainer("workbench.parts.statusbar" /* STATUSBAR_PART */)) : 0,
        };
        this._textFileService.write(uri_1.URI.file(path_1.join(this._envService.userDataPath, 'rapid_render.json')), JSON.stringify({
            id: PartsSplash._splashElementId,
            colorInfo,
            layoutInfo,
            baseTheme
        }), { encoding: 'utf8', overwriteEncoding: true });
        if (baseTheme !== this._lastBaseTheme || colorInfo.editorBackground !== this._lastBackground) {
            // notify the main window on background color changes: the main window sets the background color to new windows
            this._lastBaseTheme = baseTheme;
            this._lastBackground = colorInfo.editorBackground;
            // the color needs to be in hex
            const backgroundColor = this._themeService.getTheme().getColor(colorRegistry_1.editorBackground) || themes.WORKBENCH_BACKGROUND(this._themeService.getTheme());
            const payload = JSON.stringify({ baseTheme, background: color_1.Color.Format.CSS.formatHex(backgroundColor) });
            electron_1.ipcRenderer.send('vscode:changeColorTheme', this.windowService.windowId, payload);
        }
    }
    _getThemeColor(id) {
        const theme = this._themeService.getTheme();
        const color = theme.getColor(id);
        return color ? color.toString() : undefined;
    }
    _shouldSaveLayoutInfo() {
        return !browser_1.isFullscreen() && !this._envService.isExtensionDevelopment && !this._didChangeTitleBarStyle;
    }
    _removePartsSplash() {
        let element = document.getElementById(PartsSplash._splashElementId);
        if (element) {
            element.style.display = 'none';
        }
        // remove initial colors
        let defaultStyles = document.head.getElementsByClassName('initialShellColors');
        if (defaultStyles.length) {
            document.head.removeChild(defaultStyles[0]);
        }
    }
};
PartsSplash._splashElementId = 'monaco-parts-splash';
PartsSplash = __decorate([
    __param(0, themeService_1.IThemeService),
    __param(1, layoutService_1.IWorkbenchLayoutService),
    __param(2, textfiles_1.ITextFileService),
    __param(3, environment_1.IEnvironmentService),
    __param(4, windows_1.IWindowService),
    __param(5, lifecycle_2.ILifecycleService),
    __param(6, editorGroupsService_1.IEditorGroupsService),
    __param(7, configuration_1.IConfigurationService)
], PartsSplash);
platform_1.Registry.as(contributions_1.Extensions.Workbench).registerWorkbenchContribution(PartsSplash, 1 /* Starting */);
//# sourceMappingURL=partsSplash.contribution.js.map