"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
const files_1 = require("vs/platform/files/common/files");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const windows_1 = require("vs/platform/windows/common/windows");
const notification_1 = require("vs/platform/notification/common/notification");
const quickInput_1 = require("vs/platform/quickinput/common/quickInput");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const network_1 = require("vs/base/common/network");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const nls_1 = require("vs/nls");
const severity_1 = require("vs/base/common/severity");
const resources_1 = require("vs/base/common/resources");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const workspaceStats_1 = require("vs/workbench/contrib/stats/common/workspaceStats");
const workspaceStats_2 = require("vs/workbench/contrib/stats/electron-browser/workspaceStats");
const ModulesToLookFor = [
    // Packages that suggest a node server
    'express',
    'sails',
    'koa',
    'hapi',
    'socket.io',
    'restify',
    // JS frameworks
    'react',
    'react-native',
    'rnpm-plugin-windows',
    '@angular/core',
    '@ionic',
    'vue',
    'tns-core-modules',
    // Other interesting packages
    'aws-sdk',
    'aws-amplify',
    'azure',
    'azure-storage',
    'firebase',
    '@google-cloud/common',
    'heroku-cli',
    //Office and Sharepoint packages
    '@microsoft/office-js',
    '@microsoft/office-js-helpers',
    '@types/office-js',
    '@types/office-runtime',
    'office-ui-fabric-react',
    '@uifabric/icons',
    '@uifabric/merge-styles',
    '@uifabric/styling',
    '@uifabric/experiments',
    '@uifabric/utilities',
    '@microsoft/rush',
    'lerna',
    'just-task',
    'beachball'
];
const PyModulesToLookFor = [
    'azure',
    'azure-storage-common',
    'azure-storage-blob',
    'azure-storage-file',
    'azure-storage-queue',
    'azure-shell',
    'azure-cosmos',
    'azure-devtools',
    'azure-elasticluster',
    'azure-eventgrid',
    'azure-functions',
    'azure-graphrbac',
    'azure-keyvault',
    'azure-loganalytics',
    'azure-monitor',
    'azure-servicebus',
    'azure-servicefabric',
    'azure-storage',
    'azure-translator',
    'azure-iothub-device-client',
    'adal',
    'pydocumentdb',
    'botbuilder-core',
    'botbuilder-schema',
    'botframework-connector'
];
let WorkspaceStatsService = class WorkspaceStatsService {
    constructor(fileService, contextService, environmentService, windowService, notificationService, quickInputService, textFileService) {
        this.fileService = fileService;
        this.contextService = contextService;
        this.environmentService = environmentService;
        this.windowService = windowService;
        this.notificationService = notificationService;
        this.quickInputService = quickInputService;
        this.textFileService = textFileService;
    }
    getTags() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._tags) {
                this._tags = yield this.resolveWorkspaceTags(this.environmentService.configuration, rootFiles => this.handleWorkspaceFiles(rootFiles));
            }
            return this._tags;
        });
    }
    getTelemetryWorkspaceId(workspace, state) {
        function createHash(uri) {
            return crypto.createHash('sha1').update(uri.scheme === network_1.Schemas.file ? uri.fsPath : uri.toString()).digest('hex');
        }
        let workspaceId;
        switch (state) {
            case 1 /* EMPTY */:
                workspaceId = undefined;
                break;
            case 2 /* FOLDER */:
                workspaceId = createHash(workspace.folders[0].uri);
                break;
            case 3 /* WORKSPACE */:
                if (workspace.configuration) {
                    workspaceId = createHash(workspace.configuration);
                }
        }
        return workspaceId;
    }
    getHashedRemotesFromUri(workspaceUri, stripEndingDotGit = false) {
        const path = workspaceUri.path;
        const uri = workspaceUri.with({ path: `${path !== '/' ? path : ''}/.git/config` });
        return this.fileService.exists(uri).then(exists => {
            if (!exists) {
                return [];
            }
            return this.textFileService.read(uri, { acceptTextOnly: true }).then(content => workspaceStats_2.getHashedRemotesFromConfig(content.value, stripEndingDotGit), err => [] // ignore missing or binary file
            );
        });
    }
    /* __GDPR__FRAGMENT__
        "WorkspaceTags" : {
            "workbench.filesToOpenOrCreate" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workbench.filesToDiff" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.id" : { "classification": "SystemMetaData", "purpose": "FeatureInsight" },
            "workspace.roots" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.empty" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.grunt" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.gulp" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.jake" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.tsconfig" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.jsconfig" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.config.xml" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.vsc.extension" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.asp<NUMBER>" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.sln" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.unity" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.express" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.sails" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.koa" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.hapi" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.socket.io" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.restify" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.rnpm-plugin-windows" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.react" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@angular/core" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.vue" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.aws-sdk" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.aws-amplify-sdk" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.azure" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.azure-storage" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@google-cloud/common" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.firebase" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.heroku-cli" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@microsoft/office-js" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@microsoft/office-js-helpers" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@types/office-js" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@types/office-runtime" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.office-ui-fabric-react" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@uifabric/icons" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@uifabric/merge-styles" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@uifabric/styling" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@uifabric/experiments" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@uifabric/utilities" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.@microsoft/rush" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.lerna" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.just-task" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.npm.beachball" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.bower" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.yeoman.code.ext" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.cordova.high" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.cordova.low" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.xamarin.android" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.xamarin.ios" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.android.cpp" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.reactNative" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.ionic" : { "classification" : "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": "true" },
            "workspace.nativeScript" : { "classification" : "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": "true" },
            "workspace.java.pom" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.requirements" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.requirements.star" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.Pipfile" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.conda" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.any-azure" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-storage-common" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-storage-blob" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-storage-file" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-storage-queue" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-mgmt" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-shell" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.pulumi-azure" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-cosmos" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-devtools" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-elasticluster" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-eventgrid" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-functions" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-graphrbac" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-keyvault" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-loganalytics" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-monitor" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-servicebus" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-servicefabric" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-storage" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-translator" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-iothub-device-client" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-ml" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.azure-cognitiveservices" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.adal" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.pydocumentdb" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.botbuilder-core" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.botbuilder-schema" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true },
            "workspace.py.botframework-connector" : { "classification": "SystemMetaData", "purpose": "FeatureInsight", "isMeasurement": true }
        }
    */
    resolveWorkspaceTags(configuration, participant) {
        const tags = Object.create(null);
        const state = this.contextService.getWorkbenchState();
        const workspace = this.contextService.getWorkspace();
        tags['workspace.id'] = this.getTelemetryWorkspaceId(workspace, state);
        const { filesToOpenOrCreate, filesToDiff } = configuration;
        tags['workbench.filesToOpenOrCreate'] = filesToOpenOrCreate && filesToOpenOrCreate.length || 0;
        tags['workbench.filesToDiff'] = filesToDiff && filesToDiff.length || 0;
        const isEmpty = state === 1 /* EMPTY */;
        tags['workspace.roots'] = isEmpty ? 0 : workspace.folders.length;
        tags['workspace.empty'] = isEmpty;
        const folders = !isEmpty ? workspace.folders.map(folder => folder.uri) : this.environmentService.appQuality !== 'stable' && this.findFolders(configuration);
        if (!folders || !folders.length || !this.fileService) {
            return Promise.resolve(tags);
        }
        return this.fileService.resolveAll(folders.map(resource => ({ resource }))).then((files) => {
            const names = [].concat(...files.map(result => result.success ? (result.stat.children || []) : [])).map(c => c.name);
            const nameSet = names.reduce((s, n) => s.add(n.toLowerCase()), new Set());
            if (participant) {
                participant(names);
            }
            tags['workspace.grunt'] = nameSet.has('gruntfile.js');
            tags['workspace.gulp'] = nameSet.has('gulpfile.js');
            tags['workspace.jake'] = nameSet.has('jakefile.js');
            tags['workspace.tsconfig'] = nameSet.has('tsconfig.json');
            tags['workspace.jsconfig'] = nameSet.has('jsconfig.json');
            tags['workspace.config.xml'] = nameSet.has('config.xml');
            tags['workspace.vsc.extension'] = nameSet.has('vsc-extension-quickstart.md');
            tags['workspace.ASP5'] = nameSet.has('project.json') && this.searchArray(names, /^.+\.cs$/i);
            tags['workspace.sln'] = this.searchArray(names, /^.+\.sln$|^.+\.csproj$/i);
            tags['workspace.unity'] = nameSet.has('assets') && nameSet.has('library') && nameSet.has('projectsettings');
            tags['workspace.npm'] = nameSet.has('package.json') || nameSet.has('node_modules');
            tags['workspace.bower'] = nameSet.has('bower.json') || nameSet.has('bower_components');
            tags['workspace.java.pom'] = nameSet.has('pom.xml');
            tags['workspace.yeoman.code.ext'] = nameSet.has('vsc-extension-quickstart.md');
            tags['workspace.py.requirements'] = nameSet.has('requirements.txt');
            tags['workspace.py.requirements.star'] = this.searchArray(names, /^(.*)requirements(.*)\.txt$/i);
            tags['workspace.py.Pipfile'] = nameSet.has('pipfile');
            tags['workspace.py.conda'] = this.searchArray(names, /^environment(\.yml$|\.yaml$)/i);
            const mainActivity = nameSet.has('mainactivity.cs') || nameSet.has('mainactivity.fs');
            const appDelegate = nameSet.has('appdelegate.cs') || nameSet.has('appdelegate.fs');
            const androidManifest = nameSet.has('androidmanifest.xml');
            const platforms = nameSet.has('platforms');
            const plugins = nameSet.has('plugins');
            const www = nameSet.has('www');
            const properties = nameSet.has('properties');
            const resources = nameSet.has('resources');
            const jni = nameSet.has('jni');
            if (tags['workspace.config.xml'] &&
                !tags['workspace.language.cs'] && !tags['workspace.language.vb'] && !tags['workspace.language.aspx']) {
                if (platforms && plugins && www) {
                    tags['workspace.cordova.high'] = true;
                }
                else {
                    tags['workspace.cordova.low'] = true;
                }
            }
            if (tags['workspace.config.xml'] &&
                !tags['workspace.language.cs'] && !tags['workspace.language.vb'] && !tags['workspace.language.aspx']) {
                if (nameSet.has('ionic.config.json')) {
                    tags['workspace.ionic'] = true;
                }
            }
            if (mainActivity && properties && resources) {
                tags['workspace.xamarin.android'] = true;
            }
            if (appDelegate && resources) {
                tags['workspace.xamarin.ios'] = true;
            }
            if (androidManifest && jni) {
                tags['workspace.android.cpp'] = true;
            }
            function getFilePromises(filename, fileService, textFileService, contentHandler) {
                return !nameSet.has(filename) ? [] : folders.map(workspaceUri => {
                    const uri = workspaceUri.with({ path: `${workspaceUri.path !== '/' ? workspaceUri.path : ''}/${filename}` });
                    return fileService.exists(uri).then(exists => {
                        if (!exists) {
                            return undefined;
                        }
                        return textFileService.read(uri, { acceptTextOnly: true }).then(contentHandler);
                    }, err => {
                        // Ignore missing file
                    });
                });
            }
            function addPythonTags(packageName) {
                if (PyModulesToLookFor.indexOf(packageName) > -1) {
                    tags['workspace.py.' + packageName] = true;
                }
                // cognitive services has a lot of tiny packages. e.g. 'azure-cognitiveservices-search-autosuggest'
                if (packageName.indexOf('azure-cognitiveservices') > -1) {
                    tags['workspace.py.azure-cognitiveservices'] = true;
                }
                if (packageName.indexOf('azure-mgmt') > -1) {
                    tags['workspace.py.azure-mgmt'] = true;
                }
                if (packageName.indexOf('azure-ml') > -1) {
                    tags['workspace.py.azure-ml'] = true;
                }
                if (!tags['workspace.py.any-azure']) {
                    tags['workspace.py.any-azure'] = /azure/i.test(packageName);
                }
            }
            const requirementsTxtPromises = getFilePromises('requirements.txt', this.fileService, this.textFileService, content => {
                const dependencies = content.value.split(/\r\n|\r|\n/);
                for (let dependency of dependencies) {
                    // Dependencies in requirements.txt can have 3 formats: `foo==3.1, foo>=3.1, foo`
                    const format1 = dependency.split('==');
                    const format2 = dependency.split('>=');
                    const packageName = (format1.length === 2 ? format1[0] : format2[0]).trim();
                    addPythonTags(packageName);
                }
            });
            const pipfilePromises = getFilePromises('pipfile', this.fileService, this.textFileService, content => {
                let dependencies = content.value.split(/\r\n|\r|\n/);
                // We're only interested in the '[packages]' section of the Pipfile
                dependencies = dependencies.slice(dependencies.indexOf('[packages]') + 1);
                for (let dependency of dependencies) {
                    if (dependency.trim().indexOf('[') > -1) {
                        break;
                    }
                    // All dependencies in Pipfiles follow the format: `<package> = <version, or git repo, or something else>`
                    if (dependency.indexOf('=') === -1) {
                        continue;
                    }
                    const packageName = dependency.split('=')[0].trim();
                    addPythonTags(packageName);
                }
            });
            const packageJsonPromises = getFilePromises('package.json', this.fileService, this.textFileService, content => {
                try {
                    const packageJsonContents = JSON.parse(content.value);
                    let dependencies = packageJsonContents['dependencies'];
                    let devDependencies = packageJsonContents['devDependencies'];
                    for (let module of ModulesToLookFor) {
                        if ('react-native' === module) {
                            if ((dependencies && dependencies[module]) || (devDependencies && devDependencies[module])) {
                                tags['workspace.reactNative'] = true;
                            }
                        }
                        else if ('tns-core-modules' === module) {
                            if ((dependencies && dependencies[module]) || (devDependencies && devDependencies[module])) {
                                tags['workspace.nativescript'] = true;
                            }
                        }
                        else {
                            if ((dependencies && dependencies[module]) || (devDependencies && devDependencies[module])) {
                                tags['workspace.npm.' + module] = true;
                            }
                        }
                    }
                }
                catch (e) {
                    // Ignore errors when resolving file or parsing file contents
                }
            });
            return Promise.all([...packageJsonPromises, ...requirementsTxtPromises, ...pipfilePromises]).then(() => tags);
        });
    }
    handleWorkspaceFiles(rootFiles) {
        const state = this.contextService.getWorkbenchState();
        const workspace = this.contextService.getWorkspace();
        // Handle top-level workspace files for local single folder workspace
        if (state === 2 /* FOLDER */) {
            const workspaceFiles = rootFiles.filter(workspaces_1.hasWorkspaceFileExtension);
            if (workspaceFiles.length > 0) {
                this.doHandleWorkspaceFiles(workspace.folders[0].uri, workspaceFiles);
            }
        }
    }
    doHandleWorkspaceFiles(folder, workspaces) {
        const neverShowAgain = { id: 'workspaces.dontPromptToOpen', scope: notification_1.NeverShowAgainScope.WORKSPACE, isSecondary: true };
        // Prompt to open one workspace
        if (workspaces.length === 1) {
            const workspaceFile = workspaces[0];
            this.notificationService.prompt(severity_1.default.Info, nls_1.localize('workspaceFound', "This folder contains a workspace file '{0}'. Do you want to open it? [Learn more]({1}) about workspace files.", workspaceFile, 'https://go.microsoft.com/fwlink/?linkid=2025315'), [{
                    label: nls_1.localize('openWorkspace', "Open Workspace"),
                    run: () => this.windowService.openWindow([{ workspaceUri: resources_1.joinPath(folder, workspaceFile) }])
                }], { neverShowAgain });
        }
        // Prompt to select a workspace from many
        else if (workspaces.length > 1) {
            this.notificationService.prompt(severity_1.default.Info, nls_1.localize('workspacesFound', "This folder contains multiple workspace files. Do you want to open one? [Learn more]({0}) about workspace files.", 'https://go.microsoft.com/fwlink/?linkid=2025315'), [{
                    label: nls_1.localize('selectWorkspace', "Select Workspace"),
                    run: () => {
                        this.quickInputService.pick(workspaces.map(workspace => ({ label: workspace })), { placeHolder: nls_1.localize('selectToOpen', "Select a workspace to open") }).then(pick => {
                            if (pick) {
                                this.windowService.openWindow([{ workspaceUri: resources_1.joinPath(folder, pick.label) }]);
                            }
                        });
                    }
                }], { neverShowAgain });
        }
    }
    findFolders(configuration) {
        const folder = this.findFolder(configuration);
        return folder && [folder];
    }
    findFolder({ filesToOpenOrCreate, filesToDiff }) {
        if (filesToOpenOrCreate && filesToOpenOrCreate.length) {
            return this.parentURI(filesToOpenOrCreate[0].fileUri);
        }
        else if (filesToDiff && filesToDiff.length) {
            return this.parentURI(filesToDiff[0].fileUri);
        }
        return undefined;
    }
    parentURI(uri) {
        if (!uri) {
            return undefined;
        }
        const path = uri.path;
        const i = path.lastIndexOf('/');
        return i !== -1 ? uri.with({ path: path.substr(0, i) }) : undefined;
    }
    searchArray(arr, regEx) {
        return arr.some(v => v.search(regEx) > -1) || undefined;
    }
};
WorkspaceStatsService = __decorate([
    __param(0, files_1.IFileService),
    __param(1, workspace_1.IWorkspaceContextService),
    __param(2, environmentService_1.IWorkbenchEnvironmentService),
    __param(3, windows_1.IWindowService),
    __param(4, notification_1.INotificationService),
    __param(5, quickInput_1.IQuickInputService),
    __param(6, textfiles_1.ITextFileService)
], WorkspaceStatsService);
exports.WorkspaceStatsService = WorkspaceStatsService;
extensions_1.registerSingleton(workspaceStats_1.IWorkspaceStatsService, WorkspaceStatsService, true);
//# sourceMappingURL=workspaceStatsService.js.map