"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const tasks_1 = require("vs/workbench/contrib/tasks/common/tasks");
const terminalTaskSystem_1 = require("./terminalTaskSystem");
const abstractTaskService_1 = require("vs/workbench/contrib/tasks/browser/abstractTaskService");
const taskService_1 = require("vs/workbench/contrib/tasks/common/taskService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
class TaskService extends abstractTaskService_1.AbstractTaskService {
    getTaskSystem() {
        if (this._taskSystem) {
            return this._taskSystem;
        }
        if (this.executionEngine === tasks_1.ExecutionEngine.Terminal) {
            this._taskSystem = this.createTerminalTaskSystem();
        }
        else {
            throw new Error(TaskService.ProcessTaskSystemSupportMessage);
        }
        this._taskSystemListener = this._taskSystem.onDidStateChange((event) => {
            if (this._taskSystem) {
                this._taskRunningState.set(this._taskSystem.isActiveSync());
            }
            this._onDidStateChange.fire(event);
        });
        return this._taskSystem;
    }
    updateWorkspaceTasks(runSource = 1 /* User */) {
        this._workspaceTasksPromise = this.computeWorkspaceTasks(runSource).then(value => {
            if (this.executionEngine !== tasks_1.ExecutionEngine.Terminal || ((this._taskSystem !== undefined) && !(this._taskSystem instanceof terminalTaskSystem_1.TerminalTaskSystem))) {
                throw new Error(TaskService.ProcessTaskSystemSupportMessage);
            }
            return value;
        });
    }
    computeLegacyConfiguration(workspaceFolder) {
        throw new Error(TaskService.ProcessTaskSystemSupportMessage);
    }
    versionAndEngineCompatible(filter) {
        return this.executionEngine === tasks_1.ExecutionEngine.Terminal;
    }
}
exports.TaskService = TaskService;
TaskService.ProcessTaskSystemSupportMessage = nls.localize('taskService.processTaskSystem', 'Process task system is not support in the web.');
extensions_1.registerSingleton(taskService_1.ITaskService, TaskService, true);
//# sourceMappingURL=taskService.js.map