"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const terminal_1 = require("vs/workbench/contrib/terminal/browser/terminal");
const event_1 = require("vs/base/common/event");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
let Terminal;
let WebLinksAddon;
let SearchAddon;
class TerminalInstanceService {
    constructor() {
        this._onRequestDefaultShellAndArgs = new event_1.Emitter();
    }
    get onRequestDefaultShellAndArgs() { return this._onRequestDefaultShellAndArgs.event; }
    getXtermConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Terminal) {
                Terminal = (yield Promise.resolve().then(() => require('xterm'))).Terminal;
            }
            return Terminal;
        });
    }
    getXtermWebLinksConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!WebLinksAddon) {
                WebLinksAddon = (yield Promise.resolve().then(() => require('xterm-addon-web-links'))).WebLinksAddon;
            }
            return WebLinksAddon;
        });
    }
    getXtermSearchConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!SearchAddon) {
                SearchAddon = (yield Promise.resolve().then(() => require('xterm-addon-search'))).SearchAddon;
            }
            return SearchAddon;
        });
    }
    createWindowsShellHelper() {
        throw new Error('Not implemented');
    }
    createTerminalProcess() {
        throw new Error('Not implemented');
    }
    getDefaultShellAndArgs(useAutomationShell) {
        return new Promise(r => this._onRequestDefaultShellAndArgs.fire({
            useAutomationShell,
            callback: (shell, args) => r({ shell, args })
        }));
    }
    getMainProcessParentEnv() {
        return __awaiter(this, void 0, void 0, function* () {
            return {};
        });
    }
}
exports.TerminalInstanceService = TerminalInstanceService;
extensions_1.registerSingleton(terminal_1.ITerminalInstanceService, TerminalInstanceService, true);
//# sourceMappingURL=terminalInstanceService.js.map