"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const nls = require("vs/nls");
const actions_1 = require("vs/platform/actions/common/actions");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
function setupTerminalMenu() {
    // View menu
    actions_1.MenuRegistry.appendMenuItem(26 /* MenubarViewMenu */, {
        group: '4_panels',
        command: {
            id: "workbench.action.terminal.toggleTerminal" /* TOGGLE */,
            title: nls.localize({ key: 'miToggleIntegratedTerminal', comment: ['&& denotes a mnemonic'] }, "&&Terminal")
        },
        order: 3
    });
    // Manage
    const createGroup = '1_create';
    actions_1.MenuRegistry.appendMenuItem(25 /* MenubarTerminalMenu */, {
        group: createGroup,
        command: {
            id: "workbench.action.terminal.new" /* NEW */,
            title: nls.localize({ key: 'miNewTerminal', comment: ['&& denotes a mnemonic'] }, "&&New Terminal")
        },
        order: 1
    });
    actions_1.MenuRegistry.appendMenuItem(25 /* MenubarTerminalMenu */, {
        group: createGroup,
        command: {
            id: "workbench.action.terminal.split" /* SPLIT */,
            title: nls.localize({ key: 'miSplitTerminal', comment: ['&& denotes a mnemonic'] }, "&&Split Terminal"),
            precondition: contextkey_1.ContextKeyExpr.has('terminalIsOpen')
        },
        order: 2
    });
    // Run
    const runGroup = '2_run';
    actions_1.MenuRegistry.appendMenuItem(25 /* MenubarTerminalMenu */, {
        group: runGroup,
        command: {
            id: "workbench.action.terminal.runActiveFile" /* RUN_ACTIVE_FILE */,
            title: nls.localize({ key: 'miRunActiveFile', comment: ['&& denotes a mnemonic'] }, "Run &&Active File")
        },
        order: 3
    });
    actions_1.MenuRegistry.appendMenuItem(25 /* MenubarTerminalMenu */, {
        group: runGroup,
        command: {
            id: "workbench.action.terminal.runSelectedText" /* RUN_SELECTED_TEXT */,
            title: nls.localize({ key: 'miRunSelectedText', comment: ['&& denotes a mnemonic'] }, "Run &&Selected Text")
        },
        order: 4
    });
}
exports.setupTerminalMenu = setupTerminalMenu;
//# sourceMappingURL=terminalMenu.js.map