"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const terminal_1 = require("vs/workbench/contrib/terminal/common/terminal");
const windowsShellHelper_1 = require("vs/workbench/contrib/terminal/electron-browser/windowsShellHelper");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const platform_1 = require("vs/base/common/platform");
const terminalProcess_1 = require("vs/workbench/contrib/terminal/node/terminalProcess");
const terminal_2 = require("vs/workbench/contrib/terminal/node/terminal");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const terminalEnvironment_1 = require("vs/workbench/contrib/terminal/common/terminalEnvironment");
const storage_1 = require("vs/platform/storage/common/storage");
const terminalEnvironment_2 = require("vs/workbench/contrib/terminal/node/terminalEnvironment");
const configurationResolver_1 = require("vs/workbench/services/configurationResolver/common/configurationResolver");
const history_1 = require("vs/workbench/services/history/common/history");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const log_1 = require("vs/platform/log/common/log");
let Terminal;
let WebLinksAddon;
let SearchAddon;
let TerminalInstanceService = class TerminalInstanceService {
    constructor(_instantiationService, _configurationService, _storageService, _configurationResolverService, _workspaceContextService, _historyService, _logService) {
        this._instantiationService = _instantiationService;
        this._configurationService = _configurationService;
        this._storageService = _storageService;
        this._configurationResolverService = _configurationResolverService;
        this._workspaceContextService = _workspaceContextService;
        this._historyService = _historyService;
        this._logService = _logService;
    }
    getXtermConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!Terminal) {
                Terminal = (yield Promise.resolve().then(() => require('xterm'))).Terminal;
            }
            return Terminal;
        });
    }
    getXtermWebLinksConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!WebLinksAddon) {
                WebLinksAddon = (yield Promise.resolve().then(() => require('xterm-addon-web-links'))).WebLinksAddon;
            }
            return WebLinksAddon;
        });
    }
    getXtermSearchConstructor() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!SearchAddon) {
                SearchAddon = (yield Promise.resolve().then(() => require('xterm-addon-search'))).SearchAddon;
            }
            return SearchAddon;
        });
    }
    createWindowsShellHelper(shellProcessId, instance, xterm) {
        return new windowsShellHelper_1.WindowsShellHelper(shellProcessId, instance, xterm);
    }
    createTerminalProcess(shellLaunchConfig, cwd, cols, rows, env, windowsEnableConpty) {
        return this._instantiationService.createInstance(terminalProcess_1.TerminalProcess, shellLaunchConfig, cwd, cols, rows, env, windowsEnableConpty);
    }
    _isWorkspaceShellAllowed() {
        return this._storageService.getBoolean(terminal_1.IS_WORKSPACE_SHELL_ALLOWED_STORAGE_KEY, 1 /* WORKSPACE */, false);
    }
    getDefaultShellAndArgs(useAutomationShell, platformOverride = platform_1.platform) {
        const isWorkspaceShellAllowed = this._isWorkspaceShellAllowed();
        const activeWorkspaceRootUri = this._historyService.getLastActiveWorkspaceRoot();
        let lastActiveWorkspace = activeWorkspaceRootUri ? this._workspaceContextService.getWorkspaceFolder(activeWorkspaceRootUri) : undefined;
        lastActiveWorkspace = lastActiveWorkspace === null ? undefined : lastActiveWorkspace;
        const shell = terminalEnvironment_1.getDefaultShell((key) => this._configurationService.inspect(key), isWorkspaceShellAllowed, terminal_2.getSystemShell(platformOverride), process.env.hasOwnProperty('PROCESSOR_ARCHITEW6432'), process.env.windir, lastActiveWorkspace, this._configurationResolverService, this._logService, useAutomationShell, platformOverride);
        const args = terminalEnvironment_1.getDefaultShellArgs((key) => this._configurationService.inspect(key), isWorkspaceShellAllowed, useAutomationShell, lastActiveWorkspace, this._configurationResolverService, this._logService, platformOverride);
        return Promise.resolve({ shell, args });
    }
    getMainProcessParentEnv() {
        return terminalEnvironment_2.getMainProcessParentEnv();
    }
};
TerminalInstanceService = __decorate([
    __param(0, instantiation_1.IInstantiationService),
    __param(1, configuration_1.IConfigurationService),
    __param(2, storage_1.IStorageService),
    __param(3, configurationResolver_1.IConfigurationResolverService),
    __param(4, workspace_1.IWorkspaceContextService),
    __param(5, history_1.IHistoryService),
    __param(6, log_1.ILogService)
], TerminalInstanceService);
exports.TerminalInstanceService = TerminalInstanceService;
//# sourceMappingURL=terminalInstanceService.js.map