"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const colorRegistry_1 = require("vs/platform/theme/common/colorRegistry");
const platform_1 = require("vs/platform/registry/common/platform");
const terminalColorRegistry_1 = require("vs/workbench/contrib/terminal/common/terminalColorRegistry");
const color_1 = require("vs/base/common/color");
terminalColorRegistry_1.registerColors();
let themingRegistry = platform_1.Registry.as(colorRegistry_1.Extensions.ColorContribution);
function getMockTheme(type) {
    let theme = {
        selector: '',
        label: '',
        type: type,
        getColor: (colorId) => themingRegistry.resolveDefaultColor(colorId, theme),
        defines: () => true
    };
    return theme;
}
suite('Workbench - TerminalColorRegistry', () => {
    test('hc colors', function () {
        let theme = getMockTheme('hc');
        let colors = terminalColorRegistry_1.ansiColorIdentifiers.map(colorId => color_1.Color.Format.CSS.formatHexA(theme.getColor(colorId), true));
        assert.deepEqual(colors, [
            '#000000',
            '#cd0000',
            '#00cd00',
            '#cdcd00',
            '#0000ee',
            '#cd00cd',
            '#00cdcd',
            '#e5e5e5',
            '#7f7f7f',
            '#ff0000',
            '#00ff00',
            '#ffff00',
            '#5c5cff',
            '#ff00ff',
            '#00ffff',
            '#ffffff'
        ], 'The high contrast terminal colors should be used when the hc theme is active');
    });
    test('light colors', function () {
        let theme = getMockTheme('light');
        let colors = terminalColorRegistry_1.ansiColorIdentifiers.map(colorId => color_1.Color.Format.CSS.formatHexA(theme.getColor(colorId), true));
        assert.deepEqual(colors, [
            '#000000',
            '#cd3131',
            '#00bc00',
            '#949800',
            '#0451a5',
            '#bc05bc',
            '#0598bc',
            '#555555',
            '#666666',
            '#cd3131',
            '#14ce14',
            '#b5ba00',
            '#0451a5',
            '#bc05bc',
            '#0598bc',
            '#a5a5a5'
        ], 'The light terminal colors should be used when the light theme is active');
    });
    test('dark colors', function () {
        let theme = getMockTheme('dark');
        let colors = terminalColorRegistry_1.ansiColorIdentifiers.map(colorId => color_1.Color.Format.CSS.formatHexA(theme.getColor(colorId), true));
        assert.deepEqual(colors, [
            '#000000',
            '#cd3131',
            '#0dbc79',
            '#e5e510',
            '#2472c8',
            '#bc3fbc',
            '#11a8cd',
            '#e5e5e5',
            '#666666',
            '#f14c4c',
            '#23d18b',
            '#f5f543',
            '#3b8eea',
            '#d670d6',
            '#29b8db',
            '#e5e5e5'
        ], 'The dark terminal colors should be used when a dark theme is active');
    });
});
//# sourceMappingURL=terminalColorRegistry.test.js.map