"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("vs/base/common/errors");
const platform_1 = require("vs/base/common/platform");
const uri_1 = require("vs/base/common/uri");
const modes_1 = require("vs/editor/common/modes");
const tokenization_1 = require("vs/editor/common/modes/supports/tokenization");
const modeService_1 = require("vs/editor/common/services/modeService");
const nls = require("vs/nls");
const environment_1 = require("vs/platform/environment/common/environment");
const keybinding_1 = require("vs/platform/keybinding/common/keybinding");
const opener_1 = require("vs/platform/opener/common/opener");
const request_1 = require("vs/platform/request/common/request");
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const productService_1 = require("vs/platform/product/common/productService");
const webviewEditorService_1 = require("vs/workbench/contrib/webview/browser/webviewEditorService");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const keybindingParser_1 = require("vs/base/common/keybindingParser");
const cancellation_1 = require("vs/base/common/cancellation");
const extensions_1 = require("vs/workbench/services/extensions/common/extensions");
const editorGroupsService_1 = require("vs/workbench/services/editor/common/editorGroupsService");
const uuid_1 = require("vs/base/common/uuid");
const markdownDocumentRenderer_1 = require("vs/workbench/common/markdownDocumentRenderer");
let ReleaseNotesManager = class ReleaseNotesManager {
    constructor(_environmentService, _keybindingService, _modeService, _openerService, _requestService, _telemetryService, _editorService, _editorGroupService, _webviewEditorService, _extensionService, _productService) {
        this._environmentService = _environmentService;
        this._keybindingService = _keybindingService;
        this._modeService = _modeService;
        this._openerService = _openerService;
        this._requestService = _requestService;
        this._telemetryService = _telemetryService;
        this._editorService = _editorService;
        this._editorGroupService = _editorGroupService;
        this._webviewEditorService = _webviewEditorService;
        this._extensionService = _extensionService;
        this._productService = _productService;
        this._releaseNotesCache = new Map();
        this._currentReleaseNotes = undefined;
        modes_1.TokenizationRegistry.onDidChange(() => __awaiter(this, void 0, void 0, function* () {
            if (!this._currentReleaseNotes || !this._lastText) {
                return;
            }
            const html = yield this.renderBody(this._lastText);
            if (this._currentReleaseNotes) {
                this._currentReleaseNotes.webview.html = html;
            }
        }));
    }
    show(accessor, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const releaseNoteText = yield this.loadReleaseNotes(version);
            this._lastText = releaseNoteText;
            const html = yield this.renderBody(releaseNoteText);
            const title = nls.localize('releaseNotesInputName', "Release Notes: {0}", version);
            const activeControl = this._editorService.activeControl;
            if (this._currentReleaseNotes) {
                this._currentReleaseNotes.setName(title);
                this._currentReleaseNotes.webview.html = html;
                this._webviewEditorService.revealWebview(this._currentReleaseNotes, activeControl ? activeControl.group : this._editorGroupService.activeGroup, false);
            }
            else {
                this._currentReleaseNotes = this._webviewEditorService.createWebview(uuid_1.generateUuid(), 'releaseNotes', title, { group: editorService_1.ACTIVE_GROUP, preserveFocus: false }, {
                    tryRestoreScrollPosition: true,
                    enableFindWidget: true,
                    localResourceRoots: [
                        uri_1.URI.parse(require.toUrl('./media'))
                    ]
                }, undefined);
                this._currentReleaseNotes.webview.onDidClickLink(uri => this.onDidClickLink(uri));
                this._currentReleaseNotes.onDispose(() => { this._currentReleaseNotes = undefined; });
                const iconPath = uri_1.URI.parse(require.toUrl('./media/code-icon.svg'));
                this._currentReleaseNotes.iconPath = {
                    light: iconPath,
                    dark: iconPath
                };
                this._currentReleaseNotes.webview.html = html;
            }
            return true;
        });
    }
    loadReleaseNotes(version) {
        const match = /^(\d+\.\d+)\./.exec(version);
        if (!match) {
            return Promise.reject(new Error('not found'));
        }
        const versionLabel = match[1].replace(/\./g, '_');
        const baseUrl = 'https://code.visualstudio.com/raw';
        const url = `${baseUrl}/v${versionLabel}.md`;
        const unassigned = nls.localize('unassigned', "unassigned");
        const patchKeybindings = (text) => {
            const kb = (match, kb) => {
                const keybinding = this._keybindingService.lookupKeybinding(kb);
                if (!keybinding) {
                    return unassigned;
                }
                return keybinding.getLabel() || unassigned;
            };
            const kbstyle = (match, kb) => {
                const keybinding = keybindingParser_1.KeybindingParser.parseKeybinding(kb, platform_1.OS);
                if (!keybinding) {
                    return unassigned;
                }
                const resolvedKeybindings = this._keybindingService.resolveKeybinding(keybinding);
                if (resolvedKeybindings.length === 0) {
                    return unassigned;
                }
                return resolvedKeybindings[0].getLabel() || unassigned;
            };
            return text
                .replace(/kb\(([a-z.\d\-]+)\)/gi, kb)
                .replace(/kbstyle\(([^\)]+)\)/gi, kbstyle);
        };
        if (!this._releaseNotesCache.has(version)) {
            this._releaseNotesCache.set(version, this._requestService.request({ url }, cancellation_1.CancellationToken.None)
                .then(request_1.asText)
                .then(text => {
                if (!text || !/^#\s/.test(text)) { // release notes always starts with `#` followed by whitespace
                    return Promise.reject(new Error('Invalid release notes'));
                }
                return Promise.resolve(text);
            })
                .then(text => patchKeybindings(text)));
        }
        return this._releaseNotesCache.get(version);
    }
    onDidClickLink(uri) {
        this.addGAParameters(uri, 'ReleaseNotes')
            .then(updated => this._openerService.open(updated))
            .then(undefined, errors_1.onUnexpectedError);
    }
    addGAParameters(uri, origin, experiment = '1') {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._environmentService.isBuilt && !this._environmentService.isExtensionDevelopment && !this._environmentService.args['disable-telemetry'] && !!this._productService.enableTelemetry) {
                if (uri.scheme === 'https' && uri.authority === 'code.visualstudio.com') {
                    const info = yield this._telemetryService.getTelemetryInfo();
                    return uri.with({ query: `${uri.query ? uri.query + '&' : ''}utm_source=VsCode&utm_medium=${encodeURIComponent(origin)}&utm_campaign=${encodeURIComponent(info.instanceId)}&utm_content=${encodeURIComponent(experiment)}` });
                }
            }
            return uri;
        });
    }
    renderBody(text) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = yield markdownDocumentRenderer_1.renderMarkdownDocument(text, this._extensionService, this._modeService);
            const colorMap = modes_1.TokenizationRegistry.getColorMap();
            const css = colorMap ? tokenization_1.generateTokensCSSForColorMap(colorMap) : '';
            const styleSheetPath = require.toUrl('./media/markdown.css').replace('file://', 'vscode-resource://');
            return `<!DOCTYPE html>
		<html>
			<head>
				<base href="https://code.visualstudio.com/raw/">
				<meta http-equiv="Content-type" content="text/html;charset=UTF-8">
				<meta http-equiv="Content-Security-Policy" content="default-src 'none'; img-src https: data:; media-src https:; script-src 'none'; style-src vscode-resource: https: 'unsafe-inline'; child-src 'none'; frame-src 'none';">
				<link rel="stylesheet" type="text/css" href="${styleSheetPath}">
				<style>${css}</style>
			</head>
			<body>${content}</body>
		</html>`;
        });
    }
};
ReleaseNotesManager = __decorate([
    __param(0, environment_1.IEnvironmentService),
    __param(1, keybinding_1.IKeybindingService),
    __param(2, modeService_1.IModeService),
    __param(3, opener_1.IOpenerService),
    __param(4, request_1.IRequestService),
    __param(5, telemetry_1.ITelemetryService),
    __param(6, editorService_1.IEditorService),
    __param(7, editorGroupsService_1.IEditorGroupsService),
    __param(8, webviewEditorService_1.IWebviewEditorService),
    __param(9, extensions_1.IExtensionService),
    __param(10, productService_1.IProductService)
], ReleaseNotesManager);
exports.ReleaseNotesManager = ReleaseNotesManager;
//# sourceMappingURL=releaseNotesEditor.js.map