"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const contributions_1 = require("vs/workbench/common/contributions");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const nls_1 = require("vs/nls");
const lifecycle_1 = require("vs/base/common/lifecycle");
const commands_1 = require("vs/platform/commands/common/commands");
const platform_1 = require("vs/platform/registry/common/platform");
const configurationRegistry_1 = require("vs/platform/configuration/common/configurationRegistry");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const actions_1 = require("vs/platform/actions/common/actions");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const activity_1 = require("vs/workbench/services/activity/common/activity");
const activity_2 = require("vs/workbench/common/activity");
const notification_1 = require("vs/platform/notification/common/notification");
const uri_1 = require("vs/base/common/uri");
const amd_1 = require("vs/base/common/amd");
const resources_1 = require("vs/workbench/common/resources");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const event_1 = require("vs/base/common/event");
const history_1 = require("vs/workbench/services/history/common/history");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const resources_2 = require("vs/base/common/resources");
const instantiation_1 = require("vs/platform/instantiation/common/instantiation");
const platform_2 = require("vs/base/common/platform");
const userDataSyncService_1 = require("vs/platform/userDataSync/common/userDataSyncService");
const CONTEXT_SYNC_STATE = new contextkey_1.RawContextKey('syncStatus', userDataSync_1.SyncStatus.Uninitialized);
platform_1.Registry.as(configurationRegistry_1.Extensions.Configuration)
    .registerConfiguration({
    id: 'userConfiguration',
    order: 30,
    title: nls_1.localize('userConfiguration', "User Configuration"),
    type: 'object',
    properties: {
        'userConfiguration.enableSync': {
            type: 'boolean',
            description: nls_1.localize('userConfiguration.enableSync', "When enabled, synchronises User Configuration: Settings, Keybindings, Extensions & Snippets."),
            default: true,
            scope: 1 /* APPLICATION */
        }
    }
});
let UserDataAutoSyncContribution = class UserDataAutoSyncContribution extends lifecycle_1.Disposable {
    constructor(instantiationService) {
        super();
        if (platform_2.isWeb) {
            instantiationService.createInstance(userDataSyncService_1.UserDataAutoSync);
        }
    }
};
UserDataAutoSyncContribution = __decorate([
    __param(0, instantiation_1.IInstantiationService)
], UserDataAutoSyncContribution);
const SYNC_PUSH_LIGHT_ICON_URI = uri_1.URI.parse(amd_1.registerAndGetAmdImageURL(`vs/workbench/contrib/userDataSync/browser/media/check-light.svg`));
const SYNC_PUSH_DARK_ICON_URI = uri_1.URI.parse(amd_1.registerAndGetAmdImageURL(`vs/workbench/contrib/userDataSync/browser/media/check-dark.svg`));
let SyncActionsContribution = class SyncActionsContribution extends lifecycle_1.Disposable {
    constructor(userDataSyncService, contextKeyService, activityService, notificationService, configurationService, editorService, textFileService, historyService, workbenchEnvironmentService) {
        super();
        this.userDataSyncService = userDataSyncService;
        this.activityService = activityService;
        this.notificationService = notificationService;
        this.configurationService = configurationService;
        this.editorService = editorService;
        this.textFileService = textFileService;
        this.historyService = historyService;
        this.workbenchEnvironmentService = workbenchEnvironmentService;
        this.badgeDisposable = this._register(new lifecycle_1.MutableDisposable());
        this.conflictsWarningDisposable = this._register(new lifecycle_1.MutableDisposable());
        this.syncEnablementContext = CONTEXT_SYNC_STATE.bindTo(contextKeyService);
        this.onDidChangeStatus(userDataSyncService.status);
        this._register(event_1.Event.debounce(userDataSyncService.onDidChangeStatus, () => undefined, 500)(status => this.onDidChangeStatus(userDataSyncService.status)));
        this.registerActions();
    }
    onDidChangeStatus(status) {
        this.syncEnablementContext.set(status);
        let badge = undefined;
        let clazz;
        if (status === userDataSync_1.SyncStatus.HasConflicts) {
            badge = new activity_1.NumberBadge(1, () => nls_1.localize('resolve conflicts', "Resolve Conflicts"));
        }
        else if (status === userDataSync_1.SyncStatus.Syncing) {
            badge = new activity_1.ProgressBadge(() => nls_1.localize('syncing', "Synchronising User Configuration..."));
            clazz = 'progress-badge';
        }
        this.badgeDisposable.clear();
        if (badge) {
            this.badgeDisposable.value = this.activityService.showActivity(activity_2.GLOBAL_ACTIVITY_ID, badge, clazz);
        }
        if (status === userDataSync_1.SyncStatus.HasConflicts) {
            if (!this.conflictsWarningDisposable.value) {
                const handle = this.notificationService.prompt(notification_1.Severity.Warning, nls_1.localize('conflicts detected', "Unable to sync due to conflicts. Please resolve them to continue."), [
                    {
                        label: nls_1.localize('resolve', "Resolve Conflicts"),
                        run: () => this.handleConflicts()
                    }
                ]);
                this.conflictsWarningDisposable.value = lifecycle_1.toDisposable(() => handle.close());
                handle.onDidClose(() => this.conflictsWarningDisposable.clear());
            }
        }
        else {
            this.conflictsWarningDisposable.clear();
        }
    }
    continueSync() {
        return __awaiter(this, void 0, void 0, function* () {
            // Get the preview editor
            const editorInput = this.editorService.editors.filter(input => resources_2.isEqual(input.getResource(), this.workbenchEnvironmentService.settingsSyncPreviewResource))[0];
            // Save the preview
            if (editorInput && editorInput.isDirty()) {
                yield this.textFileService.save(editorInput.getResource());
            }
            try {
                // Continue Sync
                yield this.userDataSyncService.sync(true);
            }
            catch (error) {
                this.notificationService.error(error);
                return;
            }
            // Close the preview editor
            if (editorInput) {
                editorInput.dispose();
            }
        });
    }
    handleConflicts() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.userDataSyncService.conflictsSource === userDataSync_1.SyncSource.Settings) {
                const resourceInput = {
                    resource: this.workbenchEnvironmentService.settingsSyncPreviewResource,
                    options: {
                        preserveFocus: false,
                        pinned: false,
                        revealIfVisible: true,
                    },
                    mode: 'jsonc'
                };
                this.editorService.openEditor(resourceInput)
                    .then(editor => {
                    this.historyService.remove(resourceInput);
                    if (editor && editor.input) {
                        // Trigger sync after closing the conflicts editor.
                        const disposable = editor.input.onDispose(() => {
                            disposable.dispose();
                            this.userDataSyncService.sync(true);
                        });
                    }
                });
            }
        });
    }
    registerActions() {
        const startSyncMenuItem = {
            group: '5_sync',
            command: {
                id: 'workbench.userData.actions.syncStart',
                title: nls_1.localize('start sync', "Sync: Start")
            },
            when: contextkey_1.ContextKeyExpr.and(CONTEXT_SYNC_STATE.notEqualsTo(userDataSync_1.SyncStatus.Uninitialized), contextkey_1.ContextKeyExpr.not('config.userConfiguration.enableSync')),
        };
        commands_1.CommandsRegistry.registerCommand(startSyncMenuItem.command.id, () => this.configurationService.updateValue('userConfiguration.enableSync', true));
        actions_1.MenuRegistry.appendMenuItem(45 /* GlobalActivity */, startSyncMenuItem);
        actions_1.MenuRegistry.appendMenuItem(0 /* CommandPalette */, startSyncMenuItem);
        const stopSyncMenuItem = {
            group: '5_sync',
            command: {
                id: 'workbench.userData.actions.stopSync',
                title: nls_1.localize('stop sync', "Sync: Stop")
            },
            when: contextkey_1.ContextKeyExpr.and(CONTEXT_SYNC_STATE.notEqualsTo(userDataSync_1.SyncStatus.Uninitialized), contextkey_1.ContextKeyExpr.has('config.userConfiguration.enableSync')),
        };
        commands_1.CommandsRegistry.registerCommand(stopSyncMenuItem.command.id, () => this.configurationService.updateValue('userConfiguration.enableSync', false));
        actions_1.MenuRegistry.appendMenuItem(45 /* GlobalActivity */, stopSyncMenuItem);
        actions_1.MenuRegistry.appendMenuItem(0 /* CommandPalette */, stopSyncMenuItem);
        const resolveConflictsMenuItem = {
            group: '5_sync',
            command: {
                id: 'sync.resolveConflicts',
                title: nls_1.localize('resolveConflicts', "Sync: Resolve Conflicts"),
            },
            when: CONTEXT_SYNC_STATE.isEqualTo(userDataSync_1.SyncStatus.HasConflicts),
        };
        commands_1.CommandsRegistry.registerCommand(resolveConflictsMenuItem.command.id, () => this.handleConflicts());
        actions_1.MenuRegistry.appendMenuItem(45 /* GlobalActivity */, resolveConflictsMenuItem);
        actions_1.MenuRegistry.appendMenuItem(0 /* CommandPalette */, resolveConflictsMenuItem);
        const continueSyncCommandId = 'workbench.userData.actions.continueSync';
        commands_1.CommandsRegistry.registerCommand(continueSyncCommandId, () => this.continueSync());
        actions_1.MenuRegistry.appendMenuItem(0 /* CommandPalette */, {
            command: {
                id: continueSyncCommandId,
                title: nls_1.localize('continue sync', "Sync: Continue")
            },
            when: contextkey_1.ContextKeyExpr.and(CONTEXT_SYNC_STATE.isEqualTo(userDataSync_1.SyncStatus.HasConflicts)),
        });
        actions_1.MenuRegistry.appendMenuItem(8 /* EditorTitle */, {
            command: {
                id: continueSyncCommandId,
                title: nls_1.localize('continue sync', "Sync: Continue"),
                iconLocation: {
                    light: SYNC_PUSH_LIGHT_ICON_URI,
                    dark: SYNC_PUSH_DARK_ICON_URI
                }
            },
            group: 'navigation',
            order: 1,
            when: contextkey_1.ContextKeyExpr.and(CONTEXT_SYNC_STATE.isEqualTo(userDataSync_1.SyncStatus.HasConflicts), resources_1.ResourceContextKey.Resource.isEqualTo(this.workbenchEnvironmentService.settingsSyncPreviewResource.toString())),
        });
    }
};
SyncActionsContribution = __decorate([
    __param(0, userDataSync_1.IUserDataSyncService),
    __param(1, contextkey_1.IContextKeyService),
    __param(2, activity_1.IActivityService),
    __param(3, notification_1.INotificationService),
    __param(4, configuration_1.IConfigurationService),
    __param(5, editorService_1.IEditorService),
    __param(6, textfiles_1.ITextFileService),
    __param(7, history_1.IHistoryService),
    __param(8, environmentService_1.IWorkbenchEnvironmentService)
], SyncActionsContribution);
const workbenchRegistry = platform_1.Registry.as(contributions_1.Extensions.Workbench);
workbenchRegistry.registerWorkbenchContribution(SyncActionsContribution, 1 /* Starting */);
workbenchRegistry.registerWorkbenchContribution(UserDataAutoSyncContribution, 3 /* Restored */);
//# sourceMappingURL=userDataSync.contribution.js.map