"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const actions_1 = require("vs/base/common/actions");
const nls = require("vs/nls");
const editorExtensions_1 = require("vs/editor/browser/editorExtensions");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const webviewElement_1 = require("vs/workbench/contrib/webview/electron-browser/webviewElement");
class OpenWebviewDeveloperToolsAction extends actions_1.Action {
    constructor(id, label) {
        super(id, label);
    }
    run() {
        const elements = document.querySelectorAll('webview.ready');
        for (let i = 0; i < elements.length; i++) {
            try {
                elements.item(i).openDevTools();
            }
            catch (e) {
                console.error(e);
            }
        }
        return Promise.resolve(true);
    }
}
exports.OpenWebviewDeveloperToolsAction = OpenWebviewDeveloperToolsAction;
OpenWebviewDeveloperToolsAction.ID = 'workbench.action.webview.openDeveloperTools';
OpenWebviewDeveloperToolsAction.ALIAS = 'Open Webview Developer Tools';
OpenWebviewDeveloperToolsAction.LABEL = nls.localize('openToolsLabel', "Open Webview Developer Tools");
class SelectAllWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.selectAll());
    }
}
exports.SelectAllWebviewEditorCommand = SelectAllWebviewEditorCommand;
SelectAllWebviewEditorCommand.ID = 'editor.action.webvieweditor.selectAll';
class CopyWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, _args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.copy());
    }
}
exports.CopyWebviewEditorCommand = CopyWebviewEditorCommand;
CopyWebviewEditorCommand.ID = 'editor.action.webvieweditor.copy';
class PasteWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, _args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.paste());
    }
}
exports.PasteWebviewEditorCommand = PasteWebviewEditorCommand;
PasteWebviewEditorCommand.ID = 'editor.action.webvieweditor.paste';
class CutWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, _args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.cut());
    }
}
exports.CutWebviewEditorCommand = CutWebviewEditorCommand;
CutWebviewEditorCommand.ID = 'editor.action.webvieweditor.cut';
class UndoWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.undo());
    }
}
exports.UndoWebviewEditorCommand = UndoWebviewEditorCommand;
UndoWebviewEditorCommand.ID = 'editor.action.webvieweditor.undo';
class RedoWebviewEditorCommand extends editorExtensions_1.Command {
    runCommand(accessor, args) {
        withActiveWebviewBasedWebview(accessor, webview => webview.redo());
    }
}
exports.RedoWebviewEditorCommand = RedoWebviewEditorCommand;
RedoWebviewEditorCommand.ID = 'editor.action.webvieweditor.redo';
function getActiveWebviewEditor(accessor) {
    const editorService = accessor.get(editorService_1.IEditorService);
    const activeControl = editorService.activeControl;
    return activeControl.isWebviewEditor ? activeControl : undefined;
}
function withActiveWebviewBasedWebview(accessor, f) {
    const webViewEditor = getActiveWebviewEditor(accessor);
    if (webViewEditor) {
        webViewEditor.withWebview(webview => {
            if (webview instanceof webviewElement_1.ElectronWebviewBasedWebview) {
                f(webview);
            }
            else if (webview.getInnerWebview) {
                const innerWebview = webview.getInnerWebview();
                if (innerWebview instanceof webviewElement_1.ElectronWebviewBasedWebview) {
                    f(innerWebview);
                }
            }
        });
    }
}
//# sourceMappingURL=webviewCommands.js.map