"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const walkThroughPart_1 = require("vs/workbench/contrib/welcome/walkThrough/browser/walkThroughPart");
const editorContextKeys_1 = require("vs/editor/common/editorContextKeys");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
exports.WalkThroughArrowUp = {
    id: 'workbench.action.interactivePlayground.arrowUp',
    weight: 200 /* WorkbenchContrib */,
    when: contextkey_1.ContextKeyExpr.and(walkThroughPart_1.WALK_THROUGH_FOCUS, editorContextKeys_1.EditorContextKeys.editorTextFocus.toNegated()),
    primary: 16 /* UpArrow */,
    handler: accessor => {
        const editorService = accessor.get(editorService_1.IEditorService);
        const activeControl = editorService.activeControl;
        if (activeControl instanceof walkThroughPart_1.WalkThroughPart) {
            activeControl.arrowUp();
        }
    }
};
exports.WalkThroughArrowDown = {
    id: 'workbench.action.interactivePlayground.arrowDown',
    weight: 200 /* WorkbenchContrib */,
    when: contextkey_1.ContextKeyExpr.and(walkThroughPart_1.WALK_THROUGH_FOCUS, editorContextKeys_1.EditorContextKeys.editorTextFocus.toNegated()),
    primary: 18 /* DownArrow */,
    handler: accessor => {
        const editorService = accessor.get(editorService_1.IEditorService);
        const activeControl = editorService.activeControl;
        if (activeControl instanceof walkThroughPart_1.WalkThroughPart) {
            activeControl.arrowDown();
        }
    }
};
exports.WalkThroughPageUp = {
    id: 'workbench.action.interactivePlayground.pageUp',
    weight: 200 /* WorkbenchContrib */,
    when: contextkey_1.ContextKeyExpr.and(walkThroughPart_1.WALK_THROUGH_FOCUS, editorContextKeys_1.EditorContextKeys.editorTextFocus.toNegated()),
    primary: 11 /* PageUp */,
    handler: accessor => {
        const editorService = accessor.get(editorService_1.IEditorService);
        const activeControl = editorService.activeControl;
        if (activeControl instanceof walkThroughPart_1.WalkThroughPart) {
            activeControl.pageUp();
        }
    }
};
exports.WalkThroughPageDown = {
    id: 'workbench.action.interactivePlayground.pageDown',
    weight: 200 /* WorkbenchContrib */,
    when: contextkey_1.ContextKeyExpr.and(walkThroughPart_1.WALK_THROUGH_FOCUS, editorContextKeys_1.EditorContextKeys.editorTextFocus.toNegated()),
    primary: 12 /* PageDown */,
    handler: accessor => {
        const editorService = accessor.get(editorService_1.IEditorService);
        const activeControl = editorService.activeControl;
        if (activeControl instanceof walkThroughPart_1.WalkThroughPart) {
            activeControl.pageDown();
        }
    }
};
//# sourceMappingURL=walkThroughActions.js.map