"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const actions_1 = require("vs/base/common/actions");
const nls = require("vs/nls");
const windows_1 = require("vs/platform/windows/common/windows");
const workspace_1 = require("vs/platform/workspace/common/workspace");
const workspaceEditing_1 = require("vs/workbench/services/workspace/common/workspaceEditing");
const workspaces_1 = require("vs/platform/workspaces/common/workspaces");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const notification_1 = require("vs/platform/notification/common/notification");
let SaveWorkspaceAsAction = class SaveWorkspaceAsAction extends actions_1.Action {
    constructor(id, label, contextService, workspaceEditingService) {
        super(id, label);
        this.contextService = contextService;
        this.workspaceEditingService = workspaceEditingService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const configPathUri = yield this.workspaceEditingService.pickNewWorkspacePath();
            if (configPathUri) {
                switch (this.contextService.getWorkbenchState()) {
                    case 1 /* EMPTY */:
                    case 2 /* FOLDER */:
                        const folders = this.contextService.getWorkspace().folders.map(folder => ({ uri: folder.uri }));
                        return this.workspaceEditingService.createAndEnterWorkspace(folders, configPathUri);
                    case 3 /* WORKSPACE */:
                        return this.workspaceEditingService.saveAndEnterWorkspace(configPathUri);
                }
            }
        });
    }
};
SaveWorkspaceAsAction.ID = 'workbench.action.saveWorkspaceAs';
SaveWorkspaceAsAction.LABEL = nls.localize('saveWorkspaceAsAction', "Save Workspace As...");
SaveWorkspaceAsAction = __decorate([
    __param(2, workspace_1.IWorkspaceContextService),
    __param(3, workspaceEditing_1.IWorkspaceEditingService)
], SaveWorkspaceAsAction);
exports.SaveWorkspaceAsAction = SaveWorkspaceAsAction;
let DuplicateWorkspaceInNewWindowAction = class DuplicateWorkspaceInNewWindowAction extends actions_1.Action {
    constructor(id, label, workspaceContextService, workspaceEditingService, windowService, workspacesService, environmentService) {
        super(id, label);
        this.workspaceContextService = workspaceContextService;
        this.workspaceEditingService = workspaceEditingService;
        this.windowService = windowService;
        this.workspacesService = workspacesService;
        this.environmentService = environmentService;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const folders = this.workspaceContextService.getWorkspace().folders;
            const remoteAuthority = this.environmentService.configuration.remoteAuthority;
            const newWorkspace = yield this.workspacesService.createUntitledWorkspace(folders, remoteAuthority);
            yield this.workspaceEditingService.copyWorkspaceSettings(newWorkspace);
            return this.windowService.openWindow([{ workspaceUri: newWorkspace.configPath }], { forceNewWindow: true });
        });
    }
};
DuplicateWorkspaceInNewWindowAction.ID = 'workbench.action.duplicateWorkspaceInNewWindow';
DuplicateWorkspaceInNewWindowAction.LABEL = nls.localize('duplicateWorkspaceInNewWindow', "Duplicate Workspace in New Window");
DuplicateWorkspaceInNewWindowAction = __decorate([
    __param(2, workspace_1.IWorkspaceContextService),
    __param(3, workspaceEditing_1.IWorkspaceEditingService),
    __param(4, windows_1.IWindowService),
    __param(5, workspaces_1.IWorkspacesService),
    __param(6, environmentService_1.IWorkbenchEnvironmentService)
], DuplicateWorkspaceInNewWindowAction);
exports.DuplicateWorkspaceInNewWindowAction = DuplicateWorkspaceInNewWindowAction;
let CloseWorkspaceAction = class CloseWorkspaceAction extends actions_1.Action {
    constructor(id, label, contextService, notificationService, windowService) {
        super(id, label);
        this.contextService = contextService;
        this.notificationService = notificationService;
        this.windowService = windowService;
    }
    run() {
        if (this.contextService.getWorkbenchState() === 1 /* EMPTY */) {
            this.notificationService.info(nls.localize('noWorkspaceOpened', "There is currently no workspace opened in this instance to close."));
            return Promise.resolve(undefined);
        }
        return this.windowService.closeWorkspace();
    }
};
CloseWorkspaceAction.ID = 'workbench.action.closeFolder';
CloseWorkspaceAction.LABEL = nls.localize('closeWorkspace', "Close Workspace");
CloseWorkspaceAction = __decorate([
    __param(2, workspace_1.IWorkspaceContextService),
    __param(3, notification_1.INotificationService),
    __param(4, windows_1.IWindowService)
], CloseWorkspaceAction);
exports.CloseWorkspaceAction = CloseWorkspaceAction;
//# sourceMappingURL=workspaceActions.js.map