"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const editorBrowser_1 = require("vs/editor/browser/editorBrowser");
const codeEditorServiceImpl_1 = require("vs/editor/browser/services/codeEditorServiceImpl");
const themeService_1 = require("vs/platform/theme/common/themeService");
const editor_1 = require("vs/workbench/common/editor");
const editorService_1 = require("vs/workbench/services/editor/common/editorService");
const codeEditorService_1 = require("vs/editor/browser/services/codeEditorService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
let CodeEditorService = class CodeEditorService extends codeEditorServiceImpl_1.CodeEditorServiceImpl {
    constructor(editorService, themeService) {
        super(themeService);
        this.editorService = editorService;
    }
    getActiveCodeEditor() {
        const activeTextEditorWidget = this.editorService.activeTextEditorWidget;
        if (editorBrowser_1.isCodeEditor(activeTextEditorWidget)) {
            return activeTextEditorWidget;
        }
        if (editorBrowser_1.isDiffEditor(activeTextEditorWidget)) {
            return activeTextEditorWidget.getModifiedEditor();
        }
        return undefined;
    }
    openCodeEditor(input, source, sideBySide) {
        // Special case: If the active editor is a diff editor and the request to open originates and
        // targets the modified side of it, we just apply the request there to prevent opening the modified
        // side as separate editor.
        const activeTextEditorWidget = this.editorService.activeTextEditorWidget;
        if (!sideBySide && // we need the current active group to be the taret
            editorBrowser_1.isDiffEditor(activeTextEditorWidget) && // we only support this for active text diff editors
            input.options && // we need options to apply
            input.resource && // we need a request resource to compare with
            activeTextEditorWidget.getModel() && // we need a target model to compare with
            source === activeTextEditorWidget.getModifiedEditor() && // we need the source of this request to be the modified side of the diff editor
            input.resource.toString() === activeTextEditorWidget.getModel().modified.uri.toString() // we need the input resources to match with modified side
        ) {
            const targetEditor = activeTextEditorWidget.getModifiedEditor();
            const textOptions = editor_1.TextEditorOptions.create(input.options);
            textOptions.apply(targetEditor, 0 /* Smooth */);
            return Promise.resolve(targetEditor);
        }
        // Open using our normal editor service
        return this.doOpenCodeEditor(input, source, sideBySide);
    }
    doOpenCodeEditor(input, _source, sideBySide) {
        return __awaiter(this, void 0, void 0, function* () {
            const control = yield this.editorService.openEditor(input, sideBySide ? editorService_1.SIDE_GROUP : editorService_1.ACTIVE_GROUP);
            if (control) {
                const widget = control.getControl();
                if (editorBrowser_1.isCodeEditor(widget)) {
                    return widget;
                }
            }
            return undefined;
        });
    }
};
CodeEditorService = __decorate([
    __param(0, editorService_1.IEditorService),
    __param(1, themeService_1.IThemeService)
], CodeEditorService);
exports.CodeEditorService = CodeEditorService;
extensions_1.registerSingleton(codeEditorService_1.ICodeEditorService, CodeEditorService, true);
//# sourceMappingURL=codeEditorService.js.map