"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const extHostOutput_1 = require("vs/workbench/api/common/extHostOutput");
const extHostWorkspace_1 = require("vs/workbench/api/common/extHostWorkspace");
const extHostDecorations_1 = require("vs/workbench/api/common/extHostDecorations");
const extHostConfiguration_1 = require("vs/workbench/api/common/extHostConfiguration");
const extHostCommands_1 = require("vs/workbench/api/common/extHostCommands");
const extHostDocumentsAndEditors_1 = require("vs/workbench/api/common/extHostDocumentsAndEditors");
const extHostTerminalService_1 = require("vs/workbench/api/common/extHostTerminalService");
const extHostTask_1 = require("vs/workbench/api/common/extHostTask");
const extHostDebugService_1 = require("vs/workbench/api/common/extHostDebugService");
const extHostSearch_1 = require("vs/workbench/api/common/extHostSearch");
const extHostStoragePaths_1 = require("vs/workbench/api/common/extHostStoragePaths");
const extHostExtensionService_1 = require("vs/workbench/api/common/extHostExtensionService");
const extHostStorage_1 = require("vs/workbench/api/common/extHostStorage");
const extHostExtensionService_2 = require("vs/workbench/api/worker/extHostExtensionService");
const log_1 = require("vs/platform/log/common/log");
const extHostLogService_1 = require("vs/workbench/api/worker/extHostLogService");
// register singleton services
extensions_1.registerSingleton(log_1.ILogService, extHostLogService_1.ExtHostLogService);
extensions_1.registerSingleton(extHostOutput_1.IExtHostOutputService, extHostOutput_1.ExtHostOutputService);
extensions_1.registerSingleton(extHostWorkspace_1.IExtHostWorkspace, extHostWorkspace_1.ExtHostWorkspace);
extensions_1.registerSingleton(extHostDecorations_1.IExtHostDecorations, extHostDecorations_1.ExtHostDecorations);
extensions_1.registerSingleton(extHostConfiguration_1.IExtHostConfiguration, extHostConfiguration_1.ExtHostConfiguration);
extensions_1.registerSingleton(extHostCommands_1.IExtHostCommands, extHostCommands_1.ExtHostCommands);
extensions_1.registerSingleton(extHostDocumentsAndEditors_1.IExtHostDocumentsAndEditors, extHostDocumentsAndEditors_1.ExtHostDocumentsAndEditors);
extensions_1.registerSingleton(extHostStorage_1.IExtHostStorage, extHostStorage_1.ExtHostStorage);
extensions_1.registerSingleton(extHostExtensionService_1.IExtHostExtensionService, extHostExtensionService_2.ExtHostExtensionService);
// register services that only throw errors
function NotImplementedProxy(name) {
    return class {
        constructor() {
            return new Proxy({}, {
                get(target, prop) {
                    if (target[prop]) {
                        return target[prop];
                    }
                    throw new Error(`Not Implemented: ${name}->${String(prop)}`);
                }
            });
        }
    };
}
extensions_1.registerSingleton(extHostTerminalService_1.IExtHostTerminalService, extHostTerminalService_1.WorkerExtHostTerminalService);
extensions_1.registerSingleton(extHostTask_1.IExtHostTask, extHostTask_1.WorkerExtHostTask);
extensions_1.registerSingleton(extHostDebugService_1.IExtHostDebugService, class extends NotImplementedProxy(extHostDebugService_1.IExtHostDebugService) {
});
extensions_1.registerSingleton(extHostSearch_1.IExtHostSearch, class extends NotImplementedProxy(extHostSearch_1.IExtHostSearch) {
});
extensions_1.registerSingleton(extHostStoragePaths_1.IExtensionStoragePaths, class extends NotImplementedProxy(extHostStoragePaths_1.IExtensionStoragePaths) {
    constructor() {
        super(...arguments);
        this.whenReady = Promise.resolve();
    }
});
//# sourceMappingURL=extHost.services.js.map