"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const host_1 = require("vs/workbench/services/host/browser/host");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const layoutService_1 = require("vs/workbench/services/layout/browser/layoutService");
let BrowserHostService = class BrowserHostService {
    constructor(layoutService) {
        this.layoutService = layoutService;
        //#region Window
        this.windowCount = Promise.resolve(1);
    }
    openEmptyWindow(options) {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO@Ben delegate to embedder
            const targetHref = `${document.location.origin}${document.location.pathname}?ew=true`;
            if (options && options.reuse) {
                window.location.href = targetHref;
            }
            else {
                window.open(targetHref);
            }
        });
    }
    toggleFullScreen() {
        return __awaiter(this, void 0, void 0, function* () {
            const target = this.layoutService.getWorkbenchElement();
            // Chromium
            if (document.fullscreen !== undefined) {
                if (!document.fullscreen) {
                    try {
                        return yield target.requestFullscreen();
                    }
                    catch (error) {
                        console.warn('Toggle Full Screen failed'); // https://developer.mozilla.org/en-US/docs/Web/API/Element/requestFullscreen
                    }
                }
                else {
                    try {
                        return yield document.exitFullscreen();
                    }
                    catch (error) {
                        console.warn('Exit Full Screen failed');
                    }
                }
            }
            // Safari and Edge 14 are all using webkit prefix
            if (document.webkitIsFullScreen !== undefined) {
                try {
                    if (!document.webkitIsFullScreen) {
                        target.webkitRequestFullscreen(); // it's async, but doesn't return a real promise.
                    }
                    else {
                        document.webkitExitFullscreen(); // it's async, but doesn't return a real promise.
                    }
                }
                catch (_a) {
                    console.warn('Enter/Exit Full Screen failed');
                }
            }
        });
    }
    //#endregion
    restart() {
        return __awaiter(this, void 0, void 0, function* () {
            this.reload();
        });
    }
    reload() {
        return __awaiter(this, void 0, void 0, function* () {
            window.location.reload();
        });
    }
};
BrowserHostService = __decorate([
    __param(0, layoutService_1.IWorkbenchLayoutService)
], BrowserHostService);
exports.BrowserHostService = BrowserHostService;
extensions_1.registerSingleton(host_1.IHostService, BrowserHostService, true);
//# sourceMappingURL=browserHostService.js.map