"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
const assert = require("assert");
require("vs/workbench/services/keybinding/browser/keyboardLayouts/en.darwin"); // 15%
require("vs/workbench/services/keybinding/browser/keyboardLayouts/de.darwin");
const __contribution_1 = require("vs/workbench/services/keybinding/browser/keyboardLayouts/_.contribution");
const keymapService_1 = require("../browser/keymapService");
const keymapInfo_1 = require("../common/keymapInfo");
const instantiationServiceMock_1 = require("vs/platform/instantiation/test/common/instantiationServiceMock");
const notification_1 = require("vs/platform/notification/common/notification");
const commands_1 = require("vs/platform/commands/common/commands");
const storage_1 = require("vs/platform/storage/common/storage");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const testNotificationService_1 = require("vs/platform/notification/test/common/testNotificationService");
class TestKeyboardMapperFactory extends keymapService_1.BrowserKeyboardMapperFactoryBase {
    constructor(notificationService, storageService, commandService) {
        // super(notificationService, storageService, commandService);
        super();
        const keymapInfos = __contribution_1.KeyboardLayoutContribution.INSTANCE.layoutInfos;
        this._keymapInfos.push(...keymapInfos.map(info => (new keymapInfo_1.KeymapInfo(info.layout, info.secondaryLayouts, info.mapping, info.isUserKeyboardLayout))));
        this._mru = this._keymapInfos;
        this._initialized = true;
        this.onKeyboardLayoutChanged();
        const usLayout = this.getUSStandardLayout();
        if (usLayout) {
            this.setActiveKeyMapping(usLayout.mapping);
        }
    }
}
suite('keyboard layout loader', () => {
    let instantiationService = new instantiationServiceMock_1.TestInstantiationService();
    let notitifcationService = instantiationService.stub(notification_1.INotificationService, new testNotificationService_1.TestNotificationService());
    let storageService = instantiationService.stub(storage_1.IStorageService, new workbenchTestServices_1.TestStorageService());
    let commandService = instantiationService.stub(commands_1.ICommandService, {});
    let instance = new TestKeyboardMapperFactory(notitifcationService, storageService, commandService);
    test('load default US keyboard layout', () => {
        assert.notEqual(instance.activeKeyboardLayout, null);
    });
    test('isKeyMappingActive', () => {
        instance.setUSKeyboardLayout();
        assert.equal(instance.isKeyMappingActive({
            KeyA: {
                value: 'a',
                valueIsDeadKey: false,
                withShift: 'A',
                withShiftIsDeadKey: false,
                withAltGr: 'å',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Å',
                withShiftAltGrIsDeadKey: false
            }
        }), true);
        assert.equal(instance.isKeyMappingActive({
            KeyA: {
                value: 'a',
                valueIsDeadKey: false,
                withShift: 'A',
                withShiftIsDeadKey: false,
                withAltGr: 'å',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Å',
                withShiftAltGrIsDeadKey: false
            },
            KeyZ: {
                value: 'z',
                valueIsDeadKey: false,
                withShift: 'Z',
                withShiftIsDeadKey: false,
                withAltGr: 'Ω',
                withAltGrIsDeadKey: false,
                withShiftAltGr: '¸',
                withShiftAltGrIsDeadKey: false
            }
        }), true);
        assert.equal(instance.isKeyMappingActive({
            KeyZ: {
                value: 'y',
                valueIsDeadKey: false,
                withShift: 'Y',
                withShiftIsDeadKey: false,
                withAltGr: '¥',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Ÿ',
                withShiftAltGrIsDeadKey: false
            },
        }), false);
    });
    test('Switch keymapping', () => {
        instance.setActiveKeyMapping({
            KeyZ: {
                value: 'y',
                valueIsDeadKey: false,
                withShift: 'Y',
                withShiftIsDeadKey: false,
                withAltGr: '¥',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Ÿ',
                withShiftAltGrIsDeadKey: false
            }
        });
        assert.equal(!!instance.activeKeyboardLayout.isUSStandard, false);
        assert.equal(instance.isKeyMappingActive({
            KeyZ: {
                value: 'y',
                valueIsDeadKey: false,
                withShift: 'Y',
                withShiftIsDeadKey: false,
                withAltGr: '¥',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Ÿ',
                withShiftAltGrIsDeadKey: false
            },
        }), true);
        instance.setUSKeyboardLayout();
        assert.equal(instance.activeKeyboardLayout.isUSStandard, true);
    });
    test('Switch keyboard layout info', () => {
        instance.setKeyboardLayout('com.apple.keylayout.German');
        assert.equal(!!instance.activeKeyboardLayout.isUSStandard, false);
        assert.equal(instance.isKeyMappingActive({
            KeyZ: {
                value: 'y',
                valueIsDeadKey: false,
                withShift: 'Y',
                withShiftIsDeadKey: false,
                withAltGr: '¥',
                withAltGrIsDeadKey: false,
                withShiftAltGr: 'Ÿ',
                withShiftAltGrIsDeadKey: false
            },
        }), true);
        instance.setUSKeyboardLayout();
        assert.equal(instance.activeKeyboardLayout.isUSStandard, true);
    });
});
//# sourceMappingURL=browserKeyboardMapper.test.js.map