"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cancellation_1 = require("vs/base/common/cancellation");
const ripgrepTextSearchEngine_1 = require("vs/workbench/services/search/node/ripgrepTextSearchEngine");
class RipgrepSearchProvider {
    constructor(outputChannel) {
        this.outputChannel = outputChannel;
        this.inProgress = new Set();
        process.once('exit', () => this.dispose());
    }
    provideTextSearchResults(query, options, progress, token) {
        const engine = new ripgrepTextSearchEngine_1.RipgrepTextSearchEngine(this.outputChannel);
        return this.withToken(token, token => engine.provideTextSearchResults(query, options, progress, token));
    }
    withToken(token, fn) {
        return __awaiter(this, void 0, void 0, function* () {
            const merged = mergedTokenSource(token);
            this.inProgress.add(merged);
            const result = yield fn(merged.token);
            this.inProgress.delete(merged);
            return result;
        });
    }
    dispose() {
        this.inProgress.forEach(engine => engine.cancel());
    }
}
exports.RipgrepSearchProvider = RipgrepSearchProvider;
function mergedTokenSource(token) {
    const tokenSource = new cancellation_1.CancellationTokenSource();
    token.onCancellationRequested(() => tokenSource.cancel());
    return tokenSource;
}
//# sourceMappingURL=ripgrepSearchProvider.js.map