"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const lifecycle_1 = require("vs/base/common/lifecycle");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const log_1 = require("vs/platform/log/common/log");
const telemetryService_1 = require("vs/platform/telemetry/common/telemetryService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const storage_1 = require("vs/platform/storage/common/storage");
const workbenchCommonProperties_1 = require("vs/platform/telemetry/browser/workbenchCommonProperties");
const productService_1 = require("vs/platform/product/common/productService");
const applicationinsights_web_1 = require("@microsoft/applicationinsights-web");
class WebTelemetryAppender {
    constructor(aiKey, _logService) {
        this._logService = _logService;
        const initConfig = {
            config: {
                instrumentationKey: aiKey,
                endpointUrl: 'https://vortex.data.microsoft.com/collect/v1',
                emitLineDelimitedJson: true,
                autoTrackPageVisitTime: false,
                disableExceptionTracking: true,
                disableAjaxTracking: true
            }
        };
        this._aiClient = new applicationinsights_web_1.ApplicationInsights(initConfig);
        this._aiClient.loadAppInsights();
    }
    log(eventName, data) {
        if (!this._aiClient) {
            return;
        }
        data = telemetryUtils_1.validateTelemetryData(data);
        this._logService.trace(`telemetry/${eventName}`, data);
        this._aiClient.trackEvent({
            name: 'monacoworkbench/' + eventName,
            properties: data.properties,
            measurements: data.measurements
        });
    }
    flush() {
        if (this._aiClient) {
            return new Promise(resolve => {
                this._aiClient.flush();
                this._aiClient = undefined;
                resolve(undefined);
            });
        }
        return Promise.resolve();
    }
}
exports.WebTelemetryAppender = WebTelemetryAppender;
let TelemetryService = class TelemetryService extends lifecycle_1.Disposable {
    constructor(environmentService, logService, configurationService, storageService, productService) {
        super();
        const aiKey = productService.aiConfig && productService.aiConfig.asimovKey;
        if (!environmentService.isExtensionDevelopment && !environmentService.args['disable-telemetry'] && !!productService.enableTelemetry && !!aiKey) {
            const config = {
                appender: telemetryUtils_1.combinedAppender(new WebTelemetryAppender(aiKey, logService), new telemetryUtils_1.LogAppender(logService)),
                commonProperties: workbenchCommonProperties_1.resolveWorkbenchCommonProperties(storageService, productService.commit, productService.version, environmentService.configuration.machineId, environmentService.configuration.remoteAuthority),
                piiPaths: [environmentService.appRoot]
            };
            this.impl = this._register(new telemetryService_1.TelemetryService(config, configurationService));
        }
        else {
            this.impl = telemetryUtils_1.NullTelemetryService;
        }
    }
    setEnabled(value) {
        return this.impl.setEnabled(value);
    }
    get isOptedIn() {
        return this.impl.isOptedIn;
    }
    publicLog(eventName, data, anonymizeFilePaths) {
        return this.impl.publicLog(eventName, data, anonymizeFilePaths);
    }
    publicLog2(eventName, data, anonymizeFilePaths) {
        return this.publicLog(eventName, data, anonymizeFilePaths);
    }
    getTelemetryInfo() {
        return this.impl.getTelemetryInfo();
    }
};
TelemetryService = __decorate([
    __param(0, environmentService_1.IWorkbenchEnvironmentService),
    __param(1, log_1.ILogService),
    __param(2, configuration_1.IConfigurationService),
    __param(3, storage_1.IStorageService),
    __param(4, productService_1.IProductService)
], TelemetryService);
exports.TelemetryService = TelemetryService;
extensions_1.registerSingleton(telemetry_1.ITelemetryService, TelemetryService);
//# sourceMappingURL=telemetryService.js.map