"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const telemetry_1 = require("vs/platform/telemetry/common/telemetry");
const telemetryUtils_1 = require("vs/platform/telemetry/common/telemetryUtils");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const lifecycle_1 = require("vs/base/common/lifecycle");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const productService_1 = require("vs/platform/product/common/productService");
const sharedProcessService_1 = require("vs/platform/ipc/electron-browser/sharedProcessService");
const telemetryIpc_1 = require("vs/platform/telemetry/node/telemetryIpc");
const log_1 = require("vs/platform/log/common/log");
const storage_1 = require("vs/platform/storage/common/storage");
const workbenchCommonProperties_1 = require("vs/platform/telemetry/node/workbenchCommonProperties");
const telemetryService_1 = require("vs/platform/telemetry/common/telemetryService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
let TelemetryService = class TelemetryService extends lifecycle_1.Disposable {
    constructor(environmentService, productService, sharedProcessService, logService, storageService, configurationService) {
        super();
        if (!environmentService.isExtensionDevelopment && !environmentService.args['disable-telemetry'] && !!productService.enableTelemetry) {
            const channel = sharedProcessService.getChannel('telemetryAppender');
            const config = {
                appender: telemetryUtils_1.combinedAppender(new telemetryIpc_1.TelemetryAppenderClient(channel), new telemetryUtils_1.LogAppender(logService)),
                commonProperties: workbenchCommonProperties_1.resolveWorkbenchCommonProperties(storageService, productService.commit, productService.version, environmentService.configuration.machineId, productService.msftInternalDomains, environmentService.installSourcePath, environmentService.configuration.remoteAuthority),
                piiPaths: environmentService.extensionsPath ? [environmentService.appRoot, environmentService.extensionsPath] : [environmentService.appRoot]
            };
            this.impl = this._register(new telemetryService_1.TelemetryService(config, configurationService));
        }
        else {
            this.impl = telemetryUtils_1.NullTelemetryService;
        }
    }
    setEnabled(value) {
        return this.impl.setEnabled(value);
    }
    get isOptedIn() {
        return this.impl.isOptedIn;
    }
    publicLog(eventName, data, anonymizeFilePaths) {
        return this.impl.publicLog(eventName, data, anonymizeFilePaths);
    }
    publicLog2(eventName, data, anonymizeFilePaths) {
        return this.publicLog(eventName, data, anonymizeFilePaths);
    }
    getTelemetryInfo() {
        return this.impl.getTelemetryInfo();
    }
};
TelemetryService = __decorate([
    __param(0, environmentService_1.IWorkbenchEnvironmentService),
    __param(1, productService_1.IProductService),
    __param(2, sharedProcessService_1.ISharedProcessService),
    __param(3, log_1.ILogService),
    __param(4, storage_1.IStorageService),
    __param(5, configuration_1.IConfigurationService)
], TelemetryService);
exports.TelemetryService = TelemetryService;
extensions_1.registerSingleton(telemetry_1.ITelemetryService, TelemetryService);
//# sourceMappingURL=telemetryService.js.map