"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const textFileService_1 = require("vs/workbench/services/textfile/browser/textFileService");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const network_1 = require("vs/base/common/network");
class BrowserTextFileService extends textFileService_1.AbstractTextFileService {
    constructor() {
        super(...arguments);
        this.encoding = {
            getPreferredWriteEncoding() {
                return { encoding: 'utf8', hasBOM: false };
            }
        };
    }
    onBeforeShutdown(reason) {
        // Web: we cannot perform long running in the shutdown phase
        // As such we need to check sync if there are any dirty files
        // that have not been backed up yet and then prevent the shutdown
        // if that is the case.
        return this.doBeforeShutdownSync();
    }
    doBeforeShutdownSync() {
        if (this.models.getAll().some(model => model.hasState(2 /* PENDING_SAVE */) || model.hasState(3 /* PENDING_AUTO_SAVE */))) {
            return true; // files are pending to be saved: veto
        }
        const dirtyResources = this.getDirty();
        if (!dirtyResources.length) {
            return false; // no dirty: no veto
        }
        if (!this.isHotExitEnabled) {
            return true; // dirty without backup: veto
        }
        for (const dirtyResource of dirtyResources) {
            let hasBackup = false;
            if (this.fileService.canHandleResource(dirtyResource)) {
                const model = this.models.get(dirtyResource);
                hasBackup = !!(model && model.hasBackup());
            }
            else if (dirtyResource.scheme === network_1.Schemas.untitled) {
                hasBackup = this.untitledEditorService.hasBackup(dirtyResource);
            }
            if (!hasBackup) {
                console.warn('Unload prevented: pending backups');
                return true; // dirty without backup: veto
            }
        }
        return false; // dirty with backups: no veto
    }
}
exports.BrowserTextFileService = BrowserTextFileService;
extensions_1.registerSingleton(textfiles_1.ITextFileService, BrowserTextFileService);
//# sourceMappingURL=browserTextFileService.js.map