"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const uri_1 = require("vs/base/common/uri");
const resourceEditorInput_1 = require("vs/workbench/common/editor/resourceEditorInput");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const utils_1 = require("vs/base/test/common/utils");
const resolverService_1 = require("vs/editor/common/services/resolverService");
const modelService_1 = require("vs/editor/common/services/modelService");
const modeService_1 = require("vs/editor/common/services/modeService");
const textFileEditorModel_1 = require("vs/workbench/services/textfile/common/textFileEditorModel");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const untitledEditorService_1 = require("vs/workbench/services/untitled/common/untitledEditorService");
const event_1 = require("vs/base/common/event");
const async_1 = require("vs/base/common/async");
let ServiceAccessor = class ServiceAccessor {
    constructor(textModelResolverService, modelService, modeService, textFileService, untitledEditorService) {
        this.textModelResolverService = textModelResolverService;
        this.modelService = modelService;
        this.modeService = modeService;
        this.textFileService = textFileService;
        this.untitledEditorService = untitledEditorService;
    }
};
ServiceAccessor = __decorate([
    __param(0, resolverService_1.ITextModelService),
    __param(1, modelService_1.IModelService),
    __param(2, modeService_1.IModeService),
    __param(3, textfiles_1.ITextFileService),
    __param(4, untitledEditorService_1.IUntitledEditorService)
], ServiceAccessor);
suite('Workbench - TextModelResolverService', () => {
    let instantiationService;
    let accessor;
    let model;
    setup(() => {
        instantiationService = workbenchTestServices_1.workbenchInstantiationService();
        accessor = instantiationService.createInstance(ServiceAccessor);
    });
    teardown(() => {
        if (model) {
            model.dispose();
            model = (undefined);
        }
        accessor.textFileService.models.clear();
        accessor.textFileService.models.dispose();
        accessor.untitledEditorService.revertAll();
    });
    test('resolve resource', () => __awaiter(void 0, void 0, void 0, function* () {
        const dispose = accessor.textModelResolverService.registerTextModelContentProvider('test', {
            provideTextContent: function (resource) {
                if (resource.scheme === 'test') {
                    let modelContent = 'Hello Test';
                    let languageSelection = accessor.modeService.create('json');
                    return Promise.resolve(accessor.modelService.createModel(modelContent, languageSelection, resource));
                }
                return Promise.resolve(null);
            }
        });
        let resource = uri_1.URI.from({ scheme: 'test', authority: null, path: 'thePath' });
        let input = instantiationService.createInstance(resourceEditorInput_1.ResourceEditorInput, 'The Name', 'The Description', resource, undefined);
        const model = yield input.resolve();
        assert.ok(model);
        assert.equal(textfiles_1.snapshotToString((model.createSnapshot())), 'Hello Test');
        let disposed = false;
        let disposedPromise = new Promise(resolve => {
            event_1.Event.once(model.onDispose)(() => {
                disposed = true;
                resolve();
            });
        });
        input.dispose();
        yield disposedPromise;
        assert.equal(disposed, true);
        dispose.dispose();
    }));
    test('resolve file', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const textModel = instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, utils_1.toResource.call(this, '/path/file_resolver.txt'), 'utf8', undefined);
            accessor.textFileService.models.add(textModel.getResource(), textModel);
            yield textModel.load();
            const ref = yield accessor.textModelResolverService.createModelReference(textModel.getResource());
            const model = ref.object;
            const editorModel = model.textEditorModel;
            assert.ok(editorModel);
            assert.equal(editorModel.getValue(), 'Hello Html');
            let disposed = false;
            event_1.Event.once(model.onDispose)(() => {
                disposed = true;
            });
            ref.dispose();
            yield async_1.timeout(0); // due to the reference resolving the model first which is async
            assert.equal(disposed, true);
        });
    });
    test('resolve untitled', () => __awaiter(void 0, void 0, void 0, function* () {
        const service = accessor.untitledEditorService;
        const input = service.createOrGet();
        yield input.resolve();
        const ref = yield accessor.textModelResolverService.createModelReference(input.getResource());
        const model = ref.object;
        const editorModel = model.textEditorModel;
        assert.ok(editorModel);
        ref.dispose();
        input.dispose();
    }));
    test('even loading documents should be refcounted', () => __awaiter(void 0, void 0, void 0, function* () {
        let resolveModel;
        let waitForIt = new Promise(c => resolveModel = c);
        const disposable = accessor.textModelResolverService.registerTextModelContentProvider('test', {
            provideTextContent: (resource) => __awaiter(void 0, void 0, void 0, function* () {
                yield waitForIt;
                let modelContent = 'Hello Test';
                let languageSelection = accessor.modeService.create('json');
                return accessor.modelService.createModel(modelContent, languageSelection, resource);
            })
        });
        const uri = uri_1.URI.from({ scheme: 'test', authority: null, path: 'thePath' });
        const modelRefPromise1 = accessor.textModelResolverService.createModelReference(uri);
        const modelRefPromise2 = accessor.textModelResolverService.createModelReference(uri);
        resolveModel();
        const modelRef1 = yield modelRefPromise1;
        const model1 = modelRef1.object;
        const modelRef2 = yield modelRefPromise2;
        const model2 = modelRef2.object;
        const textModel = model1.textEditorModel;
        assert.equal(model1, model2, 'they are the same model');
        assert(!textModel.isDisposed(), 'the text model should not be disposed');
        modelRef1.dispose();
        assert(!textModel.isDisposed(), 'the text model should still not be disposed');
        let p1 = new Promise(resolve => textModel.onWillDispose(resolve));
        modelRef2.dispose();
        yield p1;
        assert(textModel.isDisposed(), 'the text model should finally be disposed');
        disposable.dispose();
    }));
});
//# sourceMappingURL=textModelResolverService.test.js.map