"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const objects = require("vs/base/common/objects");
const json_1 = require("vs/base/common/json");
const editOperation_1 = require("vs/editor/common/core/editOperation");
const modeService_1 = require("vs/editor/common/services/modeService");
const jsonEdit_1 = require("vs/base/common/jsonEdit");
const range_1 = require("vs/editor/common/core/range");
const selection_1 = require("vs/editor/common/core/selection");
const modelService_1 = require("vs/editor/common/services/modelService");
const position_1 = require("vs/editor/common/core/position");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
let SettingsMergeService = class SettingsMergeService {
    constructor(modelService, modeService) {
        this.modelService = modelService;
        this.modeService = modeService;
    }
    merge(localContent, remoteContent, baseContent) {
        return __awaiter(this, void 0, void 0, function* () {
            const local = json_1.parse(localContent);
            const remote = json_1.parse(remoteContent);
            const base = baseContent ? json_1.parse(baseContent) : null;
            const localToRemote = this.compare(local, remote);
            if (localToRemote.added.size === 0 && localToRemote.removed.size === 0 && localToRemote.updated.size === 0) {
                // No changes found between local and remote.
                return { mergeContent: localContent, hasChanges: false, hasConflicts: false };
            }
            const conflicts = new Set();
            const baseToLocal = base ? this.compare(base, local) : { added: Object.keys(local).reduce((r, k) => { r.add(k); return r; }, new Set()), removed: new Set(), updated: new Set() };
            const baseToRemote = base ? this.compare(base, remote) : { added: Object.keys(remote).reduce((r, k) => { r.add(k); return r; }, new Set()), removed: new Set(), updated: new Set() };
            const settingsPreviewModel = this.modelService.createModel(localContent, this.modeService.create('jsonc'));
            // Removed settings in Local
            for (const key of baseToLocal.removed.keys()) {
                // Got updated in remote
                if (baseToRemote.updated.has(key)) {
                    conflicts.add(key);
                }
            }
            // Removed settings in Remote
            for (const key of baseToRemote.removed.keys()) {
                if (conflicts.has(key)) {
                    continue;
                }
                // Got updated in local
                if (baseToLocal.updated.has(key)) {
                    conflicts.add(key);
                }
                else {
                    this.editSetting(settingsPreviewModel, key, undefined);
                }
            }
            // Added settings in Local
            for (const key of baseToLocal.added.keys()) {
                if (conflicts.has(key)) {
                    continue;
                }
                // Got added in remote
                if (baseToRemote.added.has(key)) {
                    // Has different value
                    if (localToRemote.updated.has(key)) {
                        conflicts.add(key);
                    }
                }
            }
            // Added settings in remote
            for (const key of baseToRemote.added.keys()) {
                if (conflicts.has(key)) {
                    continue;
                }
                // Got added in local
                if (baseToLocal.added.has(key)) {
                    // Has different value
                    if (localToRemote.updated.has(key)) {
                        conflicts.add(key);
                    }
                }
                else {
                    this.editSetting(settingsPreviewModel, key, remote[key]);
                }
            }
            // Updated settings in Local
            for (const key of baseToLocal.updated.keys()) {
                if (conflicts.has(key)) {
                    continue;
                }
                // Got updated in remote
                if (baseToRemote.updated.has(key)) {
                    // Has different value
                    if (localToRemote.updated.has(key)) {
                        conflicts.add(key);
                    }
                }
            }
            // Updated settings in Remote
            for (const key of baseToRemote.updated.keys()) {
                if (conflicts.has(key)) {
                    continue;
                }
                // Got updated in local
                if (baseToLocal.updated.has(key)) {
                    // Has different value
                    if (localToRemote.updated.has(key)) {
                        conflicts.add(key);
                    }
                }
                else {
                    this.editSetting(settingsPreviewModel, key, remote[key]);
                }
            }
            for (const key of conflicts.keys()) {
                const tree = json_1.parseTree(settingsPreviewModel.getValue());
                const valueNode = json_1.findNodeAtLocation(tree, [key]);
                const eol = settingsPreviewModel.getEOL();
                const remoteEdit = jsonEdit_1.setProperty(`{${eol}\t${eol}}`, [key], remote[key], { tabSize: 4, insertSpaces: false, eol: eol })[0];
                const remoteContent = remoteEdit ? `${remoteEdit.content.substring(remoteEdit.offset + remoteEdit.length + 1)},${eol}` : '';
                if (valueNode) {
                    // Updated in Local and Remote with different value
                    const keyPosition = settingsPreviewModel.getPositionAt(valueNode.parent.offset);
                    const valuePosition = settingsPreviewModel.getPositionAt(valueNode.offset + valueNode.length);
                    const editOperations = [
                        editOperation_1.EditOperation.insert(new position_1.Position(keyPosition.lineNumber - 1, settingsPreviewModel.getLineMaxColumn(keyPosition.lineNumber - 1)), `${eol}<<<<<<< local`),
                        editOperation_1.EditOperation.insert(new position_1.Position(valuePosition.lineNumber, settingsPreviewModel.getLineMaxColumn(valuePosition.lineNumber)), `${eol}=======${eol}${remoteContent}>>>>>>> remote`)
                    ];
                    settingsPreviewModel.pushEditOperations([new selection_1.Selection(keyPosition.lineNumber, keyPosition.column, keyPosition.lineNumber, keyPosition.column)], editOperations, () => []);
                }
                else {
                    // Removed in Local, but updated in Remote
                    const position = new position_1.Position(settingsPreviewModel.getLineCount() - 1, settingsPreviewModel.getLineMaxColumn(settingsPreviewModel.getLineCount() - 1));
                    const editOperations = [
                        editOperation_1.EditOperation.insert(position, `${eol}<<<<<<< local${eol}=======${eol}${remoteContent}>>>>>>> remote`)
                    ];
                    settingsPreviewModel.pushEditOperations([new selection_1.Selection(position.lineNumber, position.column, position.lineNumber, position.column)], editOperations, () => []);
                }
            }
            return { mergeContent: settingsPreviewModel.getValue(), hasChanges: true, hasConflicts: conflicts.size > 0 };
        });
    }
    editSetting(model, key, value) {
        const insertSpaces = false;
        const tabSize = 4;
        const eol = model.getEOL();
        const edit = jsonEdit_1.setProperty(model.getValue(), [key], value, { tabSize, insertSpaces, eol })[0];
        if (edit) {
            const startPosition = model.getPositionAt(edit.offset);
            const endPosition = model.getPositionAt(edit.offset + edit.length);
            const range = new range_1.Range(startPosition.lineNumber, startPosition.column, endPosition.lineNumber, endPosition.column);
            let currentText = model.getValueInRange(range);
            if (edit.content !== currentText) {
                const editOperation = currentText ? editOperation_1.EditOperation.replace(range, edit.content) : editOperation_1.EditOperation.insert(startPosition, edit.content);
                model.pushEditOperations([new selection_1.Selection(startPosition.lineNumber, startPosition.column, startPosition.lineNumber, startPosition.column)], [editOperation], () => []);
            }
        }
    }
    compare(from, to) {
        const fromKeys = Object.keys(from);
        const toKeys = Object.keys(to);
        const added = toKeys.filter(key => fromKeys.indexOf(key) === -1).reduce((r, key) => { r.add(key); return r; }, new Set());
        const removed = fromKeys.filter(key => toKeys.indexOf(key) === -1).reduce((r, key) => { r.add(key); return r; }, new Set());
        const updated = new Set();
        for (const key of fromKeys) {
            if (removed.has(key)) {
                continue;
            }
            const value1 = from[key];
            const value2 = to[key];
            if (!objects.equals(value1, value2)) {
                updated.add(key);
            }
        }
        return { added, removed, updated };
    }
};
SettingsMergeService = __decorate([
    __param(0, modelService_1.IModelService),
    __param(1, modeService_1.IModeService)
], SettingsMergeService);
extensions_1.registerSingleton(userDataSync_1.ISettingsMergeService, SettingsMergeService);
//# sourceMappingURL=settingsMergeService.js.map