"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const userDataSync_1 = require("vs/platform/userDataSync/common/userDataSync");
const sharedProcessService_1 = require("vs/platform/ipc/electron-browser/sharedProcessService");
const lifecycle_1 = require("vs/base/common/lifecycle");
const event_1 = require("vs/base/common/event");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
let UserDataSyncService = class UserDataSyncService extends lifecycle_1.Disposable {
    constructor(sharedProcessService) {
        super();
        this._status = userDataSync_1.SyncStatus.Uninitialized;
        this._onDidChangeStatus = this._register(new event_1.Emitter());
        this.onDidChangeStatus = this._onDidChangeStatus.event;
        this._conflictsSource = null;
        this.channel = sharedProcessService.getChannel('userDataSync');
        this.channel.call('_getInitialStatus').then(status => {
            this.updateStatus(status);
            this._register(this.channel.listen('onDidChangeStatus')(status => this.updateStatus(status)));
        });
    }
    get status() { return this._status; }
    get onDidChangeLocal() { return this.channel.listen('onDidChangeLocal'); }
    get conflictsSource() { return this._conflictsSource; }
    sync(_continue) {
        return this.channel.call('sync', [_continue]);
    }
    updateStatus(status) {
        return __awaiter(this, void 0, void 0, function* () {
            this._conflictsSource = yield this.channel.call('getConflictsSource');
            this._status = status;
            this._onDidChangeStatus.fire(status);
        });
    }
};
UserDataSyncService = __decorate([
    __param(0, sharedProcessService_1.ISharedProcessService)
], UserDataSyncService);
exports.UserDataSyncService = UserDataSyncService;
extensions_1.registerSingleton(userDataSync_1.IUserDataSyncService, UserDataSyncService);
//# sourceMappingURL=userDataSyncService.js.map