"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const event_1 = require("vs/base/common/event");
const windows_1 = require("vs/platform/windows/common/windows");
const lifecycle_1 = require("vs/base/common/lifecycle");
const label_1 = require("vs/platform/label/common/label");
const environmentService_1 = require("vs/workbench/services/environment/common/environmentService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
let WindowService = class WindowService extends lifecycle_1.Disposable {
    constructor(environmentService, windowsService, labelService) {
        super();
        this.windowsService = windowsService;
        this.labelService = labelService;
        this._windowId = environmentService.configuration.windowId;
        this.remoteAuthority = environmentService.configuration.remoteAuthority;
        const onThisWindowFocus = event_1.Event.map(event_1.Event.filter(windowsService.onWindowFocus, id => id === this._windowId), _ => true);
        const onThisWindowBlur = event_1.Event.map(event_1.Event.filter(windowsService.onWindowBlur, id => id === this._windowId), _ => false);
        const onThisWindowMaximize = event_1.Event.map(event_1.Event.filter(windowsService.onWindowMaximize, id => id === this._windowId), _ => true);
        const onThisWindowUnmaximize = event_1.Event.map(event_1.Event.filter(windowsService.onWindowUnmaximize, id => id === this._windowId), _ => false);
        this.onDidChangeFocus = event_1.Event.any(onThisWindowFocus, onThisWindowBlur);
        this.onDidChangeMaximize = event_1.Event.any(onThisWindowMaximize, onThisWindowUnmaximize);
        this._hasFocus = document.hasFocus();
        this.isFocused().then(focused => this._hasFocus = focused);
        this._register(this.onDidChangeFocus(focus => this._hasFocus = focus));
    }
    get hasFocus() { return this._hasFocus; }
    get windowId() {
        return this._windowId;
    }
    closeWorkspace() {
        return this.windowsService.closeWorkspace(this.windowId);
    }
    enterWorkspace(path) {
        return this.windowsService.enterWorkspace(this.windowId, path);
    }
    openWindow(uris, options = {}) {
        if (!!this.remoteAuthority) {
            uris.forEach(u => u.label = u.label || this.getRecentLabel(u));
        }
        return this.windowsService.openWindow(this.windowId, uris, options);
    }
    closeWindow() {
        return this.windowsService.closeWindow(this.windowId);
    }
    getRecentlyOpened() {
        return this.windowsService.getRecentlyOpened(this.windowId);
    }
    addRecentlyOpened(recents) {
        return this.windowsService.addRecentlyOpened(recents);
    }
    removeFromRecentlyOpened(paths) {
        return this.windowsService.removeFromRecentlyOpened(paths);
    }
    focusWindow() {
        return this.windowsService.focusWindow(this.windowId);
    }
    isFocused() {
        return this.windowsService.isFocused(this.windowId);
    }
    isMaximized() {
        return this.windowsService.isMaximized(this.windowId);
    }
    maximizeWindow() {
        return this.windowsService.maximizeWindow(this.windowId);
    }
    unmaximizeWindow() {
        return this.windowsService.unmaximizeWindow(this.windowId);
    }
    minimizeWindow() {
        return this.windowsService.minimizeWindow(this.windowId);
    }
    onWindowTitleDoubleClick() {
        return this.windowsService.onWindowTitleDoubleClick(this.windowId);
    }
    updateTouchBar(items) {
        return this.windowsService.updateTouchBar(this.windowId, items);
    }
    getRecentLabel(u) {
        if (windows_1.isFolderToOpen(u)) {
            return this.labelService.getWorkspaceLabel(u.folderUri, { verbose: true });
        }
        else if (windows_1.isWorkspaceToOpen(u)) {
            return this.labelService.getWorkspaceLabel({ id: '', configPath: u.workspaceUri }, { verbose: true });
        }
        else {
            return this.labelService.getUriLabel(u.fileUri);
        }
    }
};
WindowService = __decorate([
    __param(0, environmentService_1.IWorkbenchEnvironmentService),
    __param(1, windows_1.IWindowsService),
    __param(2, label_1.ILabelService)
], WindowService);
exports.WindowService = WindowService;
extensions_1.registerSingleton(windows_1.IWindowService, WindowService);
//# sourceMappingURL=windowService.js.map