"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const instantiationServiceMock_1 = require("vs/platform/instantiation/test/common/instantiationServiceMock");
const editor_1 = require("vs/workbench/common/editor");
const textEditorModel_1 = require("vs/workbench/common/editor/textEditorModel");
const modeService_1 = require("vs/editor/common/services/modeService");
const modeServiceImpl_1 = require("vs/editor/common/services/modeServiceImpl");
const configuration_1 = require("vs/platform/configuration/common/configuration");
const testConfigurationService_1 = require("vs/platform/configuration/test/common/testConfigurationService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const textModel_1 = require("vs/editor/common/model/textModel");
const resourceConfiguration_1 = require("vs/editor/common/services/resourceConfiguration");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
class MyEditorModel extends editor_1.EditorModel {
}
class MyTextEditorModel extends textEditorModel_1.BaseTextEditorModel {
    createTextEditorModel(value, resource, preferredMode) {
        return super.createTextEditorModel(value, resource, preferredMode);
    }
    isReadonly() {
        return false;
    }
}
suite('Workbench editor model', () => {
    let instantiationService;
    let modeService;
    setup(() => {
        instantiationService = new instantiationServiceMock_1.TestInstantiationService();
        modeService = instantiationService.stub(modeService_1.IModeService, modeServiceImpl_1.ModeServiceImpl);
    });
    test('EditorModel', () => __awaiter(void 0, void 0, void 0, function* () {
        let counter = 0;
        let m = new MyEditorModel();
        m.onDispose(() => {
            assert(true);
            counter++;
        });
        const model = yield m.load();
        assert(model === m);
        assert.strictEqual(m.isResolved(), true);
        m.dispose();
        assert.equal(counter, 1);
    }));
    test('BaseTextEditorModel', () => __awaiter(void 0, void 0, void 0, function* () {
        let modelService = stubModelService(instantiationService);
        let m = new MyTextEditorModel(modelService, modeService);
        const model = yield m.load();
        assert(model === m);
        model.createTextEditorModel(textModel_1.createTextBufferFactory('foo'), null, 'text/plain');
        assert.strictEqual(m.isResolved(), true);
        m.dispose();
    }));
    function stubModelService(instantiationService) {
        instantiationService.stub(configuration_1.IConfigurationService, new testConfigurationService_1.TestConfigurationService());
        instantiationService.stub(resourceConfiguration_1.ITextResourcePropertiesService, new workbenchTestServices_1.TestTextResourcePropertiesService(instantiationService.get(configuration_1.IConfigurationService)));
        return instantiationService.createInstance(modelServiceImpl_1.ModelServiceImpl);
    }
});
//# sourceMappingURL=editorModel.test.js.map