"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const uri_1 = require("vs/base/common/uri");
const resourceEditorInput_1 = require("vs/workbench/common/editor/resourceEditorInput");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const modelService_1 = require("vs/editor/common/services/modelService");
const modeService_1 = require("vs/editor/common/services/modeService");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
const modesRegistry_1 = require("vs/editor/common/modes/modesRegistry");
let ServiceAccessor = class ServiceAccessor {
    constructor(modelService, modeService) {
        this.modelService = modelService;
        this.modeService = modeService;
    }
};
ServiceAccessor = __decorate([
    __param(0, modelService_1.IModelService),
    __param(1, modeService_1.IModeService)
], ServiceAccessor);
suite('Workbench resource editor input', () => {
    let instantiationService;
    let accessor;
    setup(() => {
        instantiationService = workbenchTestServices_1.workbenchInstantiationService();
        accessor = instantiationService.createInstance(ServiceAccessor);
    });
    test('basics', () => __awaiter(void 0, void 0, void 0, function* () {
        const resource = uri_1.URI.from({ scheme: 'inmemory', authority: null, path: 'thePath' });
        accessor.modelService.createModel('function test() {}', accessor.modeService.create('text'), resource);
        const input = instantiationService.createInstance(resourceEditorInput_1.ResourceEditorInput, 'The Name', 'The Description', resource, undefined);
        const model = yield input.resolve();
        assert.ok(model);
        assert.equal(textfiles_1.snapshotToString((model.createSnapshot())), 'function test() {}');
    }));
    test('custom mode', () => __awaiter(void 0, void 0, void 0, function* () {
        modesRegistry_1.ModesRegistry.registerLanguage({
            id: 'resource-input-test',
        });
        const resource = uri_1.URI.from({ scheme: 'inmemory', authority: null, path: 'thePath' });
        accessor.modelService.createModel('function test() {}', accessor.modeService.create('text'), resource);
        const input = instantiationService.createInstance(resourceEditorInput_1.ResourceEditorInput, 'The Name', 'The Description', resource, 'resource-input-test');
        const model = yield input.resolve();
        assert.ok(model);
        assert.equal(model.textEditorModel.getModeId(), 'resource-input-test');
        input.setMode('text');
        assert.equal(model.textEditorModel.getModeId(), modesRegistry_1.PLAINTEXT_MODE_ID);
    }));
});
//# sourceMappingURL=resourceEditorInput.test.js.map