"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const mainThreadDocuments_1 = require("vs/workbench/api/browser/mainThreadDocuments");
const textModel_1 = require("vs/editor/common/model/textModel");
const async_1 = require("vs/base/common/async");
suite('BoundModelReferenceCollection', () => {
    let col = new mainThreadDocuments_1.BoundModelReferenceCollection(15, 75);
    teardown(() => {
        col.dispose();
    });
    test('max age', () => __awaiter(void 0, void 0, void 0, function* () {
        let didDispose = false;
        col.add({
            object: { textEditorModel: textModel_1.TextModel.createFromString('farboo') },
            dispose() {
                didDispose = true;
            }
        });
        yield async_1.timeout(30);
        assert.equal(didDispose, true);
    }));
    test('max size', () => {
        let disposed = [];
        col.add({
            object: { textEditorModel: textModel_1.TextModel.createFromString('farboo') },
            dispose() {
                disposed.push(0);
            }
        });
        col.add({
            object: { textEditorModel: textModel_1.TextModel.createFromString('boofar') },
            dispose() {
                disposed.push(1);
            }
        });
        col.add({
            object: { textEditorModel: textModel_1.TextModel.createFromString(new Array(71).join('x')) },
            dispose() {
                disposed.push(2);
            }
        });
        assert.deepEqual(disposed, [0, 1]);
    });
});
//# sourceMappingURL=mainThreadDocuments.test.js.map