"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("assert");
const mainThreadSaveParticipant_1 = require("vs/workbench/api/browser/mainThreadSaveParticipant");
const testConfigurationService_1 = require("vs/platform/configuration/test/common/testConfigurationService");
const workbenchTestServices_1 = require("vs/workbench/test/workbenchTestServices");
const utils_1 = require("vs/base/test/common/utils");
const modelService_1 = require("vs/editor/common/services/modelService");
const range_1 = require("vs/editor/common/core/range");
const selection_1 = require("vs/editor/common/core/selection");
const textFileEditorModel_1 = require("vs/workbench/services/textfile/common/textFileEditorModel");
const textfiles_1 = require("vs/workbench/services/textfile/common/textfiles");
let ServiceAccessor = class ServiceAccessor {
    constructor(textFileService, modelService) {
        this.textFileService = textFileService;
        this.modelService = modelService;
    }
};
ServiceAccessor = __decorate([
    __param(0, textfiles_1.ITextFileService), __param(1, modelService_1.IModelService)
], ServiceAccessor);
suite('MainThreadSaveParticipant', function () {
    let instantiationService;
    let accessor;
    setup(() => {
        instantiationService = workbenchTestServices_1.workbenchInstantiationService();
        accessor = instantiationService.createInstance(ServiceAccessor);
    });
    teardown(() => {
        accessor.textFileService.models.clear();
        textFileEditorModel_1.TextFileEditorModel.setSaveParticipant(null); // reset any set participant
    });
    test('insert final new line', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const model = instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, utils_1.toResource.call(this, '/path/final_new_line.txt'), 'utf8', undefined);
            yield model.load();
            const configService = new testConfigurationService_1.TestConfigurationService();
            configService.setUserConfiguration('files', { 'insertFinalNewline': true });
            const participant = new mainThreadSaveParticipant_1.FinalNewLineParticipant(configService, undefined);
            // No new line for empty lines
            let lineContent = '';
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), lineContent);
            // No new line if last line already empty
            lineContent = `Hello New Line${model.textEditorModel.getEOL()}`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), lineContent);
            // New empty line added (single line)
            lineContent = 'Hello New Line';
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${lineContent}${model.textEditorModel.getEOL()}`);
            // New empty line added (multi line)
            lineContent = `Hello New Line${model.textEditorModel.getEOL()}Hello New Line${model.textEditorModel.getEOL()}Hello New Line`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${lineContent}${model.textEditorModel.getEOL()}`);
        });
    });
    test('trim final new lines', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const model = instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, utils_1.toResource.call(this, '/path/trim_final_new_line.txt'), 'utf8', undefined);
            yield model.load();
            const configService = new testConfigurationService_1.TestConfigurationService();
            configService.setUserConfiguration('files', { 'trimFinalNewlines': true });
            const participant = new mainThreadSaveParticipant_1.TrimFinalNewLinesParticipant(configService, undefined);
            const textContent = 'Trim New Line';
            const eol = `${model.textEditorModel.getEOL()}`;
            // No new line removal if last line is not new line
            let lineContent = `${textContent}`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), lineContent);
            // No new line removal if last line is single new line
            lineContent = `${textContent}${eol}`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), lineContent);
            // Remove new line (single line with two new lines)
            lineContent = `${textContent}${eol}${eol}`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}${eol}`);
            // Remove new lines (multiple lines with multiple new lines)
            lineContent = `${textContent}${eol}${textContent}${eol}${eol}${eol}`;
            model.textEditorModel.setValue(lineContent);
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}${eol}${textContent}${eol}`);
        });
    });
    test('trim final new lines bug#39750', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const model = instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, utils_1.toResource.call(this, '/path/trim_final_new_line.txt'), 'utf8', undefined);
            yield model.load();
            const configService = new testConfigurationService_1.TestConfigurationService();
            configService.setUserConfiguration('files', { 'trimFinalNewlines': true });
            const participant = new mainThreadSaveParticipant_1.TrimFinalNewLinesParticipant(configService, undefined);
            const textContent = 'Trim New Line';
            // single line
            let lineContent = `${textContent}`;
            model.textEditorModel.setValue(lineContent);
            // apply edits and push to undo stack.
            let textEdits = [{ range: new range_1.Range(1, 14, 1, 14), text: '.', forceMoveMarkers: false }];
            model.textEditorModel.pushEditOperations([new selection_1.Selection(1, 14, 1, 14)], textEdits, () => { return [new selection_1.Selection(1, 15, 1, 15)]; });
            // undo
            model.textEditorModel.undo();
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}`);
            // trim final new lines should not mess the undo stack
            yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            model.textEditorModel.redo();
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}.`);
        });
    });
    test('trim final new lines bug#46075', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const model = instantiationService.createInstance(textFileEditorModel_1.TextFileEditorModel, utils_1.toResource.call(this, '/path/trim_final_new_line.txt'), 'utf8', undefined);
            yield model.load();
            const configService = new testConfigurationService_1.TestConfigurationService();
            configService.setUserConfiguration('files', { 'trimFinalNewlines': true });
            const participant = new mainThreadSaveParticipant_1.TrimFinalNewLinesParticipant(configService, undefined);
            const textContent = 'Test';
            const eol = `${model.textEditorModel.getEOL()}`;
            let content = `${textContent}${eol}${eol}`;
            model.textEditorModel.setValue(content);
            // save many times
            for (let i = 0; i < 10; i++) {
                yield participant.participate(model, { reason: 1 /* EXPLICIT */ });
            }
            // confirm trimming
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}${eol}`);
            // undo should go back to previous content immediately
            model.textEditorModel.undo();
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}${eol}${eol}`);
            model.textEditorModel.redo();
            assert.equal(textfiles_1.snapshotToString(model.createSnapshot()), `${textContent}${eol}`);
        });
    });
});
//# sourceMappingURL=mainThreadSaveParticipant.test.js.map