"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
//#region --- editor/workbench core
require("vs/editor/editor.all");
require("vs/workbench/api/browser/extensionHost.contribution");
require("vs/workbench/browser/workbench.contribution");
//#endregion
//#region --- workbench actions
require("vs/workbench/browser/actions/developerActions");
require("vs/workbench/browser/actions/helpActions");
require("vs/workbench/browser/actions/layoutActions");
require("vs/workbench/browser/actions/listCommands");
require("vs/workbench/browser/actions/navigationActions");
require("vs/workbench/browser/actions/windowActions");
require("vs/workbench/browser/actions/workspaceActions");
require("vs/workbench/browser/actions/workspaceCommands");
require("vs/workbench/browser/parts/quickopen/quickOpenActions");
require("vs/workbench/browser/parts/quickinput/quickInputActions");
//#endregion
//#region --- API Extension Points
require("vs/workbench/api/common/menusExtensionPoint");
require("vs/workbench/api/common/configurationExtensionPoint");
require("vs/workbench/api/browser/viewsExtensionPoint");
//#endregion
//#region --- workbench parts
require("vs/workbench/browser/parts/quickinput/quickInput");
require("vs/workbench/browser/parts/quickopen/quickOpenController");
require("vs/workbench/browser/parts/titlebar/titlebarPart");
require("vs/workbench/browser/parts/editor/editorPart");
require("vs/workbench/browser/parts/activitybar/activitybarPart");
require("vs/workbench/browser/parts/panel/panelPart");
require("vs/workbench/browser/parts/sidebar/sidebarPart");
require("vs/workbench/browser/parts/statusbar/statusbarPart");
require("vs/workbench/browser/parts/views/views");
//#endregion
//#region --- workbench services
require("vs/workbench/services/extensions/browser/extensionUrlHandler");
require("vs/workbench/services/bulkEdit/browser/bulkEditService");
require("vs/workbench/services/keybinding/common/keybindingEditing");
require("vs/workbench/services/decorations/browser/decorationsService");
require("vs/workbench/services/progress/browser/progressService");
require("vs/workbench/services/editor/browser/codeEditorService");
require("vs/workbench/services/preferences/browser/preferencesService");
require("vs/workbench/services/configuration/common/jsonEditingService");
require("vs/workbench/services/textmodelResolver/common/textModelResolverService");
require("vs/workbench/services/dialogs/browser/fileDialogService");
require("vs/workbench/services/editor/browser/editorService");
require("vs/workbench/services/history/browser/history");
require("vs/workbench/services/activity/browser/activityService");
require("vs/workbench/services/keybinding/browser/keybindingService");
require("vs/workbench/services/untitled/common/untitledEditorService");
require("vs/workbench/services/textfile/common/textResourcePropertiesService");
require("vs/workbench/services/mode/common/workbenchModeService");
require("vs/workbench/services/commands/common/commandService");
require("vs/workbench/services/themes/browser/workbenchThemeService");
require("vs/workbench/services/label/common/labelService");
require("vs/workbench/services/extensionManagement/common/extensionEnablementService");
require("vs/workbench/services/notification/common/notificationService");
require("vs/workbench/services/extensions/common/staticExtensions");
require("vs/workbench/services/userDataSync/common/settingsMergeService");
require("vs/workbench/services/workspace/browser/workspaceEditingService");
const extensions_1 = require("vs/platform/instantiation/common/extensions");
const extensionGalleryService_1 = require("vs/platform/extensionManagement/common/extensionGalleryService");
const extensionManagement_1 = require("vs/platform/extensionManagement/common/extensionManagement");
const contextViewService_1 = require("vs/platform/contextview/browser/contextViewService");
const contextView_1 = require("vs/platform/contextview/browser/contextView");
const listService_1 = require("vs/platform/list/browser/listService");
const editorWorkerService_1 = require("vs/editor/common/services/editorWorkerService");
const editorWorkerServiceImpl_1 = require("vs/editor/common/services/editorWorkerServiceImpl");
const markerDecorationsServiceImpl_1 = require("vs/editor/common/services/markerDecorationsServiceImpl");
const markersDecorationService_1 = require("vs/editor/common/services/markersDecorationService");
const markers_1 = require("vs/platform/markers/common/markers");
const markerService_1 = require("vs/platform/markers/common/markerService");
const contextKeyService_1 = require("vs/platform/contextkey/browser/contextKeyService");
const contextkey_1 = require("vs/platform/contextkey/common/contextkey");
const modelService_1 = require("vs/editor/common/services/modelService");
const modelServiceImpl_1 = require("vs/editor/common/services/modelServiceImpl");
const resourceConfiguration_1 = require("vs/editor/common/services/resourceConfiguration");
const resourceConfigurationImpl_1 = require("vs/editor/common/services/resourceConfigurationImpl");
const actions_1 = require("vs/platform/actions/common/actions");
const menuService_1 = require("vs/platform/actions/common/menuService");
const download_1 = require("vs/platform/download/common/download");
const downloadService_1 = require("vs/platform/download/common/downloadService");
const openerService_1 = require("vs/editor/browser/services/openerService");
const opener_1 = require("vs/platform/opener/common/opener");
extensions_1.registerSingleton(extensionManagement_1.IExtensionGalleryService, extensionGalleryService_1.ExtensionGalleryService, true);
extensions_1.registerSingleton(contextView_1.IContextViewService, contextViewService_1.ContextViewService, true);
extensions_1.registerSingleton(listService_1.IListService, listService_1.ListService, true);
extensions_1.registerSingleton(editorWorkerService_1.IEditorWorkerService, editorWorkerServiceImpl_1.EditorWorkerServiceImpl);
extensions_1.registerSingleton(markersDecorationService_1.IMarkerDecorationsService, markerDecorationsServiceImpl_1.MarkerDecorationsService);
extensions_1.registerSingleton(markers_1.IMarkerService, markerService_1.MarkerService, true);
extensions_1.registerSingleton(contextkey_1.IContextKeyService, contextKeyService_1.ContextKeyService);
extensions_1.registerSingleton(modelService_1.IModelService, modelServiceImpl_1.ModelServiceImpl, true);
extensions_1.registerSingleton(resourceConfiguration_1.ITextResourceConfigurationService, resourceConfigurationImpl_1.TextResourceConfigurationService);
extensions_1.registerSingleton(actions_1.IMenuService, menuService_1.MenuService, true);
extensions_1.registerSingleton(download_1.IDownloadService, downloadService_1.DownloadService, true);
extensions_1.registerSingleton(opener_1.IOpenerService, openerService_1.OpenerService, true);
//#endregion
//#region --- workbench contributions
// Workspace File Watching
require("vs/workbench/services/files/common/workspaceWatcher");
// Telemetry
require("vs/workbench/contrib/telemetry/browser/telemetry.contribution");
// Preferences
require("vs/workbench/contrib/preferences/browser/preferences.contribution");
require("vs/workbench/contrib/preferences/browser/keybindingsEditorContribution");
require("vs/workbench/contrib/preferences/browser/preferencesSearch");
// Logs
require("vs/workbench/contrib/logs/common/logs.contribution");
// Quick Open Handlers
require("vs/workbench/contrib/quickopen/browser/quickopen.contribution");
// Explorer
require("vs/workbench/contrib/files/browser/explorerViewlet");
require("vs/workbench/contrib/files/browser/fileActions.contribution");
require("vs/workbench/contrib/files/browser/files.contribution");
// Backup
require("vs/workbench/contrib/backup/common/backup.contribution");
// Search
require("vs/workbench/contrib/search/browser/search.contribution");
require("vs/workbench/contrib/search/browser/searchView");
require("vs/workbench/contrib/search/browser/openAnythingHandler");
// SCM
require("vs/workbench/contrib/scm/browser/scm.contribution");
require("vs/workbench/contrib/scm/browser/scmViewlet");
// Debug
require("vs/workbench/contrib/debug/browser/debug.contribution");
require("vs/workbench/contrib/debug/browser/debugQuickOpen");
require("vs/workbench/contrib/debug/browser/debugEditorContribution");
require("vs/workbench/contrib/debug/browser/breakpointEditorContribution");
require("vs/workbench/contrib/debug/browser/repl");
require("vs/workbench/contrib/debug/browser/debugViewlet");
// Markers
require("vs/workbench/contrib/markers/browser/markers.contribution");
// Comments
require("vs/workbench/contrib/comments/browser/comments.contribution");
// URL Support
require("vs/workbench/contrib/url/common/url.contribution");
// Webview
require("vs/workbench/contrib/webview/browser/webview.contribution");
require("vs/workbench/contrib/customEditor/browser/webviewEditor.contribution");
// Extensions Management
require("vs/workbench/contrib/extensions/browser/extensions.contribution");
require("vs/workbench/contrib/extensions/browser/extensionsQuickOpen");
require("vs/workbench/contrib/extensions/browser/extensionsViewlet");
// Output Panel
require("vs/workbench/contrib/output/browser/output.contribution");
require("vs/workbench/contrib/output/browser/outputPanel");
// Terminal
require("vs/workbench/contrib/terminal/browser/terminal.contribution");
require("vs/workbench/contrib/terminal/browser/terminalQuickOpen");
require("vs/workbench/contrib/terminal/browser/terminalPanel");
// Relauncher
require("vs/workbench/contrib/relauncher/browser/relauncher.contribution");
// Tasks
require("vs/workbench/contrib/tasks/browser/task.contribution");
// Remote
require("vs/workbench/contrib/remote/common/remote.contribution");
require("vs/workbench/contrib/remote/browser/remote");
// Emmet
require("vs/workbench/contrib/emmet/browser/emmet.contribution");
// CodeEditor Contributions
require("vs/workbench/contrib/codeEditor/browser/codeEditor.contribution");
// Execution
require("vs/workbench/contrib/externalTerminal/browser/externalTerminal.contribution");
// Snippets
require("vs/workbench/contrib/snippets/browser/snippets.contribution");
require("vs/workbench/contrib/snippets/browser/snippetsService");
require("vs/workbench/contrib/snippets/browser/insertSnippet");
require("vs/workbench/contrib/snippets/browser/configureSnippets");
require("vs/workbench/contrib/snippets/browser/tabCompletion");
// Formatter Help
require("vs/workbench/contrib/format/browser/format.contribution");
// Themes
require("vs/workbench/contrib/themes/browser/themes.contribution");
// Update
require("vs/workbench/contrib/update/browser/update.contribution");
// Watermark
require("vs/workbench/contrib/watermark/browser/watermark");
// Surveys
require("vs/workbench/contrib/surveys/browser/nps.contribution");
require("vs/workbench/contrib/surveys/browser/languageSurveys.contribution");
// Welcome
require("vs/workbench/contrib/welcome/overlay/browser/welcomeOverlay");
require("vs/workbench/contrib/welcome/page/browser/welcomePage.contribution");
require("vs/workbench/contrib/welcome/walkThrough/browser/walkThrough.contribution");
require("vs/workbench/contrib/welcome/gettingStarted/browser/gettingStarted.contribution");
// Call Hierarchy
require("vs/workbench/contrib/callHierarchy/browser/callHierarchy.contribution");
// Outline
require("vs/workbench/contrib/outline/browser/outline.contribution");
// Experiments
require("vs/workbench/contrib/experiments/browser/experiments.contribution");
// Send a Smile
require("vs/workbench/contrib/feedback/browser/feedback.contribution");
// User Data Sync
require("vs/workbench/contrib/userDataSync/browser/userDataSync.contribution");
//#endregion
//# sourceMappingURL=workbench.common.main.js.map